unit PatientSelectionDlg;

{ Patient Selection Package requires the following RPCS to be on your
  B-Type menu option:

    1. DVBAB TEAM PATIENTS
    2. DG SENSITIVE RECORD ACCESS DBIA #3402
    3. DG SENSITIVE RECORD BULLETIN DBIA #3403

  Patient Selection Package will Raise an exception if any of the following
  conditions are NOT met during creation:

  Exception: ERstrctdPtntError
    1. User has Restricted Patient Selection but is assigned no OE/RR List
    2. User has Restricted Patient Selection but has no patients in their list.
    3. RPC, DVBAB TEAM PATIENTS could not be accessed to check for restricted
       patient access.

  Exception: EPtntDFNInvalidError
    1. Patient DFN passed was not found in PATIENT File #2. Applicable for
       non-interactive patient lookups.

  Exception: EBrokerError
    1. Broker is NOT connected prior to calling Patient Selection.
       NOTE: Broker Object is required to have connection prior to calling API. }


interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Fmcntrls, FMcmpnts, CCOWRPCBroker, Diaccess, StrUtils,
  ExtCtrls, PatientSelectionMethods, TRPCB, clsPatientInfo, MFunStr, untMisc,
  VA508AccessibilityManager;

type
  {------ Custom Error Exceptions ------}
  ERstrctdPtntError = class(Exception);
  EPtntDFNInvalidError = class(Exception);
  TfrmPtSlctn = class(TForm)
    txtbxPtName: TEdit;
    fmLstBxPtnts: TFMListBox;
    fmLblPtntName: TFMLabel;
    btnOk: TButton;
    btnCancel: TButton;
    fmLstrPtnt: TFMLister;
    fmGetsPtntInfo: TFMGets;
    tmrPtntLkup: TTimer;
    tmrLstBxAutoCmplte: TTimer;
    fmGetsNewPrsnInfo: TFMGets;
    fmGetsPtntAddrssInfo: TFMGets;
    tmrEnsrBrkrCnnctd: TTimer;
    tmrWndwClose: TTimer;
    fmLstrCnfdntlAddrssCat: TFMLister;
    stlblOthrMsg: TStaticText;
    stlblVeteran: TStaticText;
    stlblSex: TStaticText;
    stlblDOB_Age: TStaticText;
    stlblSSN: TStaticText;
    stlblPtntName: TStaticText;
    Panel1: TPanel;
    Label1: TLabel;
    Panel2: TPanel;
    Label2: TLabel;
    fmGetsNmCmpnts: TFMGets;
    procedure fmLstBxPtntsClick(Sender: TObject);
    procedure txtbxPtNameKeyPress(Sender: TObject; var Key: Char);
    procedure txtbxPtNameKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure tmrPtntLkupTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lstBxProc(var tMsg: TMessage);
    procedure tmrLstBxAutoCmplteTimer(Sender: TObject);
    procedure fmLstBxPtntsKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure fmLstBxPtntsKeyPress(Sender: TObject; var Key: Char);
    procedure btnOkClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure fmLstBxPtntsDblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure txtbxPtNameEnter(Sender: TObject);
    procedure txtbxPtNameDblClick(Sender: TObject);
    procedure tmrWndwCloseTimer(Sender: TObject);
    procedure tmrEnsrBrkrCnnctdTimer(Sender: TObject);
  private
    { Private declarations }
    _CCOWRPCBrkr: TCCOWRPCBroker;
    _MnuOptn: String;  { Option Name^Menu test - Used if sensitive patient selected }
    _wndProc: TWndMethod;
    _lstBxSrch, _sensPtntMsg, _cnfdtnlAddrssCat: String;
    _lstBxTpIndx, _sensRslt: Integer;
    _dfltLstLoaded, _sensitivePtnt, _RstrctdUsr: Boolean;
    _SlctdPtnt: TPatientInfo;
    _RstrctdPtntLst: TStringList;
    procedure CheckRstrctdUser;
    procedure CreateRstrctdPtntList(Results: TStrings);
    procedure initBrkrCmpnts;
    procedure getPatientAddressInfo;
    procedure getPatients(ClrLists: Boolean; Indx, PrtStr: String);
    procedure getMorePtntRecs;
    procedure popPatientInfo(DFN: String);
    procedure resetPatientInfo;
    procedure ResetWndwTmr;
    procedure setupForm;
    function hasConfidentialCategory(Ctgry: String): Boolean;
    function isSensitiveRec(DFN: String): Integer;
    function isUserRstrctd(): Boolean;
    function UseTmpCnfdntlAddrssInfo(StartDte, EndDte: String): Boolean;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; RPCBrkr: TCCOWRPCBroker; TmrVal: Cardinal;
         MnuOptn: String; CnfAddCat: String); reintroduce;
    destructor Destroy; override;
    function GetSlctdPatientInfo(): TPatientInfo; overload;
    function GetSlctdPatientInfo(DFN: String): TPatientInfo; overload;
  end;

var
  frmPtSlctn: TfrmPtSlctn;

implementation

{$R *.dfm}

{ PUBLIC }

constructor TfrmPtSlctn.Create(AOwner: TComponent; RPCBrkr: TCCOWRPCBroker;
     TmrVal: Cardinal; MnuOptn: String; CnfAddCat: String);
begin
  inherited Create(AOwner);
  _CCOWRPCBrkr := RPCBrkr;
  _MnuOptn := MnuOptn;
  _wndProc := fmLstBxPtnts.WindowProc;
  { Addtnl Restriction on when to use Active Confidential Address - Based on Category }
  _cnfdtnlAddrssCat := CnfAddCat;
  fmLstBxPtnts.WindowProc := lstBxProc;
  tmrWndwClose.Interval := (TmrVal - 60000);  { Gives time for calling app to exit }
  setupForm;
end;

destructor TfrmPtSlctn.Destroy;
begin
  try
    _SlctdPtnt.Free;
  except  { Do Nothing on Exceptions }
  end;
  inherited;
end;

function TfrmPtSlctn.GetSlctdPatientInfo(): TPatientInfo;
begin
  Result := _SlctdPtnt;
end;

function TfrmPtSlctn.GetSlctdPatientInfo(DFN: string): TPatientInfo;
{ Allows retrieval of Patient Info without displaying the dialog or
  user interaction. }
begin
  popPatientInfo(DFN);  { Create Patient Object }
  getPatientAddressInfo;  { Populate Patient with Address Info }
  Result := _SlctdPtnt;
end;

{ EVENTS }

procedure TfrmPtSlctn.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  _RstrctdPtntLst.Free;
end;

procedure TfrmPtSlctn.FormShow(Sender: TObject);
begin
  ResetWndwTmr;
  if Not _RstrctdUsr then
    getPatients(True,'','');
  txtbxPtName.SetFocus;
end;

procedure TfrmPtSlctn.fmLstBxPtntsClick(Sender: TObject);
var
  DFN: String;
begin
  ResetWndwTmr;
  if (fmLstBxPtnts.ItemIndex <> -1) then
  begin
    txtbxPtName.Text := fmLstBxPtnts.Items[fmLstBxPtnts.ItemIndex];
    if _RstrctdUsr then  { Determine Patient DFN for selected record }
      DFN := _RstrctdPtntLst.Values[IntToStr(fmLstBxPtnts.ItemIndex)]
    else
      DFN := fmLstBxPtnts.GetSelectedRecord.IEN;
    { Only update if record changes to reduce screen flickering }
    if Not SameText(DFN, fmLblPtntName.IENS) then
      popPatientInfo(DFN);  { Get Patient Info }
  end;  { End If ItemIndex Not -1 }
end;

procedure TfrmPtSlctn.fmLstBxPtntsDblClick(Sender: TObject);
begin
  ResetWndwTmr;
  if (fmLstBxPtnts.ItemIndex <> -1) then
  begin
    fmLstBxPtntsClick(self);
    btnOkClick(self);
  end;  { End If Item Index is not -1 }
end;

procedure TfrmPtSlctn.fmLstBxPtntsKeyPress(Sender: TObject; var Key: Char);
var
 ordKey: Integer;
begin
  ResetWndwTmr;
  tmrLstBxAutoCmplte.Enabled := False;
  ordKey := Ord(Key);
  if ((ordKey < 32) or (ordKey > 126)) then Exit;
  if (ordKey = 32) and SameText(_lstBxSrch,'') then exit;  { No Space in first position }
  if (ordKey = 8) then  { BackSpace Key Pressed }
    _lstBxSrch := Copy(_lstBxSrch, 1, Length(_lstBxSrch)-1)
  else if ((ordKey > 31) and (ordKey < 127)) then
    _lstBxSrch := _lstBxSrch + Key;
  txtbxPtName.Text := _lstBxSrch;
  if (Not SameText(_lstBxSrch,'')) then
    tmrLstBxAutoCmplte.Enabled := True
  else if (Not _dfltLstLoaded)  then
    GetPatients(True,'','');  { Restore initial list if not already loaded }
end;

procedure TfrmPtSlctn.fmLstBxPtntsKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_DOWN) or (Key = VK_NEXT) then
  begin
    getMorePtntRecs;
    exit;
  end;  { End If Arrow Down or Page Down Keys Pressed }
end;

procedure TfrmPtSlctn.lstBxProc(var tMsg: TMessage);
begin
  _wndProc(tMsg);
 if ((tMsg.Msg = WM_VSCROLL) or (tMsg.Msg = WM_MOUSEWHEEL)) then  { Display more records if applicable }
   getMorePtntRecs;
end;

procedure TfrmPtSlctn.tmrEnsrBrkrCnnctdTimer(Sender: TObject);
begin
  { Close Window if Broker cannot be reconnected }
  if Not EnsureBrokerConnected(_CCOWRPCBrkr) then
    Close;
end;

procedure TfrmPtSlctn.tmrLstBxAutoCmplteTimer(Sender: TObject);
var
  indx: Integer;
begin
  tmrLstBxAutoCmplte.Enabled := False;
  _lstBxSrch := '';
  if Not SameText(txtbxPtName.Text , '') then
  begin
    { Check to see if patient already in list when typing in listbox,
      check is not done in tmrPtntLkupTimer because 'B' X-Ref returns
      aliases which doesn't allow list to grow from A-Z incrementally }
    indx := findPatientIndex(txtbxPtName.Text, fmLstBxPtnts);
    if (indx = -1) then
      txtbxPtName.Perform(WM_KEYUP,VK_RETURN,0)
    else
    begin
      fmLstBxPtnts.ItemIndex := indx;
      fmLstBxPtntsClick(self);
      fmLstBxPtnts.SetFocus;  
    end;  { End If Index found for item }
  end;  { End If Search Val Exists }
end;

procedure TfrmPtSlctn.tmrPtntLkupTimer(Sender: TObject);
var
 ptLkUpVal,indx: String;
begin
  indx := '';
  ptLkUpVal := txtbxPtName.Text;
  tmrPtntLkUp.Enabled := False;
  if isLookUpByIEN(ptLkUpVal) then  { Remove tick for IEN lookup }
  begin
    ptLkUpVal := Copy(ptLkUpVal, 2, Length(ptLkUpVal)-1);
    fmLstBxPtnts.ItemIndex := -1;
    fmGetsPtntInfo.IENS := ptLkUpVal;
    fmGetsPtntInfo.GetData;
    try
      ptLkUpVal := fmGetsPtntInfo.GetField('.09').FMDBInternal;  { SSN Value }
    except
      fmLstBxPtnts.Items.Clear;  { Clear to indicate no results found }
      _dfltLstLoaded := False;  { Default values not loaded }
      ptLkUpVal := '';  { Patient didn't exist, set null to prevent lister call }
    end;  { End Try/Except }
  end;  { End If Lookup By IEN - Retrieve SSN  }
  if isLookUpBySSN(ptLkUpVal) then
    indx := 'SSN'
  else if isLookUpByL5(ptLkUpVal) then
  begin
    if (Length(ptLkUpVal) = 4) then
      indx := 'BS'    { Last 4 of SSN }
    else
      indx := 'BS5';  { First Letter of Last Name and Last 4 of SSN }
  end;  { End If LookUp By Last 4 }
  if Not SameText(ptLkUpVal,'') then
  begin
    getPatients(True, indx, ptLkUpVal);
    if fmLstBxPtnts.Enabled then
    begin
      fmLstBxPtnts.ItemIndex := findPatientIndex(txtbxPtName.Text, fmLstBxPtnts);
      if (fmLstBxPtnts.ItemIndex = -1) then
        fmLstBxPtnts.ItemIndex := 0;
      fmLstBxPtntsClick(self);  { Display Patient Info }
      fmLstBxPtnts.SetFocus;
    end;  { End If List Box has Items }
  end;  { End If Patient Search Value is not null }
  if (SameText(ptLkUpVal, '') or (Not fmLstBxPtnts.Enabled)) then
  begin
    txtbxPtName.SelectAll;
    txtbxPtName.SetFocus;
  end;  { End If No Search Val or No Items in List Box }
end;

procedure TfrmPtSlctn.tmrWndwCloseTimer(Sender: TObject);
begin
  { Shutdown Patient Selection window to allow app to close -
    tmr is set just short of Main Application's Shutdown Timer value }
  Close;
end;

procedure TfrmPtSlctn.txtbxPtNameDblClick(Sender: TObject);
begin
  ResetWndwTmr;
  txtbxPtName.SelectAll;
end;

procedure TfrmPtSlctn.txtbxPtNameEnter(Sender: TObject);
begin
  txtbxPtName.SelectAll;
end;

procedure TfrmPtSlctn.txtbxPtNameKeyPress(Sender: TObject; var Key: Char);
begin
  ResetWndwTmr;
  if SameText(txtbxPtName.Text,'') then
  begin
    if (Key = char(32)) then
      Key := char(0)  { Don't Allow space in first position }
  end  { End If No Text tringList }
  else
  begin
    // Don't allow a space after a comma to help in picking a patient
    If ((Copy(txtbxPtName.Text, Length(txtbxPtName.Text), 1) = ',') And (key = ' ')) Then
      key := char(0);
    if key = char(13) Then
    begin
      tmrPtntLkupTimer(self);
      key := char(0);
    end;  { End If Carriage Return Entered }
  end;  { End if Text Entered }
end;

procedure TfrmPtSlctn.txtbxPtNameKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  tmrPtntLkUp.Enabled := False;
  { Enable timer based on type of search }
  if (isLookUpByIEN(txtbxPtName.Text) and (Length(txtbxPtName.Text) > 1)) then
    tmrPtntLkUp.Enabled := True
  else if isLookUpBySSN(txtbxPtName.Text) then
    tmrPtntLkUp.Enabled := True
  else if isLookUpByL5(txtbxPtName.Text) then
    tmrPtntLkUp.Enabled := True
  else if Length(txtbxPtName.Text) > 2 then
    tmrPtntLkUp.Enabled := True;
  if (SameText(txtbxPtName.Text, '') and (Not _dfltLstLoaded))  then
    GetPatients(True,'','');  { Restore initial list if not already loaded }
end;

procedure TfrmPtSlctn.btnOkClick(Sender: TObject);
var
  msgBffr: Array[0..7000] Of char;
  msg: String;
  err: Boolean;
begin
  ResetWndwTmr;
  getPatientAddressInfo;  { Retrieve/Store Patient Address Info }
  if _sensitivePtnt then
  begin
    _sensPtntMsg := _SensPtntMsg + #13#10 + 'Do you wish to continue?';
    StrPCopy(msgBffr, _sensPtntMsg);
    { Ensure user wants to continue with sensitive record }
    if Application.MessageBox(msgBffr, 'Sensitive Record', MB_YESNO) = IDYES  then
    begin
      if (_sensRslt = 2) then
      begin
        err := False;
        { Supported RPC: DBIA #3403 }
        _CCOWRPCBrkr.RemoteProcedure := 'DG SENSITIVE RECORD BULLETIN';
        _CCOWRPCBrkr.Param[1].Value := _SlctdPtnt.PatientDFN;
        _CCOWRPCBrkr.Param[1].PType := literal;
        _CCOWRPCBrkr.Param[2].Value := _MnuOptn;
        _CCOWRPCBrkr.Param[2].PType := literal;
        try
          _CCOWRPCBrkr.Call;
          if SameText(_CCOWRPCBrkr.Results[0], '0') then
            err := True
        except
          On EBrokerError Do
            err := True;
        end;  { End Try/Except }
        if err then
          Application.MessageBox('There was a problem generating the DG SENSITIVE RECORD e-mail.' +
                 #13#10#13#10 + 'Please notify IRM.', 'RPC Error', MB_OK);
      end;  { Record Sensitive Patient Access in Security Log and generate msg }
    end { End If User wants to select sensitive patient record }
    else
      exit;
  end;  { End If Sensitive Patient }
  if _SlctdPtnt.IsPatientDeceased() then
  begin
    msg := 'This patient died on ' + _SlctdPtnt.PatientDOD +
             #13#10 + 'Do you wish to continue?';
    StrPCopy(msgBffr, msg);
    if Application.MessageBox(msgBffr, 'Deceased Patient', MB_YESNO) = IDNO then
      exit;
  end;  { End If Patient is deceased }
  ModalResult := mrOK;
end;

procedure TfrmPtSlctn.btnCancelClick(Sender: TObject);
begin
  _SlctdPtnt.Free;  { Ensures nothing returned when GetSlctdPatientInfo called }
end;

{ PRIVATE Functions / Procedures }

procedure TfrmPtSlctn.initBrkrCmpnts;
begin
  fmLstrPtnt.RPCBroker := _CCOWRPCBrkr;
  fmGetsPtntInfo.RPCBroker := _CCOWRPCBrkr;
  fmGetsNewPrsnInfo.RPCBroker := _CCOWRPCBrkr;
  fmGetsPtntAddrssInfo.RPCBroker := _CCOWRPCBrkr;
  fmLstrCnfdntlAddrssCat.RPCBroker := _CCOWRPCBrkr;
  fmGetsNmCmpnts.RPCBroker := _CCOWRPCBrkr;
end;

procedure TfrmPtSlctn.getPatientAddressInfo;
begin
  { Get Address Info for Selected Patient }
  fmGetsPtntAddrssInfo.IENS := _SlctdPtnt.PatientDFN;
  fmGetsPtntAddrssInfo.GetData;
  { Determine if Confidential or Temporary Address should be used }
  if (UseTmpCnfdntlAddrssInfo(fmGetsPtntAddrssInfo.GetField('.1417').FMDBInternal,
       fmGetsPtntAddrssInfo.GetField('.1418').FMDBInternal) and
       hasConfidentialCategory(_cnfdtnlAddrssCat)) then
    with _SlctdPtnt do
    begin
      PatientAddress1 := fmGetsPtntAddrssInfo.GetField('.1411').FMDBExternal;
      PatientAddress2 := fmGetsPtntAddrssInfo.GetField('.1412').FMDBExternal;
      PatientAddress3 := fmGetsPtntAddrssInfo.GetField('.1413').FMDBExternal;
      PatientAddressType := 'C';
      PatientCity := fmGetsPtntAddrssInfo.GetField('.1414').FMDBExternal;
      if Not SameText(fmGetsPtntAddrssInfo.GetField('.1415').FMDBExternal, '') then
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.1415').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.1416').FMDBExternal;
      end  { End USA Address Info }
      else
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.14114').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.14115').FMDBExternal;
        PatientCountry := fmGetsPtntAddrssInfo.GetField('.14116').FMDBExternal;
      end;  { End Non-USA Address Info }
    end  { End Confidential Patient Address Info }
  else if UseTmpCnfdntlAddrssInfo(fmGetsPtntAddrssInfo.GetField('.1217').FMDBInternal,
       fmGetsPtntAddrssInfo.GetField('.1218').FMDBInternal) then
    with _SlctdPtnt do
    begin
      PatientAddress1 := fmGetsPtntAddrssInfo.GetField('.1211').FMDBExternal;
      PatientAddress2 := fmGetsPtntAddrssInfo.GetField('.1212').FMDBExternal;
      PatientAddress3 := fmGetsPtntAddrssInfo.GetField('.1213').FMDBExternal;
      PatientAddressType := 'T';
      PatientCity := fmGetsPtntAddrssInfo.GetField('.1214').FMDBExternal;
      if Not SameText(fmGetsPtntAddrssInfo.GetField('.1215').FMDBExternal, '') then
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.1215').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.1216').FMDBExternal;
      end  { End USA Address Info }
      else
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.1221').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.1222').FMDBExternal;
        PatientCountry := fmGetsPtntAddrssInfo.GetField('.1223').FMDBExternal;
      end;  { End Non-USA Address Info }
      PatientHomePhone := fmGetsPtntAddrssInfo.GetField('.1219').FMDBExternal;
    end  { End Temporary Patient Address Info }
  else  { Use Permanent Address Info }
    with _SlctdPtnt do
    begin
      PatientAddress1 := fmGetsPtntAddrssInfo.GetField('.111').FMDBExternal;
      PatientAddress2 := fmGetsPtntAddrssInfo.GetField('.112').FMDBExternal;
      PatientAddress3 := fmGetsPtntAddrssInfo.GetField('.113').FMDBExternal;
      PatientAddressType := 'P';
      PatientCity := fmGetsPtntAddrssInfo.GetField('.114').FMDBExternal;
      if Not SameText(fmGetsPtntAddrssInfo.GetField('.115').FMDBExternal, '') then
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.115').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.116').FMDBExternal;
      end  { End USA Address Info }
      else
      begin
        PatientStateOrProvince := fmGetsPtntAddrssInfo.GetField('.1171').FMDBExternal;
        PatientZip := fmGetsPtntAddrssInfo.GetField('.1172').FMDBExternal;
        PatientCountry := fmGetsPtntAddrssInfo.GetField('.1173').FMDBExternal;
      end;  { End Non-USA Address Info }
    end;  { End Permanent Patient Address Info }
    { Use Permanent Phone Number if No temporary phone number assigned }
    if SameText(_SlctdPtnt.PatientHomePhone, '') then
      _SlctdPtnt.PatientHomePhone := fmGetsPtntAddrssInfo.GetField('.131').FMDBExternal;
    _SlctdPtnt.PatientCellPhone := fmGetsPtntAddrssInfo.GetField('.134').FMDBExternal;
    _SlctdPtnt.PatientBadAddressType := fmGetsPtntAddrssInfo.GetField('.121').FMDBExternal;
end;

function TfrmPtSlctn.UseTmpCnfdntlAddrssInfo(StartDte: string; EndDte: string): Boolean;
var
  cDte, sDte, eDte: TDateTime;
begin
  Result := True;
  cDte := Date;  { Get Current System Date }
  if (SameText(StartDte, '') and SameText(EndDte, '')) then
    Result := False;
  if Not SameText(StartDte, '') then
    try
      sDte := StrToDate(FMInternalToDelphiDate(StartDte));
      if (cDte < sDte) then
        Result := False;
    except  { Do Nothing on EConvertErrors }
    end;  { End Try/Except Block }
  if (Result and Not SameText(EndDte, '')) then
    try
      eDte := StrToDate(FMInternalToDelphiDate(EndDte));
      if (cDte > eDte) then
        Result := False;
    except  { Do Nothing on EConvertErrors }
    end;  { End Try/Except Block }
end;

procedure TfrmPtSlctn.getPatients(ClrLists: Boolean; Indx,PrtStr: String);
begin
  if ClrLists then
  begin
    fmLstBxPtnts.Items.Clear;
    fmLstrPtnt.Results.Clear;
    _lstBxTpIndx := 0;
  end;  { End If Lists should be cleared }
  fmLstrPtnt.FMIndex := Indx;
  fmLstrPtnt.PartList.Clear;
  fmLstrPtnt.PartList.Add(UpperCase(PrtStr));
  fmLstBxPtnts.GetList;
  _dfltLstLoaded := SameText(PrtStr,'');
  fmLstBxPtnts.Enabled := (fmLstBxPtnts.Items.Count > 0);
  { Set index to 1st item in list when no Partial String to Search on
    (Default List - Direct Call), other calls Index is set in tmrPtntLkupTimer }
  if (fmLstBxPtnts.Enabled and SameText(PrtStr, '')) then
  begin
    fmLstBxPtnts.ItemIndex := 0;
    fmLstBxPtntsClick(self);  { Display Patient Info }
    fmLstBxPtnts.SetFocus;
  end;  { End If List Box Enabled }
end;

procedure TfrmPtSlctn.getMorePtntRecs;
var
  vsblItms: Integer;
begin
  if _lstBxTpIndx <> fmLstBxPtnts.TopIndex then
  begin
    _lstBxTpIndx := fmLstBxPtnts.TopIndex;
    vsblItms := fmLstBxPtnts.Height Div fmLstBxPtnts.ItemHeight;
    if (vsblItms * 2 + 1) + fmLstBxPtnts.TopIndex >= fmLstBxPtnts.Items.Count then
      if fmLstrPtnt.More then
      begin
        fmLstrPtnt.GetMore(fmLstBxPtnts.Items);
        fmLstBxPtnts.SortResults;
      end;
  end;  { End if different Top Index }
end;

procedure TfrmPtSlctn.resetPatientInfo;
begin
  fmLblPtntName.Caption := '';
  stlblPtntName.Caption := '';
  stlblSSN.Caption := '';
  stlblDOB_Age.Caption := '';
  stlblSex.Caption := '';
  stlblVeteran.Caption := '';
end;

procedure TfrmPtSlctn.popPatientInfo(DFN: String);
var
  fldVal: String;
  msgBffr: Array[0..7000] Of char;
begin
  _SlctdPtnt.Free;
  resetPatientInfo;  { Clear Old Data before displaying new patient info }
  stlblOthrMsg.Visible := False;
  stlblOthrMsg.Caption := '';
  if Not SameText(DFN,'') then
  begin
    _sensRslt := isSensitiveRec(DFN);  { Determine if Patients Rec Sensitive }
    if ((_sensRslt < 0) or (_sensRslt > 2)) then
    begin
      btnOk.Enabled := False;
      if Not (_sensRslt < 0) then
      begin
        StrPCopy(msgBffr, _sensPtntMsg);
        Application.MessageBox(msgBffr, 'Access Denied', MB_OK);
      end;  { End If Access Own Record or SSN missing from Record }
      exit;
    end;  { End If Access Denied to Patient Record }
    fmGetsPtntInfo.IENS := DFN;
    fmGetsPtntInfo.GetAndFill;  { Only Populates Name, label is hidden }
    stlblPtntName.Caption := fmLblPtntName.Caption; { Name displayed in 508 Static Text }
    { Ensure DFN Exists - applicable when DFN Passed into Dialog (Non-Interactive) }
    if fmGetsPtntInfo.Results.Count = 0 then
      raise EPtntDFNInvalidError.Create('Patient DFN, ' + DFN + ', is invalid.');
    if (_sensRslt > 0) then
    begin
      _sensitivePtnt := True;
      stlblOthrMsg.Caption := 'Sensitive Record';
      stlblSSN.Visible := False;
      stlblDOB_Age.Visible := False;
      stlblSex.Visible := False;
      stlblVeteran.Visible := False;
    end  { End If Patient Record Sensitive }
    else
    begin
      { Manually populate most fields as manipulation required}
      fldVal := fmGetsPtntInfo.GetField('.09').FMDBInternal;
      if Not SameText(fldVal,'') then
        stlblSSN.Caption := MidStr(fldVal,1,3) + '-' + MidStr(fldVal,4,2) + '-' +
                           MidStr(fldVal,6,Length(fldval));
      stlblSSN.Visible := Not SameText(fldVal,'');
      fldVal := fmGetsPtntInfo.GetField('.03').FMDBExternal;
      if Not SameText(fldVal,'') then
        stlblDOB_Age.Caption :=  fldVal + ' (' +
                                fmGetsPtntInfo.GetField('.033').FMDBInternal + ')';
      stlblDOB_Age.Visible := Not SameText(fldVal,'');
      fldVal := fmGetsPtntInfo.GetField('.02').FMDBExternal;
      if Not SameText(fldVal,'') then
        stlblSex.Caption := Uppercase(MidStr(fldVal,1,1)) +
                           Lowercase(MidStr(fldVal,2,Length(fldVal)));
      stlblSex.Visible := Not SameText(fldVal,'');
      if SameText(UpperCase(fmGetsPtntInfo.GetField('1901').FMDBInternal),'Y') then
        stlblVeteran.Caption := 'Veteran';
      stlblVeteran.Visible := SameText(UpperCase(fmGetsPtntInfo.GetField('1901').FMDBInternal),'Y');
      { Patient Deceased Check - Display if not sensitive record }
      if (Not SameText(fmGetsPtntInfo.GetField('.351').FMDBExternal, '') and
           SameText(stlblOthrMsg.Caption,'')) then
        stlblOthrMsg.Caption := 'Patient is deceased';
    end;  { End If Patient Record NOT Sensitive }
    stlblOthrMsg.Visible := (Not SameText(stlblOthrMsg.Caption,''));
    btnOk.Enabled := True;  { At this point record is known to exist }
    { Create Patient Object }
    _SlctdPtnt := TPatientInfo.Create(DFN,
                                      fmGetsPtntInfo.GetField('.01').FMDBInternal,
                                      fmGetsPtntInfo.GetField('.09').FMDBInternal,
                                      fmGetsPtntInfo.GetField('991.01').FMDBInternal,
                                      fmGetsPtntInfo.GetField('.03').FMDBExternal,
                                      fmGetsPtntInfo.GetField('.033').FMDBInternal,
                                      fmGetsPtntInfo.GetField('.02').FMDBExternal,
                                      fmGetsPtntInfo.GetField('.351').FMDBExternal,
                                      fmGetsPtntInfo.GetField('.313').FMDBExternal,
                                      fmGetsPtntInfo.GetField('1.01').FMDBInternal,
                                      fmGetsPtntInfo.GetField('.121').FMDBInternal,
                                      SameText(fmGetsPtntInfo.GetField('1901').FMDBInternal,'Y'));
    if (Length(_SlctdPtnt.PatientNameComponents) > 1) then
    begin
      fmGetsNmCmpnts.IENS := _SlctdPtnt.PatientNameComponents;
      fmGetsNmCmpnts.GetData;
      _SlctdPtnt.PatientLastName := fmGetsNmCmpnts.GetField('1').FMDBExternal;
      _SlctdPtnt.PatientFirstName := fmGetsNmCmpnts.GetField('2').FMDBExternal;
      _SlctdPtnt.PatientMiddleName := fmGetsNmCmpnts.GetField('3').FMDBExternal;
    end;

  end;  { End If Patient IEN exists }
end;

procedure TfrmPtSlctn.setupForm;
begin
  _lstBxSrch := '';
  _dfltLstLoaded := False;
  fmLstBxPtnts.Items.Clear;
  initBrkrCmpnts;  { Associate Passed Broker with Broker enabled components }
  resetPatientInfo;  { Clear Patient Info }
  if isUserRstrctd() then  { Restricted Patient List Checks }
  begin
    CheckRstrctdUser;
    Caption := 'Restricted Patient Selection';
  end;  { End If Restricted User }
end;

function TfrmPtSlctn.isSensitiveRec(DFN: String): Integer;
{ Returns -1 if Error
           0 if NOT Sensitive Record
           1 if Sensitive Record
           2 if Sensitive Record Display Warning
           3 if Employees Record Deny Access
           4 if SSN undefiend Deny Access }
var
  i: Integer;
begin
  _sensitivePtnt := False;
  _sensPtntMsg := '';
  { Supported RPC: DBIA #3402 }
  _CCOWRPCBrkr.RemoteProcedure := 'DG SENSITIVE RECORD ACCESS';
  _CCOWRPCBrkr.Param[0].Value := DFN;
  _CCOWRPCBrkr.Param[0].PType := literal;
  try
    _CCOWRPCBrkr.Call;
    Result := StrToInt(_CCOWRPCBrkr.Results[0]);
    if (Result = -1) then
      Application.MessageBox('DFN for patient or DUZ for user was not defined, or the RPC failed.',
                             'RPC Error', MB_OK);
    { Save RPC Message Off for display later if applicable }
    if _CCOWRPCBrkr.Results.Count > 0 then
      for i := 1 to _CCOWRPCBrkr.Results.Count - 1 do
        _sensPtntMsg := _sensPtntMsg + _CCOWRPCBrkr.Results[i] + Char(13);
  except
    On EBrokerError Do
    begin
      Application.MessageBox('RPC: DG SENSITIVE RECORD ACCESS could not be accessed!',
                             'Broker Error', MB_OK);
      Result := -1;
    end;  { End EBrokerError }
  end;  { End Try/Except }
end;

function TfrmPtSlctn.isUserRstrctd(): Boolean;
begin
  fmGetsNewPrsnInfo.IENS := _CCOWRPCBrkr.User.DUZ;
  fmGetsNewPrsnInfo.GetData;
  _RstrctdUsr := SameText(fmGetsNewPrsnInfo.GetField('101.01').FMDBInternal, '1');
  Result := _RstrctdUsr;
end;

function TfrmPtSlctn.hasConfidentialCategory(Ctgry: string): Boolean;
var
  i: Integer;
  IEN: String;
begin
  Result := True;
  { Perform Additional Restriction Check on Confidential Address based on
    the Requested Category that needs to be active. }
  if (Not SameText(Ctgry, '')) then
  begin
    Result := False;
    fmLstrCnfdntlAddrssCat.IENS := ',' + _SlctdPtnt.PatientDFN + ',';
    fmLstrCnfdntlAddrssCat.Screen := 'I $P($G(^(0)),"^")="' + Ctgry + '"';
    fmLstrCnfdntlAddrssCat.GetList(nil);
    for i := 0 to fmLstrCnfdntlAddrssCat.Results.Count - 1 do
    begin
      IEN := fmLstrCnfdntlAddrssCat.Results[i];
      Result := SameText(UpperCase(fmLstrCnfdntlAddrssCat.GetRecord(IEN).
           GetField('1').FMDBExternal), 'Y');
    end;  { End Results For Loop }
  end;  { End If Category Not Null }
end;

procedure TfrmPtSlctn.CheckRstrctdUser;
begin
  if SameText(fmGetsNewPrsnInfo.GetField('101.02').FMDBInternal, '') then
    raise ERstrctdPtntError.Create('You are restricted to a set of patients, but have no patient selection list.');
  { Ensure user has patients in restricted list }
  _CCOWRPCBrkr.RemoteProcedure := 'DVBAB TEAM PATIENTS';
  _CCOWRPCBrkr.Param[0].Value := fmGetsNewPrsnInfo.GetField('101.02').FMDBInternal;
  _CCOWRPCBrkr.Param[0].PType := literal;
  try
    _CCOWRPCBrkr.Call;
    if ((_CCOWRPCBrkr.Results.Count = 0) or
        (SameText(_CCOWRPCBrkr.Results[0], '^NO PATIENTS FOUND.'))) then
      raise ERstrctdPtntError.Create('Your restricted patient list is empty.')
    else
      CreateRstrctdPtntList(_CCOWRPCBrkr.Results);
  except
    On EBrokerError Do
      raise ERstrctdPtntError.Create('RPC: DVBAB TEAM PATIENTS could not be accessed!');
  end;  { End Try/Except }
end;

procedure TfrmPtSlctn.CreateRstrctdPtntList(Results: TStrings);
var
  i: Integer;
  tmpLst: TStringList;
begin
  tmpLst := TStringList.Create;
  tmpLst.Sorted := True;
  _RstrctdPtntLst := TStringList.Create;
  { Sort Restricted Patient List by Name }
  for i := 0 to Results.Count - 1 do
    tmpLst.Add(Piece(Results[i], '^', 2) + '^' + Piece(Results[i], '^', 1));
  { Add sorted entries to Patient Selection List Box and Create Hash Table Lkup }
  for i := 0 to tmpLst.Count - 1 do
  begin
    fmLstBxPtnts.Items.Add(Piece(tmpLst[i], '^', 1));
    { TStringList Used as HashTable for lookups Indx corresponds to DFN }
    _RstrctdPtntLst.Values[IntToStr(i)] := Piece(tmpLst[i], '^', 2);
  end;  { End For Loop }
  tmpLst.Free;
end;

procedure TfrmPtSlctn.ResetWndwTmr;
begin
  tmrWndwClose.Enabled := False;
  tmrEnsrBrkrCnnctd.Enabled := False;
  tmrWndwClose.Enabled := True;
  tmrEnsrBrkrCnnctd.Enabled := True;
end;

end.
