unit clsPatientInfo;

interface

uses SysUtils, StrUtils;

type
  TPatientInfo = class(TObject)
    private
    { Private Declarations }
    _DFN, _Name, _SSN
    , _ICN                            // CodeCR570: DONE: LMS - Added for VLER communication
    , _DOB, _AGE, _SEX, _DOD, _ClaimNum: String;
    _NameComponents, _Last, _First, _Middle: String;
    _Addrss1, _Addrss2, _Addrss3, _City, _StateProv, _Zip, _Cntry: String;
    _HomePhone, _CellPhone, _BadAddressType, _AddrssType: String;
    _VTRN: Boolean;
    function SntceCase(val: String): String;
    public
    { Public Declarations }
    constructor Create(DFN, Name, SSN
         , ICN                        // CodeCR570: DONE: LMS - Added for VLER communication
         , DOB, AGE, SEX, DOD, ClaimNum,
         NameComponents, BadAddressType: String;
         VTRN: Boolean); overload;  { Allow for Empty/Default Constructor }
    procedure CopyFrom(APtnt: TPatientInfo);
    property PatientDFN: String read _DFN;
    property PatientNameComponents: String read _NameComponents;
    property PatientLastName: String read _Last write _Last;
    property PatientFirstName: String read _First write _First;
    property PatientMiddleName: String read _Middle write _Middle;
    property PatientName: String read _Name;
    property PatientSSN: String read _SSN;
    property PatientICN: String read _ICN;         // CodeCR570: DONE: LMS - Added for VLER communication
    property PatientDOB: String read _DOB;
    property PatientAGE: String read _AGE;
    property PatientSEX: String read _SEX;
    property PatientVeteran: Boolean read _VTRN;
    property PatientDOD: String read _DOD;
    property PatientClaimFolderNumber: String read _ClaimNum;
    property PatientAddress1: String read _Addrss1 write _Addrss1;
    property PatientAddress2: String read _Addrss2 write _Addrss2;
    property PatientAddress3: String read _Addrss3 write _Addrss3;
    { P: Permanent, C: Confidential, or T: Temporary Address Loaded }
    property PatientAddressType: String read _AddrssType write _AddrssType;
    property PatientCity: String read _City write _City;
    property PatientStateOrProvince: String read _StateProv write _StateProv;
    property PatientZip: String read _Zip write _Zip;
    property PatientCountry: String read _Cntry write _Cntry;
    property PatientHomePhone: String read _HomePhone write _HomePhone;
    property PatientCellPhone: String read _CellPhone write _CellPhone;
    property PatientBadAddressType: String read _BadAddressType write _BadAddressType;
    function HasBadAddress(): Boolean;
    function IsPatientDeceased(): Boolean;
    function GetExternalSSN(): String;
  end;

implementation

constructor TPatientInfo.Create(DFN: String; Name: string; SSN: string; ICN: string;
                                 DOB: string; AGE: String; SEX: string;
                                 DOD: String; ClaimNum,
                                 NameComponents, BadAddressType: String; VTRN: Boolean);
begin
  inherited Create;
  _DFN := DFN;
  _Name := Name;
  _NameComponents := NameComponents;
  _SSN := SSN;
  _ICN := ICN;
  _DOB := DOB;
  _AGE := AGE;
  _SEX := SntceCase(SEX);
  _DOD := DOD;
  _ClaimNum := ClaimNum;
  _VTRN := VTRN;
  _BadAddressType := BadAddressType;  // DONE: LMS CodeCR536
end;

procedure TPatientInfo.CopyFrom(APtnt: TPatientInfo);
begin
  { Deep Copy Patient Object }
  _DFN := APtnt._DFN;
  _Name := APtnt._Name;
  _NameComponents := APtnt._NameComponents;
  _Last := APtnt._Last;
  _First := APtnt._First;
  _Middle := APtnt._Middle;
  _SSN := APtnt._SSN;
  _ICN := APtnt._ICN;
  _DOB := APtnt._DOB;
  _AGE := APtnt._AGE;
  _SEX := APtnt._SEX;
  _DOD := APtnt._DOD;
  _VTRN := APtnt._VTRN;
  _ClaimNum := APtnt._ClaimNum;
  { Deep Copy Address Info }
  _Addrss1 := APtnt._Addrss1;
  _Addrss2 := APtnt._Addrss2;
  _Addrss3 := APtnt._Addrss3;
  _AddrssType := APtnt._AddrssType;
  _City := APtnt._City;
  _StateProv := APtnt._StateProv;
  _Zip := APtnt._Zip;
  _Cntry := APtnt._Cntry;
  _HomePhone := APtnt._HomePhone;
  _CellPhone := APtnt._CellPhone;
  _BadAddressType := APtnt._BadAddressType;
end;


{ PRIVATE FUNCTIONS / PROCEDURES }

function TPatientInfo.SntceCase(val: string): String;
begin
  Result := '';
  if (Length(val) > 2) then
    Result := Uppercase(MidStr(val, 1, 1)) +
               Lowercase(MidStr(val, 2, Length(val)));
end;

{ PUBLIC FUNCTIONS / PROCEDURES }

function TPatientInfo.HasBadAddress(): Boolean;
begin
  Result := (Not SameText(_BadAddressType, ''));
end;

function TPatientInfo.IsPatientDeceased(): Boolean;
begin
  Result := (Not SameText(_DOD, ''));
end;

function TPatientInfo.GetExternalSSN(): String;
begin
  Result := Copy(_SSN, 1, 3) + '-' + Copy(_SSN, 4, 2) + '-' +
             Copy(_SSN, 6, Length(_SSN));
end;

end.
