unit CAPRIHelpMgr;

(*
  Help File related function(s).

  This unit started with the need to fetch help file locally if it was on
    a network drive and the help type was CHM. This is due to a security
    restriction added in Windows.
*)

interface
  procedure ChangeApplicationHelpFileToLocal(var strOriginalHelp, strLocalHelp: String; var LocalAvailable: Boolean);


implementation

uses windows, sysutils
     , untMiscMthds      // CodeCR496 - LMS - 2014-07-14 consolidate/refactor all GetTempDir into one location, so all can benefit from Win 7 specific handling.
     ;

procedure ChangeApplicationHelpFileToLocal(var strOriginalHelp, strLocalHelp: String; var LocalAvailable: Boolean);

  (*
  =========================================================================
  Could do the copy in a background thread if needed in the future.
  Determine Local Help File Path and name
  If not exists or different {
    Copy from Application path to Local path.
  }
  If not exists {
    raise error?? or set some flag so user can be notified by main thread - currently not doing this.
      in case of an unexpected error, the help file will continue to point to network, users will get
      error pages in CHM files and could call it in to help desk, and application will not be giving
      old/incorrect help.
  }
  If exists assign to application.

  // CodeCR381 - LMS 2012-11-15
  =========================================================================
  *)

(*  // CodeCR496 - LMS - 2014-07-14 consolidate/refactor all GetTempDir into one for Win 7 specific handling
  function GetTempDir: String;
  var
    TmpPath: PChar;
    PathLen : Integer;
  begin
    try
      GetMem(TmpPath, MAX_PATH);
      FillChar(TmpPath^, MAX_PATH, #0);
      PathLen := GetTempPath(MAX_PATH, TmpPath);
      if (PathLen >= MAX_PATH) then begin
        FillChar(TmpPath^, MAX_PATH, #0);
        PathLen := GetWindowsDirectory(TmpPath,MAX_PATH);   // Forcing this in case of invalid path. GetTempPath will return WindowsDirectory in some cases anyway.
      end;
      Result := ExpandFileName(TmpPath);
    finally
      FreeMem(TmpPath, MAX_PATH);
    end;
  end;
*)

  function GetFileSizeVCL(const FileName: String): Cardinal;  // Ideally should be unsigned int64 but there have been reported problems with Uint64 in some of the newer Delphi versions. Assume the file is smaller than 2GB. If we are copying larger than that across WAN, we've got a problem.
  var
    SearchRec : TSearchRec;
  begin
    Result := 0;
    if FindFirst(FileName,faAnyFile,SearchRec) = 0 then
      Result := SearchRec.Size;
  end;

  function getFileDateTimeVCL(const FileName: String): Integer;
  begin
    result := FileAge(FileName);  // FileAge is deprecated but the simplest and still works.
  end;
begin
  LocalAvailable := false;
  if ((copy(strOriginalHelp, 1, 2) = '\\') or (GetDriveType(PChar(extractFilePath(strOriginalHelp))) = DRIVE_REMOTE)) then
  begin
    if FileExists(strOriginalHelp) then
    begin
      strLocalHelp := GetTempDir;
      if (copy(strLocalHelp, length(strLocalHelp),1) <> '\') then strLocalHelp := strLocalHelp + '\';
      strLocalHelp := strLocalHelp + extractFileName(strOriginalHelp);

      LocalAvailable := FileExists(strLocalHelp);

      LocalAvailable := LocalAvailable and (getFileDateTimeVCL(strOriginalHelp) = getFileDateTimeVCL(strLocalHelp)) and (GetFileSizeVCL(strOriginalHelp) = GetFileSizeVCL(strLocalHelp));

      if not LocalAvailable then begin
        LocalAvailable := Windows.CopyFile(PChar(strOriginalHelp), PChar(strLocalHelp), False);
        LocalAvailable := LocalAvailable and (getFileDateTimeVCL(strOriginalHelp) = getFileDateTimeVCL(strLocalHelp)) and (GetFileSizeVCL(strOriginalHelp) = GetFileSizeVCL(strLocalHelp));
      end;
    end;
  end;
end;

end.
