unit Manifests;

// added functionality to count and display number of manifests available for pending / under review
// exam results from vendors.  This will be called by main and displayed in the alerts form.

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Fmcntrls, xmldom, XMLIntf, msxmldom, XMLDoc,
  ComCtrls, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
  Fmcmpnts, Diaccess
  , VlerDasIdHTTP
  , VlerDasClient, ExtCtrls
  ;

type
  TfrmManifests = class(TForm)
    btnCCR: TButton;
    btnPendExamResults: TButton;
    XMLManifests: TXMLDocument;
    lbxIPR: TListBox;
    btnGetPatient: TButton;
    ed_SSN: TEdit;
    XMLManifestDoc: TXMLDocument;
    txtGridHeader: TStaticText;
    btnClose: TButton;
    edtPatientName: TLabeledEdit;
    procedure FormShow(Sender: TObject);
    procedure btnPendExamResultsClick(Sender: TObject);
    procedure lbxIPRClick(Sender: TObject);
    procedure btnGetPatientClick(Sender: TObject);
    procedure btnCCRClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FHTTPServer: TIdHTTPvler;
    procedure getManifestList;
    procedure getManifestURL;
    procedure getManifestDoc(manifestURL: string; inum: Integer);
    procedure getPatientExams;
    procedure RemoveManifestFiles;
  public
    { Public declarations }
    function getManifestCnt: Integer;
  end;

var
  frmManifests: TfrmManifests;

implementation
uses
  main, clsPatientInfo, Patient
  , IdURI
  , untBrkrMthds
  , untMiscMthds
  , dialogscapri
  , MFunStr
  , VA508AccessibilityRouter
  ;

{$R *.DFM}

type
  TSSNInfo = class
    SSN: String;
  end;

var
  tmpPatientInfo: TPatientInfoBucket;


function getChildNodes(node: IDOMNode): string;
var
  J: Integer;
begin
  Result := '';
  for J := 0 to node.childNodes.length - 1 do
    if (node.childNodes.item[J].nodeType = TEXT_NODE) or
      (node.childNodes.item[J].nodeType = CDATA_SECTION_NODE) then
      Result := Result + node.childNodes.item[J].nodeValue;
end;

procedure TfrmManifests.RemoveManifestFiles;
var
  strManifestFile, strtmpfile: string;
  strTempDir: string;
  I: Integer;
  tbool: Boolean;

  procedure RemoveFile(inManifestFile: string);
  begin
    if inManifestFile > '' then begin
      if Pos(strTempDir, inManifestFile) > 0 then
        tbool := DeleteFile(inManifestFile)
      else
        tbool := DeleteFile(strTempDir + inManifestFile);
    end;
  end;

begin
  strTempDir := GetTempDir;
  strManifestFile := FileSearch('manifests.xml', strTempDir);
  RemoveFile(strManifestFile);

  for I := 0 to 100 do begin
    strtmpfile := 'manifestdoc_' + IntToStr(i) + '.xml';
    strManifestFile := FileSearch(strtmpfile, strTempDir);
    if strManifestFile > '' then begin
      RemoveFile(strManifestFile);
    end
    else
      break;
  end;

end;

procedure TfrmManifests.getManifestList;
var
  strXml: string;
  strURL: string;
  strURLEnc: string;
  strStationNo: string;
begin
  strStationNo := GetDivisionOfCBOC(frmMain.RPCBroker1.User.Division);
  strURL := TIdHTTPVler.GetVlerDasURL() +
    '/ecrud/v1/core/compensationAndPension.manifest.notification?query={"feed.entry.content.StationNumber":"'
    + strStationNo
    + '","feed.entry.content.Status":"Open"}';
  strURLEnc := TIdURI.URLEncode(strURL);
  try
    strXML := FHTTPServer.Get(strURLEnc);
    XMLManifests.LoadFromXML(strXml);
    XMLManifests.Active := True;
    if PARAM_SAVE_XML then begin
      RemoveManifestFiles;
      XMLManifests.SaveToFile(GetTempDir + '\' + 'Manifests.xml');              // debug only
    end;
  except                                                                        // soft fault any error
  end;

  lbxIPR.Clear;
end;                                                                            // getManifestList

procedure TfrmManifests.getManifestDoc(manifestURL: string; inum: Integer);
var
  strURL, strURLEnc: string;
  strXml: string;
begin
  strURL := manifestURL;                                                        // todo: lms check, if this needs to be transformed.
  strURLEnc := TIdURI.URLEncode(strURL);
  try
    strXML := FHTTPServer.Get(strURLEnc);
    XMLManifestDoc.Active := False;
    XMLManifestDoc.LoadFromXML(strXml);
    XMLManifestDoc.Active := True;
    if PARAM_SAVE_XML then
      XMLManifestDoc.SaveToFile(GetTempDir + '\' + 'ManifestDoc_' + IntToStr(inum) + '.xml'); // debug only
  except                                                                        // soft fault any errors
  end;
end;                                                                            // getManifestDoc

procedure TfrmManifests.getPatientExams;
var
  rNode, cNode, examsNode, eNode: IXMLNode;
  pNode, xNode: IXMLNode;
  patientstr, examstr, SSN, firstName, middleName, lastName, examName, examValue: string;
  I, J, idx: Integer;
  inpstr: string;
  icnt: Integer;
  tmpobj: TObject;
  ASSNInfo: TSSNInfo;
begin
  idx := -1;

  rNode := XMLManifestDoc.DocumentElement;
  if rNode.HasChildNodes then begin
    cNode := rNode.ChildNodes['ResultManifest'];
    pNode := cNode.ChildNodes['Patient'];
    patientstr := pNode.NodeName;
    if patientstr = 'Patient' then begin

      // patient name
      SSN := pNode.ChildNodes['SSN'].Text;
      firstName := pNode.ChildNodes['FirstName'].Text;
      middleName := pNode.ChildNodes['MiddleName'].Text;
      lastName := pNode.ChildNodes['LastName'].Text;
      inpstr := lastName + ', ' + firstName + ' ' + middleName;                 // rpk 8/7/2014

      // check for duplicate patient; add if no duplicate
      idx := -1;
      for icnt := 0 to lbxIpr.Count - 1 do begin
        tmpobj := lbxIpr.Items.Objects[icnt];
        if tmpobj <> nil then begin
          if SSN = TSSNInfo(tmpobj).SSN then begin
            idx := icnt;
            break;
          end;
        end;
      end;
      if idx < 0 then begin
        ASSNInfo := TSSNInfo.Create;
        ASSNInfo.SSN := SSN;                                                    // Top level patient record - no exam info.
        J := lbxIPR.Items.AddObject(inpstr, ASSNInfo);
      end
      else
        // set idx to position after patient, where exams should be inserted
        Inc(idx);
    end;
  end;

  // exams list
  examsNode := cNode.ChildNodes['Exams'];
  for I := 0 to examsNode.ChildNodes.Count - 1 do begin
    // each exam
    eNode := examsNode.ChildNodes[i];
    examstr := eNode.NodeName;
    if examstr = 'Exam' then begin
      xNode := eNode.ChildNodes['ExamName'];
      if xNode <> nil then begin
        examName := xNode.NodeName;
        if (xNode.NodeValue <> '') then begin
          examValue := xNode.NodeValue;
          // add exam name as indented column under patient
          inpstr := '   ' + examValue;
          // if patient already exists, add exams to patient; object is nil for exams
          if idx >= 0 then
            lbxIPR.Items.InsertObject(idx, inpstr, nil)
          else
            J := lbxIPR.Items.AddObject(inpstr, nil);
        end;
      end;
    end;
  end;
end;                                                                            // getPatientExams

procedure TfrmManifests.getManifestURL;
var
  manifestURL: string;
  tmpstr: string;
  I: Integer;
  rNode, dNode, fNode, eNode, lNode, hNode: IXMLNode;
begin
  if not XMLManifests.Active then
    getManifestList;

  rNode := XMLManifests.DocumentElement;
  if rNode.HasChildNodes then begin
    for I := 0 to rNode.ChildNodes.Count - 1 do begin
      dNode := rNode.ChildNodes[i];
      fNode := dNode.ChildNodes['feed'];
      if fNode <> nil then begin
        eNode := fNode.ChildNodes['entry'];
        if eNode <> nil then begin
          lNode := eNode.ChildNodes['link'];
          if lNode <> nil then begin
            hNode := lNode.AttributeNodes['href'];
            if hNode <> nil then begin
              tmpstr := tmpstr + ':' + hNode.Text;
              ManifestURL := hNode.Text;
              getManifestDoc(ManifestURL, I);
              getPatientExams;
            end;
          end;
        end;
      end;
    end;
  end;
end;                                                                            // getManifestURL

function TfrmManifests.getManifestCnt: Integer;
var
  rNode: IXMLNode;
begin
  Result := -1;

  // re-load XML data from DAS
  XMLManifests.Active := False;

  getManifestList;

  if XMLManifests.Active then begin
    rNode := XMLManifests.DocumentElement;
    Result := 0;
    if rNode.HasChildNodes then
      Result := rNode.ChildNodes.Count;
  end;
end;                                                                            // getManifestCnt


procedure TfrmManifests.btnPendExamResultsClick(Sender: TObject);
var
  icnt: Integer;
begin
  lbxIPR.Clear;
  edtPatientName.Text := '';                                                    // rpk 10/9/2014
  icnt := getManifestCnt;
  if icnt > 0 then begin
    if not XMLManifests.Active then
      getManifestList;
    getManifestURL;
  end;
end;

procedure TfrmManifests.btnCCRClick(Sender: TObject);
begin
  // make sure the TmpPatientInfo is loaded
  btnGetPatient.Click;

  // After patient info is loaded in main, run CCR
  frmMain.actCCRLaunchExecute(Self);

end;

procedure TfrmManifests.btnGetPatientClick(Sender: TObject);
var
  tbool: Boolean;
begin
  if ed_SSN.Text > '' then begin
//    tmpPatientInfo := TPatientInfoBucket.Create(ed_SSN.Text)
    try
      tbool := TPatientInfoBucket.CreatePatientInfoBucket(ed_SSN.Text, tmpPatientInfo); // CodeCR701 rpk 2/9/2015
    except                                                                      // CodeCR701 rpk 2/9/2015
      tmpPatientInfo := nil;                                                    // CodeCR701 rpk 2/9/2015
    end;                                                                        // CodeCR701 rpk 2/9/2015
  end;

  if (tmpPatientInfo <> nil) and (tmpPatientInfo.SSN > '') then begin
    frmMain.PatientInfo := tmpPatientInfo;
    edtPatientName.Text := tmpPatientInfo.Name;
    btnCCR.Enabled := True;
  end
  else begin
//    ShowMessageCAPRI('Patient Not Found.');
    edtPatientName.Text := '';
    btnCCR.Enabled := False;
  end;
end;                                                                            // btnGetPatientClick

procedure TfrmManifests.FormCreate(Sender: TObject);
begin
  tmpPatientInfo := nil;
  FHTTPServer := TIdHTTPvler.Create(Self, frmMain.RPCBroker1);
end;

procedure TfrmManifests.FormDestroy(Sender: TObject);
var
  i: Integer;
  ASSNInfo: TSSNInfo;
begin
  for I := 0 to lbxIPR.Count - 1 do
    if lbxIPR.Items.Objects[i] <> nil then begin
      ASSNInfo := TSSNInfo(lbxIpr.Items.Objects[i]);
      ASSNInfo.Free;
    end;

  lbxIPR.Clear;

end;

procedure TfrmManifests.FormShow(Sender: TObject);
begin
  txtGridHeader.Caption := Format('%-23.23s  %-4s%-25.25s',
    ['Patient', '   ', 'Exam Name']);
  btnCCR.Enabled := False;
  btnPendExamResults.Click;
end;

procedure TfrmManifests.lbxIPRClick(Sender: TObject);
var
  x: integer;
  ssnstr: string;
  tmpobj: TObject;
begin
  // Deal with a sub-form component being clicked
  // a leading blank space indicates that this row has an exam name
  // move index up to parent patient row
  if Copy(lbxIPR.Items[lbxIPR.ItemIndex], 1, 1) = ' ' then
  begin
    x := lbxIPR.ItemIndex;
    repeat
      dec(x);
    until Copy(lbxIPR.Items[x], 1, 1) <> ' ';
    lbxIPR.ItemIndex := x;
  end;
  x := lbxIPR.ItemIndex;
  tmpobj := lbxIpr.Items.Objects[x];
  if tmpobj <> nil then begin
    ssnstr := TSSNInfo(tmpobj).SSN;
    ed_ssn.Text := ssnstr;
    ed_ssn.Text := TSSNInfo(tmpobj).SSN;
    btnGetPatient.Click;
  end;
end;                                                                            // lbxIPRClick

end.

