unit Patient;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FMCmpnts, Diaccess;

type
  TfrmPatient = class(TForm)
    FMGets1: TFMGets;
    FMFindOne1: TFMFindOne;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TPatientInfoBucket = class(TObject)
  public
    DFN, Name, SSN
      , ICN                                                                     // CodeCR568: DONE: LMS - Added for VLER communication
      , DOB, AGE, SEX, DOD, ClaimNum: string;
    NameComponents, Last, First, Middle: string;
    Address1, Address2, Address3, City, StateProv, Zip, Country: string;
    HomePhone, CellPhone, BadAddressType, AddrssType: string;
    VTRN: Boolean;
    
    // For Patient SSN, get Fileman Patient record, then call
    // constructor to create PatientInfoBucket
    class function CreatePatientInfoBucket(inPatientSSN: string;                // CodeCR701 rpk 2/9/2015
      var outPatientInfoBucket: TPatientInfoBucket): Boolean;

    constructor Create(inIENStr: String);                                       // CodeCR701 rpk 2/9/2015
  end;

  TPatient = class
  public
    FirstName: string;
    MiddleName: string;
    LastName: string;

    Birthdate: string;
    Gender: string;

    IEN: string;
    SocialSecurityNumber: string;
    IntegrationControlNumber: string;
    ClaimNumber: string;
  end;

var
  ShortMonthNames: array[1..12] of string;                                      // LMS - Optimization

implementation
uses
  mfunstr, dialogscapri;

{$R *.dfm}

var
//  ShortMonthNames: array[1..12] of string;                                      // LMS - Optimization
  frmPatient: TfrmPatient;


class function TPatientInfoBucket.CreatePatientInfoBucket(inPatientSSN: string; // CodeCR701 rpk 2/9/2015
  var outPatientInfoBucket: TPatientInfoBucket): Boolean;                       // CodeCR701 rpk 2/9/2015
var
  IENStr: String;
begin
  // FMGets1 is a TFMGets, w/FileNumber=2
  // FMFindOne1 is a TFMFindOne component.
  Result := False;

  if outPatientInfoBucket <> nil then
    FreeAndNil(outPatientInfoBucket);

  if inPatientSSN = '' then begin  // rpk 2/11/2015
    ShowMessageCAPRI('Patient SSN is empty.');
    exit;
  end;

  if frmPatient = nil then
    frmPatient := TfrmPatient.Create(Application);

  try
    frmPatient.FMGets1.IENS := '';
    frmPatient.FMFindOne1.Value := inPatientSSN;
    frmPatient.FMFindOne1.FileNumber := '2';                                    // patient file
    IENStr := frmPatient.FMFindOne1.GetIEN;
    if (IENStr = '') or (frmPatient.FMFindOne1.ErrorList.Count > 0) then begin  // error occurred
      frmPatient.FMFindOne1.DisplayErrors;
      ShowMessageCAPRI('Error finding patient record IEN for SSN ' + inPatientSSN);
    end
    else if IENStr = '0' then begin                                             // no match
      ShowMessageCAPRI('No matching patient record found in Vista for SSN ' + inPatientSSN);
      // debug exception handling
//      raise Exception.Create('No patient record found for SSN ' + inPatientSSN);
    end
    else begin                                                                  // found patient record
      frmPatient.FMGets1.IENS := IENStr;
      frmPatient.FMGets1.GetAndFill;                                            // load the data fields
      outPatientInfoBucket := TPatientInfoBucket.Create(IENStr);                // create the patientinfobucket object
      Result := True;
    end;
  finally
    FreeAndNil(frmPatient);
  end;
end;                                                                            // CreatePatientInfoBucket

constructor TPatientInfoBucket.Create(inIENStr: String);                        // CodeCR701 rpk 2/9/2015

  function monthNum(aStr: string): smallint;
  var
    ii: smallint;
  begin
    result := 0;
    for ii := 1 to 12 do begin
      if AnsiCompareText(aStr, ShortMonthNames[ii]) = 0 then begin
        Result := ii;
        break;
      end;
    end;
  end;

  function encTextToDate(dIn: string): TDateTime;
  begin
    Result := 0;
    if (length(dIn) > 0) then try
      TryEncodeDate(
        strToInt(piece(dIn, ',', 2))
        , monthNum(piece(dIn, ' ', 1))
        , strToInt(piece(piece(dIn, ' ', 2), ',', 1))
        , Result);
    except
      // If there's invalid date in the database.
    end;
  end;
var
  tempname, tempaddr: string;
  TempAddrActive: Boolean;
  TempDate, TempDateFrom, TempDateTo: TDateTime;
begin
  inherited Create;
  if frmPatient = nil then
    exit;

  DFN := inIENStr;                                                              // populate patientinfobucket members
  Name := frmPatient.FMGets1.GetField('.01').FMDBExternal;                      // from patient record fields
  tempname := Name;
  Last := Piece(tempname, ',', 1);
  tempname := Piece(tempname, ',', 2);
  First := Piece(tempname, ' ', 1);
  Middle := Piece(tempname, ' ', 2);
  SSN := frmPatient.FMGets1.GetField('.09').FMDBExternal;
  ICN := frmPatient.FMGets1.GetField('991.01').FMDBExternal;                    // CodeCR568: DONE: LMS - Added for VLER communication
  DOB := frmPatient.FMGets1.GetField('.03').FMDBExternal;                       // tmpPatientInfo.DOB 'APR 15, 1948'
  AGE := frmPatient.FMGets1.GetField('.033').FMDBExternal;
  SEX := frmPatient.FMGets1.GetField('.02').FMDBExternal;
  DOD := frmPatient.FMGets1.GetField('.351').FMDBExternal;
  ClaimNum := frmPatient.FMGets1.GetField('.313').FMDBExternal;
  NameComponents := frmPatient.FMGets1.GetField('1.01').FMDBExternal;
  BadAddressType := frmPatient.FMGets1.GetField('.121').FMDBExternal;
  VTRN := frmPatient.FMGets1.GetField('1901').FMDBExternal = 'YES';

  tempaddr := frmPatient.FMGets1.GetField('.12105').FMDBExternal;
  TempAddrActive := (tempaddr = 'Y') or (tempaddr = 'YES');

  TempDateFrom := now - 36400;
  TempDate := TempDateFrom;
  TempDateFrom := encTextToDate(frmPatient.FMGets1.GetField('.1217').FMDBExternal);
  TempDateTo := encTextToDate(frmPatient.FMGets1.GetField('.1218').FMDBExternal);

//  Conversion did return a new date different from TempDate and from is prior to Date and To is after Date
  TempAddrActive := TempAddrActive and
    ((TempDateFrom <> TempDate) and
    (TempDateFrom <= Date) and
    (TempDateTo <> TempDate) and
    (TempDateTo >= Date));

  if TempAddrActive then begin
    Address1 := frmPatient.FMGets1.GetField('.1211').FMDBExternal;
    Address2 := frmPatient.FMGets1.GetField('.1212').FMDBExternal;
    Address3 := frmPatient.FMGets1.GetField('.1213').FMDBExternal;
    City := frmPatient.FMGets1.GetField('.1214').FMDBExternal;
    StateProv := frmPatient.FMGets1.GetField('.1215').FMDBExternal +
      frmPatient.FMGets1.GetField('.1221').FMDBExternal;
    Zip := frmPatient.FMGets1.GetField('.1216').FMDBExternal +
      frmPatient.FMGets1.GetField('.1222').FMDBExternal;
    Country := frmPatient.FMGets1.GetField('.1223').FMDBExternal;
    HomePhone := frmPatient.FMGets1.GetField('.1219').FMDBExternal;
  end
  else begin
    Address1 := frmPatient.FMGets1.GetField('.111').FMDBExternal;
    Address2 := frmPatient.FMGets1.GetField('.112').FMDBExternal;
    Address3 := frmPatient.FMGets1.GetField('.113').FMDBExternal;
    City := frmPatient.FMGets1.GetField('.114').FMDBExternal;
    StateProv := frmPatient.FMGets1.GetField('.115').FMDBExternal +
      frmPatient.FMGets1.GetField('.1171').FMDBExternal;
    Zip := frmPatient.FMGets1.GetField('.116').FMDBExternal;
    Country := frmPatient.FMGets1.GetField('.1173').FMDBExternal;
    HomePhone := frmPatient.FMGets1.GetField('.131').FMDBExternal;
  end;
  CellPhone := frmPatient.FMGets1.GetField('.134').FMDBExternal;
end;                                                                            // constructor

initialization
  ShortMonthNames[1] := 'JAN';                                                  // DONE: LMS - moved to a unit level array.. not reinstantiate on each call. In newer versions of Delphi this can be removed and replaced with FormatSettings.ShortMonthNames
  ShortMonthNames[2] := 'FEB';
  ShortMonthNames[3] := 'MAR';
  ShortMonthNames[4] := 'APR';
  ShortMonthNames[5] := 'MAY';
  ShortMonthNames[6] := 'JUN';
  ShortMonthNames[7] := 'JUL';
  ShortMonthNames[8] := 'AUG';
  ShortMonthNames[9] := 'SEP';
  ShortMonthNames[10] := 'OCT';
  ShortMonthNames[11] := 'NOV';
  ShortMonthNames[12] := 'DEC';

end.

