unit clsSFTPConn;

interface

uses Windows, Classes, Forms, SysUtils, Controls, ComCtrls,
     { Timer Component }
     ExtCtrls,
     { VistA Components }
     CCOWRPCBrokerCAPRI  // DONE: CodeCR457 - LMS
     , MFunStr, dialogsCAPRI, untAddtnlMMthds, untBrkrMthds, Trpcb,
     Fmcmpnts,
     { Secure BlackBox Components }
     SBLicenseManager, SBSimpleSftp, SBSSHConstants, SBSSHKeyStorage,
     SBSftpCommon
     , SBUtils
     , clsVendConn
     ;

type
  TSFTPConn = class(TVendorConnect)
  private
  { Private Declarations }
    _ElSBLicenseManager1: TElSBLicenseManager;
    _SFTPELClient: TElSimpleSFTPClient;
    //FCCOWBrkr: TCCOWRPCBrokerCAPRI;
    FCurDir: String;
    _DwnLdFiles: Boolean;
    _tmrKeepAlive: TTimer;
    //_RemoteIEN: String;
    procedure Connect;
    procedure Disconnect;
    procedure ReConnect;
    procedure CreateDirectory(DirPath: string);
    procedure ResetKeepAliveTmr;
    procedure SetDefaultFolder;
  private
    procedure CheckConnect;
    function DeepCreateDirectory(dirPath: string): String;
    function CheckDirectoryExists(dirPth: string): Boolean;
    function EnsureDirectoryExists(dirPth: string): Boolean;
  protected
  { Protected Declarations }
    function GetVndrConnInfo(Vndr: String): Boolean; override;
    procedure SFTPConnKeyValidate(Sender: TObject; ServerKey: TElSSHKey;
         var Validate: Boolean);
    procedure tmrKeepAliveTimer(Sender: TObject);
  public
  { Public Declarations }
    constructor Create(RmtVndrIEN: String; DownLoadFiles: Boolean; AOwnerBroker: TCCOWRPCBrokerCAPRI); override;
    destructor Destroy; override;
    function AbsPath(fleNme: string): String;

    function GetFileDirContents: TList; override;
    function DownloadFiles(VndrFles: TStrings): Boolean; override;
    procedure ArchiveFiles(VndrFles: TStrings); override;
    procedure UploadFile(UpldExmFle: String); override;

    procedure Copy(ACon: TSFTPConn);
    procedure Upld2DlDir(UlFile: String);
    //property RemoteIEN: String read _RemoteIEN;
    //property CCOWBrkr: TCCOWRPCBrokerCAPRI read FCCOWBrkr;
    property DwnLdFiles: Boolean read _DwnLdFiles;
  end;

implementation

uses
  untMiscMthds
  , untConstVals
  ;


{ Temporary Trial Key }
const LICENSEKEY: String = '485E29B696E96A099866A1D9E58B6814415B97CFADC17' +
                           '0C5C887408E8DB982003C884B8328BD6B5E6397B202EE' +
                           '728308D1652BE29D72DE2199DE11D180E668225B978F3' +
                           '8E5A06E36A87F8CB026827A80177B1C009E7E229C3DDB' +
                           'AD8D263457AD5ADC670972D65E2C41CF63FCB35385C41' +
                           '89482EB363BBD12046ABDC502E086924CC82637F98E73' +
                           '7696C145F972BBF50A41A395252EF61932CA739BFBB94' +
                           'E482B05BCD414D5FBF80537DA69C748B1F3F7B7AC7814' +
                           '428EB6CD82FF1D507E5B97D1D08E57A138E3BECE31E6F' +
                           'B735)3EE8AC6E6C93AF251BFE7063EF3E466620464D95' +
                           '6186EEA80718DA24FB0CFF6E4C3B3971219D17CC11EB4' +
                           'EC9A117B03BAFB1FDC';

{ Public }

{ Create SFTP Connection Object
    Input: RmtVndrIEN: IEN for Vendor SFTP Information on CLAIMS System (Remote)
                       Field #3 in File #396.45 (2507 EXAM CONTRACTORS)
           DwnLdFles: Indicates whether operation is for downloading or uploading
           CCOWBrkr: Home Broker Connection }
constructor TSFTPConn.Create(RmtVndrIEN: String; DownLoadFiles: Boolean; AOwnerBroker: TCCOWRPCBrokerCAPRI);
begin
  //inherited Create(RmtVndrIEN, DownLoadFiles, AOwnerBroker);
  inherited;

  if HaveVendorInfo then begin
    if (VLEREnabled = 'N') or (VLEREnabled = '') then begin
      //FCCOWBrkr := AOwnerBroker;
      //_RemoteIEN := RmtVndrIEN;
      _DwnLdFiles := DownLoadFiles;  { Determines Initial Directory Location }
      _ElSBLicenseManager1 := TElSBLicenseManager.Create(Self);   // DONE: LMS memory leak potential
      //_ElSBLicenseManager1.LicenseKey := LICENSEKEY;

      SetLicenseKey('08DB9BC0CC8D00E239DB6E38789AB248CCA30DBEFA056B77C7DA12AC71272DAF' +
      'DB99B03316120C5A3FBAF469E9ED6B55A620F3C92DB7D827B98BB54787DA2658' +
      '8DB21F304439B87845EA1C12CCA99D5CF5CA9A5BA4E209D62885A01EFAD2D75B' +
      'CF1D708A7FCE2F6BDB48A37DEDF5626A167AFA477394857BA9FEB3685D32ED60' +
      '3B57B20C5DC1C63C83D4B709B6AA9CF332FE3CEFD4359B07347A94D42E877D25' +
      'FEE1AC61B9610D59A4F0BD20E3FBA4C4834A76CBFD877672CA440AE9D535A228' +
      'EA12056A3C026DA7F3F0213AEDB97C077375FF063A5E22A6953222BCF261CA9F' +
      '614885AF8B59A85CB5B11CEE113EAFD520045F0E648CC4645A235B34500456C7');

      _SFTPELClient := TElSimpleSFTPClient.Create(Self);  // DONE : LMS Added owner, so no chance of memory leak

      with _SFTPELClient do
      begin
        KeepAlivePeriod := 60000;
        OnKeyValidate := SFTPConnKeyValidate;
        UserName := FUserName;
        Password := FPassword;
        Address := FAddress;
        Port := FPort;
      end;

      _tmrKeepAlive := TTimer.Create(Self);               // DONE : LMS Added owner, so no chance of memory leak
      _tmrKeepAlive.OnTimer := tmrKeepAliveTimer;
      _tmrKeepAlive.Interval := (_SFTPELClient.KeepAlivePeriod - 1000);
      _tmrKeepAlive.Enabled := False;

      // CheckConnect;  { Try and connect to Vendor's SFTP Server }  // DONE: LMS Changed call to standardized common execution pathway
      SetDefaultFolder;  { Try and Set Default Folder based on Connection Request }
    end;
  end  { End If Vendor Connection Info Retrieved Successfully }
  else
    raise Exception.Create('Unable to retrieve Vendor SFTP Connection Info from '+
         'central CLAIMS server.' + #13#10 + 'Try to complete this action later. ' +
         'If the problem persists contact IRM.');
end;

destructor TSFTPConn.Destroy;
begin
  Disconnect; { From SFTP Server if applicable }
  if _tmrKeepAlive <> nil then _tmrKeepAlive.Free;
  if _SFTPELClient <> nil then _SFTPELClient.Free;
  if _ElSBLicenseManager1 <> nil then _ElSBLicenseManager1.Free;
  inherited;
end;

function TSFTPConn.GetFileDirContents: TList;
begin
  // Result := inherited GetFileDirContents;  //  Currently we know that the base class method only ensures connection. This is being handled in CheckConnect but this is placeholder for future use
  _tmrKeepAlive.Enabled := False;
  Result := TList.Create;

  CheckConnect;

  _SFTPELClient.ListDirectory(FCurDir, Result);
  ResetKeepAliveTmr;
end;

function TSFTPConn.AbsPath(fleNme: string): String;
begin
  Result := FCurDir + fleNme;
  if (Length(FCurDir) = 0) or ((FCurDir[Length(FCurDir)] <> '\') and
       (FCurDir[Length(FCurDir)] <> '/')) then
    Result := FCurDir + '/' + fleNme;
end;

procedure TSFTPConn.SFTPConnKeyValidate(Sender: TObject; ServerKey: TElSSHKey;
  var Validate: Boolean);
begin
  Validate := True;
end;

function TSFTPConn.DownloadFiles(VndrFles: TStrings): Boolean;
var
  i: Integer;
  itm: String;
begin
  Result := inherited DownloadFiles(VndrFles);
  try
    _tmrKeepAlive.Enabled := False;

    CheckConnect;

    for i := 0 to VndrFles.Count - 1 do  { At least 1 file had to be selected }
    begin
      itm := VndrFles[i];
      try
        _SFTPELClient.DownloadFile(Piece(itm, '^'), Piece(itm, '^', 2));
      except
        on E: Exception do
        begin
          Result := False;
          raise Exception.Create('Error occurred downloading file: ' + #13#10 + E.Message);
        end;  { End Exception Handling }
      end;  { End Inner Try/Except Block }
    end;  { End For Loop of Selected Vendor Files }
  finally
    ResetKeepAliveTmr;
  end;  { End Outer Try/Finally Block }
end;

procedure TSFTPConn.ArchiveFiles(VndrFles: TStrings);
var
  i: Integer;
  dirPth, itm, flePth: String;
begin
  inherited;
  _tmrKeepAlive.Enabled := False;

  CheckConnect;

  { Create Archived Path - Includes VistA Instance # & Date of Archive}
  dirPth := FVndrDfltPth + '/' + CCRArchiveVendorResultsAtVendorFolder
       + '/' + GetDivisionOfCBOC(FCCOWBrkr.User.Division) // DONE: LMS Piece(_CCOWBrkr.User.Division, '^', 3)
       + '/' + FormatDateTime('yyyy-mm-dd', Date);    // DONE: LMS changed archive path from m_dd_yyyy to better sorting yyyy_mm_dd
  { If Archived Directory Structure Exists or was successfully created }
  if EnsureDirectoryExists(dirPth) then
  begin
    for i := 0 to VndrFles.Count - 1 do  { At least 1 file had to be selected }
    begin
      itm := Piece(VndrFles[i], '^', 1);  { Get File Location on Vendor SFTP Server }
      { Move File from Download to Archived Folder on the SFTP server }
      try
        { Set Full Path (Including File Name) where file will be moved to }
        flePth := dirPth + '/' + Piece(itm, '/', Occurrences('/', itm));
        { File Exists, then Append Time to prevent exception as file cannot be
          overwritten using this method. Assumes history of documents archived
          should be maintained. }
        if (_SFTPELClient.FileExists(flePth)) then
          flePth := dirPth + '/' + Piece(Piece(itm, '/', Occurrences('/', itm)), '.', 1) +
               '_' + FormatDateTime('hhnnss', Now) + '.' +
               Piece(Piece(itm, '/', Occurrences('/', itm)), '.', 2);
        _SFTPELClient.RenameFile(itm, flePth);  { Execute Move of File }  // DONE: LMS (now does with datetime in name) doesn't handle existance of prior file with same name for same day
      except
        on E: Exception do
        begin
          ShowMessageCAPRI('Unable to Archive the file: ' +
               Piece(itm, '/', Occurrences('/', itm)) + '.' + #13#10#13#10 + E.Message +
               #13#10#13#10 + 'Contact Vendor to move file(s) manually to appropriate ' +
               'Archived folder location.');
        end;  { End Exception Handling Block }
      end;  { End Try/Except Block - Error Occurred Transferring File }
    end;  { End For Loop }
  end;  { End If Directory Structure was successfully created }
  ResetKeepAliveTmr;
end;

procedure TSFTPConn.Upld2DlDir(UlFile: String);
var
  LocalFile, RemoteFile: String;
begin
  LocalFile := '\Temp\Results.txt';
  RemoteFile := '/Download/500/' + UlFile;
  try
    _tmrKeepAlive.Enabled := False;

    CheckConnect;

    _DwnLdFiles := True;
    _SFTPElClient.ASCIIMode := True;
    try
      //_SFTPElClient.UploadFile(LocalFile, RemoteFile, SBUtils.ftmOverwrite); // DONE: LMS method signature changed, so replacing with delete and then push
      try
        _SFTPElClient.RemoveFile(RemoteFile);
      except
        // DONE: LMS Not handling error conditions yet
      end;
      _SFTPElClient.UploadFile(LocalFile, RemoteFile); // , ftmOverwrite)
    except
      on E: Exception do
        raise Exception.Create('Transmission of Results Upload Error: ' + #13#10 + '  ' + E.Message);
    end;
  finally
    ResetKeepAliveTmr;
  end;
end;

{ Upload Exam Request to Appropriate Vendor's SFTP Server in the UPLOAD folder
  Input: UpldExmFle: TFileName of the exam to upload
      Exception Raised if Exam cannot be uploaded }
procedure TSFTPConn.UploadFile(UpldExmFle: String);
begin
  inherited;
  try
    _tmrKeepAlive.Enabled := False;

    CheckConnect;

    { Uploaded File to UPLOAD folder on Vendors SFTP Server }
    try
      if (Not SameText(UpldExmFle, '')) then
        //_SFTPELClient.UploadFile(UpldExmFle, AbsPath(ExtractFileName(UpldExmFle)),
        //     SBUtils.ftmOverwrite);
        //_SFTPElClient.UploadFile(LocalFile, RemoteFile, SBUtils.ftmOverwrite);  // DONE: LMS method signature changed, so replacing with delete and then push
        try
          _SFTPElClient.RemoveFile(AbsPath(ExtractFileName(UpldExmFle)));  // DONE: LMS (not a problem due to timestamps in file names) definitely a problem in deleting exam that's been sent already and the vendor hasn't processed it.
        except
          // DONE: LMS Not handling error conditions yet
        end;
        _SFTPElClient.UploadFile(UpldExmFle, AbsPath(ExtractFileName(UpldExmFle))); // , ftmOverwrite)
    except
      on E: Exception do
      begin
        raise Exception.Create('Request Transmission Upload Error: ' + #13#10 + '  ' + E.Message);
      end;  { End Exception Handling }
    end;  { End Inner Try/Except Block }
  finally
    ResetKeepAliveTmr;
  end;  { End Outer Try/Finally Block }
end;

{ Private }

function TSFTPConn.GetVndrConnInfo(Vndr: string): Boolean;
begin
  Result := inherited GetVndrConnInfo(Vndr);
  // placeholder for future additions
end;

procedure TSFTPConn.CheckConnect;
{ Use this procedure to check connection prior to all calls to _SFTPELClient. This will ensure }
begin
  EnsureBrokerConnected(CCOWRPCBroker);      // SFTP operations are prone to time outs. VLER could too.. This is preventative.
  Application.ProcessMessages;  // DONE: LMS this would allow any pending timers (and RPC broker keep alives) to fire and get handled.
  // todo: lms (later) could set current time after connection and assume it's active within 5 seconds of the last call.
  try
    if (Not _SFTPELClient.Active) then  // DONE: LMS missing reconnect in many places refactored into one call.
      ReConnect;  { Any kind of Inactivity }
  except
    Application.ProcessMessages;  // DONE: LMS Allow one more time.

    if (Not _SFTPELClient.Active) then  // DONE: LMS double check to make sure SFTP connection is really active.
      ReConnect;  { Any kind of Inactivity }
  end;
end;

procedure TSFTPConn.Connect;
begin
  _SFTPELClient.CompressionAlgorithms[SSH_CA_ZLIB] := True;
  { Public Key Authentication - Not Currently in use... }
  //sftpClient.AuthenticationTypes := sftpClient.AuthenticationTypes and not SSH_AUTH_TYPE_PUBLICKEY;
  try
    _SFTPELClient.Open;
    ResetKeepAliveTmr;
  except
    on E: Exception do
    begin
      raise Exception.Create('SFTP Connection Error: ' + #13#10 + '  ' + E.Message);
    end;  { End Exception Handling }
  end;  { End Try/Except Block }
end;

procedure TSFTPConn.Copy(ACon: TSFTPConn);
begin
  _ElSBLicenseManager1 := ACon._ElSBLicenseManager1;
  _SFTPELClient := ACon._SFTPELClient;
  FCCOWBrkr := ACon.FCCOWBrkr;
  FCurDir := ACon.FCurDir;
  FVndrDfltPth := ACon.FVndrDfltPth;
  _DwnLdFiles := ACon._DwnLdFiles;
  _tmrKeepAlive := ACon._tmrKeepAlive;
  //_RemoteIEN := ACon._RemoteIEN;
end;

procedure TSFTPConn.Disconnect;
begin
  if _SFTPELClient.Active then
    _SFTPELClient.Close(True);
end;

procedure TSFTPConn.ReConnect;
begin
  Connect;  { Use Existing Credentials }
end;

procedure TSFTPConn.SetDefaultFolder;
var
  msgDir, tempRoot: String;
begin
  CheckConnect;

  tempRoot := _SFTPELClient.RequestAbsolutePath(FVndrDfltPth);  // DONE: Move it up for both, download and upload   // TODO: LMS doing RequestAbsolutePath on the whole path would fail if a part of it didn't exist. We can only assume the vendor provided root exists.
  if tempRoot = '/' then tempRoot := '';    // TODO: LMS if / at end of any path, remove it

  msgDir := CCRToVendorFolder; // 'Upload ';

  if _DwnLdFiles then
  begin
    msgDir := CCRFromVendorFolder ; // 'Download ';

    FCurDir := tempRoot + '/' + CCRFromVendorFolder
         + '/' + GetDivisionOfCBOC(FCCOWBrkr.User.Division); // DONE: LMS Piece(_CCOWBrkr.User.Division, '^', 3));
  end  { End If Connection to Download Completed Exams }
  else
  begin
    FCurDir := tempRoot + '/' + CCRToVendorFolder;
  end;  { End Else Connection to Upload Exam Requests }

  if not CheckDirectoryExists(FCurDir) then
  begin
    try
      DeepCreateDirectory(FCurDir);
    except on E2: exception do
      raise Exception.Create( msgDir + ' directory on the SFTP server for Station ' +
           'Number: ' + GetDivisionOfCBOC(FCCOWBrkr.User.Division) + // DONE: LMS Piece(_CCOWBrkr.User.Division, '^', 3) +
           ', does NOT exist, or ' + #13#10 +
           'the current directory path is invalid: ' + FCurDir + #13#10 +
           'Error: ' + E2.Message );
    end;  { End Exception Handling }
  end;
end;

function TSFTPConn.DeepCreateDirectory(dirPath: string): String;
var
  lvl, lvlDeep, itms, itms2: Integer;
  tmpHandle: TSBSFTPFileHandle;
  tmpPath, tmpPathDeep : string;
begin
  Result := '';

  // DONE: LMS vendors were allowed non-virtual roots so, paths could be way deeper than original code of 2;
  // DONE: LMS vendors also don't give us permission to top level in non-virtual roots, so we can't start making folders at top LEVEL.
  //         on LHI change the root archived folder to a different name and then try to call this. It needs to try to create the longest path first and then on error keep reducing parts until it can make one or all the way to top (then error). When it makes one then it needs to start making one longer until full path is made. If full path isn't made then error.
  itms := Occurrences('/', dirPath) - 1;
  //if (copy(dirPath,0,2) = '//') then
  //  itms := itms - (itms2 * 2);
  lvl := itms; // - 1;      //  TODO: LMS still unconfirmed if all scenarios work

  while (Result = '') and (lvl >= 1) do begin
    if CheckDirectoryExists(dirPath) then begin
      Result := dirPath;  // full success;
      exit;
    end;

    tmpPath := Piece(dirPath, '/', 1, lvl);
    if CheckDirectoryExists(tmpPath) then begin
      // if exists then create lower levels, else create this level
      for lvlDeep := lvl + 1 to itms + 1 do try   // + 1
        tmpPathDeep := Piece(dirPath, '/', 1, lvlDeep);
        CreateDirectory(tmpPathDeep);
      except
        on E: Exception do begin
          Result := E.Message + ' ' ;  // give up
          // break;
        end;
      end;
    end else begin
      try
        CreateDirectory(tmpPath);
      except
        on E: Exception do begin
          dec(lvl);                // try one level higher
          if lvl < 1 then begin
            Result := E.Message + ' ' ;   // at "/" level.. give up
            // break;
          end;
        end;
      end;
    end;
  end;
  //if (Result = '') then Result := tmpPath; // full success;
end;

{ LMS: Simple wrapper just to check if directory exists, but not make one.}
function TSFTPConn.CheckDirectoryExists(dirPth: string): Boolean;
begin
  Result := True;
  try
    _SFTPELClient.CloseHandle(_SFTPELClient.OpenDirectory(dirPth));
  except
    Result := false;
  end;
end;

{  Determines if the Directory structure exists on the server,
   if NOT the structure will be created.
   Input: Directory structure to check existence for or to create
   Output: True: Directory Exists or was successfully created
           False: Error occurred creating the directory }
function TSFTPConn.EnsureDirectoryExists(dirPth: string): Boolean;
var
  //i, itms: Integer;
  CreateMsg : string;
begin
  CheckConnect;

  Result := True;
  try
    _SFTPELClient.CloseHandle(_SFTPELClient.OpenDirectory(dirPth));
  except
    CreateMsg := DeepCreateDirectory(dirPth);
    if (CreateMsg <> dirPth) then               // DONE: LMS expecting a blank as success is bad practice. So changed the Deep(CreateDirectory) call to return full path created.
    begin
      ShowMessageCAPRI('Unable to create the Archive directory: ' + dirPth
           //'.'
           // + #13#10 + dirPth
           + #13#10 + 'Files will NOT be moved out of the download folder. '
           + #13#10#13#10
           + CreateMsg);
      Result := False;
    end;
    (*
    i := 2; // DONE: LMS vendors were allowed non-virtual roots so, paths could be way deeper than original code of 2;
    // DONE: LMS vendors also don't give us permission to top level in non-virtual roots, so we can't start making folders at top LEVEL.
    //         on LHI change the root archived folder to a different name and then try to call this. It needs to try to create the longest path first and then on error keep reducing parts until it can make one or all the way to top (then error). When it makes one then it needs to start making one longer until full path is made. If full path isn't made then error.
    itms := Occurrences('/', dirPth);
    repeat
      try
        _SFTPELClient.CloseHandle(_SFTPELClient.OpenDirectory(Piece(dirPth, '/', 1, i)));
      except
        try
          CreateDirectory(Piece(dirPth, '/', 1, i));
        except
          on E: Exception do
          begin
            ShowMessageCAPRI('Unable to create the Archive directory: ' + Piece(dirPth, '/', 1, i) +
                 '.' + #13#10 + 'Files will NOT be moved out of the download folder. ' +
                 #13#10#13#10 + E.Message);
            Result := False;
          end;  { End Exception Handling Block }
        end;  { End Inner Try/Except Block }
      end;  { End Outer Try/Except Block }
      inc(i);
    until ((Not Result) or (i > itms));
    *)
  end;  { End Try/Except Block - Check if complete Dir Structure Exists }
end;

procedure TSFTPConn.CreateDirectory(DirPath: string);
var
  fldrAttributes: TElSftpFileAttributes;
begin
  CheckConnect;

  fldrAttributes := TELSftpFileAttributes.Create;
  with fldrAttributes do
  begin
    UserRead := True;
    UserWrite := True;
    UserExecute := False;
    GroupRead := True;
    GroupWrite := True;
    GroupExecute := False;
    OtherRead := True;
    OtherWrite := True;
    OtherExecute := False;
    FileType := ftDirectory;
  end;  { End With Folder Attributes }
  try
    try
      _SFTPELClient.MakeDirectory(DirPath, fldrAttributes);
    except
      on E: Exception do
      begin
        raise Exception.Create(E.Message);
      end;  { End Exception Block }
    end;  { End Inner Try/Except Block }
  finally
    fldrAttributes.Free;
  end;  { End Outer Try/Finally Block }
end;

procedure TSFTPConn.tmrKeepAliveTimer(Sender: TObject);
begin
  Disconnect;
  _tmrKeepAlive.Enabled := False;
end;

procedure TSFTPConn.ResetKeepAliveTmr;
begin
  _tmrKeepAlive.Enabled := False;
  _tmrKeepAlive.Enabled := True;
end;

end.
