unit clsVLERConn;

interface

uses Classes, Forms, SysUtils
     , IdHTTP, IdURI, CCOWRPCBrokerCAPRI
     , VlerDasIdHTTP
     , vlerCCRExams
     , clsVendConn
     , XMLDoc
     ;

type TVLERConn = class(TVendorConnect)
  private
    FIdHTTPVler: TIdHTTPVler;
    XMLManifests: TXMLDocument;
    procedure SetIdHTTPVler(const Value: TIdHTTPVler);
  protected
    function GetVndrConnInfo(Vndr: String): Boolean; override;
  public
  { Public Declarations }
    constructor Create(RmtVndrIEN: String; DownLoadFiles: Boolean; AOwnerBroker: TCCOWRPCBrokerCAPRI); override;
    destructor Destroy; override;

    function GetFileDirContents: TList; override;
    function DownloadFiles(VndrFles: TStrings): Boolean; override;
    procedure ArchiveFiles(VndrFles: TStrings); override;
    procedure UploadFile(UploadExamFile: String); override;
    procedure UploadFileMultiPart(UploadExamFile: String);
    function GetExamFileListForPatient(CurrentPatientSSN: string; var frmvlerCCRExams : TfrmvlerCCRExams): TList;

    property IdHTTPVler: TIdHTTPVler read FIdHTTPVler write SetIdHTTPVler;
  end;

type
  TVlerResultFileInfo = class(TComponent)       // Similar functions to TElSftpFileInfo (a single result, text on VLER presented to CAPRI as a file)
  private
    FFileName: string;
    FExamResultObjectId: string;
    FExamName: string;
    FManifestObjectId: string;
    FExamId: string;
    //FFileDateTime: string;
    function GetFileDateTime: TDateTime;
    function GetFileDateTimeStr: string;
    function ConvertVLERFileTimeToDateTime(vFileTimeStamp: string): TDateTime;
    //procedure SetExamId(const Value: string);
    //FFileDateTime: string;  // File name has the Status date time in it -- so not using FFileDateTime
  protected
  public
    //constructor Create(AOwner : TComponent); override;
    //destructor Destroy; override;
    procedure CopyFrom(Source: TVlerResultFileInfo);
    property ExamName : string read FExamName write FExamName;
    property ExamId : string read FExamId write FExamId;          // never populated by VLER data. Only generated in vista, in case of exam sent by vendor that doesn't exist. Normally one would use the file name to find the examID, but if this value is non-blank, then it overrides the value derived from filename.
    property FileName: string read FFileName write FFileName;
    property FileDateTimeStr: string read GetFileDateTimeStr;
    property FileDateTime: TDatetime read GetFileDateTime;
    property ExamResultObjectId : string read FExamResultObjectId write FExamResultObjectId;
    property ManifestObjectId : string read FManifestObjectId write FManifestObjectId;
  end;

type TVLERExamInfo = class
  public
  ExamName : string;
  ExamID : string;
  DocumentUUID : string;
  RequestUUID : string;
  SSN : String;
  ManifestDateTime : string;      // This is the date the exam was submitted to VA by vendor and considered equivalent of the SFTP file date/time
  ManifestObjectId : string;
  ExamResultObjectId : string;
  ExamTextFileAttachmentObjectId : string;
  ExamFileName : string;      // File name has the Status date time in it -- so not using ExamFileDateTime : TDateTime;
  procedure CopyFrom(Source: TVLERExamInfo);
end;

function StripMSAndZFromISO8601(AStr: string) : string;

implementation

uses
  untMiscMthds
  , untConstVals
  , main
  , MFunStr
  , IdMultipartFormData
  , untBrkrMthds
  , Windows
  ;

function StripMSAndZFromISO8601(AStr: string) : string;
begin
  if pos('.',aStr) > 0 then
    Result := copy(aStr,0,pos('.',aStr)-1)
  else
    if pos('Z',AnsiUpperCase(aStr)) > 0 then
      Result := copy(aStr,0,pos('.',aStr)-1)
end;

constructor TVLERConn.Create(RmtVndrIEN: String; DownLoadFiles: Boolean; AOwnerBroker: TCCOWRPCBrokerCAPRI);
begin
  //inherited Create(RmtVndrIEN, DownLoadFiles, AOwnerBroker);
  inherited;
  FIdHTTPVler := TIdHTTPVler.Create(AOwnerBroker, AOwnerBroker);
end;

destructor TVLERConn.Destroy;
begin
  //  FIdHTTPVler.Destroy; // Owner destroys - not needed
  inherited;
end;

function TVLERConn.DownloadFiles(VndrFles: TStrings): Boolean;
begin
  Result := inherited DownloadFiles(VndrFles);
        // VLER files download is not implemented here.
end;

procedure TVLERConn.ArchiveFiles(VndrFles: TStrings);
begin
  inherited;
        // VLER does not require an archive process.
end;

{ Upload Exam Request to Appropriate Vendor via VLER DAS
  Input: UpldExmFle: TFileName of the exam to upload
      Exception Raised if Exam cannot be uploaded }
procedure TVLERConn.UploadFile(UploadExamFile: String);
var
  UploadFileStream : TFileStream;
  ResponseToFile : TFileStream;
  AURL: string;
  hadException : boolean;
  ResponseOutputFile: string;
begin
  inherited;
  hadException := false;
  UploadFileStream := TFileStream.Create(UploadExamFile, fmOpenRead);
  ResponseOutputFile := UploadExamFile+'.out.txt';
  ResponseToFile := MakeOutputFileStreamReady(ResponseOutputFile);
  try
    try
      AURL := TIdHTTPVler.GetVlerDasURL();
      AURL := AURL + '/ecrud/v1/core/compensationAndPension';
      //  IdHTTPVler.InitVLERBaseRequestHeader;         // LMS - Standard known header and other values, now default.
      try
        IdHTTPVler.Post(AURL,UploadFileStream,ResponseToFile,[201]);    // Added new Post method versions to  -- WAS :VLER sends 201 when a post succeeds and base post class does treat it like an exception. Added this method to trap the error before it free the response content as well.
      except on excp:EIdHTTPProtocolException do             // trap only http 201 (it's ok on post)
        begin
          hadException := not (excp.ErrorCode = 201);
          if hadException then
            raise;
          end;
      end;
    finally
      if UploadFileStream <> nil then UploadFileStream.Free;
      if ResponseToFile <> nil then ResponseToFile.Free;       // DONE: LMS Make sure the file is saved if SAVEXML parameter is set.
    end;
  finally
    if (not (hadException or PARAM_SAVE_XML))
    {$IFDEF CAPRIDEVELOPER}
      and false
    {$ENDIF}
    then begin
      try
        sysutils.DeleteFile(UploadExamFile);
        sysutils.DeleteFile(ResponseOutputFile);
      except
      end;
    end;
  end;
end;

{ Upload Exam Rejection and Completion to VLER/DAS
  Input: UpldExmFle: TFileName of the exam to upload
      Exception Raised if Exam cannot be uploaded }
procedure TVLERConn.UploadFileMultiPart(UploadExamFile: String);
var
  ResponseOutputFile: string;
  //UpldFileStrm : TFileStream;
  ResponseToFile : TFileStream;
  AURL: string;
  hadException : boolean;
  PostStream: TIdMultiPartFormDataStream;
begin
  EnsureBrokerConnected(CCOWRPCBroker);      // SFTP operations are prone to time outs. VLER could too.. This is preventative.
  //UpldFileStrm := TFileStream.Create(UpldExmFle, fmOpenRead);

  AURL := TIdHTTPVler.GetVlerDasURL();
  AURL := AURL + '/ecrud/v1/core/disabilityBenefitsQuestionnaires/findCopyInsert';

  PostStream:= TIdMultiPartFormDataStream.Create;
  PostStream.AddFile('file', UploadExamFile, 'text/xml'); // VLER specified. wants this string, not GetMIMETypeFromFile(UpldExmFle));

  ResponseOutputFile := UploadExamFile+'.out.txt';
  ResponseToFile := MakeOutputFileStreamReady(ResponseOutputFile);

  try
    try
      try
        //IdHTTPVler.Request.ContentType := 'multipart/form-data'; //  required by VLER
        IdHTTPVler.Post(AURL, PostStream, ResponseToFile, [201]);    // Added new Post method versions to  -- WAS :VLER sends 201 when a post succeeds and base post class does treat it like an exception. Added this method to trap the error before it free the response content as well.
      except on excp:EIdHTTPProtocolException do             // trap only http 201 (it's ok on post)
        begin
          hadException := not (excp.ErrorCode = 201);
          if hadException then
            raise;
          end;
      end;
    finally
      //UpldFileStrm.Free;
      if PostStream <> nil then PostStream.Free;
      if ResponseToFile <> nil then ResponseToFile.Free;       // DONE: LMS Make sure the file is saved if SAVEXML parameter is set.
    end;
  finally
    if (not (hadException or PARAM_SAVE_XML))
    {$IFDEF CAPRIDEVELOPER}
      and false
    {$ENDIF}
    then begin
      try
        sysutils.DeleteFile(UploadExamFile);
        sysutils.DeleteFile(ResponseOutputFile);
      except
      end;
    end;
  end;
end;

{ Private }
function TVLERConn.GetVndrConnInfo(Vndr: string): Boolean;
begin
  Result := inherited GetVndrConnInfo(Vndr);
  // placeholder for future additions
end;

procedure TVLERConn.SetIdHTTPVler(const Value: TIdHTTPVler);
begin
  if assigned(FIdHTTPVler) then
    if (FIdHTTPVler <> Value) then
      FIdHTTPVler.Free;
  FIdHTTPVler := Value;
end;

{ TVlerResultInfo }

//constructor TVlerResultFileInfo.Create(AOwner: TComponent);
//begin
  //inherited;
  //
//end;

//destructor TVlerResultFileInfo.Destroy;
//begin
  //
  //inherited;
//end;

function TVlerResultFileInfo.ConvertVLERFileTimeToDateTime(vFileTimeStamp: string): TDateTime;
var
  ISOFormatSettings: TFormatSettings;
begin
  vFileTimeStamp := copy(vFileTimeStamp, 1, length(vFileTimeStamp) - 4);
  // remove .txt extension
  vFileTimeStamp := StringReplace(vFileTimeStamp, 'T', ' ', []);
  // ,[rfReplaceAll]);
  vFileTimeStamp := StringReplace(vFileTimeStamp, '-', '/', [rfReplaceAll]);
  //vFileDate := StrToDateTime('08/24/2014 13:01:49'); // vFileTimeStamp, // vFileTimeStamp  ** temp
  // Big promise but unfinished  -> IdDateTimeStamp1 := TIdDateTimeStamp.Create(self);
  //   DONE: LMS using StrToDateTime --  will have to hand do this..
  {
      try
        IdDateTimeStamp1.SetFromISO8601(vFileTimeStamp);
        vFileDate := IdDateTimeStamp1.GetAsTDateTime;
      finally
        IdDateTimeStamp1.free;
      end;
      }
  GetLocaleFormatSettings(LOCALE_SYSTEM_DEFAULT, ISOFormatSettings);
  ISOFormatSettings.ShortDateFormat := 'yyyy/mm/dd';
  Result := StrToDateTime(vFileTimeStamp, ISOFormatSettings);
end;

function TVlerResultFileInfo.GetFileDateTimeStr: string;
begin
  {
  Original file creation
        Result := vler_ExamIdNode.Text
                          + '$' + vler_PhysicalSourceNode.Text
                          + '$' + StringReplace(vStatusDate,':','#',[rfReplaceAll])  // fileDateTime ":" won't save on disk
                          + '.txt';
  }
  Result := StringReplace(piece(FFileName,'$',3),'#',':',[rfReplaceAll]);
  Result := piece(Result,'!');  // remove filecounter
end;

procedure TVlerResultFileInfo.CopyFrom(Source: TVlerResultFileInfo);
begin
  { Deep Copy VlerResultFileInfo Object }
  self.ExamName := Source.ExamName;
  self.ManifestObjectId := Source.ManifestObjectId;
  self.ExamResultObjectId := Source.ExamResultObjectId;
  self.FileName := Source.FileName;
end;

function TVlerResultFileInfo.GetFileDateTime: TDateTime;
var
  vFileTimeStr: string;
begin
  vFileTimeStr := GetFileDateTimeStr; // StringReplace(piece(FFileName,'$',3),'#',':',[rfReplaceAll]);
  Result := ConvertVLERFileTimeToDateTime(vFileTimeStr);
end;

function TVLERConn.GetExamFileListForPatient(CurrentPatientSSN: string; var frmvlerCCRExams : TfrmvlerCCRExams): TList;  // Get VLER contents ... treat VLER like a SFTP file system
begin
  EnsureBrokerConnected(CCOWRPCBroker);      // SFTP operations are prone to time outs. VLER could too.. This is preventative.

  // VLER specific code (get a list of exams with results for vendor RmtVndrIEN)
  TfrmvlerCCRExams.EnsurefrmvlerCCRExamsReady(frmvlerCCRExams);
  //if not Assigned(frmvlerCCRExams) then
    //Application.CreateForm(TfrmvlerCCRExams, frmvlerCCRExams);

  frmvlerCCRExams.CcrGetManifests(CurrentPatientSSN);
  frmvlerCCRExams.LimitToPatient(CurrentPatientSSN);

  Result := TList.Create;   // A list of TVlerResultInfo, except the very first object is a pointer to a list of manifestID's marked InProcess

  frmvlerCCRExams.FetchAllExamsTextToFiles(Result);

  //frmvlerCCRExams.RemoveDupeNotificForSameExams;

  frmvlerCCRExams.FillExamResultList(Result);
end;

function TVLERConn.GetFileDirContents: TList;
begin
  // Not implemented
  Result := inherited GetFileDirContents;
end;

{ TVLERExamInfo }

procedure TVLERExamInfo.CopyFrom(Source: TVLERExamInfo);
begin
  { Deep Copy VLERExamInfo Object }
  self.ExamName := Source.ExamName;
  self.ExamID := Source.ExamID;
  self.DocumentUUID := Source.DocumentUUID;
  self.RequestUUID := Source.RequestUUID;
  self.SSN := Source.SSN;
  self.ManifestDateTime := Source.ManifestDateTime;
  self.ManifestObjectId := Source.ManifestObjectId;
  self.ExamResultObjectId := Source.ExamResultObjectId;
  self.ExamTextFileAttachmentObjectId := Source.ExamTextFileAttachmentObjectId;
  self.ExamFileName := Source.ExamFileName;
end;

end.
