unit eCrud;

interface

uses Windows, Classes, Dialogs, xmldom, XMLIntf, msxmldom, XMLDoc, XsdEngineLite,
  XmlConfig, XsdComplexType, XmlComponentBindings,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP
  , ExtCtrls, SysUtils, Variants
  , VlerDasClient
  , VlerDasIdHTTP
  , CCOWRPCBrokerCAPRI
  , SynCommons, UnitDocumentMethods
  , untMiscMthds
  ;

//const  LMS not used
//  NS_GETnc = 'nc="http://niem.gov/niem/niem-core/2.0"';

type
  PVLERAttachNodeVal = ^TVLERAttachNodeVal;
  TVLERAttachNodeVal = record
    strncDocumentFileControlID: string;                                         //used a unique id in this Record to be search in TList
    strncBinaryDescriptionText: string;                                         //prefix of attachment filename
    strncBinaryFormatStandardName: string;                                      //file type such as image/jpeg
    strncBinaryLocationURI: string;                                             //Is the location within the "fs" collection that contains the file.when query "fs" passing this value the store file will be returned.
    strncBinarySizeValue: string;                                               //Is the size of the file
    strncBinaryCategoryText: string;                                            //Category group the image pertains to. (ex: X-Rays)
  end;

  TeCRUD = class(TComponent)
  private
    FeCRUDVersion: string;
    FdbName: string;
    FcollectionName: string;
    FmaxRecords: string;
    FSpecifyResponseFields: string;
    FSort: string;
    FHTTPServer: TIdHTTPvler;
    FXMLDocument: TXMLdocument;
    FVLERAttachNodeList: TList;                                                 // rpk 9/11/2014
    FOpenDocList: TOpenDocList;
    pVLERAttachNodeValue: PVLERAttachNodeVal;                                   // rpk 9/11/2014
    function SelectNodes(xnRoot: IXmlNode; const nodePath: WideString): IXMLNodeList;
//    function NodeFound(var aNode: IXmlNode; aNodeName: string; var FoundNode: IXmlNode): boolean;
    function NodeFound(aNode: IXmlNode; aNodeName: string; var FoundNode: IXmlNode): boolean;
  public
    constructor Create(AOwner: TComponent; ABroker: TCCOWRPCBrokerCAPRI); reintroduce; overload; virtual; // LMS - Corrected declaration
    destructor Destroy; override;                                               // LMS - Corrected declaration
    function getExamBy(UserInputType: string; SearchParams: TStringList; var aResultStrlst: TStringList): boolean;
    procedure getExamAttachments(aBinaryLocURI: string; var strURLQryAttachment: string);
//    procedure getExamAttachmentList(var aTList: TList; pRecord: PVLERAttachNodeVal);
    procedure setExamAttachmentList(inTList: TList);
    procedure setOpenDocumentList(inDocList: TOpenDocList);
    procedure OpenDocumentFromXMLNode(ancBinaryLocationURI: string;
      aFilename: string; aFileType: string; AttachmentSQL: string; BinarySzValue: integer);
    procedure CloseOpenAttachments();
    function getExamReviewDocuments(URLandQuery: string; var aResult: TStringList; includeText: Boolean): Boolean;
    //function GetTempDirectory: String;  // CodeCR496 - LMS - 2014-07-14
//    function FreeTList(var aTList: TList): Boolean;

  published
//    property HTTPServer: TIdHTTP read FHTTPServer write FHTTPServer;  // DONE: LMS Needs a setter method to free existing object.
    property HTTPServer: TIdHTTPVler read FHTTPServer;                          // IdHTTPVler object used by TeCRUD class to connect to the server (proxy or direct to VLER).
    property XMLDocument: TXMLDocument read FXMLDocument write FXMLDocument;    // TODO: LMS - maybe - Needs a setter method to free existing object.

    property eCRUDVersion: string read FeCRUDVersion write FeCRUDVersion;
       //interface version number
    property dbName: string read FdbName write FdbName;
       //the VLER Data Store (VDS) MongoDB database name where the data for this operation is to
       //be read
    property collectionName: string read FcollectionName write FcollectionName;
       //for MongoDB read requests: this is the VDS, MongoDB collection name, for GridFS partition,
       // this will always have the value of fs.
    //property queryString : string read FqueryString write FqueryString;
       //the optional query= request parameter, accepts a {queryString} which is a key-value
       // pair of the selection criteria in a JSON format.
    property maxRecords: string read FmaxRecords write FmaxRecords;
       //- the optional limit= request integer parameter, accepts a {maxRecords}
       // value which sets the maximum number of results that can be sent back in the response.
       // When not specified: will default to 100 max records, when zero: implies no limits and all records will be returned and when negative: the absolute value will be considered.
    property SpecifyResponseFields: string read FSpecifyResponseFields write FSpecifyResponseFields;
       //the optional fields= request parameter, accepts a {columnProjections} value which
       // sets the fields that can be included/not included in the response. When not specified
       // will return all fields in the document
    property Sort: string read FSort write FSort;

  end;

//var
//  FVLERAttachNodeList: TList;
//  pVLERAttachNodeValue: PVLERAttachNodeVal;
//  aOpenDocList: TOpenDocList;

implementation
uses
  Main,
  dialogscapri;
{ TeCRUD }

procedure TeCRUD.CloseOpenAttachments;
begin
  if Assigned(FOpenDocList) then
    FOpenDocList.CloseMyDocuments;
end;

constructor TeCRUD.Create(AOwner: TComponent; ABroker: TCCOWRPCBrokerCAPRI);
begin
  inherited Create(AOwner);
  FHTTPServer := TIdHTTPvler.Create(self, ABroker);

  //Header from http://www.sitepoint.com/web-foundations/mime-types-complete-list/
  {FHTTPServer.Request.CustomHeaders.Add('Accept : application/msword');
  FHTTPServer.Request.CustomHeaders.Add('Accept : application/mswrite');
  FHTTPServer.Request.CustomHeaders.Add('Accept : application/pdf');
  FHTTPServer.Request.CustomHeaders.Add('Accept : application/rtf');
  FHTTPServer.Request.CustomHeaders.Add('Accept : application/X-rtf');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/jpeg');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/pjpeg');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/BMP');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/gif');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/tiff');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/x-tiff');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/vnd.dwg');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/x-dwg');
  FHTTPServer.Request.CustomHeaders.Add('Accept : image/png');
  FHTTPServer.Request.CustomHeaders.Add('Accept : text/richtext');
   }

  // Make sure there's no memory leak
  // FXMLDocument:= XMLDOM;
  eCRUDVersion := 'v1';
  dbName := 'core';
  collectionName := 'disabilityBenefitsQuestionnaires';
  maxRecords := '100';
//  FOpenDocList := nil;
end;

destructor TeCRUD.Destroy;
begin
//  FreeTList(FVLERAttachNodeList);
//  if assigned(aOpenDocList) then
//    aOpenDocList.Free;
//  if assigned(FOpenDocList) then
//    FOpenDocList.Free;
  inherited Destroy;
end;

{ function TeCRUD.FreeTList(var aTList: TList): Boolean;
var
  I: Integer;
begin
  result := true;
  if assigned(aTList) then begin
    try
      try
        for I := 0 to aTList.Count - 1 do
      // aTList[i] = new(pVLERAttachNodeValue)
          dispose(PVLERAttachNodeVal(aTList[i]));
        aTlist.Clear;
//    aTlist.Free;
      except
        on E: Exception do
          result := False;
      end;
    finally
//    aTlist.Clear;
      FreeAndNil(aTList);
    end;
  end; // if assigned
end; }

(*
procedure TeCRUD.EstablishHTTPServerSettings;
begin
  theHTTPServer.Request.Clear;
  theHTTPServer.Request.Accept := FHTTPAcceptHeaderXML; //'application/xml' - DEFAULT is JASON
  theHTTPServer.Request.ContentType := FHTTPContentType;
  theHTTPServer.Request.ContentEncoding := FHTTPContentEncoding;
  theHTTPServer.Request.CustomHeaders.Clear;
  theHTTPServer.Request.CustomHeaders.Add('authToken:' + FVlerDasToken); //CodeCR540 JRL
end;
*)




//procedure TeCRUD.getExamAttachmentList(var aTList: TList; pRecord: PVLERAttachNodeVal);

procedure TeCRUD.setExamAttachmentList(inTList: TList);
begin
//  aTList := FVLERAttachNodeList;
//  pRecord := pVLERAttachNodeValue;
  FVLERAttachNodeList := inTList;
end;

procedure TeCRUD.setOpenDocumentList(inDocList: TOpenDocList);
begin
  FOpenDocList := inDocList;
end;

procedure TeCRUD.getExamAttachments(aBinaryLocURI: string; var strURLQryAttachment: string);
begin
   //Create GET URL with query
  strURLQryAttachment := TIdHTTPVler.GetVlerDasURL() + '/' + 'ecrud' + '/' + FeCRUDVersion + '/' +
    FdbName + '/' + FcollectionName + '/' + aBinaryLocURI;

   // EstablishHTTPServerSettings();
end;

function TeCRUD.getExamBy(UserInputType: string; SearchParams: TStringList; var aResultStrlst: TStringList): boolean;
var
  strURLQRY: string;
  MiddleName: string;
  includeText: Boolean;
     (****************************************************************
      !!!sudo code example of implementing monogoDB FIND $elemMatch !!!!
       cld:Claim.cld:CommonData.nc:Person.nc:PersonName.nc:PersonName: {
          { field: { $regex: /acme.*corp/i}
        }
     *******************************************************************)
begin
  includeText := True;                                                          // all getExamBy calls include text files.

   //Create GET URL with query
  strURLQRY := TIdHTTPVler.GetVlerDasURL()
    + '/' + 'ecrud'
    + '/' + FeCRUDVersion
    + '/' + FdbName
    + '/' + FcollectionName;

   //assign query based on user input on frmVlerGetExam
  if UserInputType = 'SSN' then                                                 //search by SSN
  begin
    strURLQRY := strURLQRY +
      '?query=%7b%22cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID%22:%22' +
      SearchParams.Strings[0] + '%22,' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime%22:%7b%22$gte%22:%22$Date(' + SearchParams.Strings[1] +
      ')%22,' + '%22$lte%22:%22$Date(' + SearchParams.Strings[2] + ')%22%7d%7d';
  end
  else if UserInputType = 'FACILITY' then                                       //search by facility
  begin
    strURLQRY := strURLQRY +
      '?query=%7b%22cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID%22:%22' +
      SearchParams.Strings[0] + '%22,' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime%22:%7b%22$gte%22:%22$Date(' + SearchParams.Strings[1] +
      ')%22,' + '%22$lte%22:%22$Date(' + SearchParams.Strings[2] + ')%22%7d%7d';
  end
  else if UserInputType = 'SSN_FACILITY' then                                   //search by SNN and facility
  begin
     //SSN qry
    strURLQRY := strURLQRY +
      '?query=%7b%22cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID%22:%22' +
      SearchParams.Strings[0] + '%22,' +

     //Facility qry
    '%22cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID%22:%22' +
      SearchParams.Strings[1] + '%22,' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

     //Date Range
    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime%22:%7b%22$gte%22:%22$Date(' + SearchParams.Strings[2] +
      ')%22,' + '%22$lte%22:%22$Date(' + SearchParams.Strings[3] + ')%22%7d%7d';
  end
  else if UserInputType = 'PHYSICIAN' then                                      //search by physician
  begin
    if SearchParams.Strings[1] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName :=
        '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[1] + '","$options":"i"},';
    strURLQRY := strURLQRY +
      // physician name
    '?query={"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' +
      SearchParams.Strings[0] + '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[2] + '","$options":"i"}}},' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

      // date range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[3] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[4] + ')"}}';

  end
  else if UserInputType = 'SSN_PHYSICIAN' then                                  //search by SSN and PHYSICIAN
  begin
    if SearchParams.Strings[2] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName :=
//      '"$options":"i"},"cld:Claim.cld:CommonData.nc:Person.nc:PersonName.nc:PersonMiddleName":{"$regex":"^' + SearchParams.Strings[2] + '",';
      '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[2] + '","$options":"i"},';

     //SSN qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[0] + '",' +

     //Physician qry
//    '"cld:Claim.cld:CommonData.nc:Person.nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[1] + '",' +
    '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' +
      SearchParams.Strings[1] + '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[3] + '","$options":"i"}}},' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

     //Date Range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' + SearchParams.Strings[4] +
      ')",' + '"$lte":"$Date(' + SearchParams.Strings[5] + ')"}}';

  end
  else if UserInputType = 'PHYSICIAN_FACILITY' then                             //search by Physician and Facility
  begin
    if SearchParams.Strings[1] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName := '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[1] + '","$options":"i"},';
     //Facility qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[3] + '",' +

     //Physician qry
    '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[0] +
      '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[2] + '","$options":"i"}}},' +

      // filter Document Status = Completed or Rejected
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },' +

     //Date Range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[4] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[5] + ')"}}';

  end
  else if UserInputType = 'EXAMSTATUS' then                                     //search by ExamStatus
  begin
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' +
      SearchParams.Strings[0] + '",' +
      '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[1] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[2] + ')"}}';

  end
  else if UserInputType = 'SSN_EXAMSTATUS' then                                 //search by SSN and ExamStatus
  begin
    //SSN qry
    strURLQRY := strURLQRY +
      '?query=%7b%22cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID%22:%22' +
      SearchParams.Strings[0] + '%22,' +

    //Exam status
    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText%22:%22' +
      SearchParams.Strings[1] + '%22,' +

     //Date Range
    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime%22:%7b%22$gte%22:%22$Date(' + SearchParams.Strings[2] +
      ')%22,' + '%22$lte%22:%22$Date(' + SearchParams.Strings[3] + ')%22%7d%7d';
  end
  else if UserInputType = 'PHYSICIAN_EXAMSTATUS' then                           //search by Physician and ExamStatus
  begin
    if SearchParams.Strings[1] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName := '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[1] + '","$options":"i"},';
     //Physician qry
    strURLQRY := strURLQRY +
      // physician name
    '?query={"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' +
      SearchParams.Strings[0] + '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[2] + '","$options":"i"}}},' +

     //Exam status
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' +
      SearchParams.Strings[3] + '",' +

      // date range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[4] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[5] + ')"}}';

  end
  else if UserInputType = 'SSN_PHYSICIAN_EXAMSTATUS' then                       //search by SSN, PHYSICIAN and EXAM STATUS
  begin
    if SearchParams.Strings[2] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName :=
//      '"$options":"i"},"cld:Claim.cld:CommonData.nc:Person.nc:PersonName.nc:PersonMiddleName":{"$regex":"^' + SearchParams.Strings[2] + '",';
      '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[2] + '","$options":"i"},';
    //SSN qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[0] + '",' +

    //Physician qry
//    '"cld:Claim.cld:CommonData.nc:Person.nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[1] + '",' +
    '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' +
      SearchParams.Strings[1] + '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[3] + '","$options":"i"}}},' +

    // Exam Status
//    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' +
//      SearchParams.Strings[4] + '",' +
    // Exam status
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' +
      SearchParams.Strings[4] + '",' +

     //Date Range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[5] + ')",' + '"$lte":"$Date(' +
      SearchParams.Strings[6] + ')"}}';

  end
  else if UserInputType = 'FACILITY_EXAMSTATUS' then                            //search by Facility and ExamStatus
  begin
    strURLQRY := strURLQRY +
     //Facility
    '?query=%7b%22cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID%22:%22' + SearchParams.Strings[0] + '%22,' +

     //Exam status
    '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText%22:%22' + SearchParams.Strings[1] +

     //Date
    '%22,' + '%22cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime%22:%7b%22$gte%22:%22$Date(' + SearchParams.Strings[2] +
      ')%22,' + '%22$lte%22:%22$Date(' + SearchParams.Strings[3] + ')%22%7d%7d';
  end
  else if UserInputType = 'FACILITY_PHYSICIAN_EXAMSTATUS' then                  //Search by Facility and Physician and ExamStatus
  begin
    if SearchParams.Strings[2] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName := '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[2] + '","$options":"i"},';

     //Facility qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[0] + '",' +

     //Physician qry
    '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[1] +
      '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[3] + '","$options":"i"}}},' +

     //Exam status
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' + SearchParams.Strings[4] + '",' +

      // date range
    '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[5] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[6] + ')"}}';

  end
  else if UserInputType = 'SSN_FACILITY_PHYSICIAN' then                         //Search by SSN and Facility and Physician
  begin
    if SearchParams.Strings[3] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName := '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[3] + '","$options":"i"},';

     //SSN qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID":"' + SearchParams.Strings[0] + '",';

     //Facility qry
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[1] + '",';

     //Physician qry
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[2] +
      '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[4] + '","$options":"i"}}},';

      // filter Document Status = Completed or Rejected
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText": { "$in": ["Completed", "Rejected"] },';

      // date range
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[5] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[6] + ')"}}';

  end
  else if UserInputType = 'SSN_FACILITY_PHYSICIAN_EXAMSTATUS' then              //Search by SSN and Facility and Physician and ExamStatus
  begin
    if SearchParams.Strings[3] = '' then                                        // Don't send middle name if it doesn't exist, it causes query issues
      MiddleName := ''
    else
      MiddleName := '"nc:PersonName.nc:PersonMiddleName":{"$regex":"^' +
        SearchParams.Strings[3] + '","$options":"i"},';

     //SSN qry
    strURLQRY := strURLQRY +
      '?query={"cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID":"' + SearchParams.Strings[0] + '",';

     //Facility qry
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Facility.nc:FacilityIdentification.nc:IdentificationID":"' +
      SearchParams.Strings[1] + '",';

     //Physician qry
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Person":{"$elemMatch":{"-s:id":"two",' +
      '"nc:PersonName.nc:PersonGivenName":{"$regex":"^' + SearchParams.Strings[2] +
      '","$options":"i"},' +
      MiddleName +
      '"nc:PersonName.nc:PersonSurName":{"$regex":"^' +
      SearchParams.Strings[4] + '","$options":"i"}}},';

     //Exam status
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentStatus.nc:StatusText":"' + SearchParams.Strings[5] + '",';

      // date range
    strURLQRY := strURLQRY +
      '"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":{"$gte":"$Date(' +
      SearchParams.Strings[6] + ')",' +
      '"$lte":"$Date(' + SearchParams.Strings[7] + ')"}}';

  end;

   //Result Fields, Sort condition, Limit Number of result
  //strURLQRY:=strURLQRY + '&fields='+FSpecifyResponseFields+'&sort='+FSort+'&limit='+FmaxRecords;
  strURLQRY := strURLQRY + '&limit=' + FmaxRecords;

  //Example of SSN URL Query//
  //http://DNS         4.vaco.DNS   /ecrud/v1/core/disabilityBenefitsQuestionnaires?query={"cld:Claim.cld:CommonData.nc:Person.nc:PersonSSNIdentification.nc:IdentificationID":"500950188"}&fields={"cld:Claim.cld:CommonData.nc:Document":1,"cld:Claim.cld:CommonData.vler:Client":1,"cld:Claim.cld:CommonData.nc:Person":1}&sort={"cld:Claim.cld:CommonData.nc:Document.nc:DocumentCreationDate.nc:DateTime":-1}&limit=100
  //port is optional
  Result := getExamReviewDocuments(strURLQRY, aResultStrlst, includeText);      //Execute RESTful GET and Format XML Response for Exam Review.

end;

function TeCRUD.getExamReviewDocuments(URLandQuery: string; var aResult: TStringList; includeText: Boolean): Boolean;
var
  strXML: string;
  DocFileCtrlID, DocCreateDT, DocIDUUID, ClaimID, ClientID, Person, GivenNMclient, MiddleNMclient, SurNMclient: string;
  StatusOfExamRslt, DbqDocTitleText, GivenNMSrvProv, MiddleNMSrvProv, SurNMSrvProv, FacilityNM, PersonDOB, PersonSSN: string;
  BinaryDescriptionText, BinaryFormatStandardName, BinaryLocationURI, BinarySizeValue, BinaryCategoryText, ExamID, ExamStatusDT: string;
  i, x: integer;
  xNode, aNode, docsNode, clsclaimNode, cldCommonDataNode, clsclaimIDNode, ncdocNode,
    ncDocCreationDTNode, ncDateTimeNode,
    ncDocFileControlID, ncDateTimeNodeValue, ncDocIDNode, ncIdntIDNode,
    ncDocStatusNode, ncStatusTxtNode, ncDocTitleTextNode,
    PersonElementNode, PersonSrvcProvNode, PersonSrvProvNmNode, PersonClientNmNode,
    ncFacilityNode, ncFacilityNmNode, PersonClientNode, ncPersonDOBNode, ncPersonDOBncDateNode,
    ncPersonSSNIdentNode, ncIdentIDSSNNode,
    ncBinaryDescriptionTextNode, ncBinaryFormatStandardNameNode, ncBinaryLocationURINode,
    ncBinarySizeValueNode, ncBinaryCategoryTextNode,
    cldAttachmentsNode, ncAttachmentNode,
    vlerExamDetailNode, vlerExamIdNode,
    vlerStatusDateNode, vlerStatusDateNodencDateTime,
    cldDocumentTypeVersion: IXMLNode;
  RstlTmp, TmpStr: string;
  DestFile_pt: System.Text;
  DocTypeVer: string;
  FileNM: string;
  MS: TMemoryStream;
begin
  try
   //URLandQuery - is the formatted URL and and Query and is execute RESTful GET
    result := True;
    strXML := FHTTPServer.Get(URLandQuery);

    FXMLDocument.LoadFromXML(strXml);
    FXMLDocument.Active := True;
    AddDocumentsWrapperIfMissing(FXMLDocument);                                 // DONE: LMS - new generic call to add <documents> container for queries that return single object

  //Diagnositc Code
    if PARAM_SAVE_XML then
    begin
      FileNM := GetTempDir;                                                     // LMS - 2014-07-14 refactor in unit untMiscMthds
      FileNM := FileNM + 'XMLReplyFromVler.xml';
      MS := TMemoryStream.Create;
      FXMLDocument.SaveToStream(MS);
      MS.SaveToFile(FileNM);
      MS.Free;
    end;

    if FXMLDocument.DocumentElement.HasChildNodes then
    begin

  //VLERAttachNodeList := TVLERAttachNodeList.Create; //Create a TList of type Record to hold attachment values to
                                                     //  retrieve Exam document attachments.

        //Create dynarry to hold attachment records
        { if assigned(FVLERAttachNodeList) = True then
        begin
          FreeTList(FVLERAttachNodeList);
          FVLERAttachNodeList := TList.Create;
        end
        else
          FVLERAttachNodeList := TList.Create; }

//        if assigned(FVLERAttachNodeList) then
//          FreeTList(FVLERAttachNodeList);
//        FVLERAttachNodeList := TList.Create;

      for x := 0 to FXMLDocument.DocumentElement.ChildNodes.Count - 1 do
      begin
        RstlTmp := '';
        DocFileCtrlID := ' '; DocCreateDT := ' '; DocIDUUID := ' ';
        ClaimID := ' '; ClientID := ' '; Person := ' '; GivenNMclient := ' ';
        MiddleNMclient := ' '; SurNMclient := ' '; StatusOfExamRslt := ' '; DbqDocTitleText := ' ';
        GivenNMSrvProv := ' '; MiddleNMSrvProv := ' '; SurNMSrvProv := ' '; FacilityNM := ' ';
        PersonDOB := ' '; PersonSSN := ' '; ExamID := ' '; ExamStatusDT := ' ';
        BinaryDescriptionText := ' '; BinaryFormatStandardName := ' '; BinaryLocationURI := ' ';
        BinarySizeValue := ' '; BinaryCategoryText := ' ';


        docsNode := FXMLDocument.DocumentElement.ChildNodes.Get(x);

        TmpStr := docsNode.NodeName;
        if NodeFound(docsNode, 'cld:Claim', clsclaimNode) then
          if clsclaimNode <> nil then
            tmpstr := clsclaimNode.NodeName;
        if NodeFound(clsclaimNode, 'cld:ClaimID', clsclaimIDNode) then
        begin
          if clsclaimIDNode <> nil then
          begin
            tmpstr := clsclaimIDNode.NodeName;
            if clsclaimIDNode.NodeValue <> null then
            begin
              ClaimID := clsclaimIDNode.NodeValue;                              //Retrieved ClaimID
            end;
          end;

          if NodeFound(clsclaimNode, 'cld:DocumentTypeVersion', cldDocumentTypeVersion) then
          begin
            if cldDocumentTypeVersion <> nil then
            begin
              tmpstr := cldDocumentTypeVersion.NodeName;
              if cldDocumentTypeVersion.NodeValue <> null then
              begin
                DocTypeVer := cldDocumentTypeVersion.NodeValue;                 //Retrieved ClaimID
              end;
            end;
          end;

          ncdocnode := nil;
          vlerExamDetailNode := nil;
          PersonClientNode := nil;
          PersonSrvcProvNode := nil;
          ncFacilityNode := nil;
          if NodeFound(clsclaimNode, 'cld:CommonData', cldCommonDataNode) then
          begin

            //Nav to cld:CommonData then nc:document
            if NodeFound(cldCommonDataNode, 'nc:Document', ncdocNode) then
            begin
                     //Get Creation Date
              tmpstr := ncdocNode.NodeName;
              if NodeFound(ncdocNode, 'nc:DocumentCreationDate', ncDocCreationDTNode) then
              begin
                tmpstr := inttostr(ncDocCreationDTNode.ChildNodes.count);
                if ncDocCreationDTNode <> nil then
                begin
                  tmpstr := inttostr(ncDocCreationDTNode.ChildNodes.count);
                  tmpstr := ncDocCreationDTNode.NodeName;
                  if NodeFound(ncDocCreationDTNode, 'nc:DateTime', ncDateTimeNode) then
                  begin
                    tmpstr := ncDateTimeNode.NodeName;
                    tmpstr := inttostr(ncDateTimeNode.ChildNodes.count);

                    if ncDateTimeNode <> nil then
                    begin
                      if ncDateTimeNode.NodeValue <> null then
                      begin
                        DocCreateDT := ncDateTimeNode.NodeValue;                //Retrieved 'nc:DateTime'
                      end;
                    end;
                  end;
                end;
              end;

              if NodeFound(ncdocNode, 'nc:DocumentFileControlID', ncDocFileControlID) then
                if ncDocFileControlID <> nil then
                begin
                  tmpstr := ncDocFileControlID.NodeName;
                  if ncDocFileControlID.NodeValue <> null then
                    DocFileCtrlID := ncDocFileControlID.NodeValue;              //Retrived nc:DocumentFileControlID
                end;
              //Get Document ID (UUID)
              if NodeFound(ncdocNode, 'nc:DocumentIdentification', ncDocIDNode) then
                if ncDocIDNode <> nil then
                begin
                  tmpstr := ncDocIDNode.NodeName;
                  if NodeFound(ncDocIDNode, 'nc:IdentificationID', ncIdntIDNode) then
                    if ncIdntIDNode.NodeValue <> null then
                      DocIDUUID := ncIdntIDNode.NodeValue;                      //Retrieved nc:IdentificationID
                end;
              //Get Status of Exam Result
              if NodeFound(ncdocNode, 'nc:DocumentStatus', ncDocStatusNode) then
                if ncDocStatusNode <> nil then
                begin
                  tmpstr := ncDocStatusNode.NodeName;
//                  ncStatusTxtNode := ncDocStatusNode.ChildNodes['nc:StatusText'];
                  if NodeFound(ncDocStatusNode, 'nc:StatusText', ncStatusTxtNode) then
                    if ncStatusTxtNode.NodeValue <> null then
                      StatusOfExamRslt := ncStatusTxtNode.NodeValue;            //Retrieved nc:StatusText
                end;
              //Get DBQ Document Type
              if NodeFound(ncdocNode, 'nc:DocumentTitleText', ncDocTitleTextNode) then
                if ncDocTitleTextNode <> nil then
                begin
                  tmpstr := ncDocTitleTextNode.NodeName;
                  if ncDocTitleTextNode.NodeValue <> null then
                    DbqDocTitleText := ncDocTitleTextNode.NodeValue;            //Retrived nc:DocumentTitleText
                end;
              //DONE WITH nc:Document NODE LEVEL
            end;                                                                //end of get values from nc:document node level

            if NodeFound(cldCommonDataNode, 'vler:ExamDetail', vlerExamDetailNode) then
            begin
              tmpstr := vlerExamDetailNode.NodeName;
                            //vlerExamIdNode:=vlerExamDetailNode.ChildNodes['vler:ExamId']; //Retrived vler:ExamId
              if NodeFound(vlerExamDetailNode, 'vler:ExamId', vlerExamIdNode) then
                if vlerExamIdNode <> nil then
                begin
                  tmpstr := vlerExamIdNode.NodeName;
                  if vlerExamIdNode.NodeValue <> null then
                    ExamID := vlerExamIdNode.NodeValue;
                end;
            end;

            if vlerExamDetailNode <> nil then
            begin
              if NodeFound(vlerExamDetailNode, 'vler:StatusDate', vlerStatusDateNode) then
              begin
                tmpstr := vlerStatusDateNode.NodeName;

                if NodeFound(vlerStatusDateNode, 'nc:DateTime', vlerStatusDateNodencDateTime) then
                begin
                  tmpstr := vlerStatusDateNodencDateTime.NodeName;
                  if vlerStatusDateNodencDateTime.NodeValue <> null then
                    ExamStatusDT := vlerStatusDateNodencDateTime.NodeValue;
                end;
              end;

            end;

            // loop through CommonDataNode child nodes and match nc:Person
            for I := 0 to cldCommonDataNode.ChildNodes.Count - 1 do begin
              xNode := cldCommonDataNode.ChildNodes[I];
              tmpstr := xNode.NodeName;
              // check s:id attribute to identify person node
              if xNode.NodeName = 'nc:Person' then begin
                aNode := xNode.AttributeNodes['s:id'];
                if aNode <> nil then begin
                  if aNode.Text = 'one' then
                    PersonClientNode := xNode
                  else if aNode.Text = 'two' then
                    PersonSrvcProvNode := xNode;
                end;
              end;
            end;                                                                // for I

            if PersonSrvcProvNode <> nil then
            begin
              if NodeFound(PersonSrvcProvNode, 'nc:PersonName', PersonSrvProvNmNode) then
              begin
                tmpstr := PersonSrvProvNmNode.NodeName;

                if NodeFound(PersonSrvProvNmNode, 'nc:PersonGivenName', PersonElementNode) then
                  if PersonElementNode <> nil then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      GivenNMSrvProv := PersonElementNode.NodeValue;            //Retrived nc:PersonGivenName
                  end;
                if NodeFound(PersonSrvProvNmNode, 'nc:PersonMiddleName', PersonElementNode) then
                  if PersonElementNode <> nil then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      MiddleNMSrvProv := PersonElementNode.NodeValue;           //Retrived nc:PersonMiddleName
                  end;

                if NodeFound(PersonSrvProvNmNode, 'nc:PersonSurName', PersonElementNode) then
                  if PersonElementNode <> nil then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      SurNMSrvProv := PersonElementNode.NodeValue;              //Retrived nc:PersonSurName
                  end;
              end;                                                              // end of - if PersonSrvcProvNode <> nil then
            end;                                                                // PersonSrvcProvNode

            //Get Client/Patient Name
            if PersonClientNode <> nil then
            begin
              tmpstr := PersonClientNode.NodeName;
              //DOB
              if NodeFound(PersonClientNode, 'nc:PersonBirthDate', ncPersonDOBNode) then
              begin

                if ncPersonDOBNode <> nil then
                begin
                  tmpstr := ncPersonDOBNode.NodeName;
                  if NodeFound(ncPersonDOBNode, 'nc:Date', ncPersonDOBncDateNode) then
                  begin
                    tmpstr := ncPersonDOBNode.NodeName;
                    if ncPersonDOBncDateNode.NodeValue <> null then
                      PersonDOB := ncPersonDOBncDateNode.NodeValue;             //Retrieved nc:Date
                  end;
                end;
              end;
              if NodeFound(PersonClientNode, 'nc:PersonName', PersonClientNmNode) then
              begin
                if PersonClientNmNode <> nil then
                begin
                  tmpstr := PersonClientNmNode.NodeName;
                  if NodeFound(PersonClientNmNode, 'nc:PersonGivenName', PersonElementNode) then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      GivenNMclient := PersonElementNode.NodeValue;             //Retrieved nc:PersonGivenName
                  end;
                end;

                if NodeFound(PersonClientNmNode, 'nc:PersonMiddleName', PersonElementNode) then
                  if PersonElementNode <> nil then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      MiddleNMclient := PersonElementNode.NodeValue;            //Retrived nc:PersonMiddleName
                  end;

                if NodeFound(PersonClientNmNode, 'nc:PersonSurName', PersonElementNode) then
                  if PersonElementNode <> nil then
                  begin
                    tmpstr := PersonElementNode.NodeName;
                    if PersonElementNode.NodeValue <> null then
                      SurNMclient := PersonElementNode.NodeValue;               //Retrived nc:PersonSurName
                  end;
              end;                                                              // end of - if PersonClientNmNode <> nil then
              //SSN
              if NodeFound(PersonClientNode, 'nc:PersonSSNIdentification', ncPersonSSNIdentNode) then
              begin
                if ncPersonSSNIdentNode <> nil then
                begin
                  tmpstr := ncPersonSSNIdentNode.NodeName;
                               //ncIdentIDSSNNode:= ncPersonSSNIdentNode.ChildNodes['nc:IdentificationID'];
                  if NodeFound(ncPersonSSNIdentNode, 'nc:IdentificationID', ncIdentIDSSNNode) then
                    if ncIdentIDSSNNode <> nil then
                    begin
                      tmpstr := ncIdentIDSSNNode.NodeName;
                      if ncIdentIDSSNNode.NodeValue <> null then
                        PersonSSN := ncIdentIDSSNNode.NodeValue;                //Retrived nc:IdentificationID
                    end;
                end;
              end;
            end;                                                                // end of - if PersonClientNode <> nil then

            if NodeFound(cldCommonDataNode, 'nc:Facility', ncFacilityNode) then begin
              if ncFacilityNode <> nil then begin
                tmpstr := ncFacilityNode.NodeName;
                if NodeFound(ncFacilityNode, 'nc:FacilityName', ncFacilityNmNode) then
                  if ncFacilityNmNode <> nil then begin
                    tmpstr := ncFacilityNmNode.NodeName;
                    if ncFacilityNmNode.NodeValue <> null then
                      FacilityNM := ncFacilityNmNode.NodeValue;                 //Retrieved nc:FacilityName
                  end;
              end;
            end;
            //DONE WITH cld:CommonData NODE LEVEL
          end;                                                                  //end of get values from cld:CommonData node level


          //Get File Attachment nodes
          cldAttachmentsNode := clsclaimNode.ChildNodes['cld:Attachments'];
          if cldAttachmentsNode.HasChildNodes then
          begin
            tmpstr := cldAttachmentsNode.NodeName;
            //Loop and get all attachements
            for I := 0 to cldAttachmentsNode.ChildNodes.Count - 1 do
            begin
              ncAttachmentNode := cldAttachmentsNode.ChildNodes[I];
              if ncAttachmentNode <> nil then
              begin
                tmpstr := ncAttachmentNode.NodeName;

                if NodeFound(ncAttachmentNode, 'nc:BinaryDescriptionText', ncBinaryDescriptionTextNode) then
                  if ncBinaryDescriptionTextNode.NodeValue <> null then
                  begin
                    BinaryDescriptionText := ncBinaryDescriptionTextNode.NodeValue; //Retrieved BinaryDescriptionText
                  end;

                if NodeFound(ncAttachmentNode, 'nc:BinaryFormatStandardName', ncBinaryFormatStandardNameNode) then
                  if ncBinaryFormatStandardNameNode.NodeValue <> null then
                  begin
                    BinaryFormatStandardName := ncBinaryFormatStandardNameNode.NodeValue; //Retrived BinaryFormatStandardName
                  end;

                if NodeFound(ncAttachmentNode, 'nc:BinaryLocationURI', ncBinaryLocationURINode) then
                  if ncBinaryLocationURINode.NodeValue <> null then
                  begin
                    BinaryLocationURI := ncBinaryLocationURINode.NodeValue;     //Retrived BinaryLocationURI
                  end;

                if NodeFound(ncAttachmentNode, 'nc:BinarySizeValue', ncBinarySizeValueNode) then
                  if ncBinarySizeValueNode.NodeValue <> null then
                  begin
                    BinarySizeValue := ncBinarySizeValueNode.NodeValue;         //Retrived BinarySizeValue
                  end;

                if NodeFound(ncAttachmentNode, 'nc:BinaryCategoryText', ncBinaryCategoryTextNode) then
                  if ncBinaryCategoryTextNode.NodeValue <> null then
                  begin
                    BinaryCategoryText := ncBinaryCategoryTextNode.NodeValue;   //Retrived BinaryCategoryText
                  end;


                if Assigned(FVlerAttachNodeList) then
                  if (includeText                                               // include everything in the list
                    or ((not includeText) and (pos('text', lowercase(BinaryFormatStandardName)) = 0))
                    )
                    then begin                                                  //Now assign to VLERAttachNodeList
                            //GetMem(pVLERAttachNodeValue,sizeof(TVLERAttachNodeVal));
                    new(pVLERAttachNodeValue);
                    pVLERAttachNodeValue^.strncDocumentFileControlID := DocFileCtrlID;
                    pVLERAttachNodeValue^.strncBinaryDescriptionText := BinaryDescriptionText;
                    pVLERAttachNodeValue^.strncBinaryFormatStandardName := BinaryFormatStandardName;
                    pVLERAttachNodeValue^.strncBinaryLocationURI := BinaryLocationURI;
                    pVLERAttachNodeValue^.strncBinarySizeValue := BinarySizeValue;
                    pVLERAttachNodeValue^.strncBinaryCategoryText := BinaryCategoryText;
                    FVLERAttachNodeList.Add(pVLERAttachNodeValue);
                  end;
              end;                                                              // if ncAttachmentNode <> nil
            end;                                                                //end of - for I := 0 to ncAttachmentNode.ChildNodes.Count - 1 do
          end;                                                                  // if cldAttachmentsNode.HasChildNodes

          //DONE WITH cld:Claim NODE LEVEL
        end;                                                                    //end of get values from cld:Claim node level

        RstlTmp := '"' + GivenNMclient + '"' + ',' + '"' + MiddleNMclient + '"' + ',' + '"' + SurNMclient + '"' + ',' + '"' + PersonDOB + '"' + ',' +
          '"' + PersonSSN + '"' + ',' + '"' + DocIDUUID + '"' + ',' + '"' + ExamID + '"' + ',' + '"' + DbqDocTitleText + '"' + ',' +
          '"' + DocCreateDT + '"' + ',' + '"' + ExamStatusDT + '"' + ',' + '"' + FacilityNM + '"' + ',' +
          '"' + GivenNMSrvProv + '"' + ',' + '"' + MiddleNMSrvProv + '"' + ',' + '"' + SurNMSrvProv + '"' + ',' +
          '"' + StatusOfExamRslt + '"' + ',' + '"' + DocFileCtrlID + '"';

        aResult.Add(RstlTmp);
      end;                                                                      // for x
    end                                                                         // if FXMLDocument.DocumentElement.HasChildNodes
    else
    begin
//      MessageDlg('No Records Found!', mtInformation, [mbOK], -1);
      ShowMessageCAPRI('No Records Found!');                                    // rpk 10/16/2014
      Result := False;
    end;

  except
    on E: Exception do
    begin
//      ShowMessage('Error occurred in eCrud: ' + E.Message);
      ShowMessageCAPRI('Error occurred in eCrud: ' + E.Message);                // rpk 10/16/2014
      Result := False;
    end;
  end;
end;                                                                            // getExamReviewDocument


// function TeCRUD.NodeFound(var aNode: IXmlNode; aNodeName: string; var FoundNode: IXmlNode): boolean;

function TeCRUD.NodeFound(aNode: IXmlNode; aNodeName: string; var FoundNode: IXmlNode): boolean;
var
  I: Integer;
  xNode: IXmlNode;
begin
  Result := False;
  FoundNode := nil;

  if Assigned(aNode) then begin
    try
      FoundNode := anode.ChildNodes[anodeName];
      result := True;
    except
      ;
    end;
    if not Result then begin
      // if ChildNodes fails, loop through child nodes by index and match anodeName to NodeName
      for I := 0 to aNode.ChildNodes.Count - 1 do begin
        xNode := aNode.ChildNodes[I];
        if xNode.NodeName = aNodeName then begin
          FoundNode := xNode;
          break;
        end;
      end;
    end;
    Result := FoundNode <> nil;
  end;                                                                          // if aNode assigned then
end;                                                                            // NodeFound

//****************************************************************************************
// procedure OpenDocumentFromXMLNode save binary attachments from "FS" collection to a file
//then call OpenDocument(pass filename) to open the document based on its type and handles
//destruction of open documents
//Paramaters:
////ncBinaryLocationURI: location of file in fs collection
//afilename: is used to create TFileStream and pass to OpenDocument()
//aFileType: is used for defining XML Header (ex: Content-Type: image/jpeg)
//*****************************************************************************************

procedure TeCRUD.OpenDocumentFromXMLNode(ancBinaryLocationURI, aFilename,
  aFileType, AttachmentSQL: string; BinarySzValue: integer);
var
 //fs: TFileStream;
  MS: TMemoryStream;
  strFileExt: string;
  strFileNamePath: string;
  aAttachmentSQL: string;
  DocOpen: Boolean;
  aTmpFilename: string;
begin

  aAttachmentSQL := AttachmentSQL;
  //need a HTTPserver - aHTTPServer
  //URI nc:BinaryLocationURI location of file in fs collection
  //need afilestream
  //need afilename  nc:BinaryDescriptionText + nc:BinaryCategoryText + nc:BinaryLocationURI
  //need type of file   nc:BinaryFormatStandardName ex: Accept: image/jpeg.

   //Set path, filename and extention
   //strFileNamePath:='.\'+aFilename+'.'+strFileExt;
  try
   //Create file stream create a filestream to write binary attachment
   //fs := TFileStream.Create(strFileNamePath, fmCreate);
    MS := TMemoryStream.Create;
    try
     //HTTPServer Get executes Attachment request to "fs" collection and result to TBlobField;
      FHTTPServer.Get(aAttachmentSQL, MS);
      aFilename := FHTTPServer.ManagedIdHTTP.Response.RawHeaders.Values['Content-Disposition'];
      aFilename := Copy(aFilename, (Pos('=', aFilename) + 1), (Length(aFilename) - (Pos('=', aFilename))));
      aTmpFilename := aFilename;
      aFilename := Copy(aFilename, 1, (Pos('.', aFilename) - 1));
      strFileExt := Copy(aTmpFilename, (Pos('.', aTmpFilename)), (Length(aTmpFilename) - (Pos('.', aTmpFilename) - 1)));
     //Set path, filename
      strFileNamePath := '.\' + aFilename + FormatDateTime('mmddyyyhhmmss', Now) + strFileExt;
      if assigned(MS) then
      begin

        MS.SaveToFile(strFileNamePath);
//        if assigned(aOpenDocList) = False then
//          aOpenDocList := TOpenDocList.Create;

//        DocOpen := aOpenDocList.OpenDocument(strFileNamePath);
        if Assigned(FOpenDocList) then begin
          DocOpen := FOpenDocList.OpenDocument(strFileNamePath);
//        if DocOpen <> True then
          if not DocOpen then
            MessageDlg('Attachment ' + strFileNamePath + 'could not be opened!', mtWarning, [mbOK], -1);
        end;
      end
    finally
     //fs.Free;
      MS.Free;
    end;
  except
    on E: Exception do
    begin
//      ShowMessage(E.Message);
      ShowMessageCAPRI(E.Message);                                              // rpk 10/16/2014
    end;
  end
end;

function TeCRUD.SelectNodes(xnRoot: IXmlNode; const nodePath: WideString): IXMLNodeList;
var
  intfSelect: IDomNodeSelect;
  intfAccess: IXmlNodeAccess;
  dnlResult: IDomNodeList;
  intfDocAccess: IXmlDocumentAccess;
  doc: TXmlDocument;
  i: Integer;
  dn: IDomNode;
begin
  Result := nil;
  if not Assigned(xnRoot)
    or not Supports(xnRoot, IXmlNodeAccess, intfAccess)
    or not Supports(xnRoot.DOMNode, IDomNodeSelect, intfSelect) then
    Exit;

  dnlResult := intfSelect.selectNodes(nodePath);
  if Assigned(dnlResult) then
  begin
    Result := TXmlNodeList.Create(intfAccess.GetNodeObject, '', nil);
    if Supports(xnRoot.OwnerDocument, IXmlDocumentAccess, intfDocAccess) then
      doc := intfDocAccess.DocumentObject
    else
      doc := nil;

    for i := 0 to dnlResult.length - 1 do
    begin
      dn := dnlResult.item[i];
      Result.Add(TXmlNode.Create(dn, nil, doc));
    end;
  end;
end;

// CodeCR496 - LMS - 2014-07-14
{function TeCRUD.GetTempDirectory: String;
var
  tempFolder: array[0..MAX_PATH] of Char;
begin
  GetTempPath(MAX_PATH, @tempFolder);
  result := StrPas(tempFolder);
end;
}

end.

