unit frmMVISearch;
//------------------------------------------------------------------------------
// CodeCR267 - JRL August, 2012 - Added unit/form
//             RPM August, 2012 - Testing (Debug) Utility portion
//
// MVI Web Service Search
// 
// This form is displayed wherever an "Enterprise Search" button exists.  
// This includes the main menu, the patient selection form and the restricted
// list patient selection form.  The previous "Enterprise Search" capability
// is preserved by clicking the button "Legacy Search".
//------------------------------------------------------------------------------

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Trpcb, CCOWRPCBroker, CCOWRPCBrokerCAPRI, MFunStr,
  ComCtrls, CommCtrl, Mask, ExtCtrls, DateUtils, Grids, dialogsCAPRI,
  VA508AccessibilityManager, VAUtils;

type
  TfrmMVIEnterpriseSearch = class(TForm)
    pageSearch: TPageControl;
    tsMVISearch: TTabSheet;
    tsDebug: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    edFirstName: TEdit;
    edMiddleName: TEdit;
    edLastName: TEdit;
    edSSN: TEdit;
    btnSearch: TButton;
    btnGetTreatingFacilities: TButton;
    edMVIID: TEdit;
    edCount: TEdit;
    edResultMsg: TEdit;
    lstSearchResults: TListBox;
    edTFCount: TEdit;
    edTFMessage: TEdit;
    lstTreatingFacilities: TListBox;
    dtpDOB: TDateTimePicker;
    edInitialQuantity: TEdit;
    pnlSearchCriteria: TPanel;
    leFirstName: TLabeledEdit;
    leMiddleName: TLabeledEdit;
    leLastName: TLabeledEdit;
    leSSN: TLabeledEdit;
    calDOB: TMonthCalendar;
    btnMVISearch: TButton;
    btnLegacySearch: TButton;
    pnlSearchResults: TPanel;
    lbSearchResults: TListBox;
    lblSearchResults: TLabel;
    btnGetTreatingFacilitiesList: TButton;
    pnlTreatingFacilities: TPanel;
    lblTreatingFacilities: TLabel;
    lbTreatingFacilities: TListBox;
    btnConnectToSite: TButton;
    btnClearSearch: TButton;
    sgSearchResults: TStringGrid;
    leICN: TLabeledEdit;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    edtSearchCriteriaHeading: TEdit;
    leDOB: TLabeledEdit;
    procedure btnSearchClick(Sender: TObject);
    procedure lstSearchResultsClick(Sender: TObject);
    procedure btnGetTreatingFacilitiesClick(Sender: TObject);
    procedure edMVIIDChange(Sender: TObject);
    procedure edFirstNameChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
//    procedure meDOBExit(Sender: TObject);
    procedure calDOBClick(Sender: TObject);
    procedure btnMVISearchClick(Sender: TObject);
    procedure btnClearSearchClick(Sender: TObject);
    procedure btnLegacySearchClick(Sender: TObject);
    procedure btnGetTreatingFacilitiesListClick(Sender: TObject);
    procedure lbTreatingFacilitiesClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnConnectToSiteClick(Sender: TObject);
    procedure SearchFieldChange(Sender: TObject);
    procedure sgSearchResultsClick(Sender: TObject);
    procedure lbSearchResultsClick(Sender: TObject);
    procedure leDOBExit(Sender: TObject);
  private
    { Private declarations }
    SearchResultsList : TStringList;
    TreatingFacilitiesList : TStringList;
    SelectedFacility : string;
    TFullNameRestricted : string;
    TSSNRestricted : string;
    TDOBRestricted : string;
    TICNRestricted : string;
    TSearchResultNumParam : string;  // '1' for unattended, '10' for attended
    procedure ClearSearch;  // debug routine
    procedure ClearTreatingFacilities;
    function FMToDateConvert(DateTimeST: String): String;
    procedure GetTreatingFacilities(TheIndex : Integer);
    procedure MVIClearFIelds;
    procedure MVIClearResultsFields;
    function ValidateName : Boolean;
    function ValidateSSN : Boolean;
    procedure SetupRestrictedSearch;
    procedure Load508SearchResults;
    procedure DateComplete(var StrLikeDate : string; MaxDate : TDate; const NowIfError : Boolean = TRUE);
  public       
    { Public declarations }
    dtDOB : TDate;
    property FullNameRestricted : string read TFullNameRestricted write TFullNameRestricted;
    property SSNRestricted : string read TSSNRestricted write TSSNRestricted;
    property DOBRestricted : string read TDOBRestricted write TDOBRestricted;
    property ICNRestricted : string read TICNRestricted write TICNRestricted; // future
    property SearchResultNumParam : string read TSearchResultNumParam write TSearchResultNumParam;
  end;       

var
  frmMVIEnterpriseSearch: TfrmMVIEnterpriseSearch;

implementation

uses Main, CAPRISupport, ESSOSelect;

{$R *.dfm}

//------------------------------------------------------------------------------
// Form Create
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.FormCreate(Sender: TObject);
begin
  // string lists that contain all the piece data
  SearchResultsList := TStringList.Create;
  TreatingFacilitiesList := TStringList.Create;
end;

//------------------------------------------------------------------------------
// Form Destroy
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.FormDestroy(Sender: TObject);
begin
  FreeAndNil(SearchResultsList);
  FreeAndNil(TreatingFacilitiesList);
end;


//------------------------------------------------------------------------------
// Form Activate
//------------------------------------------------------------------------------
// This form was developed from a testing utility form.  The original testing
// utility code was retained and put on a separate tab.  To run the utility,
// add the command line parameter:  SHOWDEBUGSEARCH to the project.  This 
// parameter will show the Debug utility as well as the MVI search on separate
// tabs.  The Debug tab will show by default.
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.FormActivate(Sender: TObject);
begin
  if frmMain.MVISearchDebug then // Is the SHOWDEBUGSEARCH command line parameter set?
  begin
    tsDebug.TabVisible := TRUE;
    tsDebug.Visible := TRUE;
    tsMVISearch.TabVisible := TRUE;
    tsMVISearch.Visible := TRUE;
    pageSearch.ActivePage := tsDebug;
  end
  else  // normal search mode
  begin
    tsDebug.TabVisible := FALSE;
    tsDebug.Visible := FALSE;
    tsMVISearch.TabVisible := FALSE;
    tsMVISearch.Visible := TRUE;
    pageSearch.ActivePage := tsMVISearch;
    MVIClearFIelds;
    SearchResultNumParam := '10';
    if SSNRestricted <> '' then 
    begin
       SearchResultNumParam := '1';
       SetupRestrictedSearch;
    end;
    // Display the error message lisbox at startup instead of the stringgrid.
    // The stringgrid always shows a row selected so the listbox is a better
    // visual choice until the stringgrid has results displayed
    lbSearchResults.Clear;
    lbSearchResults.Visible := TRUE;
    sgSearchResults.Visible := FALSE;
    if VAUtils.ScreenReaderActive then
    begin
      edtSearchCriteriaHeading.SetFocus;
      calDOB.Visible := FALSE;
    end
    else
    begin
      calDOB.Visible := TRUE;
      leFirstName.SetFocus;
    end;
  end;
end;

//------------------------------------------------------------------------------
// Button - Legacy Search button clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnLegacySearchClick(Sender: TObject);
begin
  frmMain.actToolsSearchExecute(btnLegacySearch);
end;

//------------------------------------------------------------------------------
// Button - Clear Search fields button clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnClearSearchClick(Sender: TObject);
begin
  MVIClearFIelds;
  btnGetTreatingFacilitiesList.Enabled := FALSE;
  sgSearchResults.Enabled := FALSE;
  lbSearchResults.Enabled := FALSE;
  btnConnectToSite.Enabled := FALSE;
  lbTreatingFacilities.Enabled := FALSE;
end;

//------------------------------------------------------------------------------
// Button - MVI Search button clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnMVISearchClick(Sender: TObject);
var
   SearchString : string;
   I : Integer;
   ResultsCount : Integer;
   ResultsMessage : string;
begin
  MVIClearResultsFields;
  if ValidateName AND ValidateSSN then
  begin
     ResultsCount := -1;
     SearchString := leFirstName.Text + '^' +
                     leMiddleName.Text + '^' +
                     leLastName.Text + '^' +
                     leSSN.Text + '^' +
                     FMToDateConvert(FormatDateTime('mm/dd/yyyy', dtDOB));
    // Get results
    try
      frmMain.RPCBroker1.Param[0].PType := literal;
      frmMain.RPCBroker1.Param[1].PType := literal;
      if CallRPC(frmMain.RPCBroker1, 'DVBA MVI SEARCH PERSON', [SearchString,SearchResultNumParam], nil) and
         (frmMain.RPCBroker1.Results.Count > 0) then
      begin
        for I := 0 to frmMain.RPCBroker1.Results.Count - 1 do
          if (I = 0) then
          begin
            ResultsCount := StrToInt(Piece(frmMain.RPCBroker1.Results[I],'^',1));  // Get Result Count
            ResultsMessage := Piece(frmMain.RPCBroker1.Results[I],'^',2);          // Get Result Message
          end
          else
            SearchResultsList.Add(frmMain.RPCBroker1.Results[I]);                 // Add results string to StringList
      end
      else
      begin
        lbSearchResults.Items.Add('RPCBroker call failed');              // if there are no results, then the RPC failed
        sgSearchResults.Visible := FALSE;
        lbSearchResults.Enabled := TRUE;
        lbSearchResults.Visible := TRUE;
        btnGetTreatingFacilitiesList.Enabled := FALSE;
        exit;
      end;
    except on e:exception do
      begin
        lbSearchResults.Items.Clear;
        lbSearchResults.Items.Add('Error executing Enterprise Search.  Please try again.');
        lbSearchResults.Enabled := TRUE;
        lbSearchResults.Visible := TRUE;
        sgSearchResults.Visible := FALSE;
        btnGetTreatingFacilitiesList.Enabled := FALSE;
        Exit;
      end;
    end;

    if ResultsCount = 0 then // Search returned 0 results.
    begin
      //  Search returned 0 results.  Message displayed will be:
      //  1.	No match found for patient
      //  2.	More than 10 potential matches found
      //  3.   Error Message
      lbSearchResults.Items.Clear;
      lbSearchResults.Items.Add(ResultsMessage);  // display message
      lbSearchResults.Enabled := TRUE;
      lbSearchResults.Visible := TRUE;
      sgSearchResults.Visible := FALSE;
      btnGetTreatingFacilitiesList.Enabled := FALSE;
    end // Search returned 0 results
    else
    begin // successful search
      // ---- SAMPLE SEARCH DATA -------------------------------------------------
      // R(0)=2
      // R(1)=CAPRI PATIENT^111905454^01/01/1980^1062212234V192931^NI^200M^USVHA
      // R(2)=CAPRI T PATIENT^111905454^01/01/1980^1008591712V479586^NI^200M^USVHA
      // -------------------------------------------------------------------------
      // clear grid and setup column widths for display data
      sgSearchResults.ColWidths[0] := 238;
      sgSearchResults.ColWidths[1] := 100;
      sgSearchResults.ColWidths[2] := 100;
      for I := 0 to sgSearchResults.ColCount - 1 do
        sgSearchResults.Cols[I].Clear;
      // populate grid
      for I := 0 to ResultsCount-1 do
      begin
        sgSearchResults.Cells[0,I] := Piece(SearchResultsList[I],'^',1); // Full name
        sgSearchResults.Cells[1,I] := Piece(SearchResultsList[I],'^',2); // SSN
        sgSearchResults.Cells[2,I] := Piece(SearchResultsList[I],'^',3); // DOB
      end;
      // limit selectable rows in string grid to those that have values
      sgSearchResults.RowCount := ResultsCount;
      // select the first row
      sgSearchResults.Row := 0;
      btnGetTreatingFacilitiesList.Enabled := TRUE;
      // if there is an exact match (1 value returned only), populate the
      // treating facilities list for that match automatically
      if ResultsCount = 1 then
      begin
         GetTreatingFacilities(sgSearchResults.Row);
         lbTreatingFacilities.ItemIndex := -1;
         btnConnectToSite.Enabled := FALSE;
      end;

      if VAUtils.ScreenReaderActive then
      begin
        Load508SearchResults;  //use lbSearchResults to display results for 508
      end
      else
      begin
        lbSearchResults.Visible := FALSE;  // hide the "error display" listbox
        sgSearchResults.Enabled := TRUE;
        sgSearchResults.Visible := TRUE;   // show the results stringgrid
        sgSearchResults.SetFocus;
      end;
    end; // successful search
  end
end;

//------------------------------------------------------------------------------
// Button - Get Treating Facilities List button clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnGetTreatingFacilitiesListClick(
  Sender: TObject);
begin
  GetTreatingFacilities(sgSearchResults.Row);
end;

//------------------------------------------------------------------------------
// Connect to Site button clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnConnectToSiteClick(Sender: TObject);
var
  I : Integer;
  IndexIntoSiteList : Integer;
  SiteNumber : string;
begin
  IndexIntoSiteList := -1;
  for I := 0 to formESSOSelect.slUserSites.Count-1 do
  begin
    SiteNumber := Piece(formESSOSelect.slUserSites[I],'^',5);
    if SelectedFacility = SiteNumber then
       IndexIntoSiteList := I;
  end;
  // if a site match is found, populate the ItemIndex value (global value switchtosite)
  // for the slUserSites list and fill the global value switchtopatientSSN
  if IndexIntoSiteList <> -1 then
  begin
    switchtosite := IntToStr(IndexIntoSiteList);
    switchtopatientSSN := sgSearchResults.Cells[1,sgSearchResults.Row];
  end
  else // cannot connect to site - clear variables and display message
  begin
    switchtosite := '';
    switchtopatientSSN := '';
    ShowMessageCAPRI('You either do not have access to the remote site or some other problem prevented connection.' + #13#10 + 'Try connecting manually instead using FILE | SWITCH SITES.');
  end;
  Close; 
end;


//------------------------------------------------------------------------------
// Get Treating Facilities
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.GetTreatingFacilities(TheIndex: Integer);
var
  I : Integer;
  FacilityCount : Integer;
  FacilityMessage : string;
  MVISearchID : string;
begin
  TreatingFacilitiesList.Clear;
  FacilityCount := -1;
  lbTreatingFacilities.Clear;
  MVISearchID := MFunStr.Piece(SearchResultsList[TheIndex],'^',4,7);
  // Get Treating Facility list
  if CallRPC(frmMain.RPCBroker1, 'DVBA MVI GET CORRESPONDING IDS', [MVISearchID], nil) and
     (frmMain.RPCBroker1.Results.Count > 0) then
  begin
    for I := 0 to frmMain.RPCBroker1.Results.Count - 1 do
      if (I = 0) then
      begin
        FacilityCount := StrToInt(Piece(frmMain.RPCBroker1.Results[I],'^',1));
        FacilityMessage := Piece(frmMain.RPCBroker1.Results[I],'^',2);
      end
      else
        TreatingFacilitiesList.Add((frmMain.RPCBroker1.Results[I]));
  end;
  // load treating facility listbox
  for I := 0 to FacilityCount - 1 do
     lbTreatingFacilities.Items.Add(Piece(TreatingFacilitiesList[I],'^',2) +
                                    ' (' +
                                    Piece(TreatingFacilitiesList[I],'^',3) + 
                                    ')');
  lbTreatingFacilities.Enabled := True;                                   
                                    //  // if there is only one treating facility, select it and enable connect button
//  if lbTreatingFacilities.Count = 1 then 
//  begin                                  
//    lbTreatingFacilities.ItemIndex := 0;
//    btnConnectToSite.Enabled := TRUE;
//    btnConnectToSite.SetFocus;
//  end;
end;


//------------------------------------------------------------------------------
// MVI Clear Fields
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.MVIClearFIelds;
begin
  // These fields could be read-only from a previous Restricted Patient Search
  leFirstName.Enabled := TRUE;
  leMiddleName.Enabled := TRUE;
  leLastName.Enabled := TRUE;
  leSSN.Enabled := TRUE;
  leICN.Enabled := TRUE;
  leDOB.Enabled := TRUE;
  btnClearSearch.Enabled := TRUE;
  if VAUtils.ScreenReaderActive then
     calDOB.Visible := FALSE
  else         
     calDOB.Visible := TRUE;         
  // Search Criteria Fields
  leFirstName.Text := '';
  leMiddleName.Text := '';
  leLastName.Text := '';
  leSSN.Text := '';
  leDOB.Clear;
  calDOB.Date := Today;
  MVIClearResultsFields;
end;


//------------------------------------------------------------------------------
// MVI Clear Results Fields
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.MVIClearResultsFields;
var
  I : Integer;
begin
  // Search Results Fields
  sgSearchResults.ColWidths[1] := 100;
  sgSearchResults.ColWidths[2] := 100;
  for I := 0 to 2 do
    sgSearchResults.Cols[I].Clear;
  sgSearchResults.Row := 0;  
  SearchResultsList.Clear;
  // "No Matches Found" listbox
  lbSearchResults.Items.Clear;
  // Treating Facilities Field
  lbTreatingFacilities.Items.Clear;
  TreatingFacilitiesList.Clear;
end;

//------------------------------------------------------------------------------
// Setup Restricted Patient Search
//------------------------------------------------------------------------------
// If the form variables FullNameRestricted, SSNRestricted, DOBRestricted and
// ICNRestricted exist, this means that the Enterprise search is doing a 
// Restricted List Enterprise Search.  Populate the edit fields with the form
// variables, set the edit fields to read-only as no other values are permitted.
// Then allow the user to search.
// FullName is in the format:  LastName,FirstName MiddleName
// DOB is a fileman date:  YYYMMDD where YYY is the number of years after 1700
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.SetupRestrictedSearch;
var
  LastName, FirstName, MiddleName : string;
  FMYear : Integer;
  CommaLocation, SpaceLocation : Integer;
begin
  // Populate SSN Field
  leSSN.Text := SSNRestricted;
  // Populate ICN field
  leICN.Text := ICNRestricted;  // hidden field  -  for future use
  // Populate DOB field
  if DOBRestricted = '' then
     DOBRestricted := '2000101';  // default DOB is 01/01/1900
  FMYear := StrToIntDef(Copy(DOBRestricted,1,3),-1);
  if FMYear <> -1 then
     FMYear := FMYear + 1700
  else
     FMYear := 1900;
  leDOB.Text := Copy(DOBRestricted,4,2) + '/' + Copy(DOBRestricted,6,2) + '/' + IntToStr(FMYear);
  // Populate Name Fields:  Parse FullName:  LastName,FirstName Middle
  CommaLocation := Pos(',',FullNameRestricted); 
  SpaceLocation := Pos(' ',FullNameRestricted);
  LastName := Copy(FullNameRestricted,1,CommaLocation-1);
  if SpaceLocation = 0 then  // no middle name
  begin
     FirstName := Copy(FullNameRestricted,CommaLocation+1,Length(FullNameRestricted));
     MiddleName := '';
  end
  else
  begin
     FirstName := Copy(FullNameRestricted,CommaLocation+1,SpaceLocation-CommaLocation-1);    
     MiddleName := Copy(FullNameRestricted,SpaceLocation+1,Length(FullNameRestricted));
  end;
  // Make all input fields Read-Only.  Hide the calendar as no date selection is allowed
  leFirstName.Text := FirstName;
  leMiddleName.Text := MiddleName;
  leLastName.Text := LastName;
  leFirstName.Enabled := FALSE;
  leMiddleName.Enabled := FALSE;
  leLastName.Enabled := FALSE;
  leSSN.Enabled := FALSE;
  leICN.Enabled := FALSE;
  leDOB.Enabled := FALSE;
  btnClearSearch.Enabled := FALSE;
  calDOB.Visible := FALSE;
end;


//------------------------------------------------------------------------------
// Validate Field - Name
//------------------------------------------------------------------------------
function TfrmMVIEnterpriseSearch.ValidateName: Boolean;
var
  IsValid : Boolean;
begin               
  try
    IsValid := TRUE;
    if Length(leFirstName.Text) < 1 then
       IsValid := FALSE;
    if Length(leLastName.Text) < 1 then 
    IsValid := FALSE;
    if IsValid = FALSE then
    begin
       ShowMessageCAPRI('First and last name cannot be blank.  Please correct before continuing search.');
       leFirstName.SetFocus;
       Result := FALSE;
    end
    else
    begin
       Result := TRUE;  // first and last name have are not blank
    end;
  except on e:exception do
    begin
      leFirstName.SetFocus;
      Result := FALSE;
    end;
  end;          
end;

//------------------------------------------------------------------------------
// Validate Field - SSN
//------------------------------------------------------------------------------
function TfrmMVIEnterpriseSearch.ValidateSSN : Boolean;
const
  BLANK_SSN = 'SSN cannot be blank.  Please correct before continuing search.';
  NOT_A_NUMBER = 'SSN must be numeric.  Please correct before continuing search.';
  SSN_NOT_NEGATIVE = 'SSN must be a positive number.  Please correct before continuing search.';
  SSN_9_CHARS = 'SSN must be 9 numbers long.  Please correct before continuing search.';
  SSN_GROUPING_ZERO = 'No SSN grouping can be zero.  Please correct before continuing search.';
  SSN_666 = 'The first SSN grouping cannot be 666.  Please correct before continuing search.';
var
  TestSSN : Integer;
  ErrorMessage : string;
begin
  // Validate the SSN
  try
    TestSSN := StrToIntDef(leSSN.Text,-1);
    if length(leSSN.Text) = 0 then  // blank SSN is not valid
       ErrorMessage := BLANK_SSN
    else if (TestSSN < 0) then  // Not a positive integer
       ErrorMessage := NOT_A_NUMBER
    else if (TestSSN <= 0) then  // Zero or negative numbers not allowed
       ErrorMessage := SSN_NOT_NEGATIVE
    else if length(leSSN.Text) < 9 then // SSN is 9 digits long
       ErrorMessage := SSN_9_CHARS
    // No SSN grouping can be zero
    else if (StrToIntDef(Copy(leSSN.Text,1,3),-1) = -1) OR (StrToIntDef(Copy(leSSN.Text,1,3),-1) = 0) then
       ErrorMessage := SSN_GROUPING_ZERO
    else if (StrToIntDef(Copy(leSSN.Text,4,2),-1) = -1) OR (StrToIntDef(Copy(leSSN.Text,4,2),-1) = 0) then
       ErrorMessage := SSN_GROUPING_ZERO
    else if (StrToIntDef(Copy(leSSN.Text,6,4),-1) = -1) OR (StrToIntDef(Copy(leSSN.Text,6,4),-1) = 0) then
       ErrorMessage := SSN_GROUPING_ZERO
    else if (StrToIntDef(Copy(leSSN.Text,1,3),-1) = -1) OR (StrToIntDef(Copy(leSSN.Text,1,3),-1) = 666) then // First grouping can't be 666
       ErrorMessage := SSN_666
    else
       ErrorMessage := '';
    if ErrorMessage <> '' then
    begin
       leSSN.Clear;
       ShowMessageCAPRI(ErrorMessage);
       leSSN.SetFocus;
       Result := FALSE;
    end
    else
    begin
       Result := TRUE; // valid SSN
    end;
  except on e:exception do
    begin
       Result := FALSE;
       leSSN.Clear;
    end;
  end;
end;


{==========================================================================
 Item Click - Search Results listbox (508)
 Synchronize the results String grid with the results listbox
 ==========================================================================}
procedure TfrmMVIEnterpriseSearch.lbSearchResultsClick(Sender: TObject);
begin
  if VAUtils.ScreenReaderActive and
     (SearchResultsList.Count > 0) and
     (lbSearchResults.ItemIndex > -1) then
  begin
    sgSearchResults.Row := lbSearchResults.ItemIndex;
    sgSearchResultsClick(Sender);
    btnGetTreatingFacilitiesList.Enabled := TRUE;
  end;
end;

//------------------------------------------------------------------------------
// Row Click - Treating Facilities listbox
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.lbTreatingFacilitiesClick(Sender: TObject);
begin
  btnConnectToSite.Enabled := TRUE;
  SelectedFacility := Piece(TreatingFacilitiesList[lbTreatingFacilities.ItemIndex],'^',1);
end;

procedure TfrmMVIEnterpriseSearch.leDOBExit(Sender: TObject);
var
  s : string;
  SaveDate : string;
  MaxDate : TDate;
begin
  s := leDOB.Text;
  SaveDate := s;
  MaxDate := Today;
  DateComplete(s, MaxDate, FALSE);
  leDOB.Text := s;
  if UpperCase(copy(leDOB.Text,1,4)) = 'MAX ' then
  begin
    leDOB.Text := '';
    ShowMessage('Date of Birth (' + DateToStr(dtDOB) + ') must be a past date');
    leDOB.SetFocus;
  end
  else
  begin
    calDOB.Date := dtDOB;
  end;
end;

{============================================================================
 Load508SearchResults
 This procedure populates a ListBox with the contents of the string grid to
 maintain 508 compliance.
 ============================================================================}
procedure TfrmMVIEnterpriseSearch.Load508SearchResults;
var
  I: Integer;
  aFormattedLine: String;
const
  //formatted column sizes allowing 2 spaces between columns
  NameColSize = 30;
  SSNColSize = 11;
  DOBColSize = 12;
begin
  lbSearchResults.Clear;
  for I := 0 to sgSearchResults.RowCount - 1 do
  begin
    aFormattedLine := sgSearchResults.Cells[0, I];
    aFormattedLine := aFormattedLine + StringOfChar(' ', NameColSize - Length(sgSearchResults.Cells[0, I])) + (sgSearchResults.Cells[1, I]);
    aFormattedLine := aFormattedLine + StringOfChar(' ', SSNColSize - Length(sgSearchResults.Cells[1, I])) + (sgSearchResults.Cells[2, I]);
    lbSearchResults.Items.Add(aFormattedLine);
  end;

  lbSearchResults.Enabled := TRUE;
  lbSearchResults.Visible := TRUE;  // show the results listbox
  sgSearchResults.Visible := FALSE;   // hide the results stringgrid
  lbSearchResults.SetFocus;
  btnGetTreatingFacilitiesList.Enabled := FALSE;
end;

//------------------------------------------------------------------------------
// Search Results StringGrid Click
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.sgSearchResultsClick(Sender: TObject);
begin
  lbTreatingFacilities.Items.Clear;
  btnConnectToSite.Enabled := FALSE;
end;

//------------------------------------------------------------------------------
// Search Input Field Changed - Clear results fields
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.SearchFieldChange(Sender: TObject);
begin
  MVIClearResultsFields;
end;

//------------------------------------------------------------------------------
// Calendar Clicked - update Date of Birth field
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.calDOBClick(Sender: TObject);
begin
  leDOB.Text := FormatDateTime('mm/dd/yyyy', calDOB.Date);
end;


//------------------------------------------------------------------------------
// Date Complete - Date validation function
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.DateComplete(var StrLikeDate : string; MaxDate : TDate; const NowIfError : Boolean = TRUE);
var
   DateStr : string;
   Year, Month, Day : Word;
   cnt, SepCount : Integer;
begin
   DateStr := StrLikeDate;
   if DateStr = '' then
      Exit;
   SepCount := 0;
   for cnt := 1 to Length(DateStr) do
   begin
     if not (DateStr[cnt] in ['0'..'9']) then
     begin
       DateStr[cnt] := DateSeparator;
       inc(SepCount);
     end;
   end;
   while (DateStr <> '') and (DateStr[Length(DateStr)]=DateSeparator) do
   begin
     Delete(DateStr, Length(DateStr),1);
     Dec(SepCount);
   end;
   DecodeDate(Now,Year,Month,Day);
   if SepCount = 0 then
   begin
     case Length(DateStr) of
       0   : DateStr := DateToStr(Now);
       1, 2: DateStr := DateStr+DateSeparator+IntToStr(Day);
//     1, 2: DateStr := DateStr+DateSeparator+IntToStr(Month);  // use this if short date format is dd/mm/yyyy
       4   : Insert(DateSeparator, DateStr, 3);
       6, 8: begin
             Insert(DateSeparator, DateStr, 5);
             Insert(DateSeparator, DateStr, 3);
             end; 
     end; // case
   end; // if SepCount
   try
     StrLikeDate := FormatDateTime('mm/dd/yyyy',StrToDate(DateStr));
     dtDOB := StrToDate(DateStr);
   Except
     if NowIfError = TRUE then
     begin
      StrLikeDate := DateToStr(Date)
     end
     else
        StrLikeDate := '';
   end;
   // Date is valid, check for greater than minimum date
   if StrToDate(DateStr) > MaxDate then
      StrLikeDate := 'Max ' + StrLikeDate;
end;


//------------------------------------------------------------------------------
// The routines labeled "Debug" are for the original MVI Search test utility.
//------------------------------------------------------------------------------
   

//------------------------------------------------------------------------------
// Debug - Get Treatingt Facilities Button Clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnGetTreatingFacilitiesClick(Sender: TObject);
var
  I: Integer;
begin
  ClearTreatingFacilities;
  if CallRPC(frmMain.RPCBroker1, 'DVBA MVI GET CORRESPONDING IDS', [edMVIID.Text], nil) and
     (frmMain.RPCBroker1.Results.Count > 0) then
  begin
    for I := 0 to frmMain.RPCBroker1.Results.Count - 1 do
      if (I = 0) then
      begin
        edTFCount.Text := Piece(frmMain.RPCBroker1.Results[I],'^',1);
        edTFMessage.Text := Piece(frmMain.RPCBroker1.Results[I],'^',2);
      end
      else
        lstTreatingFacilities.Items.Add(frmMain.RPCBroker1.Results[I]);
  end;
end;

//------------------------------------------------------------------------------
// Debug - Search Button Clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.btnSearchClick(Sender: TObject);
var
  I: Integer;
  SearchString : string;
begin
  ClearSearch;
  frmMain.RPCBroker1.Param[0].PType := literal;
  SearchString := edFirstName.Text + '^' + edMiddleName.Text
                  + '^' + edLastName.Text + '^' + edSSN.Text
                  + '^'
                  + FMToDateConvert(FormatDateTime('mm/dd/yyyy', dtpDOB.DateTime));
  if CallRPC(frmMain.RPCBroker1, 'DVBA MVI SEARCH PERSON', [SearchString], nil) and
     (frmMain.RPCBroker1.Results.Count > 0) then
  begin
    for I := 0 to frmMain.RPCBroker1.Results.Count - 1 do
      if (I = 0) then
      begin
        edCount.Text := Piece(frmMain.RPCBroker1.Results[I],'^',1);
        edResultMsg.Text := Piece(frmMain.RPCBroker1.Results[I],'^',2);
      end
      else
        lstSearchResults.Items.Add(frmMain.RPCBroker1.Results[I]);
  end
  else
    lstSearchResults.Items.Add('RPCBroker call failed');
end;

//------------------------------------------------------------------------------
// Debug - Clear Search
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.ClearSearch;
var
  S: String;
begin
  S := ' ';
  //dtpDOB.Perform(DTM_SETFORMAT, DWord(nil), DWord(S)); {make sure you have alteast 1 empty space}
  edCount.Text := '';
  edResultMsg.Text := '';
  lstSearchResults.Items.Clear;
  edMVIID.Text := '';
  //ClearTreatingFacilities;
end;

//------------------------------------------------------------------------------
// Debug - Clear Treating Facilities
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.ClearTreatingFacilities;
begin
  edTFCount.Text := '';
  edTFMessage.Text := '';
  lstTreatingFacilities.Items.Clear;
end;

//------------------------------------------------------------------------------
// Debug - First Name Change
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.edFirstNameChange(Sender: TObject);
begin
  ClearSearch;
end;

//------------------------------------------------------------------------------
// Debug - MVI ID Change
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.edMVIIDChange(Sender: TObject);
begin
  ClearTreatingFacilities;
  if (edMVIID.Text = '') then
    btnGetTreatingFacilities.Enabled := False
  else
    btnGetTreatingFacilities.Enabled := True;

end;

//------------------------------------------------------------------------------
// Debug - list search results clicked
//------------------------------------------------------------------------------
procedure TfrmMVIEnterpriseSearch.lstSearchResultsClick(Sender: TObject);
begin
  edMVIID.Text := MFunStr.Piece(lstSearchResults.Items[lstSearchResults.ItemIndex],'^',4,7);
  btnGetTreatingFacilities.Enabled := True;
end;

//------------------------------------------------------------------------------
// Debug - Fileman to Date Convert
//------------------------------------------------------------------------------
function TfrmMVIEnterpriseSearch.FMToDateConvert(DateTimeST: String): String;
var
  Month, Day, Year: String;
  Year2: String;
begin
  // Converts mm/dd/yyyy to a Fileman date
  Month := Copy(DateTimeSt, 1, Pos('/', DateTimeSt) - 1);
  DateTimeSt := Copy(DateTimeSt, Pos('/', DateTimeSt) + 1, 7);
  Day := Copy(DateTimeSt, 1, Pos('/', DateTimeSt) - 1);
  Year := Copy(DateTimeSt, Pos('/', DateTimeSt) + 1, 4);
  Year2 := Year[1] + Year[2] + '00';
  Result := IntToStr((StrToInt(Year2) - 1700) Div 100);
  Result := Result + Year[3] + year[4] + Month + Day;
end;


end.
