//------------------------------------------------------------------------------
// Patch 193 JRL 5/11/16
// This screen was added for the Re-Route Exam Request function in Patch 193.
// It allows input of Re-Route data to accompany a request sent to another site.
//------------------------------------------------------------------------------
unit rerouterequest;
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Fmcntrls, VA508AccessibilityManager, ORCtrls, ExtCtrls,
  CAPRISupport, dialogsCAPRI, Diaccess, Fmcmpnts, Trpcb, untMiscMthds,
  CCOWRPCBroker, CCOWRPCBrokerCAPRI, ComCtrls;

type
  TfrmReRouteRequest = class(TForm)
    Panel1: TPanel;
    FMReRouteReason: TFMComboBox;
    FMReRouteDescription: TFMMemo;
    FMReRouteLister: TFMLister;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    Panel2: TPanel;
    btnCancelReRoute: TButton;
    btnRerouteRequest: TButton;
    cbxLocation: TComboBox;
    sttxtReRouteReason: TStaticText;
    sttxtReRouteDescription: TStaticText;
    sttxtReRouteLocation: TStaticText;
    sttxtVAMC: TStaticText;
    FMVAMC: TFMComboBox;
    sttxtSortByState: TStaticText;
    cbxSortByState: TCheckBox;
    Panel3: TPanel;
    FMExamList: TFMListBox;
    sttxtExams: TStaticText;
    sttxtSiteComments: TStaticText;
    FMDivisionComments: TFMMemo;
    btnTestGuestLogin: TButton;
    btnTestRemoteRPC: TButton;
    StatusBar1: TStatusBar;
    btnTestRoutingLocations: TButton;
    btnTestExamList: TButton;
    btnTestSiteComments: TButton;
    edtStationNumber: TEdit;
    procedure btnCancelReRouteClick(Sender: TObject);
    procedure btnRerouteRequestClick(Sender: TObject);
    procedure btnTestGuestLoginClick(Sender: TObject);
    procedure cbxLocationChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure btnTestRemoteRPCClick(Sender: TObject);
    procedure btnTestRoutingLocationsClick(Sender: TObject);
    procedure btnTestExamListClick(Sender: TObject);
    procedure btnTestSiteCommentsClick(Sender: TObject);
    procedure cbxSortByStateClick(Sender: TObject);
    procedure FMVAMCExit(Sender: TObject);
  private
    { Private declarations }
    RoutingLocation_IEN : String;
    VAMC_IEN : String;
    VAMC_IEN_List : TStringList;
    VAMC_Station_List : TStringList;
    VAMC_Name_List : TStringList;
    VAMC_State_List : TStringList;
    RoutingLocation_IEN_List : TStringList;
    ReRouteReasonIENList : TStringList;
    SelectedStation : String;
    function CreateReRouteRequest : Boolean;
    function  CheckUserInputValues : Boolean;
    procedure GetRoutingLocations;
    procedure GetVAMCLocations;
    procedure GetExamLists;
    procedure GetSiteComments;
    procedure GetReRouteReasons;
    procedure IsDelphiRunning;
    function  CheckIfVAMCCanHandleReroutes : Boolean;
  public
    { Public declarations }
  end;

implementation
uses
  Main
, rofinder
, unitzip
, viewexam
, essoselect
  ;

  
const
  CLOSE_FORM = '1';     // Patch 193 JRL 9/2/16
  DONT_CLOSE_FORM = '2'; // Patch 193 JRL 9/2/16

 
{$R *.dfm}

//------------------------------------------------------------------------------
// Form Create
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.FormCreate(Sender: TObject);
begin
  // Display testing buttons if Delphi IDE is running.  This is only for
  // developer testing:  btnTestRoutingLocations, btnTestGuestLogin, 
  // btnTestRemoteRPC.
  IsDelphiRunning;  
  // Create string lists
  VAMC_IEN_List := TStringList.Create;
  RoutingLocation_IEN_List := TStringList.Create;
  VAMC_Station_List := TStringList.Create;
  VAMC_Name_List := TStringList.Create;
  VAMC_State_List := TStringList.Create;
  ReRouteReasonIENList := TStringList.Create;
  // Populate initial fields
  GetReRouteReasons; // Populate Re-Routing Reasons dropdown
  GetVAMCLocations;  // Populate VAMC Locations dropdown
end;


//------------------------------------------------------------------------------
// Form Close
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  // Destroy string lists
  FreeAndNil(VAMC_IEN_List);
  FreeAndNil(RoutingLocation_IEN_List);
  FreeAndNil(VAMC_Station_List);
  FreeAndNil(VAMC_Name_List);
  FreeAndNil(VAMC_State_List);
  FreeAndNil(ReRouteReasonIENList);
end;

//------------------------------------------------------------------------------
// Cancel Re-Route Button Clicked
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnCancelReRouteClick(Sender: TObject);
begin
  // No action needed except to close the form
  frmViewExam.ReturnValue := DONT_CLOSE_FORM;
  Close;
end;

//------------------------------------------------------------------------------
// Re-Route Request button clicked
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnRerouteRequestClick(Sender: TObject);
var
  request_rerouted : Boolean;
  CloseForm : Boolean;
begin
  if CheckUserInputValues then // if all inputs are satisfied, allow re-routing
  begin
     StatusBar1.SimpleText := 'Re-Routing Request';
     request_rerouted := CreateReRouteRequest;     // send the re-routed request
     if request_rerouted = TRUE then // re-route was successful
        frmViewExam.ReturnValue := CLOSE_FORM
     else
        frmViewExam.ReturnValue := DONT_CLOSE_FORM;
     Close;
  end;  // CheckUserInputValues
  
end;

//------------------------------------------------------------------------------
// Create Re-Route Request
//------------------------------------------------------------------------------
// RPC Input:    
//   Request IEN
//   Patient IEN
//   VAMC IEN (Station Number)
//   Routing Location IEN
//   Re-Route Reason IEN
//   Re-Route Description
//
// RPC Output:  
//   1^Transmitted as message # 909761 from this site to DEVVCC.DNS      .MED.DNS   
//   0^Message transmission error! Request WILL NOT be rerouted
// -----------------------------------------------------------------------------
// FROM THE AMIE TRANSFER DOCUMENTATION:
//   If the mail message is sent out successfully, the user will see:
//   "Transmitted as message # {number} from this site to {site name}.DNS   ."
// 
//   If the message fails (and therefore the transfer fails) the user will be so informed:
//   "Message transmission error!   Request WILL NOT be sent !"
// -----------------------------------------------------------------------------
function TfrmReRouteRequest.CreateReRouteRequest : Boolean;
CONST
  RPCNAME = 'DVBA CAPRI SEND REROUTE';
var
  MailmanRPCStatus : Integer;
  MailmanMessage : String;
begin
  Result := FALSE; // default to re-route failed
  try
    Screen.Cursor := crHourGlass;
    // Setup RPC
    frmMain.RPCBroker1.Results.Clear;
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].Value := frmViewExam.edExamReferenceNumber.Text;  // Request IEN
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := PatientIEN;                              // Patient IEN
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := VAMC_IEN;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := RoutingLocation_IEN;
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[4].Value := ReRouteReasonIENList[FMReRouteReason.ItemIndex];
    frmMain.RPCBroker1.Param[4].PType := literal;
    frmMain.RPCBroker1.Param[5].Value := FMReRouteDescription.Text;
    frmMain.RPCBroker1.Param[5].Ptype := literal;
    
    frmMain.RPCBrokerCall;
    Try
      frmMain.RPCBroker1.Call;
    Except
      On e: EBrokerError Do
      begin
        ShowMessageCAPRI(RPCNAME + ' could not be accessed!  '+#10#13 + E.Message + '.');
      end;
    End;
  finally
    Screen.Cursor := crDefault;
  end;

  // Check RPC results  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    MailmanRPCStatus := StrToIntDef(Piece(frmMain.RPCBroker1.Results[0], '^', 1),0);
    MailmanMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
    StatusBar1.SimpleText := MailmanMessage;
  
    if MailmanRPCStatus > 0 then // Success, request was re-routed
    begin
      // FYI: An email is sent to the original requester upon successful re-routing (all M code)
      ShowMessageCAPRI(MailmanMessage);  
      Result := TRUE;
      Close;
    end
  end;
  // if Re-Route message failed
  if not Result then
    ShowMessageCAPRI(RPCNAME + ' returned a failure.'+#10#13+MailmanMessage);
end;

//------------------------------------------------------------------------------
// Get VAMC Locations
//------------------------------------------------------------------------------
// VAMCs are located in the Institution File.  Filtering and sorting must be 
// done to get an accurate selection list.
// -----------------------------------------------------------------------------
// RPC Input:    Sorting Flag (NAME or STATE)
// RPC Output:   State ^ VAMC sites ^ VAMC IEN ^ VAMC Station Number
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.GetVAMCLocations;
var
  i : Integer;
CONST
  RPCNAME = 'DVBA CAPRI REROUTE VAMC';
  STRINGFORMAT = '%-28s%-15s%-6s';
begin
  // update status
  StatusBar1.SimpleText := 'Retrieving Routing Locations.';
  // setup and call RPC
  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  if cbxSortByState.Checked then
     frmMain.RPCBroker1.Param[0].Value := 'STATE'
  else
     frmMain.RPCBroker1.Param[0].Value := 'NAME';
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except
    On E: EBrokerError Do
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!'+#13#10+E.Message);
  End;
  // clear all lists
  FMVAMC.Items.Clear;
  VAMC_Name_List.Clear;
  VAMC_State_List.Clear;
  VAMC_IEN_List.Clear;
  VAMC_Station_List.Clear;
  // populate all lists
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    For i := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
    begin
      VAMC_Name_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      VAMC_State_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
      VAMC_IEN_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 3));
      VAMC_Station_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 4));
    end;
  end;
  // Add data to listbox and format it into columns
  for i := 0 to VAMC_Name_List.Count - 1 do
    FMVAMC.Items.Add(Format(STRINGFORMAT,[VAMC_Name_List[i],VAMC_State_List[i],VAMC_Station_List[i]]));
  // clear status bar
  StatusBar1.SimpleText := '';
 end;


//------------------------------------------------------------------------------
// Get Re-Route Reasons
//------------------------------------------------------------------------------
// RPC:  DVBA CAPRI REROUTEREASONS
//   Input:   None
//   Output:  Re-Route Code^IEN
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.GetReRouteReasons;
const
  RPCNAME = 'DVBA CAPRI GET REROUTE CODE';
var
  i : Integer;
begin
  frmMain.RPCBroker1.Results.Clear;                                         
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;          
  frmMain.RPCBrokerCall;                                                    
  try                                                                       
    frmMain.RPCBroker1.Call;                                                
  except                                                                    
    On EBrokerError Do                                                      
       ShowMessageCAPRI('RPC: '+RPCNAME + ' could not be accessed!'); 
  end;                                                                      
  FMReRouteReason.Items.Clear;                                     
  ReRouteReasonIENList := TStringList.Create;                          
  if frmMain.RPCBroker1.Results.Count > 0 then                              
  begin                                                                     
    for i := 0 To frmMain.RPCBroker1.Results.Count - 1 do                   
    begin                                                                   
      FMReRouteReason.Items.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      ReRouteReasonIENList.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
    end;                                                                   
  end;                       
end;


//------------------------------------------------------------------------------
// Get Routing Locations
//------------------------------------------------------------------------------
// Call RPC on remote VistA site specified by Station Number passed to RPC.
// Input:    
//           RPC: XWB DIRECT RPC    
//           RPC: Station Number of remote VistA instance (VAMC)
//           Local RPC:  DVBA CAPRI GET DIVISION
//           Local RPC Version:  Not specified here, sending blank
//           Local RPC Parameter:  Station Number
// Output:   List of Routing Locations (Divisions) for the input Station Number
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.GetRoutingLocations;
CONST  
  RPC_DIRECT = 'XWB DIRECT RPC'; 
  RPCNAME = 'DVBA CAPRI GET DIVISION';
var 
  i : Integer;
begin
  StatusBar1.SimpleText := 'Retrieving Routing Locations.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  FMExamList.Items.Clear;
  
  // determine if we are re-routing locally versus to a remote VistA site
  if frmViewExam.CurrentStationNumber = SelectedStation then
  begin // Call RPC locally
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
  end
  else
  begin // Call RPC via XWB DIRECT RPC
    frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := RPCNAME;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := SelectedStation;
  end;

  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    cbxLocation.Items.Clear;
    RoutingLocation_IEN_List.Clear;
    If frmMain.RPCBroker1.Results.Count > 0 Then
    begin
      For i := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
      begin
        cbxLocation.Items.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
        RoutingLocation_IEN_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      end;
    end;
  except
    On E: EBrokerError Do
    begin
      Screen.Cursor := crDefault;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + SelectedStation + '!' +#13#10 + E.Message);
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';
end;


//------------------------------------------------------------------------------
// Get Exam Lists
//------------------------------------------------------------------------------
// Input:    
//           RPC: XWB DIRECT RPC    
//           RPC: Station Number of remote VistA instance (VAMC)
//           Local RPC:  DVBA CAPRI GET DIV EXAM
//           Local RPC Version:  Not specified here, sending blank
//           Local RPC Parameter:  Routing Location (Division) IEN
// Output:   List of Exams Available at selected site
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.GetExamLists;
CONST   
  RPC_DIRECT = 'XWB DIRECT RPC'; 
  RPCNAME = 'DVBA CAPRI GET DIV EXAM';
var
  i : Integer;
begin
  StatusBar1.SimpleText := 'Retrieving Exam Lists.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  FMExamList.Items.Clear;
  if frmViewExam.CurrentStationNumber = SelectedStation then
  begin
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := RoutingLocation_IEN;  // routing location (division) selected
  end
  else
  begin
    frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := RPCNAME;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := RoutingLocation_IEN;  // routing location (division) selected
  end;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    If frmMain.RPCBroker1.Results.Count > 0 Then
      QuickCopy(frmMain.RPCBroker1.Results, FMExamList.Items);
  except
    On EBrokerError Do
    begin
      Screen.Cursor := crHourGlass;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + SelectedStation + '!');
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';
end;


//------------------------------------------------------------------------------
// Get Site Comments
//------------------------------------------------------------------------------
// Routing Locations are located in the Institution File.  Filtering and sorting
// must be done to get an accurate selection list.
// -----------------------------------------------------------------------------
// Input:
//           RPC: XWB DIRECT RPC    
//           RPC: Station Number of remote VistA instance (VAMC)
//           Local RPC:  DVBA CAPRI GET DIV COMMENT
//           Local RPC Version:  Not specified here, sending blank
//           Local RPC Parameter:  Routing Location (Division) IEN
// Output:   List of Re-Route sites
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.GetSiteComments;
CONST  
  RPC_DIRECT = 'XWB DIRECT RPC'; 
  RPCNAME = 'DVBA CAPRI GET DIV COMMENT';
begin
  StatusBar1.SimpleText := 'Retrieving Site Comments.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  FMDivisionComments.Lines.Clear;
  if frmViewExam.CurrentStationNumber = SelectedStation then
  begin
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := RoutingLocation_IEN;  // routing location (division) selected
  end
  else
  begin
    frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := RPCNAME;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := RoutingLocation_IEN;  // routing location (division) selected
  end;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    If frmMain.RPCBroker1.Results.Count > 0 Then
      QuickCopy(frmMain.RPCBroker1.Results, FMDivisionComments.Lines);    
  except
    On EBrokerError Do
    begin
      Screen.Cursor := crDefault;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + SelectedStation + '!');
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';
end;

//------------------------------------------------------------------------------
// Check User Input Values
//------------------------------------------------------------------------------
// Check all user input values.  If values are input correctly, return TRUE.
// If values are missing, routine short circuits back to input form.
//------------------------------------------------------------------------------
function TfrmReRouteRequest.CheckUserInputValues: Boolean;
begin
  if FMReRouteReason.Text = '' then
  begin
     ShowMessageCAPRI('Please enter a Re-Routing Reason.');
     Result := FALSE;
     exit;  // make user complete form before sending
  end;
  if FMReRouteReason.Text = 'OTHER' then
  begin
    if Length(FMReRouteDescription.Lines.Text) = 0 then
    begin
       ShowMessageCAPRI('Please enter a Re-Routing Description.');
       Result := FALSE;
       exit;  // make user complete form before sending
    end;
  end;
  if FMVAMC.ItemIndex = -1 then
  begin
    ShowMessageCAPRI('Please enter a VAMC.');
    Result := FALSE;
    exit;  // make user complete form before sending
  end;
  if cbxLocation.ItemIndex = -1 then
  begin
     ShowMessageCAPRI('Please enter a Routing Location.');
     Result := FALSE;
     exit;  // make user complete form before sending
  end;
  Result := TRUE;
end;


//------------------------------------------------------------------------------
// Select VAMC
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.FMVAMCExit(Sender: TObject);
var
  buffer : String;
begin
  Application.ProcessMessages;
  if FMVAMC.ItemIndex <> -1 then
     VAMC_IEN := VAMC_IEN_list[FMVAMC.ItemIndex]
  else
     exit;
  
  SelectedStation := VAMC_Station_List[FMVAMC.ItemIndex];

  // If we can't re-route to the selected VAMC, set the VAMC to blank and 
  // give the user a message.
  if CheckIfVAMCCanHandleReroutes = FALSE then
  begin
    buffer := Copy(FMVAMC.Text,1,28);
    FMVAMC.ItemIndex := -1;
    ShowMessageCAPRI(trim(buffer) + ' cannot accept re-routed requests at this time.');
    FMVAMC.SetFocus;
  end
  else
    GetRoutingLocations;

end;

//------------------------------------------------------------------------------
// Check if VAMC Can Handle Re-Routes
//------------------------------------------------------------------------------
// If the destination site doesn't have Patch 193 installed, then it will not
// know how to receive a re-routed request.  Don't allow the user to re-route
// to a site where Patch 193 isn't installed. 
// Input:    Station Number at distant site
// Output:   
//    TRUE:  ok to re-route
//    FALSE: cannot re-route to this site 
// -----------------------------------------------------------------------------
function TfrmReRouteRequest.CheckIfVAMCCanHandleReroutes: Boolean;
const
   RPC_DIRECT = 'XWB DIRECT RPC'; 
   RPCNAME = 'DVBA CHECK PATCH';
   PATCHNUM = 'DVBA*2.7*193';
begin
  
  Result := FALSE;
  // if ItemIndex is not selected (or forced to -1), don't test.  Just fail
  if FMVAMC.ItemIndex = -1 then
     exit;
  // if we are re-routing within the same VAMC, do not check the patch.
  // if we got this far, the patch is installed.
  if SelectedStation = frmViewExam.CurrentStationNumber then
  begin
     Result := TRUE;
     Exit;
  end;
  
  try
    Screen.Cursor := crHourGlass;
    // check patch number at selected site.  If Patch 193 is not installed, error
    frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := RPCNAME;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := PATCHNUM;
    try
      frmMain.RPCBrokerCall;
      frmMain.RPCBroker1.Call;
      if frmMain.RPCBroker1.Results.Count > 0 then
        if piece(frmMain.RPCBroker1.Results[0], '^', 1) = '1' then
           Result := TRUE;
    except
      begin
        ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + SelectedStation + '!');
        Result := FALSE;
      end
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

 

//------------------------------------------------------------------------------
// Select Routing Location
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.cbxLocationChange(Sender: TObject);
begin
  Application.ProcessMessages;
  if cbxLocation.ItemIndex <> -1 then
  begin
     RoutingLocation_IEN := RoutingLocation_IEN_List[cbxLocation.ItemIndex];
     StatusBar1.SimpleText := 'Retrieving Exam Info';
     GetExamLists;
     GetSiteComments;
     StatusBar1.SimpleText := '';
  end;                            
end;



//------------------------------------------------------------------------------
// Sort By State / Name
//------------------------------------------------------------------------------
// Reload VAMC list and sort by State if checkbox is set, name if checkbox is
// empty.
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.cbxSortByStateClick(Sender: TObject);
begin
   GetVAMCLocations;
end;    

//------------------------------------------------------------------------------
// Is Delphi Running?
//------------------------------------------------------------------------------
// If Delphi is running, we are in the IDE and operating in a testing mode.  
// Display testing buttons.  If Delphi is not running, do not display test 
// buttons.
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.IsDelphiRunning;
begin
  if DebugHook <> 0 then
  begin
    btnTestGuestLogin.Visible := TRUE;
    btnTestRemoteRPC.Visible := TRUE;
    btnTestRoutingLocations.Visible := TRUE;
    btnTestExamList.Visible := TRUE;
    btnTestSiteComments.Visible := TRUE;
    edtStationNumber.Visible := TRUE;
  end
  else
  begin
    btnTestGuestLogin.Visible := FALSE;
    btnTestRemoteRPC.Visible := FALSE;
    btnTestRoutingLocations.Visible := FALSE;
    btnTestExamList.Visible := FALSE;
    btnTestSiteComments.Visible := FALSE;
    edtStationNumber.Visible := FALSE;
  end;

end;

//------------------------------------------------------------------------------
// TEST ROUTINE - TEST GUEST LOGIN TO TEST CLAIMS SERVER
//------------------------------------------------------------------------------
// This routine logs a "local user" silently onto the TEST CLAIMS system and
// retrieves data via a Fileman control as well as via a RPC.  Proof of concept
// for logging in silently to TEST CLAIMS which is a direction not used in CAPRI
// up to this point.  The audit log will login to CLAIMS, but that is not done
// on a test account.
//------------------------------------------------------------------------------
// TEST CLAIMS DATA:
//     FILE:  396.97, FIELD .01
//         @DEV/FEX TEST SYSTEM
//         ZZ 553DT
//         KANSAS CITY(DSS)
//         BAY PINES-GC
//         BAY PINES-PRRTP
//         ZZ WEST PLAINS CBOC
//         CHEYENNE-VBA
//         BAY PINES-CIOFO
//         BAY PINES NURSING HOME
//         CHEY133.DNS        
//         ORLANDO VAMC (EFF. 10/1/06)
//         CHEY133_LINUX.DNS        
//         ZZ BEST
//         ALASKA VAHSRO
//         IRON MOUNTAIN VAMC
//         CHYSHR
//         DAYTSHR
//         CHEYL147
//         CHEYL135
//------------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnTestGuestLoginClick(Sender: TObject);
var
  tempServer : String;
  tempPort : Integer;  
  tempTestMode : Boolean;
const
  TESTCLAIMSSERVER = 'VISTA.DNS   .MED.DNS   ';
  TESTPORT = '9401';
begin

  Application.ProcessMessages;
  try
    Screen.Cursor := crHourGlass;
    StatusBar1.SimpleText := 'Connecting to TEST CLAIMS server';
    tempServer := frmMain.RPCBroker1.Server;
    tempPort := frmMain.RPCBroker1.ListenerPort;
    tempTestMode := frmMain.RPCBroker1.ANUTestMode;
    frmMain.RPCBroker1.Connected := False;
    frmMain.RPCBroker1.Server := TESTCLAIMSSERVER;
    frmMain.RPCBroker1.ListenerPort := StrToInt(TESTPORT);
    frmMain.RPCBroker1.ANUTestMode := TRUE;
    if not frmMain.ConnectToServer('DVBA CAPRI GUI') then
    begin
       ShowMessageCAPRI('Cannot connect to ' + TESTCLAIMSSERVER);
       exit;
    end; 
    StatusBar1.SimpleText := 'Connected to TEST CLAIMS server';
    
    // Populate list with Fileman control data
    FMExamList.Clear;
    FMDivisionComments.Clear;
    // Populate list with RPC data             
    StatusBar1.SimpleText := 'Get Fileman Control Data.';
    FMExamList.FMLister := FMReRouteLister;
    FMReRouteLister.GetList(FMExamList.Items);

    StatusBar1.SimpleText := 'Get RPC Data.';
    frmMain.RPCBroker1.RemoteProcedure := 'DVBA CHECK PATCH';
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := 'DVBA*2.7*190';
    try
      frmMain.RPCBrokerCall;
      frmMain.RPCBroker1.Call;
      if frmMain.RPCBroker1.Results.Count > 0 then
        if piece(frmMain.RPCBroker1.Results[0], '^', 1) = '1' then
           FMDivisionComments.Lines.Add('Patch 190 installed')
        else
           FMDivisionComments.Lines.Add('Patch 190 in not installed')
    except on E: Exception do
       FMDivisionComments.Lines.Add('Exception on GetData: ' + E.Message);
    end;

  finally
    StatusBar1.SimpleText := 'Disconnect from TEST CLAIMS.  Reconnect to local server.';
    frmMain.RPCBroker1.Connected := False;
    frmMain.RPCBroker1.Server := TempServer;
    frmMain.RPCBroker1.ANUTestMode := tempTestMode;
    frmMain.RPCBroker1.ListenerPort := TempPort;
    frmMain.ConnectToServer('DVBA CAPRI GUI');
    StatusBar1.SimpleText := 'Reconnected to local server.';
    Screen.Cursor := crDefault;
  end;
  
end;


// -----------------------------------------------------------------------------
// TEST ROUTINE - TEST ROUTING LOCATIONS
// -----------------------------------------------------------------------------
// This routine calls DVBA CAPRI GET DIVISION using the station number input in
// the TEST STATION / DIVISION input field.  Only used during development on 
// the local station number.
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnTestRoutingLocationsClick(Sender: TObject);
const
  RPCNAME = 'DVBA CAPRI GET DIVISION';
var 
  i : Integer;
begin
  StatusBar1.SimpleText := 'Retrieving Routing Locations.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  cbxLocation.Items.Clear;
  RoutingLocation_IEN_List.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[0].Value := edtStationNumber.Text;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    If frmMain.RPCBroker1.Results.Count > 0 Then
    begin
      For i := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
      begin
        cbxLocation.Items.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
        RoutingLocation_IEN_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      end;
    end;
  except
    On EBrokerError Do
    begin
      Screen.Cursor := crDefault;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!');
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';
end;


// -----------------------------------------------------------------------------
// TEST ROUTINE - TEST SITE COMMENTS
// -----------------------------------------------------------------------------
// Calls DVBA CAPRI GET DIV COMMENT from the local VistA instance.  The value
// input in the TEST STATION/DIVISION input should be a local division IEN.
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnTestSiteCommentsClick(Sender: TObject);
CONST  
  RPCNAME = 'DVBA CAPRI GET DIV COMMENT';
  DIVISION = '3';  // ALBANY ON DEVFEX
var
  i : Integer;
begin
  StatusBar1.SimpleText := 'Retrieving Site Comments.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  FMDivisionComments.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].Value := DIVISION;
  frmMain.RPCBroker1.Param[0].PType := literal;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    If frmMain.RPCBroker1.Results.Count > 0 Then
      QuickCopy(frmMain.RPCBroker1.Results, FMExamList.Items);    
  except
    On EBrokerError Do
    begin
      Screen.Cursor := crDefault;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed at Division ' + DIVISION + '!');
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';
end;

// -----------------------------------------------------------------------------
// TEST ROUTINE - TEST EXAM LIST
// -----------------------------------------------------------------------------
// Calls DVBA CAPRI GET DIV EXAM from the local VistA instance.  The value
// input in the TEST STATION/DIVISION input should be a local division IEN.
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnTestExamListClick(Sender: TObject);
CONST   
  RPCNAME = 'DVBA CAPRI GET DIV EXAM';
  DIVISION = '3';  // ALBANY ON DEVFEX
begin
  StatusBar1.SimpleText := 'Retrieving Exam Lists.';
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  FMExamList.Items.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[0].Value := DIVISION;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    If frmMain.RPCBroker1.Results.Count > 0 Then
     QuickCopy(frmMain.RPCBroker1.Results, FMExamList.Items);    
  except
    On EBrokerError Do
    begin
      Screen.Cursor := crHourGlass;
      ShowMessageCAPRI(RPCNAME + ' could not be accessed at Division ' + DIVISION + '!');
    end;
  end;
  Screen.Cursor := crDefault;
  StatusBar1.SimpleText := '';

end;



// -----------------------------------------------------------------------------
// TEST ROUTINE - XWB DIRECT RPC
// -----------------------------------------------------------------------------
// This routine calls an RPC that uses HL7 to communicate between VistA 
// instances.  It was exactly what we needed, but our test accounts do not have
// this HL7 interface installed.  In addition, TEST CLAIMS does not have a 
// unique station number that is published that also causes issues.  Leaving 
// this routine here in case we come back to it.
//
// Update:  HL7 was added to our test environments between DEVFEX and TEST 
// CLAIMS.  Other dev environments may be added.  Our dev environment has
// unique station numbers between the systems it knows about (only dev, not
// production) so HL7 can communicate.  HL7 is currently deployed in all 
// production environments so no worries about using the RPC in production.
// -----------------------------------------------------------------------------
// XWB DIRECT RPC
//   Param 0: Station Number
//   Param 1: Name of RPC
//   Param 2: RPC VERSION PARAMETER (Optional)
//   Param 3: Remote RPC Parameters
// -----------------------------------------------------------------------------
procedure TfrmReRouteRequest.btnTestRemoteRPCClick(Sender: TObject);
const
   RPCNAME = 'DVBA CHECK PATCH';
   PATCH = 'DVBA*2.7*193';
var
  ReturnValue : String;
begin
  FMExamList.Items.Clear;
  frmMain.RPCBroker1.RemoteProcedure := 'XWB DIRECT RPC';
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[0].Value := edtStationNumber.Text;
  frmMain.RPCBroker1.Param[1].PType := literal;
  frmMain.RPCBroker1.Param[1].Value := RPCNAME;
  frmMain.RPCBroker1.Param[2].PType := literal;
  frmMain.RPCBroker1.Param[2].Value := '';
  frmMain.RPCBroker1.Param[3].PType := literal;
  frmMain.RPCBroker1.Param[3].Value := PATCH;
  try
    frmMain.RPCBrokerCall;
    frmMain.RPCBroker1.Call;
    if frmMain.RPCBroker1.Results.Count > 0 then
     QuickCopy(frmMain.RPCBroker1.Results, FMExamList.Items);
  except
    ShowMessage('Exception');
  end;
end;


end.
