unit UnitSiblings;

interface

uses Windows,
  Classes,
  SysUtils,
  Controls,
  StdCtrls,
  tabnotbk,
  comctrls,
  forms,
  extctrls,
  CPWMRadioGroup,
  ORCtrls,
  Graphics,
  buttons,
  PNCSCodeRun,
  ColorCapriBtn,
  Menus,
  HyperlinkLabel,
  Dialogs,
  CheckLst;

type

  TColoredControl = class(TControl); // needed to change protected Color in UpdateSibling

procedure UpdateSiblings2(baseForm: TComponent; editedControl: TControl);
procedure UpdateSiblingObject(selfControl: TControl; editedControl: TControl);
procedure CloneValue(editedControl, selfControl: TControl);
function IsCloneable(editedControl, selfControl: TControl): boolean;
procedure CloneORListBoxValue(editedControl, selfControl: TControl);
function IsNullValue(editedControl: TControl): boolean;
procedure CloneListBoxValue(editedControl, selfControl: TControl);
function IsUpdated(aControl: TControl): Boolean;

const
  SIBLING_COLOR: TColor = $00C8F4FB;
  NORMAL_COLOR: TColor = clBtnFace;
  BLANK_COLOR: TColor = clWhite;

var
  MayUpdateSiblings: boolean = true; // Default to allow UpdateSiblings
  FormRepopulated: boolean = false;
  {rpm 4/1/09 - Defect #2165 - replace timestamp/tag method of determining
                               replication status with TList }
  UpdatingControls: TList;
implementation

uses
    Main,
    PNCSMain;

var
  fUpdatingValue: boolean = false;
  PreventRecursionCtrl: TControl;

  {-----------------------------------------------------------------------------
    Procedure: UpdateSiblings2   (Based on TCPWMExam.UpdateSiblings)
    Author:    DNS   tighee
    Date:      01-Mar-2004
    Updated:   13-Sep-2005
    Arguments: editedControl : TControl
    Result:    None
    Loop through a list of all the components that represent User Controls.
    These are controls that have a numeric hint. Make the contents and selection
    of each user control the same as the selected and editted control, so user
    can ignore redundant controls on this or merged exams. Each affected control
    has it's color changed to indicated they have been handled in this fashion.
  -----------------------------------------------------------------------------}
//procedure UpdateSiblings2(baseForm: TComponent; editedControl: TControl); // Exam
//var
//  i: integer;
//  child: TComponent;
//begin
//  if MayUpdateSiblings then begin
//    try
//      PauseScript := True;
//      for i := 0 to baseForm.ComponentCount - 1 do begin
//        child := baseForm.Components[i];
//        if (child is TControl) then
//          UpdateSiblingObject(child as TControl, editedControl)
//      end;
//    finally;
//      PauseScript := False;
//    end;
//  end;
//
//end;

procedure UpdateSiblings2(baseForm: TComponent; editedControl: TControl);   { WLM 3/26/08 }
var
  selfControl: TComponent;

  i: integer;

  procedure ClickIt(aComponent: TComponent);
  begin
    if aComponent is TEdit then begin
      if @TEdit(aComponent).onChange <> nil then try
          TEdit(aComponent).onChange(TEdit(aComponent));
        except
        end;
      if @TEdit(aComponent).onClick <> nil then try
          TEdit(aComponent).onClick(TEdit(aComponent));
        except
        end;
    end
    else
      if aComponent is TCheckBox then begin
        if @TCheckbox(aComponent).onClick <> nil then
          TCheckbox(aComponent).onClick(TCheckbox(aComponent));
      end
      else
        if aComponent is TRadioButton then begin
          if @TRadioButton(aComponent).onClick <> nil then
            TRadioButton(aComponent).onClick(TRadioButton(aComponent));
        end
        else
          if aComponent is TMemo then begin
            if @TMemo(aComponent).OnClick <> nil then try
                TMemo(aComponent).OnClick(TMemo(aComponent));
              except
              end;
            if @TMemo(aComponent).OnChange <> nil then try
                TMemo(aComponent).OnChange(TMemo(aComponent));
              except
              end;
          end
          else
            if aComponent is TComboBox then begin
              if @TComboBox(aComponent).onClick <> nil then try
                  TComboBox(aComponent).onClick(TComboBox(aComponent));
                except
                end;
              if @TComboBox(aComponent).onChange <> nil then try
                  TComboBox(aComponent).onChange(TComboBox(aComponent));
                except
                end;
            end
            else
              if aComponent is TRadioGroup then begin
                if @TRadiogroup(aComponent).onClick <> nil then
                  TRadiogroup(aComponent).onClick(TRadiogroup(aComponent));
              end
              else
                if aComponent is TCPWMRadioGroup then begin
                  if @TCPWMRadiogroup(aComponent).onClick <> nil then
                    TCPWMRadiogroup(aComponent).onClick(TCPWMRadiogroup(aComponent));
                end
                else
                  if aComponent is TListBox then begin
                    if @TListBox(aComponent).onClick <> nil then
                      TListBox(aComponent).onClick(TListBox(aComponent));
                  end
                  else
                    if aComponent is TCheckListBox then begin
                      if @TCheckListBox(aComponent).onClick <> nil then
                        TCheckListBox(aComponent).onClick(TCheckListBox(aComponent));
                    end
                    else
                      if aComponent is TORListBox then begin
                        if @TORListBox(aComponent).onClick <> nil then
                          TORListBox(aComponent).onClick(TORListBox(aComponent));
                      end
                      else begin
                        Showmessage('Script error!  CAPRI doesn''t know how to handle ' + TComponent(aComponent).name + ' : ' + aComponent.ClassName + '.');
                      end;
  end;

begin
  { WLM 3/26/08 }

  if assigned(PNCSForm) and
  not FormRepopulated and
  not MergedFormMode then
  Exit;

  if PNCSForm.xEditMode.text = 'NEW' then
      Exit;


  if MayUpdateSiblings then begin
    try
      PauseScript := True;
      if (PreventRecursionCtrl <> nil) and
        (PreventRecursionCtrl.Hint = editedControl.Hint) and
        (PreventRecursionCtrl.Name = editedControl.Name) then begin
        PreventRecursionCtrl := nil;
        Exit;
      end;

      for i := 0 to baseForm.ComponentCount - 1 do begin
        selfControl := baseForm.Components[i];
        if (selfControl is TControl) and
          (Length(Trim(editedControl.Hint)) > 0) and
          {(selfControl.Tag <> editedControl.Tag) and //rpm 4/1/09 - Defect #2165}
          (NOT IsUpdated(TControl(selfControl))) and
          (trim(TControl(selfControl).Hint) = trim(editedControl.Hint)) then begin
          PreventRecursionCtrl := TControl(selfControl);
          if not fUpdatingValue then try
              fUpdatingValue := True;
              CloneValue(editedControl, TControl(selfControl)); // Make values the same , setting color as appropriate for control
              ClickIt(SelfControl);
              //  end;
            finally
              fUpdatingValue := False;
            end;
        end; { if }
      end; { for }
    finally;
      PauseScript := False;
      PreventRecursionCtrl := nil;
    end;
  end;

end;

{-----------------------------------------------------------------------------
  Procedure: UpdateSiblingObject (based on TCPWMObject.UpdateSiblings)
  Author:    DNS   tighee
  Date:      01-Mar-2004
  Updated:   13-Sep-2005
  Arguments: editedControl : TControl
  Result:    None

  Driven from the UpdateSiblings2().
  Make the contents and selection of each user control the same as the selected
  and edited control, so user can ignore redundant controls on this or merged
  exams. Each affected control has it's color changed to indicated they have
  been handled in this fashion.
-----------------------------------------------------------------------------}
procedure UpdateSiblingObject(selfControl: TControl; editedControl: TControl);
  procedure ClickIt(aComponent: TComponent);
  begin
    if aComponent is TEdit then begin
      if @TEdit(aComponent).onChange <> nil then try
          TEdit(aComponent).onChange(TEdit(aComponent));
        except
        end;
      if @TEdit(aComponent).onClick <> nil then try
          TEdit(aComponent).onClick(TEdit(aComponent));
        except
        end;
    end
    else
      if aComponent is TCheckBox then begin
        if @TCheckbox(aComponent).onClick <> nil then
          TCheckbox(aComponent).onClick(TCheckbox(aComponent));
      end
      else
        if aComponent is TRadioButton then begin
          if @TRadioButton(aComponent).onClick <> nil then
            TRadioButton(aComponent).onClick(TRadioButton(aComponent));
        end
        else
          if aComponent is TMemo then begin
            if @TMemo(aComponent).OnClick <> nil then try
                TMemo(aComponent).OnClick(TMemo(aComponent));
              except
              end;
            if @TMemo(aComponent).OnChange <> nil then try
                TMemo(aComponent).OnChange(TMemo(aComponent));
              except
              end;
          end
          else
            if aComponent is TComboBox then begin
              if @TComboBox(aComponent).onClick <> nil then try
                  TComboBox(aComponent).onClick(TComboBox(aComponent));
                except
                end;
              if @TComboBox(aComponent).onChange <> nil then try
                  TComboBox(aComponent).onChange(TComboBox(aComponent));
                except
                end;
            end
            else
              if aComponent is TRadioGroup then begin
                if @TRadiogroup(aComponent).onClick <> nil then
                  TRadiogroup(aComponent).onClick(TRadiogroup(aComponent));
              end
              else
                if aComponent is TCPWMRadioGroup then begin
                  if @TCPWMRadiogroup(aComponent).onClick <> nil then
                    TCPWMRadiogroup(aComponent).onClick(TCPWMRadiogroup(aComponent));
                end
                else
                  if aComponent is TListBox then begin
                    if @TListBox(aComponent).onClick <> nil then
                      TListBox(aComponent).onClick(TListBox(aComponent));
                  end
                  else
                    if aComponent is TCheckListBox then begin
                      if @TCheckListBox(aComponent).onClick <> nil then
                        TCheckListBox(aComponent).onClick(TCheckListBox(aComponent));
                    end
                    else
                      if aComponent is TORListBox then begin
                        if @TORListBox(aComponent).onClick <> nil then
                          TORListBox(aComponent).onClick(TORListBox(aComponent));
                      end
                      else begin
                        Showmessage('Script error!  CAPRI doesn''t know how to handle ' + TComponent(aComponent).name + ' : ' + aComponent.ClassName + '.');
                      end;
  end;
begin
  //if selfControl.Tag <> editedControl.Tag then //rpm 4/1/09 - Defect #2165
  if (NOT IsUpdated(TControl(selfControl))) then
    if not fUpdatingValue then try
        fUpdatingValue := True;
        if Length(Trim(editedControl.Hint)) > 0 then
          if trim(selfControl.Hint) = trim(editedControl.Hint) then begin
            //          ControlToCopy := selfControl.Name;
            //          ControlCopied := editedControl.Name;
            //          UserMsg := 'Copy to control ' + ControlToCopy + ' Control Copied ' + ControlCopied;
            //          Application.MessageBox(PChar(UserMsg), PChar('Common Controls'), MB_OK);
            CloneValue(editedControl, selfControl); // Make values the same , setting color as appropriate for control
            ClickIt(SelfControl);
          end;
      finally
        fUpdatingValue := False;
      end;
end;

{-----------------------------------------------------------------------------
  Procedure: IsNullValue
  Author:    DNS   tighee
  Date:      10-May-2004
  Updated:   13-Sep-2005
  Arguments: editedControl : TControl
  Result:    boolean
  Checks to see if the new value has a 'Null' state such as a blank Memo or a RadioGroup
  with no button checked
-----------------------------------------------------------------------------}
function IsNullValue(editedControl: TControl): boolean;
var
  tempTORListBox: TORListBox;
  tempTListBox: TListBox;
begin
  Result := True;

  if editedControl is TORListBox then begin
    tempTORListBox := editedControl as TORListBox;
    if tempTORListBox.MultiSelect then begin
      if tempTORListBox.SelCount > 0 then
        Result := False;
    end
    else
      if tempTORListBox.ItemIndex > -1 then
        Result := False;
    exit;
  end
  else
    if editedControl is TListBox then begin
      tempTListBox := editedControl as TListBox;
      if tempTListBox.MultiSelect then begin
        if tempTListBox.SelCount > 0 then
          Result := False;
      end
      else
        if tempTListBox.ItemIndex > -1 then
          Result := False;
      exit;
    end
    else
      if editedControl is TRadioGroup then begin
        if (editedControl as TRadioGroup).ItemIndex > -1 then
          Result := False;
        exit;
      end
      else
        if editedControl is TCPWMRadioGroup then begin
          if (editedControl as TCPWMRadioGroup).ItemIndex > -1 then
            Result := False;
          exit;
        end
        else
          if editedControl is TCheckBox then begin
            if (editedControl as TCheckBox).State <> cbUnchecked then
              Result := False;
            exit;
          end
          else
            if editedControl is TMemo then begin
              if Length(Trim((editedControl as TMemo).Text)) > 0 then
                Result := False;
              exit;
            end
            else
              if editedControl is TComboBox then begin
                if Length(Trim((editedControl as TComboBox).Text)) > 0 then
                  Result := False;
                exit;
              end
              else
                if editedControl is TEdit then begin
                  if Length(Trim((editedControl as TEdit).Text)) > 0 then
                    Result := False;
                  exit;
                end;
end;

{-----------------------------------------------------------------------------
  Procedure: CloneValue
  Author:    DNS   tighee
  Date:      01-Mar-2004
  Updated:   13-Sep-2005
  Arguments: editedControl, selfControl : TControl
  Result:    None
  Helper to UpdateSiblings. Makes appropriate actions to make the items and
  selection the same for the passed in edited control and the current control
  found to have the same hint, which equates the control as logically the same.

  selfControl is the discovered control and edited control is the actual control
  the user modified.
-----------------------------------------------------------------------------}
procedure CloneValue(editedControl, selfControl: TControl);
begin
  if editedControl is TORListBox then begin
    if ((formRepopulated = false) and ((editedControl as TORListBox).ItemIndex = -1)) then
      exit;
    if not IsCloneable(editedControl, selfControl) then
      exit;
    CloneORListBoxValue(editedControl, selfControl);
    //rpm 4/1/09 - Defect #2165
    //selfControl.Tag := editedControl.Tag;
    UpdatingControls.Add(selfControl);
    if not IsNullValue(editedControl) then begin
      TColoredControl(selfControl).Color := SIBLING_COLOR;
      TColoredControl(editedControl).Color := SIBLING_COLOR;
    end
    else begin
      TColoredControl(selfControl).Color := BLANK_COLOR;
      TColoredControl(editedControl).Color := BLANK_COLOR;
    end;
  end
  else
    if editedControl is TListBox then begin
      if ((formRepopulated = false) and ((editedControl as TListBox).ItemIndex = -1)) then
        exit;
      if not IsCloneable(editedControl, selfControl) then
        exit;
      try
        CloneListBoxValue(editedControl, selfControl);
      except
      end;
      try
        //rpm 4/1/09 - Defect #2165
        //selfControl.Tag := editedControl.Tag;
        UpdatingControls.Add(selfControl);
      except
      end;
      if not IsNullValue(editedControl) then begin
        TColoredControl(selfControl).Color := SIBLING_COLOR;
        TColoredControl(editedControl).Color := SIBLING_COLOR;
      end
      else begin
        TColoredControl(selfControl).Color := BLANK_COLOR;
        TColoredControl(editedControl).Color := BLANK_COLOR;
      end;
    end
    else
      if editedControl is TRadioGroup then begin
        if ((formRepopulated = false) and ((editedControl as TRadioGroup).ItemIndex = -1)) then
          exit;
        if not IsCloneable(editedControl, selfControl) then
          exit;
        (selfControl as TRadioGroup).ItemIndex := (editedControl as TRadioGroup).ItemIndex;
        //rpm 4/1/09 - Defect #2165
        //selfControl.Tag := editedControl.Tag;
        UpdatingControls.Add(selfControl);
        if not IsNullValue(editedControl) then begin
          TColoredControl(selfControl).Color := SIBLING_COLOR;
          TColoredControl(editedControl).Color := SIBLING_COLOR;
        end
        else begin
          TColoredControl(selfControl).Color := TColoredControl(selfControl.Parent).Color; // ET 050328  // NORMAL_COLOR;
          TColoredControl(editedControl).Color := TColoredControl(editedControl.Parent).Color; // ET 050328  // NORMAL_COLOR;
        end;
      end
      else
        if editedControl is TCPWMRadioGroup then begin
          if ((formRepopulated = false) and ((editedControl as TCPWMRadioGroup).ItemIndex = -1)) then
            exit;
          if not IsCloneable(editedControl, selfControl) then
            exit;
          (selfControl as TCPWMRadioGroup).ItemIndex := (editedControl as TCPWMRadioGroup).ItemIndex;
          //rpm 4/1/09 - Defect #2165
          //selfControl.Tag := editedControl.Tag;
          UpdatingControls.Add(selfControl);
          if not IsNullValue(editedControl) then begin
            TColoredControl(selfControl).Color := SIBLING_COLOR;
            TColoredControl(editedControl).Color := SIBLING_COLOR;
          end
          else begin
            TColoredControl(selfControl).Color := TColoredControl(selfControl.Parent).Color; // ET 050328 // NORMAL_COLOR;
            TColoredControl(editedControl).Color := TColoredControl(editedControl.Parent).Color; // ET 050328 // NORMAL_COLOR;
          end;
        end
        else
          if editedControl is TCheckBox then begin
            if ((formRepopulated = false) and ((editedControl as TCheckBox).Checked = False)) then
              exit;
            if not IsCloneable(editedControl, selfControl) then
              exit;
            (selfControl as TCheckBox).State := (editedControl as TCheckBox).State;
            //rpm 4/1/09 - Defect #2165
            //selfControl.Tag := editedControl.Tag;
            UpdatingControls.Add(selfControl);
            if not IsNullValue(editedControl) then begin
              TColoredControl(selfControl).Color := SIBLING_COLOR;
              TColoredControl(editedControl).Color := SIBLING_COLOR;
            end
            else begin
              TColoredControl(selfControl).Color := TColoredControl(selfControl.Parent).Color; // ET 050328 // NORMAL_COLOR;
              TColoredControl(editedControl).Color := TColoredControl(editedControl.Parent).Color; // ET 050328 // NORMAL_COLOR;
            end;
          end
          else
            if editedControl is TMemo then begin
              if ((formRepopulated = false) and ((editedControl as TMemo).Text = '')) then
                exit;
              if not IsCloneable(editedControl, selfControl) then
                exit;
              (selfControl as TMemo).Text := (editedControl as TMemo).Text;
              //rpm 4/1/09 - Defect #2165
              //selfControl.Tag := editedControl.Tag;
              UpdatingControls.Add(selfControl);
              if not IsNullValue(editedControl) then begin
                TColoredControl(selfControl).Color := SIBLING_COLOR;
                TColoredControl(editedControl).Color := SIBLING_COLOR;
              end
              else begin
                TColoredControl(selfControl).Color := BLANK_COLOR;
                TColoredControl(editedControl).Color := BLANK_COLOR;
                Sleep(10); // needed a delay for a 'mouse backspace' followed by a character
              end;
            end
            else
              if editedControl is TPageControl then begin
                if not IsCloneable(editedControl, selfControl) then
                  exit;
                (selfControl as TPageControl).ActivePageIndex := (editedControl as TPageControl).ActivePageIndex;
                //rpm 4/1/09 - Defect #2165
                //selfControl.Tag := editedControl.Tag;
                UpdatingControls.Add(selfControl);
              end
              else
                if editedControl is TComboBox then begin
                  if ((formRepopulated = false) and ((editedControl as TComboBox).Text = '')) then
                    exit;
                  if not IsCloneable(editedControl, selfControl) then
                    exit;
                  (selfControl as TComboBox).Items := (editedControl as TComboBox).Items;
                  (selfControl as TComboBox).ItemIndex := (editedControl as TComboBox).ItemIndex;
                  (selfControl as TComboBox).Text := (editedControl as TComboBox).Text;
                  //rpm 4/1/09 - Defect #2165
                  //selfControl.Tag := editedControl.Tag;
                  UpdatingControls.Add(selfControl);
                  if not IsNullValue(editedControl) then begin
                    TColoredControl(selfControl).Color := SIBLING_COLOR;
                    TColoredControl(editedControl).Color := SIBLING_COLOR;
                  end
                  else begin
                    TColoredControl(selfControl).Color := BLANK_COLOR;
                    TColoredControl(editedControl).Color := BLANK_COLOR;
                  end;
                end
                else
                  if editedControl is TEdit then begin
                    if ((formRepopulated = false) and ((editedControl as TEdit).Text = '')) then
                      exit;
                    if not IsCloneable(editedControl, selfControl) then
                      exit;
                    (selfControl as TEdit).Text := (editedControl as TEdit).Text;
                    //rpm 4/1/09 - Defect #2165
                    //selfControl.Tag := editedControl.Tag;
                    UpdatingControls.Add(selfControl);
                    if not IsNullValue(editedControl) then begin
                      TColoredControl(selfControl).Color := SIBLING_COLOR;
                      TColoredControl(editedControl).Color := SIBLING_COLOR;
                    end
                    else begin
                      TColoredControl(selfControl).Color := BLANK_COLOR;
                      TColoredControl(editedControl).Color := BLANK_COLOR;
                    end;
                  end;
end;

{-----------------------------------------------------------------------------
  Procedure: CloneListBoxValue
  Author:    DNS   tighee
  Date:      01-Mar-2004
  Updated:   13-Sep-2005
  Arguments: editedControl, selfControl : TControl
  Result:    None
  Helper to UpdateSiblings and CloneValue for ListBox types.

  selfControl is the discovered control and editedControl is the actual control
  the user modified.

  Called when editedControl is of TORListBox type.
-----------------------------------------------------------------------------}
procedure CloneListBoxValue(editedControl, selfControl: TControl);
var
  i: integer;
begin
  if selfControl is TListBox then begin
    (selfControl as TListBox).Clear;
    (selfControl as TListBox).Items := (editedControl as TListBox).Items;
    if (editedControl as TListBox).MultiSelect then begin
      for i := 0 to (editedControl as TListBox).Items.Count - 1 do try
          if (editedControl as TListBox).Selected[i] then
            (selfControl as TListBox).Selected[i] := True;
        except
        end;
    end;
  end
  else begin
    (selfControl as TORListBox).Clear;
    (selfControl as TORListBox).Items := (editedControl as TListBox).Items;
    if (editedControl as TListBox).SelCount > -1 then
      for i := 0 to (editedControl as TListBox).Items.Count - 1 do try
          if (editedControl as TListBox).Selected[i] then
            (selfControl as TORListBox).Selected[i] := True;
        except
        end;
  end;

  if selfControl is TListBox then
    (selfControl as TListBox).ItemIndex := (editedControl as TListBox).ItemIndex
  else
    (selfControl as TORListBox).ItemIndex := (editedControl as TListBox).ItemIndex;
end;

{-----------------------------------------------------------------------------
  Procedure: CloneORListBoxValue
  Author:    DNS   tighee
  Date:      01-Mar-2004
  Updated:   13-Sep-2005
  Arguments: editedControl, selfControl : TControl
  Result:    None
  Helper to UpdateSiblings and CloneValue for ORListBox types.

  selfControl is the discovered control and edited control is the actual control
  the user modified.

  Called when editedControl is of TORListBox type.
-----------------------------------------------------------------------------}
procedure CloneORListBoxValue(editedControl, selfControl: TControl);
var
  i: integer;
begin
  if selfControl is TORListBox then begin
    (selfControl as TORListBox).Clear;
    (selfControl as TORListBox).Items := (editedControl as TORListBox).Items;
    if (editedControl as TORListBox).SelCount > -1 then
      for i := 0 to (editedControl as TORListBox).Items.Count - 1 do
        if (editedControl as TORListBox).Selected[i] then
          (selfControl as TORListBox).Selected[i] := True;
  end
  else begin
    (selfControl as TListBox).Clear;
    (selfControl as TListBox).Items := (editedControl as TORListBox).Items;
    if (editedControl as TORListBox).SelCount > -1 then
      for i := 0 to (editedControl as TORListBox).Items.Count - 1 do
        if (editedControl as TORListBox).Selected[i] then
          (selfControl as TListBox).Selected[i] := True;
  end;

  if selfControl is TORListBox then
    (selfControl as TORListBox).ItemIndex := (editedControl as TORListBox).ItemIndex
  else
    (selfControl as TListBox).ItemIndex := (editedControl as TORListBox).ItemIndex;
end;

{-----------------------------------------------------------------------------
  Procedure: IsCloneable
  Author:    DNS   tighee
  Date:      01-Mar-2004
  Updated:   13-Sep-2005
  Arguments: editedControl : TControl
  Result:    None
  Determinies if a a pair of objects need to have thier values cloned
-----------------------------------------------------------------------------}
function IsCloneable(editedControl, selfControl: TControl): boolean;
begin

  // Special handling for TORListBox and TListBox
  if ((editedControl.ClassType = TORListBox) or (editedControl.ClassType = TListBox)) and
    ((selfControl.ClassType = TORListBox) or (selfControl.ClassType = TListBox)) then begin
    Result := True;
    exit;
  end;

  if editedControl.ClassType <> selfControl.ClassType then begin
{$IFDEF CLONE_DEBUG}
    ShowmessageCAPRI('Type Mismatch: ' + editedControl.Name + '[' + editedControl.hint + '] is ' + editedControl.ClassName + ' but ' + selfControl.Name + '[' + selfControl.hint + '] is ' + selfControl.ClassName);
{$ENDIF}
    Result := False
  end
  else
    Result := True;
end;

{==================================================================
 rpm 4/1/09
 This function searches the replicated controls list
 for a given control.
 Input:  aControl: TControl
 Output:  returns True if control found; otherwise returns False
 ==================================================================}
function IsUpdated(aControl: TControl): Boolean;
var
  I: Integer;
begin
  Result := False;
  for I := 0 to UpdatingControls.Count - 1 do
    if UpdatingControls.Items[I] = aControl then
    begin
      Result := True;
      Break;
    end;
end;

end.

