{******************************************************************}
{*     IPTERM.PAS - Telnet terminal and related classes           *}
(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is TurboPower Internet Professional
 *
 * The Initial Developer of the Original Code is
 * TurboPower Software
 *
 * Portions created by the Initial Developer are Copyright (C) 2000-2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** *)

{ Global defines potentially affecting this unit }
{$I IPDEFINE.INC}

{ Options required for this unit }
{$Z-}

{$IFOPT R+}
{$DEFINE UseRangeChecks}
{$ENDIF}

{.$DEFINE TermDebug}

{$R IPTRMVT1.R32}
{$R IPCHSVT1.R32}

Unit IpTerm;

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  ExtCtrls,
  Forms,
  Dialogs,
  ClipBrd,
  IpUtils,
  IpConst,
  IpSock;

Type
  TIpCaptureMode = ({ terminal data capture modes.. }
    cmOff, { ..no capturing of data }
    cmOn, { ..capture new data to new file }
    cmAppend); { ..capture data, appending to old file }

  TIpTerminalCursorMovedEvent =
    Procedure(aSender: TObject; aRow, aCol: Integer) Of Object;

  TIpCursorType = ({ cursor types }
    ctNone, { ..no cursor is visible }
    ctUnderline, { ..underline cursor }
    ctBlock); { ..block cursor }

  TIpVT100LineAttr = ({ special VT100 line attributes }
    ltNormal, { ..normal, no special attrs }
    ltDblHeightTop, { ..line is top half of double-height line }
    ltDblHeightBottom, { ..line is bottom half of double-height line }
    ltDblWidth); { ..line is double-width }

Const
  {Emulator commands}
  eNone = 0; { No command, ignore this char }
  eChar = 1; { No command, process the char }
  eGotoXY = 2; {X} { Absolute goto cursor position call }
  eUp = 3; {X} { Cursor up }
  eDown = 4; {X} { Cursor down }
  eRight = 5; {X} { Cursor right }
  eLeft = 6; {X} { Cursor left }
  eClearBelow = 7; {R} { Clear screen below cursor }
  eClearAbove = 8; {R} { Clear screen above cursor }
  eClearScreen = 9; {R} { Clear entire screen }
  eClearEndofLine = 10; {R} { Clear from cursor to end of line }
  eClearStartOfLine = 11; {R} { Clear from cursor to the start of line }
  eClearLine = 12; {R} { Clear entire line that cursor is on }
  eSetMode = 13; {X} { Set video mode }
  eSetBackground = 14; { Set background attribute }
  eSetForeground = 15; { Set foreground attribute }
  eSetAttribute = 16; {X} { Set video attribute (foreground and background) }
  eSaveCursorPos = 17; { Save cursor position }
  eRestoreCursorPos = 18; { Restore cursor position }
  eDeviceStatusRePORT     ; {X} { Report device status or cursor position }
  eString = 20; { Pascal style string }
  eHT = 21; { Horizontal Tab Character }
  eError = 255; { indicates a parser error }

  eAPC { } = 30; { Application programming command }
  eBEL {X} = 115; { Sound bell }
  eBS {X} = 116; { Backspace }
  eCBT {X} = 31; { Cursor backward tabulation }
  eCCH { } = 32; { Cancel character }
  eCHA {X} = 33; { Cursor horizontal absolute }
  eCHT {X} = 34; { Cursor horizontal tabulation }
  eCNL {X} = 35; { Cursor next line }
  eCPL {X} = 36; { Cursor preceding line }
  eCPR {X} = 37; { Cursor position report }
  eCR {X} = 118; { Carriage return }
  eCRM {.} = 38; { Control representation mode }
  eCTC {X} = 39; { Cursor tabulation control }
  eCUB {X} = eLeft; { Cursor backward }
  eCUD {X} = eDown; { Cursor down }
  eCUF {X} = eRight; { Cursor forward }
  eCUP {X} = eGotoXY; { Cursor position }
  eCUU {X} = eUp; { Cursor up }
  eCVT {X} = 40; { Cursor vertical tabulation }
  eDA {X} = 41; { Device attributes }
  eDAQ { } = 42; { Define area qualification }
  eDCH {X} = 43; { Delete character }
  eDCS { } = 44; { Device control string }
  eDL {X} = 45; { Delete line }
  eDMI { } = 46; { Disable manual input }
  eDSR {X} = eDeviceStatusReport; { Device status report }
  eEA { } = 47; { Erase in area }
  eEBM { } = 48; { Editing boundry mode }
  eECH {X} = 49; { Erase character }
  eED {X} = 50; { Erase in Display }
  eEF { } = 51; { Erase in field }
  eEL {X} = 52; { Erase in line }
  eEMI { } = 53; { Enable manual input }
  eENQ {X} = 114; { Enquiry request }
  eEPA { } = 54; { End of protected mode }
  eERM { } = 55; { Erasure mode }
  eESA { } = 56; { End of selected area }
  eFEAM { } = 57; { Format effector action mode }
  eFETM { } = 58; { Format effector transfer mode }
  eFNT { } = 59; { Font selection }
  eGATM { } = 60; { Guarded area transfer mode }
  eGSM { } = 61; { Graphics size modification }
  eGSS { } = 62; { Graphics size selection }
  eHEM { } = 63; { Horizontal editing mode }
  eHPA {X} = eCHA; { Horizontal position absolute }
  eHPR {X} = eCUF; { Horizontal position relative }
  eHTJ {X} = 64; { Horizontal tab with justification }
  eHTS {X} = 65; { Horizontal tabulation set }
  eHVP {X} = eCUP; { Horizontal and vertical position }
  eICH {X} = 66; { Insert character }
  eIL {X} = 67; { Insert line }
  eIND {X} = eCUD; { Index }
  eIND2 {X} = 121; { Corrected eIND (<> eCUD, eDown) new term only }
  eINT { } = 68; { Interrupt }
  eIRM {.} = 69; { Inseration-Replacement mode }
  eJFY { } = 70; { Justify }
  eKAM {.} = 71; { Keyboard action mode }
  eLF {X} = 117; { Line feed command }
  eLNM {.} = 72; { Line feed new line mode }
  eMATM { } = 73; { Multiple area transfer mode }
  eMC {.} = 74; { Media copy }
  eMW {.} = 75; { Message waiting }
  eNEL {X} = 76; { Next line }
  eNP {.} = 77; { Next page }
  eOSC { } = 78; { Operating system command }
  ePLD { } = 79; { Partial line down }
  ePLU { } = 80; { Partial line up }
  ePM { } = 81; { Privacy message }
  ePP {.} = 82; { Preceding page }
  ePU1 { } = 83; { Private use 1 }
  ePU2 { } = 84; { Private use 2 }
  ePUM { } = 85; { Positioning unit mode }
  eQUAD { } = 86; { Quad }
  eREP { } = 87; { Repeat }
  eRI {X} = 88; { Reverse index }
  eRIS {.} = 89; { Reset to initial state }
  eRM {.} = 90; { Reset mode }
  eSATM { } = 91; { Selected area transfer mode }
  eSD { } = 92; { Scroll down }
  eSEM { } = 93; { selected editing extent mode }
  eSGR {X} = eSetAttribute; { Select graphics rendition }
  eSI {X} = 120; { Invoke G0 charset }
  eSL { } = 94; { Scroll left }
  eSM {.} = eSetMode; { Set Mode }
  eSO {X} = 119; { Invoke G1 charset }
  eSPA { } = 95; { Start of protected area }
  eSPI { } = 96; { Spacing increment }
  eSR { } = 97; { Scroll right }
  eSRM { } = 98; { Send-Receive mode }
  eSRTM { } = 99; { Status report transfer mode }
  eSS2 { } = 100; { Single shift 2 }
  eSS3 { } = 101; { Single shift 3 }
  eSSA { } = 102; { Start of selected area }
  eST { } = 103; { String terminator }
  eSTS { } = 104; { Set transmit state }
  eSU { } = 105; { Scroll up }
  eTBC {X} = 106; { Tabulation clear }
  eTSM { } = 107; { Tabulation stop mode }
  eTSS { } = 108; { Thin space specification }
  eTTM { } = 109; { Transfer termination mode }
  eVEM { } = 110; { Vertical editing mode }
  eVPA {X} = 111; { Vertical position absolute }
  eVPR {X} = eCUD; { Vertical position relative }
  eVTS {X} = 112; { vertical tabulation set }

  eDECALN = 122; { DEC PRIVATE-screen alignment display }
  eDECDHL = 123; { DEC PRIVATE-Double height line }
  eDECDWL = 124; { DEC PRIVATE-Double width line }
  eDECLL = 125; { DEC PRIVATE-load LEDs }
  eDECREQTPARM = 126; { DEC PRIVATE-request terminal parameters }
  eDECSCS = 129; { DEC PRIVATE-select charset }
  eDECSTBM = 113; { DEC PRIVATE-set Top/Bottom margin }
  eDECSWL = 127; { DEC PRIVATE-single width line }
  eDECTST = 128; { DEC PRIVATE-Invoke confidence test }

Const
  { Convenient character constants (and aliases) }
  cNul = #0;
  cSoh = #1;
  cStx = #2;
  cEtx = #3;
  cEot = #4;
  cEnq = #5;
  cAck = #6;
  cBel = #7;
  cBS = #8;
  cTab = #9;
  cLF = #10;
  cVT = #11;
  cFF = #12;
  cCR = #13;
  cSO = #14;
  cSI = #15;
  cDle = #16;
  cDC1 = #17;
  cXon = #17;
  cDC2 = #18;
  cDC3 = #19;
  cXoff = #19;
  cDC4 = #20;
  cNak = #21;
  cSyn = #22;
  cEtb = #23;
  cCan = #24;
  cEM = #25;
  cSub = #26;
  cEsc = #27;
  cFS = #28;
  cGS = #29;
  cRS = #30;
  cUS = #31;

Var
  ipc_TermBufForeColor: tcolor;
  ipc_TermBufBackColor: tcolor;

Const
  { Buffer default property values }
  ipc_TermBufColCount = 80;
  ipc_TermBufRowCount = 24;
  ipc_TermBufScrollRowCount = 200;
  ipc_TermBufUseAbsAddress = True; { use absolute rows/col values }
  ipc_TermBufUseAutoWrap = True; { chars wrap at end of line }
  ipc_TermBufUseAutoWrapDelay = True; { ..but delay slightly }
  ipc_TermBufUseInsertMode = False; { no insert mode }
  ipc_TermBufUseNewLineMode = False; { LF char is a pure linefeed }
  ipc_TermBufUseScrollRegion = False; { no scroll region used }

Const
  { default property values for the terminal }
  ipc_TermActive = True;
  ipc_TermBackColor = clblack;
  ipc_TermBlinkTime = 500;
  ipc_TermBorderStyle = bsSingle;
  ipc_TermCapture = cmOff;
  ipc_TermCaptureFile = 'IPROTERM.CAP';
  ipc_TermColumnCount = ipc_TermBufColCount;
  ipc_TermCursorType = ctBlock;
  ipc_TermFontName = 'Terminal';
  ipc_TermForeColor = clsilver;
  ipc_TermHalfDuplex = False;
  ipc_TermHeight = 200;
  ipc_TermLazyByteDelay = 200;
  ipc_TermLazyTimeDelay = 100;
  ipc_TermRowCount = ipc_TermBufRowCount;
  ipc_TermScrollback = False;
  ipc_TermScrollRowCount = ipc_TermBufScrollRowCount;
  ipc_TermUseLazyDisplay = True;
  ipc_TermWantAllKeys = True;
  ipc_TermWidth = 300;
  ipc_FreezeScrollback = true; {!!.03}

  { default property values for the VT100 emulator--'power-up' values }
  ipc_VT100ANSIMode = True;
  ipc_VT100Answerback = 'iPROterm';
  ipc_VT100AppKeyMode = False;
  ipc_VT100AppKeypadMode = False;
  ipc_VT100AutoRepeat = True;
  ipc_VT100Col132Mode = False;
  ipc_VT100G0CharSet = 0;
  ipc_VT100G1CharSet = 0;
  ipc_VT100GPOMode = False;
  ipc_VT100InsertMode = ipc_TermBufUseInsertMode;
  ipc_VT100Interlace = False;
  ipc_VT100NewLineMode = ipc_TermBufUseNewLineMode;
  ipc_VT100RelOriginMode = Not ipc_TermBufUseAbsAddress;
  ipc_VT100RevScreenMode = False;
  ipc_VT100SmoothScrollMode = False;
  ipc_VT100WrapAround = ipc_TermBufUseAutoWrap;

Type
  TIpSockControlFriend = Class(TIpSockControl);

  PIptWordArray = ^TIptWordArray;
  TIptWordArray = Array[0..MaxInt Div sizeof(Word) - 1] Of Word;

  PIptLongArray = ^TIptLongArray;
  TIptLongArray = Array[0..MaxInt Div sizeof(Longint) - 1] Of Longint;

  TIpTerminalCharAttr = ({ character attributes }
    tcaBold, { ..bold }
    tcaUnderline, { ..underlined }
    tcaStrikethrough, { ..struck through as if deleted }
    tcaBlink, { ..blinking }
    tcaReverse, { ..back/foreground colors reversed }
    tcaInvisible, { ..invisible }
    tcaSelected); { ..selected }

  TIpTerminalCharAttrs = Set Of TIpTerminalCharAttr;

  TIpScrollRowsNotifyEvent = Procedure(aSender: TObject;
    aCount, aTop, aBottom: Integer) Of Object;

  TIpOnCursorMovedEvent = {!!.01}
    Procedure(ASender: TObject; {!!.01}
    Row, Col: integer) Of Object; {!!.01}

  TIpTerminalArray = Class
  Protected
    FActColCount: Integer;
    FColCount   : Integer;
    FDefaultItem: Longint;
    FItems      : PAnsiChar;
    FItemSize   : Integer;
    FRowCount   : Integer;
    Procedure taClearRows(aBuffer: PAnsiChar; aActColCount: Integer;aStartRow, aEndRow: Integer);
    Procedure taGrowArray(aRowCount, aColCount, aActColCount: Integer);
    Procedure taSetColCount(aNewCount: Integer);
    Procedure taSetRowCount(aNewCount: Integer);
  Public
    Constructor Create(aItemSize: Integer);
    Destructor Destroy; Override;
    Function GetItemPtr(aRow, aCol: Integer): Pointer;
    Procedure Clear;
    Procedure ClearItems(aRow: Integer; aFromCol, aToCol: Integer);
    Procedure DeleteItems(aCount: Integer; aRow: Integer; aCol: Integer);
    Procedure InsertItems(aCount: Integer; aRow: Integer; aCol: Integer);
    Procedure ReplaceItems(aOldItem: pointer; aNewItem: pointer);
    Procedure ScrollRows(aCount: Integer; aStartRow, aEndRow: Integer);
    Procedure SetDefaultItem(aDefaultItem: Pointer);
    Procedure WriteDupItems(aItem: Pointer; aCount: Integer;aRow, aCol: Integer);
    Procedure WriteItems(aItems: Pointer; aCount: Integer;aRow, aCol: Integer);
    Property ColCount: Integer Read FColCount Write taSetColCount;
    Property ItemSize: Integer Read FItemSize;
    Property RowCount: Integer Read FRowCount Write taSetRowCount;
  End;

Type

  TIpTerminalBuffer = Class
  Private
    FAttr: TIpTerminalCharAttrs; { current attributes }
    FBackColor: TColor; { current background color }
    FBeyondMargin: Boolean; { True if cursor's beyond right margin }
    FCharSet: Byte; { current charset }
    FColCount: Integer; { count of columns in both views }
    FCursorCol: Integer; { current internal cursor col position }
    FCursorMoved: Boolean; { True if cursor has moved }
    FCursorRow: Integer; { current internal cursor row position }
    FDefAnsiChar: AnsiChar; { default ANSI character }
    FDefAttr: TIpTerminalCharAttrs; { default attributes }
    FDefCharSet: Byte; { default charset }
    FDefBackColor: TColor; { default background color }
    FDefForeColor: TColor; { default foreground color }
    FDefWideChar: WideChar; { default wide character }
    FDisplayOriginCol: Integer; { column origin of addressable area }
    FDisplayOriginRow: Integer; { row origin of addressable area }
    FDisplayColCount: Integer; { column count in addressable area }
    FDisplayRowCount: Integer; { row count in addressable area }
    FForeColor: TColor; { current foreground color }
    FHorzTabStops: PByteArray; { bitset of horizontal tab stops }
    FInvRectList: Pointer; { list of invalid rects }
    FOnScrollRows: TIpScrollRowsNotifyEvent;
    FRowCount: Integer; { count of rows in display view }
    FSRStartRow: Integer; { start row of scrolling region }
    FSREndRow: Integer; { end row of scrolling region }
    FSVRowCount: Integer; { count of rows in scrollback view }
    FUseAbsAddress: Boolean; { True if absolute values for row/col }
    FUseAutoWrap: Boolean; { True if chars wrap to next line }
    FUseAutoWrapDelay: Boolean; { True if cursor stays at last col }
    FUseInsertMode: Boolean; { True if insert rather than replace }
    FUseNewLineMode: Boolean; { True if LF means CR+LF }
    FUseScrollRegion: Boolean; { True if limit to scroll region }
    FUseWideChars: Boolean; { True if expecting UNICODE chars }
    FVertTabStops: PByteArray; { bitset of vertical tab stops }

    FCharMatrix: TIpTerminalArray; { matrix of chars }
    FCharSetMatrix: TIpTerminalArray; { matrix of charsets }
    FAttrMatrix: TIpTerminalArray; { matrix of attrs }
    FForeColorMatrix: TIpTerminalArray; { matrix of forecolors }
    FBackColorMatrix: TIpTerminalArray; { matrix of backcolors }

    FOnCursorMoved: TIpOnCursorMovedEvent; {Cursor moved event} {!!.01}

  Protected
    Function tbGetCol: Integer;
    Function tbGetOriginCol: Integer;
    Function tbGetOriginRow: Integer;
    Function tbGetRow: Integer;

    Procedure tbSetBackColor(aValue: TColor);
    Procedure tbSetCharSet(aValue: Byte);
    Procedure tbSetDefAnsiChar(aValue: AnsiChar);
    Procedure tbSetDefBackColor(aValue: TColor);
    Procedure tbSetDefForeColor(aValue: TColor);
    Procedure tbSetForeColor(aValue: TColor);
    Procedure tbSetSVRowCount(aNewCount: Integer);
    Procedure tbSetCol(aCol: Integer);
    Procedure tbSetColCount(aNewCount: Integer);
    Procedure tbSetRow(aRow: Integer);
    Procedure tbSetRowCount(aNewCount: Integer);
    Procedure tbSetUseScrollRegion(aValue: Boolean);

    Procedure tbInvalidateRect(aFromRow, aFromCol, aToRow, aToCol: Integer);

    Function tbCvtToInternalCol(aCol: Integer; aAbsolute: Boolean): Integer;
    Function tbCvtToInternalRow(aRow: Integer; aAbsolute: Boolean): Integer;
    Function tbCvtToExternalCol(aCol: Integer; aAbsolute: Boolean): Integer;
    Function tbCvtToExternalRow(aRow: Integer; aAbsolute: Boolean): Integer;

    Function tbAtLastColumn: Boolean;
    Procedure tbMoveCursorLeftRight(aDirection: Integer;
      aWrap: Boolean; aScroll: Boolean);
    Procedure tbMoveCursorUpDown(aDirection: Integer; aScroll: Boolean);
    Procedure tbReallocBuffers(aNewRowCount: Integer; aNewColCount: Integer);
    Procedure tbScrollRows(aCount, aTop, aBottom: Integer);

    Procedure tbFireOnCursorMovedEvent; {!!.01}

  Public
    Constructor Create(aUseWideChars: Boolean);
    Destructor Destroy; Override;

    {---METHODS---}
    { character attributes }
    Procedure GetCharAttrs(Var aValue: TIpTerminalCharAttrs);
    Procedure GetDefCharAttrs(Var aValue: TIpTerminalCharAttrs);
    Procedure SetDefCharAttrs(Const aValue: TIpTerminalCharAttrs);
    Procedure SetCharAttrs(Const aValue: TIpTerminalCharAttrs);

    { cursor movement }
    Procedure MoveCursorDown(aScroll: Boolean);
    Procedure MoveCursorLeft(aWrap: Boolean; aScroll: Boolean);
    Procedure MoveCursorRight(aWrap: Boolean; aScroll: Boolean);
    Procedure MoveCursorUp(aScroll: Boolean);
    Procedure SetCursorPosition(aRow, aCol: Integer);

    { insertion/deletion }
    Procedure DeleteChars(aCount: Integer);
    Procedure DeleteLines(aCount: Integer);
    Procedure InsertChars(aCount: Integer);
    Procedure InsertLines(aCount: Integer);

    { erasing }
    Procedure EraseAll;
    Procedure EraseChars(aCount: Integer);
    Procedure EraseFromBOW;
    Procedure EraseFromBOL;
    Procedure EraseLine;
    Procedure EraseScreen;
    Procedure EraseToEOL;
    Procedure EraseToEOW;

    { horizontal tab stop control }
    Procedure SetHorzTabStop;
    Procedure ClearHorzTabStop;
    Procedure ClearAllHorzTabStops;
    Procedure DoHorzTab;
    Procedure DoBackHorzTab;

    { vertical tab stop control }
    Procedure SetVertTabStop;
    Procedure ClearVertTabStop;
    Procedure ClearAllVertTabStops;
    Procedure DoVertTab;
    Procedure DoBackVertTab;

    { scrolling regions }
    Procedure SetScrollRegion(aTopRow, aBottomRow: Integer);

    { write character/string }
    Procedure WriteChar(aCh: char);
    Procedure WriteString(Const aSt: String);

    { miscellaneous special processing }
    Procedure DoBackspace;
    Procedure DoCarriageReturn;
    Procedure DoLineFeed;
    Procedure Reset;

    { get buffer information }
    Function GetLineCharPtr(aRow: Integer): Pointer;
    Function GetLineAttrPtr(aRow: Integer): Pointer;
    Function GetLineForeColorPtr(aRow: Integer): Pointer;
    Function GetLineBackColorPtr(aRow: Integer): Pointer;
    Function GetLineCharSetPtr(aRow: Integer): Pointer;

    { getting information about changes }
    Function HasCursorMoved: Boolean;
    Function HasDisplayChanged: Boolean;
    Function GetInvalidRect(Var aRect: TRect): Boolean;

    {---PROPERTIES---}
    { color, charsets }
    Property BackColor: TColor Read FBackColor Write tbSetBackColor;
    Property CharSet: Byte Read FCharSet Write tbSetCharSet;
    Property DefAnsiChar: AnsiChar Read FDefAnsiChar Write tbSetDefAnsiChar;
    Property DefBackColor: TColor Read FDefBackColor Write tbSetDefBackColor;
    Property DefCharSet: Byte Read FDefCharSet Write FDefCharSet;
    Property DefForeColor: TColor Read FDefForeColor Write tbSetDefForeColor;
    Property ForeColor: TColor Read FForeColor Write tbSetForeColor;

    { scrollback view extent }
    Property SVRowCount: Integer Read FSVRowCount Write tbSetSVRowCount;

    { display view properties }
    Property Col: Integer Read tbGetCol Write tbSetCol;
    Property ColCount: Integer Read FColCount Write tbSetColCount;
    Property OriginCol: Integer Read tbGetOriginCol;
    Property OriginRow: Integer Read tbGetOriginRow;
    Property Row: Integer Read tbGetRow Write tbSetRow;
    Property RowCount: Integer Read FRowCount Write tbSetRowCount;

    Property UseAbsAddress: Boolean
      Read FUseAbsAddress Write FUseAbsAddress;
    Property UseAutoWrap: Boolean
      Read FUseAutoWrap Write FUseAutoWrap;
    Property UseAutoWrapDelay: Boolean
      Read FUseAutoWrapDelay Write FUseAutoWrapDelay;
    Property UseInsertMode: Boolean
      Read FUseInsertMode Write FUseInsertMode;
    Property UseNewLineMode: Boolean
      Read FUseNewLineMode Write FUseNewLineMode;
    Property UseScrollRegion: Boolean
      Read FUseScrollRegion Write tbSetUseScrollRegion;
    Property UseWideChars: Boolean Read FUseWideChars;
    Property OnScrollRows: TIpScrollRowsNotifyEvent
      Read FOnScrollRows Write FOnScrollRows;

    { OnCursorMoved
      This property is used to notify the TAdTerminal component when
      the cursor moves.  It should not be used for your own purposes
        as that may cause unexpected behaviour in the terminal }{!!.01}

    Property OnCursorMoved: TIpOnCursorMovedEvent {!!.01}
      Read FOnCursorMoved Write FOnCursorMoved; {!!.01}
  End;

Type
  PIpKeyString = ^TIpKeyString;
  TIpKeyString = String[63];

Const
  DefaultFontName: String[9] = '<Default>';

Type
  TIpKeyboardMapping = Class
  Private
    FTable: TList;
    FCount: Integer;
  Protected
    Function kbmFindPrim(Const aKey: TIpKeyString; Var aInx: Integer;
      Var aNode: Pointer): Boolean;
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Function Add(Const aKey: TIpKeyString; Const aValue: TIpKeyString): Boolean;
    Procedure Clear;
    Function Get(Const aKey: TIpKeyString): TIpKeyString;
    Procedure LoadFromFile(Const aFileName: String);
    Procedure LoadFromRes(aInstance: THandle; Const aResName: String);
    Procedure StoreToBinFile(Const aFileName: String);
{$IFDEF CompileDebugCode}
    Procedure DebugPrint(Const aFileName: String);
{$ENDIF}
    Property Count: Integer Read FCount;
  End;

Type
  TIpCharSetMapping = Class
  Private
    FTable: TList;
    FCharQueue: Pointer;
    FCount: Integer;
    FScript: Pointer;
    FScriptEnd: Pointer;
    FScriptFreeList: Pointer;
  Protected
    Procedure csmAddScriptNode(aFont: PIpKeyString);
    Function csmFindPrim(Const aCharSet: TIpKeyString; aChar: AnsiChar;
      Var aInx: Integer; Var aNode: Pointer): Boolean;
    Procedure csmFreeScript;
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Function Add(Const aCharSet: TIpKeyString; aFromCh: AnsiChar;
      aToCh: AnsiChar; Const aFont: TIpKeyString; aGlyph: AnsiChar): Boolean;
    Procedure Clear;
    Procedure GetFontNames(aList: TStrings);
    Procedure GenerateDrawScript(Const aCharSet: TIpKeyString; aText: PAnsiChar);
    Function GetNextDrawCommand(Var aFont: TIpKeyString; aText: PAnsiChar): Boolean;
    Procedure LoadFromFile(Const aFileName: String);
    Procedure LoadFromRes(aInstance: THandle; Const aResName: String);
    Procedure StoreToBinFile(Const aFileName: String);
{$IFDEF CompileDebugCode}
    Procedure DebugPrint(Const aFileName: String);
{$ENDIF}
    Property Count: Integer Read FCount;
  End;

Type
  TIpParserCmdType = ({ Parser return command types... }
    pctNone, { ..no command, unknown command or char ignored }
    pctChar, { ..single displayable character }
    pct8bitChar, { ..single character with bit 7 set }
    pctPending, { ..command being built up }
    pctComplete); { ..a complete command is ready }

  TIpVTParserState = ({ VT Parser states... }
    psIdle, { ..nothing happening }
    psGotEscape, { ..received escape char }
    psParsingANSI, { ..parsing an ESC[ sequence }
    psParsingHash, { ..parsing an ESC# sequence }
    psParsingLeftParen, { ..parsing an ESC( sequence }
    psParsingRightParen, { ..parsing an ESC) sequence }
    psParsingCharSet, { ..parsing ESC *, +, -, ., / sequence }
    psGotCommand, { ..received complete command }
    psGotInterCommand, { ..received complete intermediary command }
    psParsingCUP52); { ..received VT52 position cursor command }

Type
  PAdIntegerArray = ^TIpIntegerArray;
  TIpIntegerArray = Array[0..pred(MaxInt Div sizeof(Integer))] Of Integer;

Type
  TIpTerminalParser = Class
    { the ancestor parser class }
  Private
    FArgCount: Integer;
    FCommand: Byte;
    FUseWideChar: Boolean;
  Protected
    Function tpGetArgument(aInx: Integer): Integer; Virtual;
    Function tpGetSequence: String; Virtual;
  Public
    Constructor Create(aUseWideChar: Boolean);
    Destructor Destroy; Override;

    Function ProcessChar(aCh: AnsiChar): TIpParserCmdType; Virtual;
    Function ProcessWideChar(aCh: WideChar): TIpParserCmdType; Virtual;

    Procedure Clear; Virtual;

    Property Argument[aInx: Integer]: Integer
    Read tpGetArgument;
    Property ArgumentCount: Integer Read FArgCount;
    Property Command: Byte Read FCommand;
    Property Sequence: String Read tpGetSequence;
  End;

  TIpVT100Parser = Class(TIpTerminalParser)
    { the VT100 terminal parser }
  Private
    FArgCountMax: Integer;
    FArgs: PAdIntegerArray;
    FInVT52Mode: Boolean;
    FSavedSeq: Pointer;
    FSavedState: TIpVTParserState;
    FSequence: Pointer;
    FState: TIpVTParserState;
  Protected
    Function tpGetArgument(aInx: Integer): Integer; Override;
    Function tpGetSequence: String; Override;

    Function vtpGetArguments: Boolean;
    Function vtpParseANSISeq(aCh: char): TIpParserCmdType;
    Function vtpProcessVT52(aCh: char): TIpParserCmdType;
    Function vtpValidateArgsPrim(aMinArgs: Integer;
      aMaxArgs: Integer;
      aDefault: Integer): Boolean;

    Procedure vtpGrowArgs;
  Public
    Constructor Create(aUseWideChar: Boolean);
    Destructor Destroy; Override;

    Function ProcessChar(aCh: AnsiChar): TIpParserCmdType; Override;
    Function ProcessWideChar(aCh: WideChar): TIpParserCmdType; Override;

    Procedure Clear; Override;

    Property InVT52Mode: Boolean Read FInVT52Mode;
  End;

Type
  TIpTerminalEmulator = Class;

  TIpCustomTerminal = Class(TIpBaseWinControl)
  Private
    FActive: Boolean;
    FBlinkTime: Integer;
    FBlinkTimeCount: Integer;
    FBlinkTextVisible: Boolean;
    FBorderStyle: TBorderStyle;
    FByteQueue: Pointer;
    FCapture: TIpCaptureMode;
    FCaptureFile: String;
    FCaptureStream: TFileStream;
    FCanvas: TCanvas;
    FCharHeight: Integer; { height of char cell in pixels }
    FCharWidth: Integer; { width of char cell in pixels }
    FClientCols: Integer; { client width in columns, incl part }
    FClientFullCols: Integer; { client width in *full* columns }
    FClientRows: Integer; { client height in rows, incl part }
    FClientFullRows: Integer; { client height in *full* rows }
    FCreatedCaret: Boolean;
    FCursorType: TIpCursorType;
    FDefEmulator: TIpTerminalEmulator; { default = tty }
    FEmulator: TIpTerminalEmulator;
    FHalfDuplex: Boolean;
    FHaveSelection: Boolean;
    FHeartbeat: TTimer;
    FLazyByteCount: Integer;
    FLazyTimeCount: Integer;
    FLazyByteDelay: Integer;
    FLazyTimeDelay: Integer;
    FTermWidth: Integer;
    FTermHeight: Integer;
    FLButtonAnchor: TPoint;
    FLButtonDown: Boolean;
    FLButtonRect: TRect;
    FOnCursorMoved: TIpTerminalCursorMovedEvent;
    FOriginCol: Integer;
    FOriginRow: Integer;
    FScrollback: Boolean;
    FFreezeScrollback: boolean; {!!.03}
    FScrollHorzInfo: TScrollInfo;
    FScrollVertInfo: TScrollInfo;
    FShowingCaret: Boolean;
    FSockControl: TIpSockControl;
    FSocket: TSocket;
    FUsedRect: TRect;
    FUseLazyDisplay: Boolean;
    FUnusedRightRect: TRect;
    FUnusedBottomRect: TRect;
    FUseHScrollBar: Boolean;
    FUseVScrollBar: Boolean;
    FWantAllKeys: Boolean;
  Protected
    { get and set methods for properties }
    Function tmGetAttributes(aRow, aCol: Integer): TIpTerminalCharAttrs;
    Function tmGetBackColor(aRow, aCol: Integer): TColor;
    Function tmGetCharSet(aRow, aCol: Integer): Byte;
    Function tmGetColumns: Integer;
    Function tmGetEmulator: TIpTerminalEmulator;
    Function tmGetForeColor(aRow, aCol: Integer): TColor;
    Function tmGetIsMasterTerminal: Boolean;
    Function tmGetLine(aRow: Integer): String;
    Function tmGetRows: Integer;
    Function tmGetScrollbackRows: Integer;
    Function tmGetTelnetTermSize: String;
    Procedure tmSetActive(aValue: Boolean);
    Procedure tmSetAttributes(aRow, aCol: Integer; Const aAttr: TIpTerminalCharAttrs);
    Procedure tmSetBackColor(aRow, aCol: Integer; aValue: TColor);
    Procedure tmSetBlinkTime(aValue: Integer);
    Procedure tmSetBorderStyle(aBS: TBorderStyle);
    Procedure tmSetCapture(aValue: TIpCaptureMode);
    Procedure tmSetCaptureFile(Const aValue: String);
    Procedure tmSetCharSet(aRow, aCol: Integer; aValue: Byte);
    Procedure tmSetColumns(aValue: Integer);
    Procedure tmSetCursorType(aValue: TIpCursorType);
    Procedure tmSetForeColor(aRow, aCol: Integer; aValue: TColor);
    Procedure tmSetEmulator(aValue: TIpTerminalEmulator);
    Procedure tmSetLazyByteDelay(aValue: Integer);
    Procedure tmSetLazyTimeDelay(aValue: Integer);
    Procedure tmSetTermWidth(aValue: Integer);
    Procedure tmSetTermHeight(aValue: Integer);
    Procedure tmSetLine(aRow: Integer; Const aValue: String);
    Procedure tmSetOriginCol(aValue: Integer);
    Procedure tmSetOriginRow(aValue: Integer);
    Procedure tmSetRows(aValue: Integer);
    Procedure tmSetScrollback(aValue: Boolean);
    Procedure tmSetScrollbackRows(aValue: Integer);
    Procedure tmSetSockControl(aValue: TIpSockControl);
    Procedure tmSetSocket(aValue: TSocket);
    Procedure tmSetUseLazyDisplay(aValue: Boolean);
    Procedure tmSetWantAllKeys(aValue: Boolean);
    Procedure tmSetFreezeScrollback(v: boolean); {!!.03}

    { various miscellaneous methods }
    Procedure tmAttachToSocket;
    Procedure tmCalcExtent;
    Procedure tmDetachFromSocket;
    Procedure tmDrawDefaultText;
    Procedure tmGetFontInfo;
    Procedure tmInvalidateRow(aRow: Integer);
    Procedure tmTermResize;

    { caret methods }
    Procedure tmFreeCaret;
    Procedure tmHideCaret;
    Procedure tmMakeCaret;
    Procedure tmPositionCaret;
    Procedure tmShowCaret;

    { scrollbar stuff }
    Procedure tmInitHScrollBar;
    Procedure tmInitVScrollBar;
    Procedure tmScrollHorz(aDist: Integer);
    Procedure tmScrollVert(aDist: Integer);

    { selection stuff }
    Procedure tmGrowSelect(X, Y: Integer);
    Procedure tmMarkDeselected(aRow: Integer; aFromCol, aToCol: Integer);
    Procedure tmMarkSelected(aRow: Integer; aFromCol, aToCol: Integer);
    Function tmProcessClipboardCopy(Var Msg: TWMKeyDown): Boolean;
    Procedure tmStartSelect(X, Y: Integer);

    { overridden ancestor methods }
    Procedure KeyDown(Var Key: Word; Shift: TShiftState); Override;
    Procedure KeyPress(Var Key: Char); Override;
    Procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); Override;
    Procedure MouseMove(Shift: TShiftState; X, Y: Integer); Override;
    Procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); Override;
    Procedure PaintWindow(DC: HDC); Override;
    Procedure Notification(AComponent: TComponent; Operation: TOperation); Override;

    { message and event handlers }
    Procedure CMCtl3DChanged(Var Msg: TMessage); Message CM_CTL3DCHANGED;
    Procedure CMFontChanged(Var Msg: TMessage); Message CM_FONTCHANGED;
    Procedure CMIpSocketStatus(Var Msg: TCMIpSocketStatus); Message CM_IPSOCKETSTATUS;
    Procedure CMIpTermData(Var Msg: TCMIpTermData); Message CM_IPTERMDATA;
    Procedure CMIpTermForceSize(Var Msg: TMessage); Message CM_IPTERMFORCESIZE;
    Procedure CMIpTermStuff(Var Msg: TMessage); Message CM_IPTERMSTUFF;
    Procedure CNKeyDown(Var Msg: TWMKeyDown); Message CN_KEYDOWN;
    Procedure CNSysKeyDown(Var Msg: TWMKeyDown); Message CN_SYSKEYDOWN;
    Procedure tmBeat(Sender: TObject);
    Procedure WMCancelMode(Var Msg: TMessage); Message WM_CANCELMODE;
    Procedure WMCopy(Var Msg: TMessage); Message WM_COPY;
    Procedure WMEraseBkgnd(Var Msg: TMessage); Message WM_ERASEBKGND;
    Procedure WMGetDlgCode(Var Msg: TMessage); Message WM_GETDLGCODE;
    Procedure WMHScroll(Var Msg: TWMScroll); Message WM_HSCROLL;
    Procedure WMKillFocus(Var Msg: TWMSetFocus); Message WM_KILLFOCUS;
    Procedure WMPaint(Var Msg: TWMPaint); Message WM_PAINT;
    Procedure WMSetFocus(Var Msg: TWMSetFocus); Message WM_SETFOCUS;
    Procedure WMSize(Var Msg: TWMSize); Message WM_SIZE;
    Procedure WMVScroll(Var Msg: TWMScroll); Message WM_VSCROLL;

    { TCustomControl emulation }
    Procedure Paint;

  Public
    Constructor Create(aOwner: TComponent); Override;
    Destructor Destroy; Override;

    { overridden ancestor methods }
    Procedure CreateParams(Var Params: TCreateParams); Override;
    Procedure CreateWnd; Override;
    Procedure DestroyWnd; Override;

    { terminal methods }
    Procedure Clear;
    Procedure ClearAll;
    Procedure CopyToClipboard;
    Procedure HideSelection;
    Procedure WriteChar(aCh: AnsiChar);
    Procedure WriteString(Const aSt: String);
    Function GetSelectedText: String;

    { public properties }
    Property Attributes[aRow, aCol: Integer]: TIpTerminalCharAttrs
    Read tmGetAttributes Write tmSetAttributes;
    Property BackColor[aRow, aCol: Integer]: TColor
    Read tmGetBackColor Write tmSetBackColor;
    Property Canvas: TCanvas Read FCanvas;
    Property CharHeight: Integer Read FCharHeight;
    Property CharSet[aRow, aCol: Integer]: Byte
    Read tmGetCharSet Write tmSetCharSet;
    Property CharWidth: Integer Read FCharWidth;
    Property ClientCols: Integer Read FClientCols;
    Property ClientRows: Integer Read FClientRows;
    Property ClientOriginCol: Integer Read FOriginCol Write tmSetOriginCol;
    Property ClientOriginRow: Integer Read FOriginRow Write tmSetOriginRow;
    Property ForeColor[aRow, aCol: Integer]: TColor
    Read tmGetForeColor Write tmSetForeColor;
    Property HaveSelection: Boolean Read FHaveSelection;
    Property Line[aRow: Integer]: String Read tmGetLine Write tmSetLine;
    Property Emulator: TIpTerminalEmulator
      Read tmGetEmulator Write tmSetEmulator;
    Property Socket: TSocket
      Read FSocket Write tmSetSocket;
    Property TelnetTermSize: String Read tmGetTelnetTermSize;
  Published
    Property Active: Boolean
      Read FActive Write tmSetActive Default ipc_TermActive;
    Property BlinkTime: Integer
      Read FBlinkTime Write tmSetBlinkTime Default ipc_TermBlinkTime;
    Property BorderStyle: TBorderStyle
      Read FBorderStyle Write tmSetBorderStyle Default ipc_TermBorderStyle;
    Property Capture: TIpCaptureMode
      Read FCapture Write tmSetCapture Default ipc_TermCapture;
    Property CaptureFile: String
      Read FCaptureFile Write tmSetCaptureFile;
    Property Columns: Integer
      Read tmGetColumns Write tmSetColumns Default ipc_TermColumnCount;
    Property CursorType: TIpCursorType
      Read FCursorType Write tmSetCursorType Default ipc_TermCursorType;
    Property HalfDuplex: Boolean
      Read FHalfDuplex Write FHalfDuplex Default ipc_TermHalfDuplex;
    Property IsMasterTerminal: Boolean Read tmGetIsMasterTerminal;
    Property LazyByteDelay: Integer
      Read FLazyByteDelay Write tmSetLazyByteDelay Default ipc_TermLazyByteDelay;
    Property LazyTimeDelay: Integer
      Read FLazyTimeDelay Write tmSetLazyTimeDelay Default ipc_TermLazyTimeDelay;
    Property TermWidth: Integer
      Read FTermWidth Write tmSetTermWidth Default ipc_TermWidth;
    Property TermHeight: Integer
      Read FTermHeight Write tmSetTermHeight Default ipc_TermHeight;
    Property Rows: Integer
      Read tmGetRows Write tmSetRows Default ipc_TermRowCount;
    Property ScrollbackRows: Integer
      Read tmGetScrollbackRows Write tmSetScrollbackRows Default ipc_TermScrollRowCount;
    Property Scrollback: Boolean
      Read FScrollback Write tmSetScrollback;
    Property SockControl: TIpSockControl
      Read FSockControl Write tmSetSockControl;
    Property UseLazyDisplay: Boolean
      Read FUseLazyDisplay Write tmSetUseLazyDisplay Default ipc_TermUseLazyDisplay;
    Property WantAllKeys: Boolean
      Read FWantAllKeys Write tmSetWantAllKeys Default ipc_TermWantAllKeys;
    Property OnCursorMoved: TIpTerminalCursorMovedEvent
      Read FOnCursorMoved Write FOnCursorMoved;
    Property FreezeScrollBack: boolean {!!.03}
      Read FFreezeScrollback Write tmSetFreezeScrollback {!!.03}
      Default ipc_FreezeScrollback; {!!.03}

    { publish ancestor's properties }
    Property Align;
    Property Color;
    Property Ctl3D;
    Property Enabled;
    Property Font;
    Property Height;
    Property ParentColor;
    Property ParentCtl3D;
    Property ParentFont;
    Property TabOrder;
    Property TabStop;
    Property Visible;
    Property Width;

    { publish ancestor's events }
    Property OnClick;
    Property OnDblClick;
    Property OnExit;
    Property OnKeyDown;
    Property OnKeyPress;
    Property OnKeyUp;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  End;

  TIpTerminalEmulator = Class(TIpBaseComponent)
  Private
    FTerminalBuffer: TIpTerminalBuffer;
    FCharSetMapping: TIpCharSetMapping;
    FIsDefault: Boolean;
    FKeyboardMapping: TIpKeyboardMapping;
    FParser: TIpTerminalParser;
    FTelnetTermType: String;
    FTerminal: TIpCustomTerminal;

    { saved cursor details }
    FSavedAttrs: TIpTerminalCharAttrs;
    FSavedBackColor: TColor;
    FSavedCharSet: Byte;
    FSavedCol: Integer;
    FSavedForeColor: TColor;
    FSavedRow: Integer;

  Protected
    { property access methods }
    Function teGetNeedsUpdate: Boolean; Virtual;
    Procedure teSetTerminal(aValue: TIpCustomTerminal); Virtual;

    { overridden ancestor methods }
    Procedure Notification(AComponent: TComponent;
      Operation: TOperation); Override;

    { new virtual methods }
    Procedure teClear; Virtual;
    Procedure teClearAll; Virtual;
    Procedure teSendChar(aCh: char; aCanEcho: Boolean); Virtual;

    {Cursor Movement Methods}{!!.01}
    Procedure teHandleCursorMovement(Sender: TObject; {!!.01}
      Row: Integer; {!!.01}
      Col: Integer); {!!.01}
  Public
    Constructor Create(aOwner: TComponent); Override;
    Destructor Destroy; Override;

    Procedure BlinkPaint(aVisible: Boolean); Virtual;
    Function HasBlinkingText: Boolean; Virtual;
    Procedure KeyDown(Var Key: Word; Shift: TShiftState); Virtual;
    Procedure KeyPress(Var Key: AnsiChar); Virtual;
    Procedure LazyPaint; Virtual;
    Procedure Paint; Virtual;
    Procedure ProcessBlock(aData: Pointer; aDataLen: Longint); Virtual;

    { read-only properties for low-level objects }
    Property Buffer: TIpTerminalBuffer Read FTerminalBuffer;
    Property CharSetMapping: TIpCharSetMapping Read FCharSetMapping;
    Property KeyboardMapping: TIpKeyboardMapping Read FKeyboardMapping;
    Property Parser: TIpTerminalParser Read FParser;
    Property NeedsUpdate: Boolean Read teGetNeedsUpdate;
  Published
    Property Terminal: TIpCustomTerminal Read FTerminal Write teSetTerminal;
    Property TelnetTermType: String Read FTelnetTermType Write FTelnetTermType;
  End;

  TIpTerminal = Class(TIpCustomTerminal)
  Published
    Property Emulator;
  End;

Type
  TIpTTYEmulator = Class(TIpTerminalEmulator)
  Private
    FCellWidths: PAdIntegerArray;
    FDisplayStr: PAnsiChar;
    FDisplayStrSize: Integer;
    FPaintFreeList: Pointer;
    FRefresh: Boolean;
  Protected
    { property accessor methods }
    Function teGetNeedsUpdate: Boolean; Override;

    { overridden ancestor methods }
    Procedure teClear; Override;
    Procedure teClearAll; Override;
    Procedure teSetTerminal(aValue: TIpCustomTerminal); Override;

    { miscellaneous }
    Procedure ttyDrawChars(aRow, aStartCol, aEndCol: Integer; aVisible: Boolean);
    Procedure ttyProcessCommand(aCh: AnsiChar);

    { paint node methods }
    Procedure ttyExecutePaintScript(aRow: Integer; aScript: Pointer);
    Procedure ttyFreeAllPaintNodes;
    Procedure ttyFreePaintNode(aNode: Pointer);
    Function ttyNewPaintNode: Pointer;

  Public
    Constructor Create(aOwner: TComponent); Override;
    Destructor Destroy; Override;

    { overridden ancestor methods }
    Procedure KeyPress(Var Key: AnsiChar); Override;
    Procedure LazyPaint; Override;
    Procedure Paint; Override;
    Procedure ProcessBlock(aData: Pointer; aDataLen: Longint); Override;
  End;

Type
  TIpVT100Emulator = Class(TIpTerminalEmulator)
  Private
    FAnswerback: String;
    FBlinkers: Pointer;
    FBlinkFreeList: Pointer;
    FCellWidths: PAdIntegerArray;
    FDisplayStr: PAnsiChar;
    FDisplayStrSize: Integer;
    FDispUpperASCII: Boolean;
    FLEDs: Integer;
    FLineAttrArray: TObject;
    FPaintFreeList: Pointer;
    FRefresh: Boolean;
    FSecondaryFont: TFont;

    { modes }
    FANSIMode: Boolean;
    FAppKeyMode: Boolean;
    FAppKeypadMode: Boolean;
    FAutoRepeat: Boolean;
    FCol132Mode: Boolean;
    FGPOMode: Boolean; { graphics proc.option not supported }
    FInsertMode: Boolean;
    FInterlace: Boolean; { interlace chgs are not supported }
    FNewLineMode: Boolean;
    FRelOriginMode: Boolean;
    FRevScreenMode: Boolean;
    FSmoothScrollMode: Boolean; { smooth scrolling is not supported }
    FWrapAround: Boolean;

    { character sets }
    FUsingG1: Boolean;
    FG0CharSet: Integer;
    FG1CharSet: Integer;

  Protected
    { property accessor methods }
    Function teGetNeedsUpdate: Boolean; Override;
    Procedure vttSetCol132Mode(aValue: Boolean);
    Procedure vttSetRelOriginMode(aValue: Boolean);
    Procedure vttSetRevScreenMode(aValue: Boolean);

    { overridden ancestor methods }
    Procedure teClear; Override;
    Procedure teClearAll; Override;
    Procedure teSetTerminal(aValue: TIpCustomTerminal); Override;

    { miscellaneous }
    Procedure vttDrawChars(aRow, aStartVal, aEndVal: Integer;
      aVisible: Boolean; aCharValues: Boolean);
    Procedure vttProcessCommand;
    Function vttGenerateDECREPTPARM(aArg: Integer): String;
    Procedure vttInvalidateRow(aRow: Integer);
    Procedure vttProcess8bitChar(aCh: AnsiChar);
    Procedure vttScrollRowsHandler(aSender: TObject; aCount, aTop, aBottom: Integer);
    Procedure vttToggleNumLock;

    { blink node methods }
    Procedure vttCalcBlinkScript;
    Procedure vttClearBlinkScript;
    Procedure vttDrawBlinkOffCycle(aRow, aStartCh, aEndCh: Integer);
    Procedure vttFreeAllBlinkNodes;
    Procedure vttFreeBlinkNode(aNode: Pointer);
    Function vttNewBlinkNode: Pointer;

    { paint node methods }
    Procedure vttExecutePaintScript(aRow: Integer; aScript: Pointer);
    Procedure vttFreeAllPaintNodes;
    Procedure vttFreePaintNode(aNode: Pointer);
    Function vttNewPaintNode: Pointer;

  Public
    Constructor Create(aOwner: TComponent); Override;
    Destructor Destroy; Override;

    { overridden ancestor methods }
    Procedure BlinkPaint(aVisible: Boolean); Override;
    Function HasBlinkingText: Boolean; Override;
    Procedure KeyDown(Var Key: Word; Shift: TShiftState); Override;
    Procedure KeyPress(Var Key: AnsiChar); Override;
    Procedure LazyPaint; Override;
    Procedure Paint; Override;
    Procedure ProcessBlock(aData: Pointer; aDataLen: Longint); Override;

    { modes }
    Property ANSIMode: Boolean Read FANSIMode Write FANSIMode;
    Property AppKeyMode: Boolean Read FAppKeyMode Write FAppKeyMode;
    Property AppKeypadMode: Boolean Read FAppKeypadMode Write FAppKeypadMode;
    Property AutoRepeat: Boolean Read FAutoRepeat Write FAutoRepeat;
    Property Col132Mode: Boolean Read FCol132Mode Write vttSetCol132Mode;
    Property GPOMode: Boolean Read FGPOMode Write FGPOMode;
    Property InsertMode: Boolean Read FInsertMode Write FInsertMode;
    Property Interlace: Boolean Read FInterlace Write FInterlace;
    Property NewLineMode: Boolean Read FNewLineMode Write FNewLineMode;
    Property RelOriginMode: Boolean Read FRelOriginMode Write vttSetRelOriginMode;
    Property RevScreenMode: Boolean Read FRevScreenMode Write vttSetRevScreenMode;
    Property SmoothScrollMode: Boolean Read FSmoothScrollMode Write FSmoothScrollMode;
    Property WrapAround: Boolean Read FWrapAround Write FWrapAround;

    { miscellaneous }
    Property LEDs: Integer Read FLEDs Write FLEDs;

  Published
    Property Answerback: String Read FAnswerback Write FAnswerback;
    Property DisplayUpperASCII: Boolean Read FDispUpperASCII Write FDispUpperASCII;
  End;

Var
  TIpCustomTerminalScrollBarPresent: Boolean;
  TerminalScreenWidth, TerminalScreenHeight: Integer;

Implementation

Const
  BeatInterval = 100;

Type
  PBlinkNode = ^TBlinkNode;
  TBlinkNode = Packed Record
    bnNext: PBlinkNode;
    bnRow: Integer;
    bnStartCh: Integer;
    bnEndCh: Integer;
  End;

Type
  PPaintNode = ^TPaintNode;
  TPaintNode = Packed Record
    pnNext: PPaintNode;
    pnStart: Integer; { start column of range }
    pnEnd: Integer; { end column of range }
    pnFore: TColor; { foreground color for range }
    pnBack: TColor; { background color for range }
    pnAttr: TIpTerminalCharAttrs; { attributes for range }
    pnCSet: Byte; { charset for range }
  End;

Const
  VT52DeviceAttrs = #27'/Z'; { "VT100 acting as VT52" }
  VT100DeviceAttrs = #27'[?1;0c'; { "Base VT100, no options" }
  VT100StatusRpt = #27'[0n'; { "terminal OK" }
  VT100CursorPos = #27'[%d;%dR'; { "cursor is at row;col" }
  VT100ReportParm = #27'[%d;%d;%d;%d;%d;%d;%dx'; { report terminal parameters }

Const
  VT100CharSetNames: Array[0..4] Of String =
  ('VT100-USASCII', { charset 0 }
    'VT100-UK', { charset 1 }
    'VT100-linedraw', { charset 2 }
    'VT100-ROM1', { charset 3 }
    'VT100-ROM2'); { charset 4 }

  {===Terminal/Emulator links==========================================}
Type
  PTermEmuLink = ^TTermEmuLink;
  TTermEmuLink = Record
    telNext: PTermEmuLink;
    telTerm: TIpCustomTerminal;
    telEmu: TIpTerminalEmulator;
  End;
Var
  TermEmuLink: PTermEmuLink;
  TermEmuLinkFreeList: PTermEmuLink;

Procedure AddTermEmuLink(aTerminal: TIpCustomTerminal;
  aEmulator: TIpTerminalEmulator);
Var
  Node: PTermEmuLink;
Begin
  { if the link already exists, exit }
  Node := TermEmuLink;
  While (Node <> Nil) Do
  Begin
    If (Node^.telTerm = aTerminal) Then
      Exit;
    Node := Node^.telNext;
  End;
  { otherwise, add it }
  If (TermEmuLinkFreeList = Nil) Then
    New(Node)
  Else
  Begin
    Node := TermEmuLinkFreeList;
    TermEmuLinkFreeList := Node^.telNext;
  End;
  Node^.telTerm := aTerminal;
  Node^.telEmu := aEmulator;
  Node^.telNext := TermEmuLink;
  TermEmuLink := Node;
  { now update each object to point to the other }
  aTerminal.Emulator := aEmulator;
  aEmulator.Terminal := aTerminal;
End;

Procedure RemoveTermEmuLink(aTerminal: TIpCustomTerminal;
  aNotify: Boolean);
Var
  Dad, Node: PTermEmuLink;
  Emulator: TIpTerminalEmulator;
Begin
  { remove the link }
  Emulator := Nil;
  Dad := Nil;
  Node := TermEmuLink;
  While (Node <> Nil) Do
  Begin
    If (Node^.telTerm = aTerminal) Then
    Begin
      If (Dad = Nil) Then
        TermEmuLink := Node^.telNext
      Else
        Dad^.telNext := Node^.telNext;
      Emulator := Node^.telEmu;
      Node^.telNext := TermEmuLinkFreeList;
      TermEmuLinkFreeList := Node;
      Break;
    End;
    Dad := Node;
    Node := Node^.telNext;
  End;
  {now update each object to point to nil instead of each other}
  If aNotify And (Emulator <> Nil) Then
  Begin
    aTerminal.Emulator := Nil;
    Emulator.Terminal := Nil;
  End;
End;
{====================================================================}

{===Byte queue=======================================================}
{Notes: this byte queue was published in Algorithms Alfresco in The
        Delphi Magazine, December 1988. It is reproduced and used here
        with permission.
        Copyright (c) Julian M Bucknall, 1998. All Rights Reserved.}
Type
  TaaByteQueue = Class
  Private
    bqHead: PChar;
    bqTail: PChar;
    bqQMidPoint: PChar;
    bqQueue: PChar;
    bqQueueEnd: PChar;
  Protected
    Function GetCapacity: Integer;
    Function GetCount: Integer;
    Procedure SetCapacity(aValue: Integer);
  Public
    Constructor Create;
    Destructor Destroy; Override;

    Procedure AdvanceAfterPut(aDataLen: Integer);
    Procedure Clear;
    (***** not used yet
    procedure Get(var aData; aDataLen : Integer);
    function IsEmpty : Boolean;
    *****)
    Procedure Put(Const aData; aDataLen: Integer);
    Function PutPeek(aDataLen: Integer): Pointer;
    Function Peek(aDataLen: Integer): Pointer;
    Procedure Remove(aDataLen: integer); {!!.01}

    Property Capacity: Integer Read GetCapacity Write SetCapacity;
    Property Count: Integer Read GetCount;
  End;

Procedure NotEnoughDataError(aAvail, aReq: Integer);
Begin
  Raise Exception.Create(Format(SNotEnoughData, [aAvail, aReq]));
End;

Constructor TaaByteQueue.Create;
Begin
  Inherited Create;
  Capacity := 64;
End;

Destructor TaaByteQueue.Destroy;
Begin
  If Assigned(bqQueue) Then
    FreeMem(bqQueue, bqQueueEnd - bqQueue);
  Inherited Destroy;
End;

Procedure TaaByteQueue.AdvanceAfterPut(aDataLen: Integer);
Begin
  inc(bqTail, aDataLen);
End;

Procedure TaaByteQueue.Clear;
Begin
  bqHead := bqQueue;
  bqTail := bqQueue;
End;
(***** not used yet
procedure TaaByteQueue.Get(var aData; aDataLen : Integer);
var
  ByteCount : Integer;
begin
  {check for enough data}
  if (aDataLen > Count) then
    NotEnoughDataError(Count, aDataLen);
  {move the data}
  Move(bqHead^, aData, aDataLen);
  inc(bqHead, aDataLen);
  {if we've emptied the queue, move the head/tail pointers back}
  ByteCount := Count;
  if (ByteCount = 0) then begin
    bqHead := bqQueue;
    bqTail := bqQueue;
  end
  {if the head of the queue has moved into the overflow zone, move the
   data back, and reset the head/tail pointers}
  else if (bqHead >= bqQMidPoint) then begin
    Move(bqHead^, bqQueue^, ByteCount);
    bqHead := bqQueue;
    bqTail := bqHead + ByteCount;
  end;
end;
*****)

Function TaaByteQueue.getCapacity: Integer;
Begin
  Result := (bqQueueEnd - bqQueue) Div 2;
End;

Function TaaByteQueue.getCount: Integer;
Begin
  Result := bqTail - bqHead;
End;
(***** not used yet
function TaaByteQueue.IsEmpty : Boolean;
begin
  Result := bqHead = bqTail;
end;
*****)

Procedure TaaByteQueue.Put(Const aData; aDataLen: Integer);
Var
  ByteCount: Integer;
Begin
  {if required, grow the queue by at least doubling its size}
  ByteCount := Count;
  While (ByteCount + aDataLen > Capacity) Do
    Capacity := Capacity * 2;
  {we now have enough room, so add the new data}
  Move(aData, bqTail^, aDataLen);
  inc(bqTail, aDataLen);
End;

Function TaaByteQueue.PutPeek(aDataLen: Integer): Pointer;
Var
  ByteCount: Integer;
Begin
  {if required, grow the queue by at least doubling its size}
  ByteCount := Count;
  While (ByteCount + aDataLen > Capacity) Do
    Capacity := Capacity * 2;
  {just return the tail Pointer}
  Result := bqTail;
End;

Function TaaByteQueue.Peek(aDataLen: Integer): Pointer;
Begin
  {check for enough data}
  If (aDataLen > Count) Then
    NotEnoughDataError(Count, aDataLen);
  {just return the head Pointer}
  Result := bqHead;
End;

Procedure TaaByteQueue.Remove(aDataLen: integer); {!!.01}
Begin {!!.01}
  {check for enough data}{!!.01}
  If (aDataLen > Count) Then {!!.01}
    NotEnoughDataError(Count, aDataLen); {!!.01}
  {move the remaining data to the head}{!!.01}
  Move((bqHead + aDataLen)^, bqHead^, Count - aDataLen); {!!.01}
  bqTail := bqTail - aDataLen; {!!.01}
End; {!!.01}

Procedure TaaByteQueue.setCapacity(aValue: Integer);
Var
  ByteCount: Integer;
  NewQueue: PChar;
Begin
  {don't allow data to be lost}
  ByteCount := Count;
  If (aValue < ByteCount) Then
    aValue := ByteCount;
  {round the requested capacity to nearest 64 bytes}
  aValue := (aValue + 63) And $7FFFFFC0;
  {get a new buffer}
  GetMem(NewQueue, aValue * 2);
  {if we have data to transfer from the old buffer, do so}
  If (ByteCount <> 0) Then
    Move(bqHead^, NewQueue^, ByteCount);
  {destroy the old buffer}
  If (bqQueue <> Nil) Then
    FreeMem(bqQueue, bqQueueEnd - bqQueue);
  {set the head/tail and other pointers}
  bqQueue := NewQueue;
  bqHead := NewQueue;
  bqTail := NewQueue + ByteCount;
  bqQueueEnd := NewQueue + (aValue * 2);
  bqQMidPoint := NewQueue + aValue;
End;
{====================================================================}

{===VT100 line attributes array======================================}
Type
  TIpLineAttrArray = Class
  Private
    FArray: PAnsiChar;
    FEmulator: TIpVT100Emulator;
    FTotalLineCount: Integer;
  Protected
    Function laaGetAttr(aInx: Integer): TIpVT100LineAttr;
    Procedure laaSetAttr(aInx: Integer; aValue: TIpVT100LineAttr);
    Procedure laaResize;
  Public
    Constructor Create(aEmulator: TIpVT100Emulator);
    Destructor Destroy; Override;
    Procedure Scroll(aCount, aTop, aBottom: Integer);
    Property Attr[aInx: Integer]: TIpVT100LineAttr
    Read laaGetAttr Write laaSetAttr;
  End;

Constructor TIpLineAttrArray.Create(aEmulator: TIpVT100Emulator);
Begin
  Inherited Create;
  FEmulator := aEmulator;
  FTotalLineCount := aEmulator.Buffer.SVRowCount;
  FArray := AllocMem(FTotalLineCount * sizeof(TIpVT100LineAttr));
End;

Destructor TIpLineAttrArray.Destroy;
Begin
  If (FArray <> Nil) Then
    FreeMem(FArray, FTotalLineCount * sizeof(TIpVT100LineAttr));
  Inherited Destroy;
End;

Function TIpLineAttrArray.laaGetAttr(aInx: Integer): TIpVT100LineAttr;
Begin
  If (FTotalLineCount <> FEmulator.Buffer.SVRowCount) Then
    laaResize;
  aInx := aInx + FTotalLineCount - FEmulator.Buffer.RowCount - 1;
  If (aInx < 0) Or (aInx >= FTotalLineCount) Then
    Result := ltNormal
  Else
    Result := TIpVT100LineAttr(FArray[aInx]);
End;

Procedure TIpLineAttrArray.laaResize;
Var
  NewArray: PAnsiChar;
  NewTotalCount: Integer;
Begin
  {allocate the new array}
  NewTotalCount := FEmulator.Buffer.SVRowCount;
  NewArray := AllocMem(NewTotalCount * sizeof(TIpVT100LineAttr));
  {copy over the old array from the end, *not* the beginning}
  If (NewTotalCount < FTotalLineCount) Then
    Move(FArray[FTotalLineCount - NewTotalCount], NewArray[0],
      NewTotalCount)
  Else {FTotalLineCount <= NewTotalCount}
    Move(FArray[0], NewArray[FTotalLineCount - NewTotalCount],
      FTotalLineCount);
  {free the old array, save the new one}
  FreeMem(FArray, FTotalLineCount * sizeof(TIpVT100LineAttr));
  FArray := NewArray;
  FTotalLineCount := NewTotalCount;
End;

Procedure TIpLineAttrArray.laaSetAttr(aInx: Integer;
  aValue: TIpVT100LineAttr);
Begin
  If (FTotalLineCount <> FEmulator.Buffer.SVRowCount) Then
    laaResize;
  aInx := aInx + FTotalLineCount - FEmulator.Buffer.RowCount - 1;
  If (0 <= aInx) And (aInx < FTotalLineCount) Then
    FArray[aInx] := AnsiChar(aValue);
End;

Procedure TIpLineAttrArray.Scroll(aCount, aTop, aBottom: Integer);
Var
  i: Integer;
  ToInx: Integer;
Begin
  aTop := aTop + FTotalLineCount - FEmulator.Buffer.RowCount - 1;
  aBottom := aBottom + FTotalLineCount - FEmulator.Buffer.RowCount - 1;
  If (aCount > 0) Then {scroll upwards}
  Begin
    ToInx := aTop;
    For i := (aTop + aCount) To aBottom Do
    Begin
      FArray[ToInx] := FArray[i];
      inc(ToInx);
    End;
    For i := ToInx To aBottom Do
      FArray[i] := #0;
  End
  Else
    If (aCount < 0) Then {scroll downwards}
    Begin
      ToInx := aBottom;
      For i := (aBottom + aCount) Downto aTop Do
      Begin
        FArray[ToInx] := FArray[i];
        dec(ToInx);
      End;
      For i := ToInx Downto aTop Do
        FArray[i] := #0;
    End;
End;
{====================================================================}

{===Helper routines==================================================}

Function BoundI(aLow, X, aHigh: Integer): Integer;
Begin
  If (aLow < aHigh) Then
    If (X <= aLow) Then
      Result := aLow
    Else
      If (X >= aHigh) Then
        Result := aHigh
      Else
        Result := X
    Else {aHigh is less than aLow}
      If (X <= aHigh) Then
        Result := aHigh
      Else
        If (X >= aLow) Then
          Result := aLow
        Else
          Result := X;
End;

{===TIpTerminalArray=================================================}

Constructor TIpTerminalArray.Create(aItemSize: Integer);
Begin
  Inherited Create;
  {save a valid item size}
  Case aItemSize Of
    1, 2, 4: FItemSize := aItemSize;
  Else
    FItemSize := 1;
  End; {case}
  {set the actual column count to -1, which means 'uncalculated'}
  FActColCount := -1;
End;

Destructor TIpTerminalArray.Destroy;
Var
  inFActColCount: Integer;
  inItemSize    : Integer;
  inRowCount    : Integer;
Begin
  If (FItems <> Nil) Then
  Begin
    //An Access Violation is sometimes being thrown by FreeMem
    //I want to find out what argument is causing the problem
    //consequently, I have broken them out as follows
    inFActColCount:= FActColCount;
    inItemSize    := ItemSize;
    inRowCount    := RowCount;
    FreeMem(FItems, inRowCount * inFActColCount * inItemSize);
  End;
  Inherited Destroy;
End;

{ Clear the entire array, filling with DefaultItem }

Procedure TIpTerminalArray.Clear;
Begin
  If (FItems <> Nil) Then
    taClearRows(FItems, FActColCount, 0, pred(RowCount));
End;

{ Clear the row in between the two inclusive columns, filling }
{ with DefaultItem (equivalent to 'erase') }

Procedure TIpTerminalArray.ClearItems(aRow: Integer;
  aFromCol, aToCol: Integer);
Var
  Walker: PAnsiChar;
  Value: Longint;
  i: Integer;
Begin
  Walker := @FItems[((aRow * FActColCount) + aFromCol) * ItemSize];
  Case ItemSize Of
    1: FillChar(Walker^, succ(aToCol - aFromCol), Byte(FDefaultItem));
    2:
      Begin
        Value := Word(FDefaultItem);
        For i := 0 To (aToCol - aFromCol) Do
        Begin
          PWord(Walker)^ := Value;
          inc(Walker, 2);
        End;
      End;
    4:
      Begin
        Value := FDefaultItem;
        For i := 0 To (aToCol - aFromCol) Do
        Begin
          PLongint(Walker)^ := Value;
          inc(Walker, 4);
        End;
      End;
  End; {case}
End;

{ Delete aCount default items at aRow , aCol; items currently beyond   }
{ these positions are pulled left, and their original positions filled }
{ with default items; this action does not extend beyond the row       }

Procedure TIpTerminalArray.DeleteItems(aCount: Integer;
  aRow: Integer; aCol: Integer);
Var
  ItemCount: Integer;
  Distance: Integer;
  FromPtr: PAnsiChar;
  ToPtr: PAnsiChar;
  Value: Longint;
  i: Integer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aRow and aCol}
  If (aRow < 0) Or (aRow >= RowCount) Or
    (aCol < 0) Or (aCol >= ColCount) Then
    Raise Exception.Create(Format(SRowColOOR,
      ['TIpTerminalArray.DeleteItems', aRow, aCol]));
{$ENDIF}
  Distance := ColCount - aCol;
  If (Distance > aCount) Then
    Distance := aCount;
  ItemCount := ColCount - aCol - Distance;
  Case ItemSize Of
    1:
      Begin
        ToPtr := @FItems[(aRow * FActColCount) + aCol];
        If (ItemCount > 0) Then
        Begin
          FromPtr := ToPtr + Distance;
          Move(FromPtr^, ToPtr^, ItemCount);
        End;
        ToPtr := ToPtr + ItemCount;
        FillChar(ToPtr^, Distance, Byte(FDefaultItem));
      End;
    2:
      Begin
        ToPtr := @FItems[((aRow * FActColCount) + aCol) * 2];
        If (ItemCount > 0) Then
        Begin
          FromPtr := ToPtr + (Distance * 2);
          Move(FromPtr^, ToPtr^, ItemCount * 2);
        End;
        ToPtr := ToPtr + (ItemCount * 2);
        Value := Word(FDefaultItem);
        For i := 0 To pred(Distance) Do
        Begin
          PWord(ToPtr)^ := Value;
          inc(ToPtr, 2);
        End;
      End;
    4:
      Begin
        ToPtr := @FItems[((aRow * FActColCount) + aCol) * 4];
        If (ItemCount > 0) Then
        Begin
          FromPtr := ToPtr + (Distance * 4);
          Move(FromPtr^, ToPtr^, ItemCount * 4);
        End;
        ToPtr := ToPtr + (ItemCount * 4);
        Value := FDefaultItem;
        For i := 0 To pred(Distance) Do
        Begin
          PLongint(ToPtr)^ := Value;
          inc(ToPtr, 4);
        End;
      End;
  End; {case}
End;

{ Return a Pointer to the item at aRow, aCol; caller must properly }
{ dereference, and not memory overread }

Function TIpTerminalArray.GetItemPtr(aRow, aCol: Integer): Pointer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aRow and aCol}
  If (aRow < 0) Or (aRow >= RowCount) Or
    (aCol < 0) Or (aCol >= ColCount) Then
    Raise Exception.Create(Format(SRowColOOR, ['TIpTerminalArray.GetItemPtr', aRow, aCol]));
{$ENDIF}
  If FItems = Nil Then
    Result := Nil
  Else
  Begin
    Case ItemSize Of
      1: Result := @FItems[(aRow * FActColCount) + aCol];
      2: Result := @FItems[(aRow * FActColCount * 2) + (aCol * 2)];
      4: Result := @FItems[(aRow * FActColCount * 4) + (aCol * 4)];
    Else
      Raise Exception.Create(SInvItemSize);
      Result := Nil;
    End; {case}
  End;
End;

{ Insert aCount default items at aRow , aCol; items currently in these }
{ positions are pushed right, but not beyond the row boundary          }

Procedure TIpTerminalArray.InsertItems(aCount: Integer;
  aRow: Integer;
  aCol: Integer);
Var
  ItemCount: Integer;
  Distance: Integer;
  FromPtr: PAnsiChar;
  ToPtr: PAnsiChar;
  Value: Longint;
  i: Integer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aRow and aCol}
  If (aRow < 0) Or (aRow >= RowCount) Or (aCol < 0) Or (aCol >= ColCount) Then
    Raise Exception.Create(
      Format(SRowColOOR, ['TIpTerminalArray.InsertItems', aRow, aCol]));
{$ENDIF}
  Distance := ColCount - aCol;
  If (Distance > aCount) Then
    Distance := aCount;
  ItemCount := ColCount - aCol - Distance;
  Case ItemSize Of
    1:
      Begin
        FromPtr := @FItems[(aRow * FActColCount) + aCol];
        If (ItemCount > 0) Then
        Begin
          ToPtr := FromPtr + Distance;
          Move(FromPtr^, ToPtr^, ItemCount);
        End;
        FillChar(FromPtr^, Distance, Byte(FDefaultItem));
      End;
    2:
      Begin
        FromPtr := @FItems[((aRow * FActColCount) + aCol) * 2];
        If (ItemCount > 0) Then
        Begin
          ToPtr := FromPtr + (Distance * 2);
          Move(FromPtr^, ToPtr^, ItemCount * 2);
        End;
        Value := Word(FDefaultItem);
        For i := 0 To pred(Distance) Do
        Begin
          PWord(FromPtr)^ := Value;
          inc(FromPtr, 2);
        End;
      End;
    4:
      Begin
        FromPtr := @FItems[((aRow * FActColCount) + aCol) * 4];
        If (ItemCount > 0) Then
        Begin
          ToPtr := FromPtr + (Distance * 4);
          Move(ToPtr^, FromPtr^, ItemCount * 4);
        End;
        Value := FDefaultItem;
        For i := 0 To pred(Distance) Do
        Begin
          PLongint(FromPtr)^ := Value;
          inc(FromPtr, 4);
        End;
      End;
  End; {case}
End;

Procedure TIpTerminalArray.ReplaceItems(aOldItem: pointer; aNewItem: pointer);
Var
  Walker: PAnsiChar;
  OldValue: Longint;
  NewValue: Longint;
  Row: Integer;
  I: Integer;
Begin
  Case ItemSize Of
    1:
      Begin
        OldValue := PByte(aOldItem)^;
        NewValue := PByte(aNewItem)^;
      End;
    2:
      Begin
        OldValue := PWord(aOldItem)^;
        NewValue := PWord(aNewItem)^;
      End;
    4:
      Begin
        OldValue := PLongint(aOldItem)^;
        NewValue := PLongint(aNewItem)^;
      End;
  Else
      { dummy statements that will never get executed, however they fool }
      { the warning analyzer in the compiler }
    OldValue := 0;
    NewValue := 0;
  End; { case }
  For Row := 0 To Pred(RowCount) Do
  Begin
    Walker := @FItems[(Row * FActColCount) * ItemSize];
    Case ItemSize Of
      1: For I := 0 To pred(ColCount) Do
        Begin
          If (PByte(Walker)^ = OldValue) Then
            PByte(Walker)^ := NewValue;
          Inc(Walker);
        End;
      2: For I := 0 To pred(ColCount) Do
        Begin
          If (PWord(Walker)^ = OldValue) Then
            PWord(Walker)^ := NewValue;
          Inc(Walker, 2);
        End;
      4: For I := 0 To pred(ColCount) Do
        Begin
          If (PLongint(Walker)^ = OldValue) Then
            PLongint(Walker)^ := NewValue;
          Inc(Walker, 4);
        End;
    End; { case }
  End;
End;

{ Scroll the data by aCount rows, filling new rows with DefaultItem;  }
{ scroll just between aStartRow and aEndRow inclusive; if aCount is   }
{ positive it means scroll upwards (ie, the usual sense), if negative }
{ scroll downwards }

Procedure TIpTerminalArray.ScrollRows(aCount: Integer;
  aStartRow, aEndRow: Integer);
Var
  ThisRow: Integer;
  FromPtr: PAnsiChar;
  ToPtr: PAnsiChar;
  i: Integer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aStartRow and aEndRow}
  If (aStartRow < 0) Or (aStartRow >= RowCount) Or
    (aEndRow < 0) Or (aEndRow >= RowCount) Then
    Raise Exception.Create(Format(SRowRowOOR, [aStartRow, aEndRow]));
{$ENDIF}
  If (FItems <> Nil) And (aCount <> 0) Then
  Begin
    {make sure the end row is larger than the start row}
    If (aEndRow < aStartRow) Then
    Begin
      ThisRow := aEndRow;
      aEndRow := aStartRow;
      aStartRow := ThisRow;
    End;
    { split the code depending on whether we are scrolling upwards, }
    { aCount is +ve, or downwards, aCount is -ve }
    If (aCount > 0) Then {scroll upwards}
    Begin
      { if the number of rows to scroll is greater than the difference }
      { between the start and end rows, all we need to do is blank out }
      { all the rows between start and end inclusive, otherwise we have }
      { some scrolling to do }
      ThisRow := aStartRow;
      If (aCount <= (aEndRow - aStartRow)) Then
      Begin
        ToPtr := @FItems[ThisRow * FActColCount * ItemSize];
        FromPtr := @FItems[(ThisRow + aCount) * FActColCount * ItemSize];
        For i := 0 To (aEndRow - aStartRow - aCount) Do
        Begin
          Move(FromPtr^, ToPtr^, ColCount * ItemSize);
          inc(FromPtr, FActColCount * ItemSize);
          inc(ToPtr, FActColCount * ItemSize);
          inc(ThisRow);
        End;
      End;
      { now blank out everything from ThisRow to aEndRow }
      taClearRows(FItems, FActColCount, ThisRow, aEndRow);
    End
    Else {scroll downwards}
    Begin
      { if the number of rows to scroll is greater than the difference  }
      { between the start and end rows, all we need to do is blank out  }
      { all the rows between start and end inclusive, otherwise we have }
      { some scrolling to do }
      aCount := -aCount;
      ThisRow := aEndRow;
      If (aCount <= (aEndRow - aStartRow)) Then
      Begin
        ToPtr := @FItems[ThisRow * FActColCount * ItemSize];
        FromPtr := @FItems[(ThisRow - aCount) * FActColCount * ItemSize];
        For i := 0 To (aEndRow - aStartRow - aCount) Do
        Begin
          Move(FromPtr^, ToPtr^, ColCount * ItemSize);
          dec(FromPtr, FActColCount * ItemSize);
          dec(ToPtr, FActColCount * ItemSize);
          dec(ThisRow);
        End;
      End;
      {now blank out everything from aStartRow to ThisRow}
      taClearRows(FItems, FActColCount, aStartRow, ThisRow);
    End;
  End;
End;

{ Define the default item to be used to fill new items, eg, }
{ during scrolling, clearing or resizing }

Procedure TIpTerminalArray.SetDefaultItem(aDefaultItem: Pointer);
Begin
  Case ItemSize Of
    1: FDefaultItem := PByte(aDefaultItem)^;
    2: FDefaultItem := PWord(aDefaultItem)^;
    4: FDefaultItem := PLongint(aDefaultItem)^;
  End;
End;

Procedure TIpTerminalArray.taClearRows(aBuffer: PAnsiChar;
  aActColCount: Integer;
  aStartRow, aEndRow: Integer);
Var
  Walker: PAnsiChar;
  Value: Longint;
  DWORDCount: Integer;
  i: Integer;
Begin
  Walker := @aBuffer[aStartRow * aActColCount * ItemSize];
  If (ItemSize = 1) Then
    FillChar(Walker^,
      succ(aEndRow - aStartRow) * aActColCount,
      Byte(FDefaultItem))
  Else
  Begin
    If (ItemSize = 2) Then
    Begin
      Value := (FDefaultItem Shl 16) + Word(FDefaultItem);
      DWORDCount := (succ(aEndRow - aStartRow) * aActColCount) Div 2;
    End
    Else
    Begin
      Value := FDefaultItem;
      DWORDCount := succ(aEndRow - aStartRow) * aActColCount;
    End;
    For i := 0 To pred(DWORDCount) Do
    Begin
      PLongint(Walker)^ := Value;
      inc(Walker, 4);
    End;
  End;
End;

Procedure TIpTerminalArray.taGrowArray(aRowCount,
  aColCount,
  aActColCount: Integer);
Var
  NewArray: PAnsiChar;
  RowSize: Integer;
  NumRows: Integer;
  FromPtr: PAnsiChar;
  ToPtr: PAnsiChar;
  i: Integer;
Begin
  { make sure we have the new actual column count: this is the external }
  { column count rounded up so that the actual length of a row in bytes }
  { is a multiple of four--this makes fills and moves much faster       }
  If (aActColCount = -1) Then
  Begin
    Case ItemSize Of
      1: aActColCount := ((aColCount + 3) Div 4) * 4;
      2: aActColCount := ((aColCount + 1) Div 2) * 2;
      4: aActColCount := aColCount;
    End; {case}
  End;
  { nothing to do if either the row or actual column count is zero }
  If (aRowCount = 0) Or (aActColCount = 0) Then
    Exit;
  { equally obvious, nothing to do if neither the row and actual column }
  { count have changed }
  If (aRowCount = RowCount) And (aActColCount = FActColCount) Then
    Exit;
  { at this point we must allocate another array }
  GetMem(NewArray, aRowCount * aActColCount * ItemSize);
  { blank it all out using the current default item }
  taClearRows(NewArray, aActColCount, 0, pred(aRowCount));
  {if the old array existed, transfer over the data, row by row,
   starting at the bottom}
  If (FItems <> Nil) Then
  Begin
    {calculate the number of bytes to copy per row}
    If (ColCount < aColCount) Then
      RowSize := ColCount * ItemSize
    Else
      RowSize := aColCount * ItemSize;
    {calculate the number of rows to copy}
    If (RowCount < aRowCount) Then
      NumRows := RowCount
    Else
      NumRows := aRowCount;
    {copy the rows}
    FromPtr := @FItems[RowCount * FActColCount * ItemSize];
    ToPtr := @NewArray[aRowCount * aActColCount * ItemSize];
    For i := pred(RowCount) Downto (RowCount - NumRows) Do
    Begin
      dec(FromPtr, FActColCount * ItemSize);
      dec(ToPtr, aActColCount * ItemSize);
      Move(FromPtr^, ToPtr^, RowSize);
    End;
    {dispose of the old array}
    FreeMem(FItems, RowCount * FActColCount * ItemSize);
  End;
  {save the new array}
  FItems := NewArray;
  FActColCount := aActColCount;
End;

Procedure TIpTerminalArray.taSetColCount(aNewCount: Integer);
Begin
{$IFDEF UseRangeChecks}
  {Range check aNewCount}
  If (aNewCount < 0) Then
    Raise Exception.Create(
      Format(SLessZero, ['TIpTerminalArray.taSetColCount', aNewCount]));
{$ENDIF}
  If (aNewCount <> ColCount) Then
  Begin
    taGrowArray(RowCount, aNewCount, -1);
    FColCount := aNewCount;
  End;
End;

Procedure TIpTerminalArray.taSetRowCount(aNewCount: Integer);
Begin
{$IFDEF UseRangeChecks}
  {Range check aNewCount}
  If (aNewCount < 0) Then
    Raise Exception.Create(
      Format(SLessZero, ['TIpTerminalArray.taSetColCount', aNewCount]));
{$ENDIF}
  If (aNewCount <> RowCount) Then
  Begin
    taGrowArray(aNewCount, ColCount, FActColCount);
    FRowCount := aNewCount;
  End;
End;

{ Write aCount copies of aItem to aRow, aCol, without wrapping at }
{ the end of the row }

Procedure TIpTerminalArray.WriteDupItems(aItem: Pointer;
  aCount: Integer; aRow, aCol: Integer);
Var
  Walker: PAnsiChar;
  Value: Longint;
  i: Integer;
  ItemCount: Integer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aRow and aCol}
  If (aRow < 0) Or (aRow >= RowCount) Or (aCol < 0) Or (aCol >= ColCount) Then
    Raise Exception.Create(
      Format(SRowColOOR, ['TIpTerminalArray.WriteDupItems', aRow, aCol]));
{$ENDIF}
  If (FItems <> Nil) Then
  Begin
    ItemCount := ColCount - aCol;
    If (ItemCount > aCount) Then
      ItemCount := aCount;
    Case ItemSize Of
      1: FillChar(FItems[(aRow * FActColCount) + aCol],
          ItemCount, PByte(aItem)^);
      2:
        Begin
          Walker := @FItems[((aRow * FActColCount) + aCol) * 2];
          Value := PWord(aItem)^;
          For i := 0 To pred(ItemCount) Do
          Begin
            PWord(Walker)^ := Value;
            inc(Walker, 2);
          End;
        End;
      4:
        Begin
          Walker := @FItems[((aRow * FActColCount) + aCol) * 4];
          Value := PLongint(aItem)^;
          For i := 0 To pred(ItemCount) Do
          Begin
            PLongint(Walker)^ := Value;
            inc(Walker, 4);
          End;
        End;
    End; {case}
  End;
End;

{ Write aCount items to aRow, aCol, without wrapping at the end of the row }

Procedure TIpTerminalArray.WriteItems(aItems: Pointer;
  aCount: Integer;
  aRow, aCol: Integer);
Var
  ItemCount: Integer;
Begin
{$IFDEF UseRangeChecks}
  {Range check aRow and aCol}
  If (aRow < 0) Or (aRow >= RowCount) Or (aCol < 0) Or (aCol >= ColCount) Then
    Raise Exception.Create(
      Format(SRowColOOR, ['TIpTerminalArray.WriteItems', aRow, aCol]));
{$ENDIF}
  If (FItems <> Nil) Then
  Begin
    ItemCount := ColCount - aCol;
    If (ItemCount > aCount) Then
      ItemCount := aCount;
    Case ItemSize Of
      1: Move(aItems^,
          FItems[(aRow * FActColCount) + aCol],
          ItemCount);
      2: Move(aItems^,
          FItems[(aRow * FActColCount * 2) + (aCol * 2)],
          ItemCount * 2);
      4: Move(aItems^,
          FItems[(aRow * FActColCount * 4) + (aCol * 4)],
          ItemCount * 4);
    End; {case}
  End;
End;
{====================================================================}

{===Bitset routines==================================================}

Procedure IPTClearAllBits(aBitset: PByteArray; aBitCount: Integer);
Begin
  FillChar(aBitset^, (aBitCount + 7) Shr 3, 0);
End;

Procedure IPTClearBit(aBitset: PByteArray; aBit: Integer);
Var
  BS: PAnsiChar Absolute aBitset;
  P: PAnsiChar;
  M: Byte;
Begin
  P := BS + (aBit Shr 3);
  M := 1 Shl (Byte(aBit) And 7);
  P^ := char(Byte(P^) And Not M);
End;

Function IPTIsBitSet(aBitset: PByteArray; aBit: Integer): Boolean;
Var
  BS: PAnsiChar Absolute aBitset;
  P: PAnsiChar;
  M: Byte;
Begin
  P := BS + (aBit Shr 3);
  M := 1 Shl (Byte(aBit) And 7);
  Result := (Byte(P^) And M) <> 0;
End;

Function IPTReallocBitset(aBitset: PByteArray;
  aOldBitCount: Integer;
  aNewBitCount: Integer): PByteArray;
Var
  XferBitCount: Integer;
Begin
  Try
    If (aNewBitCount = 0) Then
      Result := Nil
    Else
    Begin
      Result := AllocMem(aNewBitCount);
      If (aBitset <> Nil) Then
      Begin
        If (aOldBitCount < aNewBitCount) Then
          XferBitCount := aOldBitCount
        Else
          XferBitCount := aNewBitCount;
        Move(aBitset^, Result^, (XferBitCount + 7) Shr 3);
      End;
    End;
    FreeMem(aBitset, (aOldBitCount + 7) Shr 3);
  Except
    Result := Nil;
  End;
End;

Procedure IPTSetBit(aBitset: PByteArray; aBit: Integer);
Var
  BS: PAnsiChar Absolute aBitSet;
  P: PAnsiChar;
  M: Byte;
Begin
  P := BS + (aBit Shr 3);
  M := 1 Shl (Byte(aBit) And 7);
  P^ := char(Byte(P^) Or M);
End;
{====================================================================}

{===Invalid rectangle routines==========================================}
Const
  RectsPerPage = 200;
Type
  PInvRect = ^TInvRect;
  TInvRect = Packed Record
    irNext: PInvRect;
    irRect: TRect;
  End;
  PInvRectPage = ^TInvRectPage;
  TInvRectPage = Packed Record
    irpNext: PInvRectPage;
    irpRects: Array[0..pred(RectsPerPage)] Of TInvRect;
  End;
Var
  InvRectFreeList: PInvRect;
  InvRectPageList: PInvRectPage;

Procedure IPTFreeInvRect(P: PInvRect);
Begin
  {push rect onto free list}
  P^.irNext := InvRectFreeList;
  InvRectFreeList := P;
End;

Procedure IPTAllocInvRectPage;
Var
  NewPage: PInvRectPage;
  i: Integer;
Begin
  {alloc new page and add it to front of page list}
  New(NewPage);
  NewPage^.irpNext := InvRectPageList;
  InvRectPageList := NewPage;
  {add all rects on this page to free list}
  For i := 0 To pred(RectsPerPage) Do
    IPTFreeInvRect(@NewPage^.irpRects[i]);
End;

Function IPTAllocInvRect: PInvRect;
Begin
  {pop top rect from free list; if none, add a whole page's worth}
  If (InvRectFreeList = Nil) Then
    IPTAllocInvRectPage;
  Result := InvRectFreeList;
  InvRectFreeList := Result^.irNext;
End;

Procedure IPTFreeInvRectPages;
Var
  Temp: PInvRectPage;
Begin
  {dispose of all rect pages}
  While (InvRectPageList <> Nil) Do
  Begin
    Temp := InvRectPageList;
    InvRectPageList := Temp^.irpNext;
    Dispose(Temp);
  End;
  {since all rects have now gone, force the rect free list to nil}
  InvRectFreeList := Nil;
End;

Procedure IPTIpdInvalidRect(Var aInvRectList: PInvRect;
  Const aRect: TRect);
Var
  NewRect: PInvRect;
Begin
  NewRect := IPTAllocInvRect;
  NewRect^.irNext := aInvRectList;
  aInvRectList := NewRect;
  NewRect^.irRect := aRect;
End;

Function IPTRemoveInvalidRect(Var aInvRectList: PInvRect;
  Var aRect: TRect): Boolean;
Var
  TopRect: PInvRect;
Begin
  If (aInvRectList = Nil) Then
    Result := False
  Else
  Begin
    Result := True;
    TopRect := aInvRectList;
    aInvRectList := TopRect^.irNext;
    aRect := TopRect^.irRect;
    IPTFreeInvRect(TopRect);
  End;
End;

Function IPTPeekInvalidRect(aInvRectList: PInvRect;
  Var aRect: TRect): Boolean;
Begin
  If (aInvRectList = Nil) Then
    Result := False
  Else
  Begin
    Result := True;
    aRect := aInvRectList^.irRect;
  End;
End;

Procedure IPTMergeInvalidRects(aInvRectList: PInvRect);
Var
  Temp: PInvRect;
  Walker: PInvRect;
  MinRect: TRect;
Begin
  If (aInvRectList = Nil) Then
    Exit;
  {performs a simple merge of all the invalid rects in the list by
   working out the rect that just covers them all; free the rects from
   the list after we read them--leaving the first for our use}
  MinRect := aInvRectList^.irRect;
  Walker := aInvRectList^.irNext;
  While (Walker <> Nil) Do
  Begin
    With Walker^.irRect Do
    Begin
      If Left < MinRect.Left Then
        MinRect.Left := Left;
      If Top < MinRect.Top Then
        MinRect.Top := Top;
      If Right > MinRect.Right Then
        MinRect.Right := Right;
      If Bottom > MinRect.Bottom Then
        MinRect.Bottom := Bottom;
    End;
    Temp := Walker;
    Walker := Walker^.irNext;
    IPTFreeInvRect(Temp);
  End;
  {MinRect now contains the smallest rectangle that covers all invalid
   rects in the list; set this minimum invalid rect into the first
   (and only) item in the list}
  aInvRectList^.irNext := Nil;
  aInvRectList^.irRect := MinRect;
End;
{====================================================================}

{===TIpTerminalBuffer================================================}

Constructor TIpTerminalBuffer.Create(aUseWideChars: Boolean);
Var
  i: Integer;
Begin
  Inherited Create;

  {set the values of the properties that define defaults}
  FDefBackColor := ipc_TermBufBackColor;
  FDefForeColor := ipc_TermBufForeColor;
  FDefAnsiChar := ' ';
  FDefWideChar := ' ';
  FDefCharSet := 0;
  FDefAttr := [];

  {set the 'power-up' values}
  FBackColor := ipc_TermBufBackColor;
  FForeColor := ipc_TermBufForeColor;
  UseAbsAddress := ipc_TermBufUseAbsAddress;
  UseAutoWrap := ipc_TermBufUseAutoWrap;
  UseAutoWrapDelay := ipc_TermBufUseAutoWrapDelay;
  UseInsertMode := ipc_TermBufUseInsertMode;
  UseNewLineMode := ipc_TermBufUseNewLineMode;
  UseScrollRegion := ipc_TermBufUseScrollRegion;

  {set up all the matrices to hold the displayed data}
  {..character matrix}
  If aUseWideChars Then
  Begin
    FCharMatrix := TIpTerminalArray.Create(sizeof(WideChar));
    FCharMatrix.SetDefaultItem(@FDefWideChar);
  End
  Else
  Begin
    FCharMatrix := TIpTerminalArray.Create(sizeof(AnsiChar));
    FCharMatrix.SetDefaultItem(@FDefAnsiChar);
  End;
  FCharMatrix.ColCount := ipc_TermBufColCount;
  FCharMatrix.RowCount := ipc_TermBufScrollRowCount;

  {..character set matrix}
  FCharSetMatrix := TIpTerminalArray.Create(sizeof(Byte));
  FCharSetMatrix.SetDefaultItem(@FDefCharSet);
  FCharSetMatrix.ColCount := ipc_TermBufColCount;
  FCharSetMatrix.RowCount := ipc_TermBufScrollRowCount;

  {..character attributes matrix}
  FAttrMatrix := TIpTerminalArray.Create(sizeof(TIpTerminalCharAttrs));
  FAttrMatrix.SetDefaultItem(@FDefAttr);
  FAttrMatrix.ColCount := ipc_TermBufColCount;
  FAttrMatrix.RowCount := ipc_TermBufScrollRowCount;

  {..character foreground color matrix}
  FForeColorMatrix := TIpTerminalArray.Create(sizeof(TColor));
  FForeColorMatrix.SetDefaultItem(@FDefForeColor);
  FForeColorMatrix.ColCount := ipc_TermBufColCount;
  FForeColorMatrix.RowCount := ipc_TermBufScrollRowCount;

  {..character background color matrix}
  FBackColorMatrix := TIpTerminalArray.Create(sizeof(TColor));
  FBackColorMatrix.SetDefaultItem(@FDefBackColor);
  FBackColorMatrix.ColCount := ipc_TermBufColCount;
  FBackColorMatrix.RowCount := ipc_TermBufScrollRowCount;

  {initialize the terminal dimensions}
  FUseWideChars := aUseWideChars;
  SVRowCount := ipc_TermBufScrollRowCount;
  ColCount := ipc_TermBufColCount;
  RowCount := ipc_TermBufRowCount;

  ClearAllHorzTabStops;
  ClearAllVertTabStops;
  i := Col;
  While (i < ColCount) Do
  Begin
    Col := i;
    SetHorzTabStop;
    inc(i, 8);
  End;
  Row := 1;
  Col := 1;

  {set up the current cursor position}
  FCursorRow := SVRowCount - RowCount;
  FDisplayOriginRow := FCursorRow;
  FCursorCol := 0;

  { add the whole screen as an invalid rect }
  FCursorMoved := True;
  tbInvalidateRect(FCursorRow, 0,
    pred(SVRowCount), pred(ColCount));
  tbFireOnCursorMovedEvent; {!!.01}
End;

Destructor TIpTerminalBuffer.Destroy;
Var
  OurRect: TRect;
Begin
  { remove all of the invalid rects and discard them }
  While IPTRemoveInvalidRect(PInvRect(FInvRectList), OurRect) Do { nothing }

  {free the tab stops}
    IPTReallocBitset(FVertTabStops, RowCount, 0);
  IPTReallocBitset(FHorzTabStops, ColCount, 0);

  { free all arrays }
  FBackColorMatrix.Free;
  FForeColorMatrix.Free;
  FAttrMatrix.Free;
  FCharSetMatrix.Free;
  FCharMatrix.Free;
  Inherited Destroy;
End;

Procedure TIpTerminalBuffer.ClearAllHorzTabStops;
Begin
  If (ColCount <> 0) Then
    IPTClearAllBits(FHorzTabStops, ColCount);
End;

Procedure TIpTerminalBuffer.ClearAllVertTabStops;
Begin
  If (RowCount <> 0) Then
    IPTClearAllBits(FVertTabStops, RowCount);
End;

Procedure TIpTerminalBuffer.ClearHorzTabStop;
Begin
  If (ColCount <> 0) Then
    IPTClearBit(FHorzTabStops, FCursorCol);
End;

Procedure TIpTerminalBuffer.ClearVertTabStop;
Begin
  If (RowCount <> 0) Then
    IPTClearBit(FVertTabStops, FCursorRow);
End;

Procedure TIpTerminalBuffer.DeleteChars(aCount: Integer);
Var
  CharCount: Integer;
Begin
  FBeyondMargin := False;
{$IFDEF UseRangeCheck}
  If (aCount <= 0) Then
    Raise Exception.Create(Format(SPosReqd, ['TIpTerminalBuffer.DeleteChars']));
{$ENDIF}
  {the actual number of characters to delete is constrained by the
   current display region}
  CharCount := FDisplayOriginCol + FDisplayColCount - FCursorCol;
  If (CharCount > aCount) Then
    CharCount := aCount;
  If (CharCount > 0) Then
  Begin
    FCharMatrix.DeleteItems(CharCount, FCursorRow, FCursorCol);
    FCharSetMatrix.DeleteItems(CharCount, FCursorRow, FCursorCol);
    FAttrMatrix.DeleteItems(CharCount, FCursorRow, FCursorCol);
    FForeColorMatrix.DeleteItems(CharCount, FCursorRow, FCursorCol);
    FBackColorMatrix.DeleteItems(CharCount, FCursorRow, FCursorCol);
    {the cursor does not move}
    tbInvalidateRect(FCursorRow, FCursorCol,
      FCursorRow, FCursorCol + CharCount - 1);
  End;
End;

Procedure TIpTerminalBuffer.DeleteLines(aCount: Integer);
Var
  MaxRow: Integer;
Begin
  FBeyondMargin := False;
  {deleting lines is equivalent to a scroll up to the current cursor
   position; we take account of any scroll region, of course}
{$IFDEF UseRangeCheck}
  If (aCount <= 0) Then
    Raise Exception.Create(Format(SPosReqd, ['TIpTerminalBuffer.DeleteLines']));
{$ENDIF}
  MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
  tbScrollRows(aCount, FCursorRow, MaxRow);
  {the cursor does not move}
  tbInvalidateRect(FCursorRow, FDisplayOriginCol,
    MaxRow, FDisplayOriginCol + FDisplayColCount - 1);
End;

Procedure TIpTerminalBuffer.DoBackHorzTab;
Var
  NewCol: Integer;
Begin
  If (ColCount > 0) Then
  Begin
    NewCol := FCursorCol;
    While (NewCol > FDisplayOriginCol) Do
    Begin
      dec(NewCol);
      If IPTIsBitSet(FHorzTabStops, NewCol) Then
      Begin
        FCursorMoved := FCursorMoved Or (FCursorCol <> NewCol);
        FCursorCol := NewCol;
        tbFireOnCursorMovedEvent; {!!.01}
        Exit;
      End;
    End;
    FCursorMoved := FCursorMoved Or (FCursorCol <> FDisplayOriginCol);
    FCursorCol := FDisplayOriginCol;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.DoBackVertTab;
Var
  NewRow: Integer;
Begin
  FBeyondMargin := False;
  If (RowCount > 0) Then
  Begin
    NewRow := FCursorRow;
    While (NewRow > FDisplayOriginRow) Do
    Begin
      dec(NewRow);
      If IPTIsBitSet(FVertTabStops, NewRow) Then
      Begin
        FCursorMoved := FCursorMoved Or (FCursorRow <> NewRow);
        FCursorRow := NewRow;
        tbFireOnCursorMovedEvent; {!!.01}
        Exit;
      End;
    End;
    FCursorMoved := FCursorMoved Or (FCursorRow <> FDisplayOriginRow);
    FCursorRow := FDisplayOriginRow;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.DoBackspace;
Begin
  FBeyondMargin := False;
  If (FCursorCol > FDisplayOriginCol) Then
  Begin
    FCursorMoved := True;
    dec(FCursorCol);
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.DoCarriageReturn;
Begin
  FBeyondMargin := False;
  FCursorMoved := FCursorMoved Or (FCursorCol <> FDisplayOriginCol);
  FCursorCol := FDisplayOriginCol;
  tbFireOnCursorMovedEvent; {!!.01}
End;

Procedure TIpTerminalBuffer.DoHorzTab;
Var
  NewCol: Integer;
  MaxCol: Integer;
Begin
  FBeyondMargin := False;
  If (ColCount > 0) Then
  Begin
    NewCol := FCursorCol;
    MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
    While (NewCol < MaxCol) Do
    Begin
      inc(NewCol);
      If IPTIsBitSet(FHorzTabStops, NewCol) Then
      Begin
        FCursorMoved := FCursorMoved Or (FCursorCol <> NewCol);
        FCursorCol := NewCol;
        tbFireOnCursorMovedEvent; {!!.01}
        Exit;
      End;
    End;
    FCursorMoved := FCursorMoved Or (FCursorCol <> MaxCol);
    FCursorCol := MaxCol;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.DoLineFeed;
Begin
  FBeyondMargin := False;
  If UseNewLineMode Then
    DoCarriageReturn;
  MoveCursorDown(True);
End;

Procedure TIpTerminalBuffer.DoVertTab;
Var
  NewRow: Integer;
  MaxRow: Integer;
Begin
  FBeyondMargin := False;
  If (RowCount > 0) Then
  Begin
    NewRow := FCursorRow;
    MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
    While (NewRow < MaxRow) Do
    Begin
      inc(NewRow);
      If IPTIsBitSet(FVertTabStops, NewRow) Then
      Begin
        FCursorMoved := FCursorMoved Or (FCursorRow <> NewRow);
        FCursorRow := NewRow;
        tbFireOnCursorMovedEvent; {!!.01}
        Exit;
      End;
    End;
    FCursorMoved := FCursorMoved Or (FCursorRow <> MaxRow);
    FCursorRow := MaxRow;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.EraseAll;
Begin
  {WARNING: this DOES NOT use the scroll region, if defined, it blanks
            out everything in the scrollback buffer}
  FBeyondMargin := False;
  FCharMatrix.Clear;
  FCharSetMatrix.Clear;
  FAttrMatrix.Clear;
  FForeColorMatrix.Clear;
  FBackColorMatrix.Clear;
  Row := 1;
  Col := 1;
  tbInvalidateRect(FCursorRow, 0,
    pred(SVRowCount), pred(ColCount));
End;

Procedure TIpTerminalBuffer.EraseChars(aCount: Integer);
Var
  CharCount: Integer;
  ToColNum: Integer;
  CurRow: Integer;
  CurCol: Integer;
  MaxCol: Integer;
  MaxRow: Integer;
Begin
  {WARNING: this uses the scroll region, if defined}

  FBeyondMargin := False;
{$IFDEF UseRangeChecks}
  If (aCount <= 0) Then
    Raise Exception.Create(Format(SPosReqd, ['TIpTerminalBuffer.EraseChars']));
{$ENDIF}
  { this is complicated by the need to erase chars on individual lines separately }
  CurRow := FCursorRow;
  CurCol := FCursorCol;
  MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
  MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
  While (aCount > 0) And (CurRow <= MaxRow) Do
  Begin
    {calculate the number of characters to erase in this row}
    CharCount := MaxCol - CurCol + 1;
    If (CharCount > aCount) Then
      CharCount := aCount;
    {calculate the final column number}
    ToColNum := CurCol + CharCount - 1;
    {erase}
    FCharMatrix.ClearItems(CurRow, CurCol, ToColNum);
    FCharSetMatrix.ClearItems(CurRow, CurCol, ToColNum);
    FAttrMatrix.ClearItems(CurRow, CurCol, ToColNum);
    FForeColorMatrix.ClearItems(CurRow, CurCol, ToColNum);
    FBackColorMatrix.ClearItems(CurRow, CurCol, ToColNum);
    tbInvalidateRect(CurRow, CurCol,
      CurRow, ToColNum);
    {set up for the next loop}
    dec(aCount, CharCount);
    inc(CurRow);
    CurCol := FDisplayOriginCol;
  End;
End;

Procedure TIpTerminalBuffer.EraseFromBOL;
Begin
  {WARNING: this uses the scroll region, if defined}

  FBeyondMargin := False;
  {set all characters from the beginning of the line, up to and
   including the cursor position, to blanks}
  FCharMatrix.ClearItems(FCursorRow, FDisplayOriginCol, FCursorCol);
  FCharSetMatrix.ClearItems(FCursorRow, FDisplayOriginCol, FCursorCol);
  FAttrMatrix.ClearItems(FCursorRow, FDisplayOriginCol, FCursorCol);
  FForeColorMatrix.ClearItems(FCursorRow, FDisplayOriginCol, FCursorCol);
  FBackColorMatrix.ClearItems(FCursorRow, FDisplayOriginCol, FCursorCol);
  tbInvalidateRect(FCursorRow, FDisplayOriginCol,
    FCursorRow, FCursorCol);
End;

Procedure TIpTerminalBuffer.EraseFromBOW;
Var
  DisplayStartRow: Integer;
Begin
  {WARNING: this DOES NOT use the scroll region, if defined, it blanks
            out everything on the window up to and including the
            cursor position}

  FBeyondMargin := False;
  {set all characters from the beginning of the line, up to and
   including the cursor position, to blanks}
  FCharMatrix.ClearItems(FCursorRow, 0, FCursorCol);
  FCharSetMatrix.ClearItems(FCursorRow, 0, FCursorCol);
  FAttrMatrix.ClearItems(FCursorRow, 0, FCursorCol);
  FForeColorMatrix.ClearItems(FCursorRow, 0, FCursorCol);
  FBackColorMatrix.ClearItems(FCursorRow, 0, FCursorCol);
  tbInvalidateRect(FCursorRow, 0,
    FCursorRow, FCursorCol);
  {now erase all previous lines, by scrolling them out of existence}
  DisplayStartRow := SVRowCount - RowCount;
  If (FCursorRow > DisplayStartRow) Then
  Begin
    tbScrollRows(FCursorRow - DisplayStartRow,
      DisplayStartRow, pred(FCursorRow));
    tbInvalidateRect(DisplayStartRow, 0,
      pred(FCursorRow), pred(ColCount));
  End;
End;

Procedure TIpTerminalBuffer.EraseLine;
Var
  MaxCol: Integer;
Begin
  {WARNING: this uses the scroll region, if defined}

  FBeyondMargin := False;
  {set all characters from the beginning to the end of the line to
   blanks}
  MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
  FCharMatrix.ClearItems(FCursorRow, FDisplayOriginCol, MaxCol);
  FCharSetMatrix.ClearItems(FCursorRow, FDisplayOriginCol, MaxCol);
  FAttrMatrix.ClearItems(FCursorRow, FDisplayOriginCol, MaxCol);
  FForeColorMatrix.ClearItems(FCursorRow, FDisplayOriginCol, MaxCol);
  FBackColorMatrix.ClearItems(FCursorRow, FDisplayOriginCol, MaxCol);
  tbInvalidateRect(FCursorRow, FDisplayOriginCol,
    FCursorRow, MaxCol);
End;

Procedure TIpTerminalBuffer.EraseScreen;
Begin
  {WARNING: this DOES NOT use the scroll region, if defined, it blanks
            out everything on the window}

  FBeyondMargin := False;
  {scroll the entire scrollback view by RowCount lines: this will have
   the effect of clearing the display view and of setting up the
   scrollback buffer with the previous screen}
  tbScrollRows(RowCount, 0, pred(SVRowCount));
  tbInvalidateRect(SVRowCount - RowCount, 0,
    pred(SVRowCount), pred(ColCount));
End;

Procedure TIpTerminalBuffer.EraseToEOL;
Var
  MaxCol: Integer;
Begin
  {WARNING: this uses the scroll region, if defined}

  FBeyondMargin := False;
  {set all characters from and including the cursor position to the
   end of the line to blanks}
  MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
  FCharMatrix.ClearItems(FCursorRow, FCursorCol, MaxCol);
  FCharSetMatrix.ClearItems(FCursorRow, FCursorCol, MaxCol);
  FAttrMatrix.ClearItems(FCursorRow, FCursorCol, MaxCol);
  FForeColorMatrix.ClearItems(FCursorRow, FCursorCol, MaxCol);
  FBackColorMatrix.ClearItems(FCursorRow, FCursorCol, MaxCol);
  tbInvalidateRect(FCursorRow, FCursorCol,
    FCursorRow, MaxCol);
End;

Procedure TIpTerminalBuffer.EraseToEOW;
Begin
  {WARNING: this DOES NOT use the scroll region, if defined, it blanks
            out everything from and including the cursor position up
            to the end of the screen}

  FBeyondMargin := False;
  {set all characters from and including the cursor position to the
   end of the line to blanks}
  FCharMatrix.ClearItems(FCursorRow, FCursorCol, pred(ColCount));
  FCharSetMatrix.ClearItems(FCursorRow, FCursorCol, pred(ColCount));
  FAttrMatrix.ClearItems(FCursorRow, FCursorCol, pred(ColCount));
  FForeColorMatrix.ClearItems(FCursorRow, FCursorCol, pred(ColCount));
  FBackColorMatrix.ClearItems(FCursorRow, FCursorCol, pred(ColCount));
  tbInvalidateRect(FCursorRow, FCursorCol,
    FCursorRow, pred(ColCount));
  {now erase all succeeding lines, by scrolling them out of existence}
  If (FCursorRow < pred(SVRowCount)) Then
  Begin
    tbScrollRows(pred(SVRowCount) - FCursorRow,
      succ(FCursorRow), pred(SVRowCount));
    tbInvalidateRect(succ(FCursorRow), 0,
      pred(SVRowCount), pred(ColCount));
  End;
End;

Procedure TIpTerminalBuffer.GetCharAttrs(Var aValue: TIpTerminalCharAttrs);
Begin
  aValue := FAttr;
End;

Procedure TIpTerminalBuffer.GetDefCharAttrs(Var aValue: TIpTerminalCharAttrs);
Begin
  aValue := FDefAttr;
End;

Function TIpTerminalBuffer.GetInvalidRect(Var aRect: TRect): Boolean;
Begin
  If (FInvRectList = Nil) Then
    Result := False
  Else
  Begin
    {if there is more than one invalid rect, merge them all into one}
    If (PInvRect(FInvRectList)^.irNext <> Nil) Then
      IPTMergeInvalidRects(FInvRectList);
    {return the first invalid rect}
    Result := IPTRemoveInvalidRect(PInvRect(FInvRectList), aRect);
  End;
End;

Function TIpTerminalBuffer.GetLineAttrPtr(aRow: Integer): Pointer;
Var
  OurRow: Integer;
Begin
  {normalize the row number to our internal system}
  OurRow := tbCvtToInternalRow(aRow, True);
{$IFDEF UseRangeChecks}
  If (OurRow < 0) Or (OurRow >= FSVRowCount) Then
    Raise Exception.Create(Format(SRowOOR, ['TIpTerminalBuffer.GetLineAttrPtr']));
{$ENDIF}
  {return the Pointer into the matrix}
  Result := FAttrMatrix.GetItemPtr(OurRow, FDisplayOriginCol);
End;

Function TIpTerminalBuffer.GetLineBackColorPtr(aRow: Integer): Pointer;
Var
  OurRow: Integer;
Begin
  {normalize the row number to our internal system}
  OurRow := tbCvtToInternalRow(aRow, True);
{$IFDEF UseRangeChecks}
  If (OurRow < 0) Or
    (OurRow >= FSVRowCount) Then
    Raise Exception.Create(Format(SRowOOR, ['TIpTerminalBuffer.GetLineBackColorPtr']));
{$ENDIF}
  {return the Pointer into the matrix}
  Result := FBackColorMatrix.GetItemPtr(OurRow, FDisplayOriginCol);
End;

Function TIpTerminalBuffer.GetLineCharPtr(aRow: Integer): Pointer;
Var
  OurRow: Integer;
Begin
  {normalize the row number to our internal system}
  OurRow := tbCvtToInternalRow(aRow, True);
{$IFDEF UseRangeChecks}
  If (OurRow < 0) Or (OurRow >= FSVRowCount) Then
    Raise Exception.Create(Format(SRowOOR, ['TIpTerminalBuffer.GetLineCharPtr']));
{$ENDIF}
  {return the Pointer into the matrix}
  Result := FCharMatrix.GetItemPtr(OurRow, FDisplayOriginCol)
End;

Function TIpTerminalBuffer.GetLineCharSetPtr(aRow: Integer): Pointer;
Var
  OurRow: Integer;
Begin
  {normalize the row number to our internal system}
  OurRow := tbCvtToInternalRow(aRow, True);
{$IFDEF UseRangeChecks}
  If (OurRow < 0) Or (OurRow >= FSVRowCount) Then
    Raise Exception.Create(Format(SRowOOR, ['TIpTerminalBuffer.GetLineCharSetPtr']));
{$ENDIF}
  {return the Pointer into the matrix}
  Result := FCharSetMatrix.GetItemPtr(OurRow, FDisplayOriginCol)
End;

Function TIpTerminalBuffer.GetLineForeColorPtr(aRow: Integer): Pointer;
Var
  OurRow: Integer;
Begin
  {normalize the row number to our internal system}
  OurRow := tbCvtToInternalRow(aRow, True);
{$IFDEF UseRangeChecks}
  If (OurRow < 0) Or (OurRow >= FSVRowCount) Then
    Raise Exception.Create(Format(SRowOOR, ['TIpTerminalBuffer.GetLineForeColorPtr']));
{$ENDIF}
  {return the Pointer into the matrix}
  Result := FForeColorMatrix.GetItemPtr(OurRow, FDisplayOriginCol)
End;

Function TIpTerminalBuffer.HasCursorMoved: Boolean;
Begin
  {return whether the cursor has moved since the last time this
   function was called; reset the internal variable}
  Result := FCursorMoved;
  FCursorMoved := False;
End;

Function TIpTerminalBuffer.HasDisplayChanged: Boolean;
Var
  DummyRect: TRect;
Begin
  Result := IPTPeekInvalidRect(PInvRect(FInvRectList), DummyRect);
End;

Procedure TIpTerminalBuffer.InsertChars(aCount: Integer);
Var
  CharCount: Integer;
Begin
  FBeyondMargin := False;
{$IFDEF UseRangeCheck}
  If (aCount <= 0) Then
    Raise Exception.Create(Format(SPosReqd, ['TIpTerminalBuffer.InsertChars']));
{$ENDIF}
  { the actual number of characters to delete is constrained by the }
  { current display region }
  CharCount := FDisplayOriginCol + FDisplayColCount - FCursorCol;
  If (CharCount > aCount) Then
    CharCount := aCount;
  If (CharCount > 0) Then
  Begin
    FCharMatrix.InsertItems(CharCount, FCursorRow, FCursorCol);
    FCharSetMatrix.InsertItems(CharCount, FCursorRow, FCursorCol);
    FAttrMatrix.InsertItems(CharCount, FCursorRow, FCursorCol);
    FForeColorMatrix.InsertItems(CharCount, FCursorRow, FCursorCol);
    FBackColorMatrix.InsertItems(CharCount, FCursorRow, FCursorCol);
    tbInvalidateRect(FCursorRow, FCursorCol,
      FCursorRow, pred(FCursorCol + CharCount));
  End;
End;

Procedure TIpTerminalBuffer.InsertLines(aCount: Integer);
Var
  MaxRow: Integer;
Begin
  FBeyondMargin := False;
  { inserting lines is equivalent to a scroll down from the current  }
  { cursor position; we take account of any scroll region, of course }
{$IFDEF UseRangeCheck}
  If (aCount <= 0) Then
    Raise Exception.Create(Format(SPosReqd, ['TIpTerminalBuffer.InsertLines']));
{$ENDIF}
  MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
  tbScrollRows(-aCount, FCursorRow, MaxRow);
  {the cursor position doesn't change as a result of inserting rows}
  tbInvalidateRect(FCursorRow, FDisplayOriginCol,
    MaxRow, pred(FDisplayOriginCol + FDisplayColCount));
End;

Procedure TIpTerminalBuffer.MoveCursorDown(aScroll: Boolean);
Begin
  FBeyondMargin := False;
  tbMoveCursorUpDown(1, aScroll);
End;

Procedure TIpTerminalBuffer.MoveCursorLeft(aWrap: Boolean;
  aScroll: Boolean);
Begin
  FBeyondMargin := False;
  tbMoveCursorLeftRight(-1, aWrap, aScroll);
End;

Procedure TIpTerminalBuffer.MoveCursorRight(aWrap: Boolean;
  aScroll: Boolean);
Begin
  FBeyondMargin := False;
  tbMoveCursorLeftRight(1, aWrap, aScroll);
End;

Procedure TIpTerminalBuffer.MoveCursorUp(aScroll: Boolean);
Begin
  FBeyondMargin := False;
  tbMoveCursorUpDown(-1, aScroll);
End;

Procedure TIpTerminalBuffer.Reset;
Var
  i: Integer;
Begin
  {set the attributes, the colors, and the charset}
  FAttr := FDefAttr;
  FForeColor := FDefForeColor;
  FBackColor := FDefBackColor;
  FCharSet := FDefCharSet;

  {set the various matrices to their 'power-up' values}
  If UseWideChars Then
  Begin
    FCharMatrix.SetDefaultItem(@FDefWideChar);
  End
  Else
  Begin
    FCharMatrix.SetDefaultItem(@FDefAnsiChar);
  End;
  FCharSetMatrix.SetDefaultItem(@FDefCharSet);
  FAttrMatrix.SetDefaultItem(@FDefAttr);
  FForeColorMatrix.SetDefaultItem(@FDefForeColor);
  FBackColorMatrix.SetDefaultItem(@FDefBackColor);

  {clear the matrices}
  FCharMatrix.Clear;
  FCharSetMatrix.Clear;
  FAttrMatrix.Clear;
  FForeColorMatrix.Clear;
  FBackColorMatrix.Clear;

  {clear all tab stops, set horizontal ones to every 8 chars}
  ClearAllHorzTabStops;
  ClearAllVertTabStops;
  i := 1;
  While (i < ColCount) Do
  Begin
    Col := i;
    SetHorzTabStop;
    inc(i, 8);
  End;

  {set the buffer modes}
  UseAbsAddress := ipc_TermBufUseAbsAddress;
  UseAutoWrap := ipc_TermBufUseAutoWrap;
  UseAutoWrapDelay := ipc_TermBufUseAutoWrapDelay;
  UseInsertMode := ipc_TermBufUseInsertMode;
  UseNewLineMode := ipc_TermBufUseNewLineMode;
  UseScrollRegion := ipc_TermBufUseScrollRegion;

  {reset the cursor position}
  FBeyondMargin := False;
  Row := 1;
  Col := 1;
End;

Procedure TIpTerminalBuffer.SetCharAttrs(Const aValue: TIpTerminalCharAttrs);
Begin
  FAttr := aValue;
  FAttrMatrix.SetDefaultItem(@FAttr);
End;

Procedure TIpTerminalBuffer.SetDefCharAttrs(Const aValue: TIpTerminalCharAttrs);
Begin
  FDefAttr := aValue;
End;

Procedure TIpTerminalBuffer.SetCursorPosition(aRow, aCol: Integer);
Begin
  FBeyondMargin := False;
  Row := aRow;
  Col := aCol;
End;

Procedure TIpTerminalBuffer.SetScrollRegion(aTopRow, aBottomRow: Integer);
Var
  Temp: Integer;
Begin
  FBeyondMargin := False;
  {if the top row is greater than the bottom row, they're out of
   order, so switch 'em round}
  If (aTopRow > aBottomRow) Then
  Begin
    Temp := aTopRow;
    aTopRow := aBottomRow;
    aBottomRow := Temp;
  End;
{$IFDEF UseRangeChecks}
  If ((aBottomRow - aTopRow) < 1) Or (aTopRow < 1) Or (aTopRow > RowCount) Or
    (aBottomRow < 1) Or (aBottomRow > RowCount) Then
    Raise Exception.Create(SInvScrollRow);
{$ENDIF}
  FSRStartRow := tbCvtToInternalRow(aTopRow, True);
  FSREndRow := tbCvtToInternalRow(aBottomRow, True);
  {force the scroll region to be used}
  If UseScrollRegion Then
    UseScrollRegion := False;
  If (aTopRow <> 1) Or (aBottomRow <> RowCount) Then
    UseScrollRegion := True;
End;

Procedure TIpTerminalBuffer.SetHorzTabStop;
Begin
  If (ColCount <> 0) Then
    IPTSetBit(FHorzTabStops, FCursorCol);
End;

Procedure TIpTerminalBuffer.SetVertTabStop;
Begin
  If (RowCount <> 0) Then
    IPTSetBit(FVertTabStops, FCursorRow);
End;

Function TIpTerminalBuffer.tbAtLastColumn: Boolean;
Var
  MaxCol: Integer;
Begin
  MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
  Result := FCursorCol = MaxCol;
End;

Function TIpTerminalBuffer.tbCvtToExternalCol(aCol: Integer;
  aAbsolute: Boolean): Integer;
Begin
  {aCol is an internal reference (ie. zero-based and absolute), we
   need the external value (ie, one-based and relative to the start of
   the addressable area)}
  If aAbsolute Then
    Result := aCol + 1
  Else
    Result := aCol - FDisplayOriginCol + 1;
End;

Function TIpTerminalBuffer.tbCvtToExternalRow(aRow: Integer;
  aAbsolute: Boolean): Integer;
Begin
  {aRow is an internal reference (ie. zero-based and absolute), we
   need the external value (ie, one-based and relative to the start of
   the addressable area)}
  If aAbsolute Then
    Result := aRow - (SVRowCount - RowCount) + 1
  Else
    Result := aRow - FDisplayOriginRow + 1;
End;

Function TIpTerminalBuffer.tbCvtToInternalCol(aCol: Integer;
  aAbsolute: Boolean): Integer;
Begin
  {aCol is an external reference (ie, one-based and relative to the
   start of the addressable area), we need the internal value (ie.
   zero-based and absolute)}
  If aAbsolute Then
    Result := aCol - 1
  Else
    Result := aCol - 1 + FDisplayOriginCol;
End;

Function TIpTerminalBuffer.tbCvtToInternalRow(aRow: Integer;
  aAbsolute: Boolean): Integer;
Begin
  {aRow is an external reference (ie, one-based and relative to the
   start of the addressable area), we need the internal value (ie.
   zero-based and absolute)}
  If aAbsolute Then
    Result := aRow - 1 + (SVRowCount - RowCount)
  Else
    Result := aRow - 1 + FDisplayOriginRow;
End;

Procedure TIpTerminalBuffer.tbFireOnCursorMovedEvent; {!!.01}
Begin {!!.01}
  If (assigned(FOnCursorMoved)) And (FCursorMoved) Then {!!.01}
    FOnCursorMoved(Self, Row, Col); {!!.01}
End; {!!.01}

Function TIpTerminalBuffer.tbGetCol: Integer;
Begin
  Result := tbCvtToExternalCol(FCursorCol, UseAbsAddress);
End;

Function TIpTerminalBuffer.tbGetOriginCol: Integer;
Begin
  Result := tbCvtToExternalCol(FDisplayOriginCol, True);
End;

Function TIpTerminalBuffer.tbGetOriginRow: Integer;
Begin
  Result := tbCvtToExternalRow(FDisplayOriginRow, True);
End;

Function TIpTerminalBuffer.tbGetRow: Integer;
Begin
  Result := tbCvtToExternalRow(FCursorRow, UseAbsAddress);
End;

Procedure TIpTerminalBuffer.tbInvalidateRect(aFromRow, aFromCol,
  aToRow, aToCol: Integer);
Var
  OurRect: TRect;
Begin
  {convert the row and column values to external values}
  OurRect.Left := aFromCol + 1;
  OurRect.Top := aFromRow + RowCount - SVRowCount + 1;
  OurRect.Right := aToCol + 1;
  OurRect.Bottom := aToRow + RowCount - SVRowCount + 1;
  IPTIpdInvalidRect(PInvRect(FInvRectList), OurRect);
End;

Procedure TIpTerminalBuffer.tbMoveCursorLeftRight(aDirection: Integer;
  aWrap: Boolean;
  aScroll: Boolean);
Var
  MaxCol: Integer;
  MaxRow: Integer;
  StartRow: Integer;
Begin
  { if wrap is off, we just advance or retard the cursor position by }
  { one without extending beyond the left or right margin; scrolling is }
  { not possible in this case }
  MaxCol := FDisplayOriginCol + FDisplayColCount - 1;
  If Not aWrap Then
  Begin
    If (aDirection < 0) Then
    Begin
      If (FCursorCol > FDisplayOriginCol) Then
      Begin
        FCursorMoved := True;
        dec(FCursorCol);
        tbFireOnCursorMovedEvent; {!!.01}
      End;
    End
    Else
    Begin
      If (FCursorCol < MaxCol) Then
      Begin
        FCursorMoved := True;
        inc(FCursorCol);
        tbFireOnCursorMovedEvent; {!!.01}
      End;
    End;
    Exit;
  End;
  MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
  { otherwise it's a wrap, as it were                                   }
  { there are several cases here                                        }
  {   1. if the new cursor position is within the same line, just move  }
  {      it as above                                                    }
  {   2. if the current cursor position is 0 and we're moving left...   }
  {      a. if we're not on the top display row, move the cursor to     }
  {         the last column of the previous row                         }
  {      b. if aScroll is False and we're on the top row, leave         }
  {         the cursor where it is                                      }
  {      c. if aScroll is True and we're on the top row, scroll the     }
  {         display down and move the cursor to the last column of the  }
  {         new top row                                                 }
  {   3. if the current cursor position is on the last column and we're }
  {      moving right...                                                }
  {      a. if we're not on the bottom display row, move the cursor to  }
  {         the first column of the next row                            }
  {      b. if aScroll is False and we're on the bottom row, leave the  }
  {         cursor where it is                                          }
  {      c. if aScroll is True and we're on the bottom, scroll the      }
  {         display up and move the cursor to the first column of the   }
  {         new bottom row                                              }
  If (aDirection < 0) Then
  Begin { moving left }
    If (FCursorCol > FDisplayOriginCol) Then
    Begin
      FCursorMoved := True;
      dec(FCursorCol);
      tbFireOnCursorMovedEvent; {!!.01}
    End
    Else
      If (FCursorRow > FDisplayOriginRow) Then
      Begin
        FCursorMoved := True;
        FCursorCol := MaxCol;
        dec(FCursorRow);
        tbFireOnCursorMovedEvent; {!!.01}
      End
      Else
        If aScroll Then
        Begin
          tbScrollRows(-1, FDisplayOriginRow, MaxRow);
          tbInvalidateRect(FDisplayOriginRow, 0, MaxRow, pred(ColCount));
          FCursorMoved := True;
          FCursorCol := MaxCol;
          tbFireOnCursorMovedEvent; {!!.01}
        End;
  End
  Else {Direction > 0}
  Begin {moving right}
    If (FCursorCol < MaxCol) Then
    Begin
      FCursorMoved := True;
      inc(FCursorCol);
      tbFireOnCursorMovedEvent; {!!.01}
    End
    Else
      If (FCursorRow < MaxRow) Then
      Begin
        FCursorMoved := True;
        FCursorCol := FDisplayOriginCol;
        inc(FCursorRow);
        tbFireOnCursorMovedEvent; {!!.01}
      End
      Else
        If aScroll Then
        Begin
          If UseScrollRegion Then
            StartRow := FDisplayOriginRow
          Else
            StartRow := 0;
          tbScrollRows(1, StartRow, MaxRow);
          tbInvalidateRect(FDisplayOriginRow, 0, MaxRow, pred(ColCount));
          FCursorMoved := True;
          FCursorCol := FDisplayOriginCol;
          tbFireOnCursorMovedEvent; {!!.01}
        End;
  End;
End;

Procedure TIpTerminalBuffer.tbMoveCursorUpDown(aDirection: Integer;
  aScroll: Boolean);
Var
  MaxRow: Integer;
  StartRow: Integer;
Begin
  MaxRow := FDisplayOriginRow + FDisplayRowCount - 1;
  If (aDirection < 0) Then
  Begin
    If (FCursorRow > FDisplayOriginRow) Then
    Begin
      FCursorMoved := True;
      dec(FCursorRow);
      tbFireOnCursorMovedEvent; {!!.01}
    End
    Else
      If aScroll Then
      Begin
        tbScrollRows(-1, FDisplayOriginRow, MaxRow);
        tbInvalidateRect(FDisplayOriginRow, 0, MaxRow, pred(ColCount));
      End;
  End
  Else {Direction > 0}
  Begin
    If (FCursorRow < MaxRow) Then
    Begin
      FCursorMoved := True;
      inc(FCursorRow);
      tbFireOnCursorMovedEvent; {!!.01}
    End
    Else
      If aScroll Then
      Begin
        If UseScrollRegion Then
          StartRow := FDisplayOriginRow
        Else
          StartRow := 0;
        tbScrollRows(1, StartRow, MaxRow);
        tbInvalidateRect(FDisplayOriginRow, 0, MaxRow, pred(ColCount));
      End;
  End;
End;

Procedure TIpTerminalBuffer.tbReallocBuffers(aNewRowCount: Integer;
  aNewColCount: Integer);
Begin
  {check for changes in row count}
  If (aNewRowCount <> SVRowCount) Then
  Begin
    FCharMatrix.RowCount := aNewRowCount;
    FCharSetMatrix.RowCount := aNewRowCount;
    FAttrMatrix.RowCount := aNewRowCount;
    FForeColorMatrix.RowCount := aNewRowCount;
    FBackColorMatrix.RowCount := aNewRowCount;
    FSVRowCount := aNewRowCount;
  End
    {otherwise it's a change in column count}
  Else
  Begin
    FCharMatrix.ColCount := aNewColCount;
    FCharSetMatrix.ColCount := aNewColCount;
    FAttrMatrix.ColCount := aNewColCount;
    FForeColorMatrix.ColCount := aNewColCount;
    FBackColorMatrix.ColCount := aNewColCount;
    FColCount := aNewColCount;
  End;
End;

Procedure TIpTerminalBuffer.tbScrollRows(aCount, aTop, aBottom: Integer);
Begin
  FCharMatrix.ScrollRows(aCount, aTop, aBottom);
  FCharSetMatrix.ScrollRows(aCount, aTop, aBottom);
  FAttrMatrix.ScrollRows(aCount, aTop, aBottom);
  FForeColorMatrix.ScrollRows(aCount, aTop, aBottom);
  FBackColorMatrix.ScrollRows(aCount, aTop, aBottom);
  If Assigned(FOnScrollRows) Then
    FOnScrollRows(Self, aCount,
      aTop + 1 - (SVRowCount - RowCount),
      aBottom + 1 - (SVRowCount - RowCount));
End;

Procedure TIpTerminalBuffer.tbSetBackColor(aValue: TColor);
Begin
  If (aValue <> BackColor) Then
  Begin
    FBackColor := aValue;
    FBackColorMatrix.SetDefaultItem(@FBackColor);
  End;
End;

Procedure TIpTerminalBuffer.tbSetCharSet(aValue: Byte);
Begin
  If (aValue <> CharSet) Then
  Begin
    FCharSet := aValue;
    FCharSetMatrix.SetDefaultItem(@FCharSet);
  End;
End;

Procedure TIpTerminalBuffer.tbSetCol(aCol: Integer);
Var
  OurCol: Integer;
Begin
  FBeyondMargin := False;
  If (aCol <> Col) Then
  Begin
    OurCol := tbCvtToInternalCol(aCol, UseAbsAddress);
    If (OurCol < 0) Then {!!.01}
      OurCol := 0 {!!.01}
    Else
      If (OurCol >= ColCount) Then {!!.01}
        OurCol := pred(ColCount); {!!.01}
    FCursorMoved := True;
    FCursorCol := OurCol;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.tbSetColCount(aNewCount: Integer);
Begin
  {only do something if the value changes}
  If (aNewCount <> ColCount) Then
  Begin
    {check the new value is sensible}
    If (aNewCount < 2) Then
      Raise Exception.Create(Format(SCountTooSmall,
        ['TIpTerminalBuffer.tbSetColCount']));
    {reallocate the tab positions bitset}
    FHorzTabStops :=
      IPTReallocBitset(FHorzTabStops, ColCount, aNewCount);
    {if the number of scrollback rows is zero, just make a note of the
     new value: there won't have been any allocations yet}
    If (SVRowCount = 0) Then
    Begin
      FColCount := aNewCount;
      FDisplayColCount := aNewCount;
    End
      {otherwise we can allocate new buffers and transfer over the old}
    Else
    Begin
      tbReallocBuffers(SVRowCount, aNewCount);
      tbInvalidateRect(SVRowCount - RowCount, 0,
        pred(SVRowCount), pred(aNewCount));
      If UseScrollRegion Then
        UseScrollRegion := False
      Else
      Begin
        FCursorRow := SVRowCount - RowCount;
        FCursorCol := 0;
        FDisplayColCount := aNewCount;
      End;
    End;
  End;
End;

Procedure TIpTerminalBuffer.tbSetDefAnsiChar(aValue: AnsiChar);
Begin
  If (aValue <> DefAnsiChar) Then
  Begin
    FDefAnsiChar := aValue;
    FCharMatrix.SetDefaultItem(@FDefAnsiChar);
  End;
End;

Procedure TIpTerminalBuffer.tbSetDefBackColor(aValue: TColor);
Begin
  If (FDefBackColor <> aValue) Then
  Begin
    FBackColorMatrix.ReplaceItems(@FDefBackColor, @aValue);
    FDefBackColor := aValue;
  End;
End;

Procedure TIpTerminalBuffer.tbSetDefForeColor(aValue: TColor);
Begin
  If (FDefForeColor <> aValue) Then
  Begin
    FForeColorMatrix.ReplaceItems(@FDefForeColor, @aValue);
    FDefForeColor := aValue;
  End;
End;

Procedure TIpTerminalBuffer.tbSetForeColor(aValue: TColor);
Begin
  If (aValue <> ForeColor) Then
  Begin
    FForeColor := aValue;
    FForeColorMatrix.SetDefaultItem(@FForeColor);
  End;
End;

Procedure TIpTerminalBuffer.tbSetRow(aRow: Integer);
Var
  OurRow: Integer;
Begin
  FBeyondMargin := False;
  If (aRow <> Row) Then
  Begin
    OurRow := tbCvtToInternalRow(aRow, UseAbsAddress);
    If (OurRow < 0) Then {!!.01}
      OurRow := 0 {!!.01}
    Else
      If (OurRow >= SVRowCount) Then {!!.01}
        OurRow := pred(SVRowCount); {!!.01}
    FCursorMoved := True;
    FCursorRow := OurRow;
    tbFireOnCursorMovedEvent; {!!.01}
  End;
End;

Procedure TIpTerminalBuffer.tbSetRowCount(aNewCount: Integer);
Begin
  { only do something if the value changes, and changes to something }
  { not greater than the scrollback view count }
  If (aNewCount <> RowCount) And
    (aNewCount <= SVRowCount) Then
  Begin
    { check the new value is sensible }
    If (aNewCount < 2) Then
      Raise Exception.Create(Format(SCountTooSmall,
        ['TIpTerminalBuffer.tbSetRowCount']));
    { changing the row count resets the scroll region }
    If UseScrollRegion Then
      UseScrollRegion := False;
    { reallocate the tab positions bitset }
    FVertTabStops :=
      IPTReallocBitset(FVertTabStops, RowCount, aNewCount);
    { set the display origin and the cursor position }
    FDisplayOriginRow := FSVRowCount - aNewCount;
    If (FCursorRow < FDisplayOriginRow) Then
      FCursorRow := FDisplayOriginRow;
    { save the new value }
    FRowCount := aNewCount;
    FDisplayRowCount := aNewCount;
  End;
End;

Procedure TIpTerminalBuffer.tbSetSVRowCount(aNewCount: Integer);
Begin
  { only do something if the value changes }
  If (aNewCount <> SVRowCount) Then
  Begin
    { check the new value is sensible }
    If (aNewCount < 2) Then
      Raise Exception.Create(Format(SCountTooSmall, ['TIpTerminalBuffer.tbSetSVRowCount']));
    { if the new scrollback view count is less than the display view }
    { count, reduce the display view count to match }
    If (aNewCount < RowCount) Then
    Begin
      FRowCount := aNewCount;
      FDisplayRowCount := aNewCount;
    End;
    {set the display origin and the cursor position}
    FDisplayOriginRow := aNewCount - RowCount;
    FCursorRow := FCursorRow - SVRowCount + aNewCount;
    { if the number of columns is zero, just make a note of the new }
    { value: there won't have been any allocations yet }
    If (ColCount = 0) Then
      FSVRowCount := aNewCount
        { otherwise we can allocate new buffers and transfer over the old }
    Else
    Begin
      tbReallocBuffers(aNewCount, ColCount);
      tbInvalidateRect(SVRowCount - RowCount, 0,
        pred(SVRowCount), pred(aNewCount));
    End;
  End;
End;

Procedure TIpTerminalBuffer.tbSetUseScrollRegion(aValue: Boolean);
Begin
  If (aValue <> UseScrollRegion) Then
  Begin
    {calculate the limits beyond which the cursor cannot move}
    If aValue {limit to scroll region} Then
    Begin
      FDisplayOriginCol := 0;
      FDisplayOriginRow := FSRStartRow;
      FDisplayColCount := ColCount;
      FDisplayRowCount := FSREndRow - FSRStartRow + 1;
    End
    Else {limit to full display area}
    Begin
      FDisplayOriginCol := 0;
      FDisplayOriginRow := SVRowCount - RowCount;
      FDisplayColCount := ColCount;
      FDisplayRowCount := RowCount;
    End;
    {rest the cursor to the top left corner of the allowed region}
    FCursorMoved := True;
    FCursorCol := FDisplayOriginCol;
    FCursorRow := FDisplayOriginRow;
    tbFireOnCursorMovedEvent; {!!.01}
    {save the property value}
    FUseScrollRegion := aValue;
  End;
End;

Procedure TIpTerminalBuffer.WriteChar(aCh: char);
Begin
  {this is performed as
    - write the character to the current cursor
    - write the current attributes, colors and charset to the current
      cursor
    - advance the cursor
   the latter operation may not do anything if UseAutoWrap is off and
   the cursor is at the end of the line, otherwise, if it's on, a
   scroll will occur}
  If FBeyondMargin Then
  Begin
    MoveCursorRight(UseAutoWrap, False);
    FBeyondMargin := False;
  End;
  If UseInsertMode Then
  Begin
    FCharMatrix.InsertItems(1, FCursorRow, FCursorCol);
    FCharSetMatrix.InsertItems(1, FCursorRow, FCursorCol);
    FAttrMatrix.InsertItems(1, FCursorRow, FCursorCol);
    FForeColorMatrix.InsertItems(1, FCursorRow, FCursorCol);
    FBackColorMatrix.InsertItems(1, FCursorRow, FCursorCol);
  End;
  FCharMatrix.WriteItems(@aCh, 1, FCursorRow, FCursorCol);
  FCharSetMatrix.WriteItems(@FCharSet, 1, FCursorRow, FCursorCol);
  FAttrMatrix.WriteItems(@FAttr, 1, FCursorRow, FCursorCol);
  FForeColorMatrix.WriteItems(@FForeColor, 1, FCursorRow, FCursorCol);
  FBackColorMatrix.WriteItems(@FBackColor, 1, FCursorRow, FCursorCol);
  tbInvalidateRect(FCursorRow, FCursorCol,
    FCursorRow, FCursorCol);
  If (Not UseAutoWrapDelay) Or (Not tbAtLastColumn) Then
    MoveCursorRight(UseAutoWrap, False)
  Else
    FBeyondMargin := True;
End;

Procedure TIpTerminalBuffer.WriteString(Const aSt: String);
Var
  i: Integer;
Begin
  For i := 1 To length(aSt) Do
  Begin
    WriteChar(aSt[i]);
  End;
End;

Const
  {The hash table sizes: these are prime numbers that suit these
   particular implementations}
  KBHashTableSize = 57; {keyboard mapping hash table size}
  CSHashTableSize = 199; {charset mapping hash table size}

  OurSignature: Longint = $33544841;
  { Note: $33544841 = AHT3 = APRO Hash Table, version 3 }
  { Use the same sig for IPRO }

Type
  PKBHashNode = ^TKBHashNode; {hash table node for keyboard maps}
  TKBHashNode = Packed Record
    kbnNext: PKBHashNode;
    kbnKey: PIpKeyString;
    kbnValue: PIpKeyString;
  End;

Type
  PCSHashNode = ^TCSHashNode; {hash table node for charset maps}
  TCSHashNode = Packed Record
    csnNext: PCSHashNode;
    csnCharSet: PIpKeyString;
    csnFont: PIpKeyString;
    csnChar: AnsiChar;
    csnGlyph: AnsiChar;
  End;

  PScriptNode = ^TScriptNode;
  TScriptNode = Packed Record
    snNext: PScriptNode;
    snFont: PIpKeyString;
    snText: PAnsiChar;
  End;

Const
  CharQueueDelta = 32;
Type
  TCharQueue = Class
  Private
    FSize: Longint;
    FLen: Longint;
    FText: PAnsiChar;
  Protected
    Function cqGetDupText: PAnsiChar;
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Procedure Add(aCh: AnsiChar);
    Procedure Clear;
    Property DupText: PAnsiChar Read cqGetDupText;
  End;

Constructor TCharQueue.Create;
Begin
  Inherited Create;
  {allocate a starter character queue}
  GetMem(FText, CharQueueDelta);
  FSize := CharQueueDelta;
  FText[0] := #0;
End;

Destructor TCharQueue.Destroy;
Begin
  If (FText <> Nil) Then
    FreeMem(FText, FSize);
  Inherited Destroy;
End;

Procedure TCharQueue.Add(aCh: AnsiChar);
Var
  NewQ: PAnsiChar;
Begin
  If (FLen = FSize - 1) Then
  Begin
    GetMem(NewQ, FSize + CharQueueDelta);
    StrCopy(NewQ, FText);
    FreeMem(FText, FSize);
    inc(FSize, CharQueueDelta);
    FText := NewQ;
  End;
  FText[FLen] := aCh;
  inc(FLen);
  FText[FLen] := #0;
End;

Procedure TCharQueue.Clear;
Begin
  FLen := 0;
  FText[0] := #0;
End;

Function TCharQueue.cqGetDupText: PAnsiChar;
Begin
  GetMem(Result, FLen + 1);
  StrCopy(Result, FText);
End;
{===Helper routines==================================================}
{Note: The ELF hash functions are described in "Practical Algorithms
       For Programmers" by Andrew Binstock and John Rex, Addison
       Wesley, with modifications in Dr Dobbs Journal, April 1996.
       They're modified to suit this implementation.}

Function HashELF(Const S: TIpKeyString): Longint;
Var
  G: Longint;
  i: Integer;
Begin
  Result := 0;
  For i := 1 To length(S) Do
  Begin
    Result := (Result Shl 4) + ord(S[i]);
    G := Result And Longint($F0000000);
    If (G <> 0) Then
      Result := Result Xor (G Shr 24);
    Result := Result And (Not G);
  End;
End;

Function HashELFPlusChar(Const S: TIpKeyString;
  C: AnsiChar): Longint;
Var
  G: Longint;
  i: Integer;
Begin
  Result := ord(C);
  G := Result And Longint($F0000000);
  If (G <> 0) Then
    Result := Result Xor (G Shr 24);
  Result := Result And (Not G);
  For i := 1 To length(S) Do
  Begin
    Result := (Result Shl 4) + ord(S[i]);
    G := Result And Longint($F0000000);
    If (G <> 0) Then
      Result := Result Xor (G Shr 24);
    Result := Result And (Not G);
  End;
End;

Function AllocKeyString(Const aSt: TIpKeyString): PIpKeyString;
Begin
  GetMem(Result, succ(length(aSt)));
  Result^ := aSt;
End;

Procedure FreeKeyString(aKS: PIpKeyString);
Begin
  If (aKS <> Nil) Then
    FreeMem(aKS, succ(length(aKS^)));
End;

Function ProcessCharSetLine(Const aLine: ShortString;
  Var aCharSet: TIpKeyString;
  Var aFromCh: AnsiChar;
  Var aToCh: AnsiChar;
  Var aFontName: TIpKeyString;
  Var aGlyph: AnsiChar): Boolean;
Var
  InWord: Boolean;
  CharInx: Integer;
  StartCh: Integer;
  QuoteMark: AnsiChar;
  WordStart: Array[0..4] Of Integer;
  WordEnd: Array[0..4] Of Integer;
  WordCount: Integer;
  WordLen: Integer;
  Chars: Array[0..4] Of AnsiChar;
  i: Integer;
  AsciiCh: Integer;
  ec: Integer;
  TestSt: String[3];
Begin
  {assumption: the line has had trailing spaces stripped, the line is
   not the empty string, the line starts with a ' ' character

  {assume we'll fail to parse the line properly}
  Result := False;

  {extract out the 5 words; if there are not at least 5 words, exit}
  QuoteMark := ' '; {needed to fool the compiler}
  StartCh := 0; {needed to fool the compiler}
  InWord := False;
  WordCount := 0;
  CharInx := 1;
  While CharInx <= length(aLine) Do
  Begin
    If InWord Then
    Begin
      If (QuoteMark = ' ') Then
      Begin
        If (aLine[CharInx] = ' ') Then
        Begin
          InWord := False;
          WordStart[WordCount] := StartCh;
          WordEnd[WordCount] := pred(CharInx);
          inc(WordCount);
          If (WordCount = 5) Then
            Break;
        End
      End
      Else {the quotemark is active}
      Begin
        If (aLine[CharInx] = QuoteMark) Then
          QuoteMark := ' ';
      End;
    End
    Else {not in a word}
    Begin
      If (aLine[CharInx] <> ' ') Then
      Begin
        InWord := True;
        StartCh := CharInx;
        QuoteMark := aLine[CharInx];
        If (QuoteMark <> '''') And (QuoteMark <> '"') Then
          QuoteMark := ' ';
      End;
    End;
    inc(CharInx);
  End;
  {when we reach this point we know where the last word ended}
  If InWord Then
  Begin
    If (QuoteMark <> ' ') Then
      Exit; {the last word had no close quote}
    WordStart[WordCount] := StartCh;
    WordEnd[WordCount] := pred(CharInx);
    inc(WordCount);
  End;
  If (WordCount <> 5) Then
    Exit;
  {fix the quoted strings}
  For i := 0 To 4 Do
  Begin
    If (aLine[WordStart[i]] = '''') Or
      (aLine[WordStart[i]] = '"') Then
    Begin
      inc(WordStart[i]);
      dec(WordEnd[i]);
      If (WordEnd[i] < WordStart[i]) Then
        Exit; {the word was either '' or ""}
    End;
  End;
  {we now know where each word can be found; the only special words
   are words 1, 2, and 4 which must be single characters, or ASCII
   values of the form \xnn}
  For i := 1 To 4 Do
    If (i <> 3) Then
    Begin
      WordLen := succ(WordEnd[i] - WordStart[i]);
      If (WordLen = 1) Then
        Chars[i] := aLine[WordStart[i]]
      Else
        If (WordLen = 4) Then
        Begin
          CharInx := WordStart[i];
          If (aLine[CharInx] <> '\') Or
            (aLine[CharInx + 1] <> 'x') Then
            Exit;
          TestSt := Copy(aLine, CharInx + 1, 3);
          TestSt[1] := '$';
          Val(TestSt, AsciiCh, ec);
          If (ec <> 0) Then
            Exit;
          Chars[i] := chr(AsciiCh);
        End
        Else
          Exit; {unknown format}
    End;
  {return values}
  aFromCh := Chars[1];
  aToCh := Chars[2];
  aGlyph := Chars[4];
  aCharSet := Copy(aLine, WordStart[0], succ(WordEnd[0] - WordStart[0]));
  aFontName := Copy(aLine, WordStart[3], succ(WordEnd[3] - WordStart[3]));
  Result := True;
End;

Constructor TIpKeyboardMapping.Create;
Begin
  Inherited Create;
  FTable := TList.Create;
  FTable.Count := KBHashTableSize;
End;

Destructor TIpKeyboardMapping.Destroy;
Begin
  If (FTable <> Nil) Then
  Begin
    Clear;
    FTable.Destroy;
  End;
  Inherited Destroy;
End;

Function TIpKeyboardMapping.Add(Const aKey: TIpKeyString;
  Const aValue: TIpKeyString): Boolean;
Var
  Inx: Integer;
  Node: PKBHashNode;
Begin
  If kbmFindPrim(aKey, Inx, Pointer(Node)) Then
    Result := False
  Else
  Begin
    Result := True;
    New(Node);
    Node^.kbnNext := FTable[Inx];
    Node^.kbnKey := AllocKeyString(aKey);
    Node^.kbnValue := AllocKeyString(aValue);
    FTable[Inx] := Node;
    inc(FCount);
  End;
End;

Procedure TIpKeyboardMapping.Clear;
Var
  i: Integer;
  Node: PKBHashNode;
  Temp: PKBHashNode;
Begin
  For i := 0 To pred(KBHashTableSize) Do
  Begin
    Node := FTable[i];
    While (Node <> Nil) Do
    Begin
      Temp := Node;
      Node := Node^.kbnNext;
      FreeKeyString(Temp^.kbnKey);
      FreeKeyString(Temp^.kbnValue);
      Dispose(Temp);
    End;
    FTable[i] := Nil;
  End;
  FCount := 0;
End;
{$IFDEF CompileDebugCode}

Procedure TIpKeyboardMapping.DebugPrint(Const aFileName: String);
Var
  F: text;
  i: Integer;
  Node: PKBHashNode;
Begin
  System.Assign(F, aFileName);
  System.Rewrite(F);

  For i := 0 To pred(KBHashTableSize) Do
  Begin
    writeln(F, '---', i, '---');
    Node := FTable[i];
    While (Node <> Nil) Do
    Begin
      writeln(F, Node^.kbnKey^: 20, Node^.kbnValue^: 20);
      Node := Node^.kbnNext;
    End;
  End;

  writeln(F);
  writeln(F, 'Count: ', Count, ' (mean: ', Count / CSHashTableSize: 5: 3, ')');

  System.Close(F);
End;
{$ENDIF}

Function TIpKeyboardMapping.Get(Const aKey: TIpKeyString): TIpKeyString;
Var
  Inx: Integer;
  Node: PKBHashNode;
Begin
  If kbmFindPrim(aKey, Inx, Pointer(Node)) Then
    Result := Node^.kbnValue^
  Else
    Result := '';
End;

Function TIpKeyboardMapping.kbmFindPrim(Const aKey: TIpKeyString;
  Var aInx: Integer;
  Var aNode: Pointer): Boolean;
Var
  Node: PKBHashNode;
Begin
  {assume we won't find aKey}
  Result := False;
  aNode := Nil;
  {calculate the index, ie hash, of the key}
  aInx := HashELF(aKey) Mod KBHashTableSize;
  {traverse the linked list at this entry, looking for the key in each
   node we encounter--a case-sensitive comparison}
  Node := FTable[aInx];
  While (Node <> Nil) Do
  Begin
    If (aKey = Node^.kbnKey^) Then
    Begin
      Result := True;
      aNode := Node;
      Exit;
    End;
    Node := Node^.kbnNext;
  End;
End;

Procedure TIpKeyboardMapping.LoadFromFile(Const aFileName: String);
Var
  Lines: TStringList;
  ActualLen: Integer;
  i: Integer;
  LineInx: Integer;
  Word1Start: Integer;
  Word1End: Integer;
  Word2Start: Integer;
  Word2End: Integer;
  LookingForStart: Boolean;
  Line: String[255];
Begin
  {clear the hash table, ready for loading}
  Clear;
  {create the stringlist to hold the keymap script}
  Lines := TStringList.Create;
  Try
    {load the keymap script}
    Lines.LoadFromFile(aFileName);
    For LineInx := 0 To pred(Lines.Count) Do
    Begin
      {get this line}
      Line := Lines[LineInx];
      {remove trailing spaces}
      ActualLen := length(Line);
      For i := ActualLen Downto 1 Do
        If (Line[i] = ' ') Then
          dec(ActualLen)
        Else
          Break;
      Line[0] := chr(ActualLen);
      {only process detail lines}
      If (Line <> '') And (Line[1] <> '*') Then
      Begin
        {identify the first 'word'}
        Word1Start := 0;
        Word1End := 0;
        LookingForStart := True;
        For i := 1 To ActualLen Do
        Begin
          If LookingForStart Then
          Begin
            If (Line[i] <> ' ') Then
            Begin
              Word1Start := i;
              LookingForStart := False;
            End;
          End
          Else {looking for end}
          Begin
            If (Line[i] = ' ') Then
            Begin
              Word1End := i - 1;
              Break;
            End;
          End;
        End;
        {if we've set Word1End then there are at least two words in
         the line, otherwise there was only one word (which we shall
         ignore)}
        If (Word1End <> 0) Then
        Begin
          {identify the second 'word'}
          Word2Start := 0;
          Word2End := 0;
          LookingForStart := True;
          For i := succ(Word1End) To ActualLen Do
          Begin
            If LookingForStart Then
            Begin
              If (Line[i] <> ' ') Then
              Begin
                Word2Start := i;
                LookingForStart := False;
              End;
            End
            Else {looking for end}
            Begin
              If (Line[i] = ' ') Then
              Begin
                Word2End := i - 1;
                Break;
              End;
            End;
          End;
          If (Word2End = 0) Then
            Word2End := ActualLen;
          {add the key and value to the hash table}
          Add(System.Copy(Line, Word1Start, succ(Word1End - Word1Start)),
            System.Copy(Line, Word2Start, succ(Word2End - Word2Start)));
        End;
      End;
    End;
  Finally
    Lines.Free;
  End;
End;

Procedure TIpKeyboardMapping.LoadFromRes(aInstance: THandle;
  Const aResName: String);
Var
  MS: TMemoryStream;
  ResInfo: THandle;
  ResHandle: THandle;
  ResNameZ: PAnsiChar;
  Res: PByteArray;
  i: Integer;
  Sig: Longint;
  ResCount: Longint;
  BytesRead: Longint;
  Key: TIpKeyString;
  Value: TIpKeyString;
Begin
  {Note: this code has been written to work with all versions of
   Delphi, both 16-bit and 32-bit. Hence it does not make use of any
   of the features available in later compilers, ie, typecasting a
   string to a PAnsiChar, or TResourceStream)

  {clear the hash table, ready for loading}
  Clear;
  {get the resource info handle}
  GetMem(ResNameZ, succ(length(aResName)));
  Try
    StrPCopy(ResNameZ, aResName);
    ResInfo := FindResource(aInstance, ResNameZ, RT_RCDATA);
  Finally
    FreeMem(ResNameZ, succ(length(aResName)));
  End;
  If (ResInfo = 0) Then
    Exit;
  {load and lock the resource}
  ResHandle := LoadResource(aInstance, ResInfo);
  If (ResHandle = 0) Then
    Exit;
  Res := LockResource(ResHandle);
  If (Res = Nil) Then
  Begin
    FreeResource(ResHandle);
    Exit;
  End;
  Try
    {create a memory stream}
    MS := TMemoryStream.Create;
    Try
      {copy the resource to our memory stream}
      MS.Write(Res^, SizeOfResource(aInstance, ResInfo));
      MS.Position := 0;
      {read the header signature, get out if it's not ours}
      BytesRead := MS.Read(Sig, sizeof(Sig));
      If (BytesRead <> sizeof(Sig)) Or (Sig <> OurSignature) Then
        Exit;
      {read the count of key/value string pairs in the resource}
      MS.Read(ResCount, sizeof(ResCount));
      {read that number of key/value string pairs and add them to the
       hash table}
      For i := 0 To pred(ResCount) Do
      Begin
        MS.Read(Key[0], 1);
        MS.Read(Key[1], ord(Key[0]));
        MS.Read(Value[0], 1);
        MS.Read(Value[1], ord(Value[0]));
        Add(Key, Value);
      End;
      {read the footer signature, clear and get out if it's not ours}
      BytesRead := MS.Read(Sig, sizeof(Sig));
      If (BytesRead <> sizeof(Sig)) Or (Sig <> OurSignature) Then
      Begin
        Clear;
        Exit;
      End;
    Finally
      MS.Free;
    End;
  Finally
    UnlockResource(ResHandle);
    FreeResource(ResHandle);
  End;
End;

Procedure TIpKeyboardMapping.StoreToBinFile(Const aFileName: String);
Var
  aFS: TFileStream;
  i: Integer;
  Node: PKBHashNode;
Begin
  {create a file stream}
  aFS := TFileStream.Create(aFileName, fmCreate);
  Try
    {write our signature as header}
    aFS.Write(OurSignature, sizeof(OurSignature));
    {write the number of key/value string pairs}
    aFS.Write(FCount, sizeof(FCount));
    {write all the key/value string pairs}
    For i := 0 To pred(KBHashTableSize) Do
    Begin
      Node := FTable[i];
      While (Node <> Nil) Do
      Begin
        aFS.Write(Node^.kbnKey^, succ(length(Node^.kbnKey^)));
        aFS.Write(Node^.kbnValue^, succ(length(Node^.kbnValue^)));
        Node := Node^.kbnNext;
      End;
    End;
    {write our signature as footer as a check}
    aFS.Write(OurSignature, sizeof(OurSignature));
  Finally
    aFS.Free;
  End;
End;

Constructor TIpCharSetMapping.Create;
Begin
  Inherited Create;
  FTable := TList.Create;
  FTable.Count := CSHashTableSize;
  FCharQueue := Pointer(TCharQueue.Create);
End;

Destructor TIpCharSetMapping.Destroy;
Var
  Temp, Walker: PScriptNode;
Begin
  {free the hash table}
  If (FTable <> Nil) Then
  Begin
    Clear;
    FTable.Destroy;
  End;
  {free the character queue}
  TCharQueue(FCharQueue).Free;
  {free the script node freelist}
  Walker := FScriptFreeList;
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.snNext;
    Dispose(Temp);
  End;
  Inherited Destroy;
End;

Function TIpCharSetMapping.Add(Const aCharSet: TIpKeyString;
  aFromCh: AnsiChar;
  aToCh: AnsiChar;
  Const aFont: TIpKeyString;
  aGlyph: AnsiChar): Boolean;
Var
  Inx: Integer;
  Node: PCSHashNode;
  Ch: AnsiChar;
  Glyph: AnsiChar;
Begin
  {we must do this in two stages: first, determine that we can add
   *all* the character mappings; second, do so}

  {stage one: check no mapping currently exists}
  Result := False;
  For Ch := aFromCh To aToCh Do
  Begin
    If csmFindPrim(aCharSet, Ch, Inx, Pointer(Node)) Then
      Exit;
  End;
  {stage two: add all charset/char mappings}
  Result := True;
  Glyph := aGlyph;
  For Ch := aFromCh To aToCh Do
  Begin
    Inx := HashELFPlusChar(aCharSet, Ch) Mod CSHashTableSize;
    New(Node);
    Node^.csnNext := FTable[Inx];
    Node^.csnCharSet := AllocKeyString(aCharSet);
    Node^.csnFont := AllocKeyString(aFont);
    Node^.csnChar := Ch;
    Node^.csnGlyph := Glyph;
    FTable[Inx] := Node;
    inc(FCount);
    inc(Glyph);
  End;
End;

Procedure TIpCharSetMapping.Clear;
Var
  i: Integer;
  Node: PCSHashNode;
  Temp: PCSHashNode;
Begin
  {free the script: in a moment there's going to be no mapping}
  csmFreeScript;
  {clear out the hash table}
  For i := 0 To pred(CSHashTableSize) Do
  Begin
    Node := FTable[i];
    While (Node <> Nil) Do
    Begin
      Temp := Node;
      Node := Node^.csnNext;
      FreeKeyString(Temp^.csnCharSet);
      FreeKeyString(Temp^.csnFont);
      Dispose(Temp);
    End;
    FTable[i] := Nil;
  End;
  FCount := 0;
End;

Procedure TIpCharSetMapping.csmAddScriptNode(aFont: PIpKeyString);
Var
  Node: PScriptNode;
Begin
  {allocate and set up the new node}
  If (FScriptFreeList = Nil) Then
    New(Node)
  Else
  Begin
    Node := FScriptFreeList;
    FScriptFreeList := Node^.snNext;
  End;
  Node^.snNext := Nil;
  Node^.snFont := aFont;
  Node^.snText := TCharQueue(FCharQueue).DupText;
  {add the node to the script}
  If (FScript <> Nil) Then
    PScriptNode(FScriptEnd)^.snNext := Node
  Else
    FScript := Node;
  {update the tail Pointer}
  FScriptEnd := Node;
End;

Function TIpCharSetMapping.csmFindPrim(Const aCharSet: TIpKeyString;
  aChar: AnsiChar;
  Var aInx: Integer;
  Var aNode: Pointer): Boolean;
Var
  Node: PCSHashNode;
Begin
  {assume we won't find aCharSet/aChar}
  Result := False;
  aNode := Nil;
  {calculate the index, ie hash, of the charset/char}
  aInx := HashELFPlusChar(aCharSet, aChar) Mod CSHashTableSize;
  {traverse the linked list at this entry, looking for the character
   in each node we encounter--a case-sensitive comparison--if we get a
   match, compare the character set name as well, again case-
   insensitive}
  Node := FTable[aInx];
  While (Node <> Nil) Do
  Begin
    If (aChar = Node^.csnChar) Then
    Begin
      If (aCharSet = Node^.csnCharSet^) Then
      Begin
        Result := True;
        aNode := Node;
        Exit;
      End;
    End;
    Node := Node^.csnNext;
  End;
End;

Procedure TIpCharSetMapping.csmFreeScript;
Var
  Walker, Temp: PScriptNode;
Begin
  Walker := FScript;
  FScript := Nil;
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.snNext;
    FreeMem(Temp^.snText, StrLen(Temp^.snText));
    {NOTE: we do NOT free the font name: it's a copy of an allocated
     string in the mapping hash table}
    Temp^.snNext := FScriptFreeList;
    FScriptFreeList := Temp;
  End;
End;
{$IFDEF CompileDebugCode}

Procedure TIpCharSetMapping.DebugPrint(Const aFileName: String);
Var
  F: text;
  i: Integer;
  Node: PCSHashNode;
Begin
  System.Assign(F, aFileName);
  System.Rewrite(F);

  For i := 0 To pred(CSHashTableSize) Do
  Begin
    writeln(F, '---', i, '---');
    Node := FTable[i];
    While (Node <> Nil) Do
    Begin
      writeln(F, Node^.csnCharSet^: 20,
        ord(Node^.csnChar): 4,
        Node^.csnFont^: 20,
        ord(Node^.csnGlyph): 4);
      Node := Node^.csnNext;
    End;
  End;

  writeln(F);
  writeln(F, 'Count: ', Count, ' (mean: ', Count / CSHashTableSize: 5: 3, ')');

  System.Close(F);
End;
{$ENDIF}

Procedure TIpCharSetMapping.GenerateDrawScript(Const aCharSet: TIpKeyString;
  aText: PAnsiChar);
Var
  i: Integer;
  Inx: Integer;
  TextLen: Integer;
  Node: PCSHashNode;
  Ch: AnsiChar;
  CurFont: PIpKeyString;
  ThisFont: PIpKeyString;
  ThisChar: AnsiChar;
Begin
  {nothing to do if the string is empty}
  TextLen := StrLen(aText);
  If (TextLen = 0) Then
    Exit;
  {destroy any current script}
  csmFreeScript;
  TCharQueue(FCharQueue).Clear;
  {we don't yet have a font name}
  CurFont := Nil;
  {read the text, char by char}
  For i := 0 To pred(TextLen) Do
  Begin
    {look up this charset/char in the hash table}
    Ch := aText[i];
    If csmFindPrim(aCharSet, Ch, Inx, Pointer(Node)) Then
    Begin
      {found it, use the named font and glyph}
      ThisFont := Node^.csnFont;
      ThisChar := Node^.csnGlyph;
    End
    Else
    Begin
      {if not found, use the default font and glyph}
      ThisFont := @DefaultFontName;
      ThisChar := Ch;
    End;
    {if the font has changed, create a script node for the previous
     font}
    If (CurFont = Nil) Then
      CurFont := ThisFont;
    If (CurFont^ <> ThisFont^) Then
    Begin
      csmAddScriptNode(CurFont);
      CurFont := ThisFont;
      TCharQueue(FCharQueue).Clear;
    End;
    {add this character to the current string}
    TCharQueue(FCharQueue).Add(ThisChar);
  End;
  {add the final script node to finish off the string}
  csmAddScriptNode(CurFont);
  TCharQueue(FCharQueue).Clear;
End;

Procedure TIpCharSetMapping.GetFontNames(aList: TStrings);
Var
  i: Integer;
  Node: PCSHashNode;
  PrevFont: String;
Begin
  aList.Clear;
  PrevFont := '';
  For i := 0 To pred(CSHashTableSize) Do
  Begin
    Node := FTable[i];
    While (Node <> Nil) Do
    Begin
      If (CompareText(Node^.csnFont^, PrevFont) <> 0) Then
      Begin
        PrevFont := Node^.csnFont^;
        If (aList.IndexOf(PrevFont) = -1) Then
          aList.Add(PrevFont);
      End;
      Node := Node^.csnNext;
    End;
  End;
End;

Function TIpCharSetMapping.GetNextDrawCommand(Var aFont: TIpKeyString;
  aText: PAnsiChar): Boolean;
Var
  Temp: PScriptNode;
Begin
  {start off with the obvious case: there's no script}
  If (FScript = Nil) Then
  Begin
    Result := False;
    Exit;
  End;
  {we'll definitely return something}
  Result := True;
  {return the data from the top node}
  aFont := PScriptNode(FScript)^.snFont^;
  StrCopy(aText, PScriptNode(FScript)^.snText);
  {unlink the top node}
  Temp := PScriptNode(FScript);
  FScript := Temp^.snNext;
  {free the unlinked top node}
  Try
    FreeMem(Temp^.snText, StrLen(Temp^.snText));
  Except End; //zzz rlm
  {NOTE: we do NOT free the font name: it's a copy of an allocated
   string in the mapping hash table}
  Temp^.snNext := FScriptFreeList;
  FScriptFreeList := Temp;
End;

Procedure TIpCharSetMapping.LoadFromFile(Const aFileName: String);
Var
  Lines: TStringList;
  ActualLen: Integer;
  i: Integer;
  LineInx: Integer;
  Line: String[255];
  CharSet: TIpKeyString;
  FontName: TIpKeyString;
  FromCh: AnsiChar;
  ToCh: AnsiChar;
  Glyph: AnsiChar;
Begin
  {clear the hash table, ready for loading}
  Clear;
  {create the stringlist to hold the mapping script}
  Lines := TStringList.Create;
  Try
    {load the mapping script}
    Lines.LoadFromFile(aFileName);
    For LineInx := 0 To pred(Lines.Count) Do
    Begin
      {get this line}
      Line := Lines[LineInx];
      {remove trailing spaces}
      ActualLen := length(Line);
      For i := ActualLen Downto 1 Do
        If (Line[i] = ' ') Then
          dec(ActualLen)
        Else
          Break;
      Line[0] := chr(ActualLen);
      {only process detail lines}
      If (Line <> '') And (Line[1] = ' ') Then
      Begin
        If ProcessCharSetLine(Line, CharSet, FromCh, ToCh, FontName, Glyph) Then
          Add(CharSet, FromCh, ToCh, FontName, Glyph);
      End;
    End;
  Finally
    Lines.Free;
  End;
End;

Procedure TIpCharSetMapping.LoadFromRes(aInstance: THandle;
  Const aResName: String);
Var
  MS: TMemoryStream;
  ResInfo: THandle;
  ResHandle: THandle;
  ResNameZ: PAnsiChar;
  Res: PByteArray;
  i: Integer;
  Sig: Longint;
  ResCount: Longint;
  BytesRead: Longint;
  CharSet: TIpKeyString;
  Font: TIpKeyString;
  Ch: AnsiChar;
  Glyph: AnsiChar;
Begin
  {Note: this code has been written to work with all versions of
   Delphi, both 16-bit and 32-bit. Hence it does not make use of any
   of the features available in later compilers, ie, typecasting a
   string to a PChar, or TResourceStream)

  {clear the hash table, ready for loading}
  Clear;
  {get the resource info handle}
  GetMem(ResNameZ, succ(length(aResName)));
  Try
    StrPCopy(ResNameZ, aResName);
    ResInfo := FindResource(aInstance, ResNameZ, RT_RCDATA);
  Finally
    FreeMem(ResNameZ, succ(length(aResName)));
  End;
  If (ResInfo = 0) Then
    Exit;
  {load and lock the resource}
  ResHandle := LoadResource(aInstance, ResInfo);
  If (ResHandle = 0) Then
    Exit;
  Res := LockResource(ResHandle);
  If (Res = Nil) Then
  Begin
    FreeResource(ResHandle);
    Exit;
  End;
  Try
    {create a memory stream}
    MS := TMemoryStream.Create;
    Try
      {copy the resource to our memory stream}
      MS.Write(Res^, SizeOfResource(aInstance, ResInfo));
      MS.Position := 0;
      {read the header signature, get out if it's not ours}
      BytesRead := MS.Read(Sig, sizeof(Sig));
      If (BytesRead <> sizeof(Sig)) Or (Sig <> OurSignature) Then
        Exit;
      {read the count of mappings in the resource}
      MS.Read(ResCount, sizeof(ResCount));
      {read that number of mappings and add them to the hash table}
      For i := 0 To pred(ResCount) Do
      Begin
        MS.Read(CharSet[0], 1);
        MS.Read(CharSet[1], ord(CharSet[0]));
        MS.Read(Font[0], 1);
        MS.Read(Font[1], ord(Font[0]));
        MS.Read(Ch, 1);
        MS.Read(Glyph, 1);
        Add(CharSet, Ch, Ch, Font, Glyph);
      End;
      {read the footer signature, clear and get out if it's not ours}
      BytesRead := MS.Read(Sig, sizeof(Sig));
      If (BytesRead <> sizeof(Sig)) Or (Sig <> OurSignature) Then
      Begin
        Clear;
        Exit;
      End;
    Finally
      MS.Free;
    End;
  Finally
    UnlockResource(ResHandle);
    FreeResource(ResHandle);
  End;
End;

Procedure TIpCharSetMapping.StoreToBinFile(Const aFileName: String);
Var
  aFS: TFileStream;
  i: Integer;
  Node: PCSHashNode;
Begin
  {create a file stream}
  aFS := TFileStream.Create(aFileName, fmCreate);
  Try
    {write our signature as header}
    aFS.Write(OurSignature, sizeof(OurSignature));
    {write the number of mappings}
    aFS.Write(FCount, sizeof(FCount));
    {write all the mappings}
    For i := 0 To pred(CSHashTableSize) Do
    Begin
      Node := FTable[i];
      While (Node <> Nil) Do
      Begin
        aFS.Write(Node^.csnCharSet^, succ(length(Node^.csnCharSet^)));
        aFS.Write(Node^.csnFont^, succ(length(Node^.csnFont^)));
        aFS.Write(Node^.csnChar, sizeof(AnsiChar));
        aFS.Write(Node^.csnGlyph, sizeof(AnsiChar));
        Node := Node^.csnNext;
      End;
    End;
    {write our signature as footer as a further check on reading}
    aFS.Write(OurSignature, sizeof(OurSignature));
  Finally
    aFS.Free;
  End;
End;

Constructor TIpTerminalParser.Create(aUseWideChar: Boolean);
Begin
  Inherited Create;
  FUseWideChar := aUseWideChar;
  FCommand := eNone;
End;

Destructor TIpTerminalParser.Destroy;
Begin
  Inherited Destroy;
End;

Procedure TIpTerminalParser.Clear;
Begin
  {do nothing at this level}
End;

Function TIpTerminalParser.ProcessChar(aCh: AnsiChar): TIpParserCmdType;
Begin
  Result := pctNone;
End;

Function TIpTerminalParser.ProcessWideChar(aCh: WideChar): TIpParserCmdType;
Begin
  Result := pctNone;
End;

Function TIpTerminalParser.tpGetArgument(aInx: Integer): Integer;
Begin
  Result := 0;
End;

Function TIpTerminalParser.tpGetSequence: String;
Begin
  Result := '';
End;

Type
  PSeq = ^TSeq;
  TSeq = Packed Record
    sSize: Longint;
    sLen: Longint;
    sText: Array[1..10000] Of AnsiChar;
  End;

Function ReAllocSeq(aSeq: PSeq; aSize: Longint): PSeq;
Var
  NewSeq: PSeq;
Begin
  If (aSize = 0) Then
    NewSeq := Nil
  Else
  Begin
    GetMem(NewSeq, 2 * sizeof(Longint) + aSize);
    NewSeq^.sSize := aSize;
    NewSeq^.sLen := 0;
    If (aSeq <> Nil) Then
    Begin
      Move(aSeq^.sText, NewSeq^.sText, aSeq^.sLen);
      NewSeq^.sLen := aSeq^.sLen;
    End;
  End;
  If (aSeq <> Nil) Then
    FreeMem(aSeq, 2 * sizeof(Longint) + aSeq^.sSize);
  Result := NewSeq;
End;

Procedure AddCharToSeq(Var aSeq: PSeq; aCh: AnsiChar);
Begin
  If (aSeq = Nil) Then
    aSeq := ReAllocSeq(aSeq, 64)
  Else
    If (aSeq^.sSize = aSeq^.sLen) Then
      aSeq := ReAllocSeq(aSeq, aSeq^.sSize + 64);
  inc(aSeq^.sLen);
  aSeq^.sText[aSeq^.sLen] := aCh;
End;

Procedure AssignSeqToChar(Var aSeq: PSeq; aCh: AnsiChar);
Begin
  If (aSeq <> Nil) Then
    aSeq^.sLen := 0;
  AddCharToSeq(aSeq, aCh);
End;

Procedure CopySeq(aFromSeq: PSeq; Var aToSeq: PSeq);
Begin
  If (aFromSeq = Nil) Then
  Begin
    If (aToSeq <> Nil) Then
      aToSeq^.sLen := 0;
  End
  Else
  Begin
    If (aToSeq = Nil) Or
      (aToSeq^.sSize < aFromSeq^.sLen) Then
      aToSeq := ReAllocSeq(aToSeq, aFromSeq^.sLen);
    If (aToSeq <> Nil) Then
    Begin
      aToSeq^.sLen := aFromSeq^.sLen;
      Move(aFromSeq^.sText, aToSeq^.sText, aFromSeq^.sLen);
    End;
  End;
End;

Procedure DelCharFromSeq(aSeq: PSeq);
Begin
  If (aSeq <> Nil) And (aSeq^.sLen > 0) Then
    dec(aSeq^.sLen);
End;

Procedure ClearSeq(aSeq: PSeq);
Begin
  If (aSeq <> Nil) Then
    aSeq^.sLen := 0;
End;

Function GetSeqLength(aSeq: PSeq): Integer;
Begin
  Result := aSeq^.sLen;
End;

Function GetStringFromSeq(aSeq: PSeq): String;
Begin
  Result := '';
  If (aSeq <> Nil) And (aSeq^.sLen > 0) Then
  Begin
    SetLength(Result, aSeq^.sLen);
    Move(aSeq^.sText, Result[1], aSeq^.sLen);
  End;
End;

Const
  DECSCLseq: String[6] = ^['[61"p';

Constructor TIpVT100Parser.Create(aUseWideChar: Boolean);
Begin
  Inherited Create(aUseWideChar);
  FArgCount := 0;
  vtpGrowArgs;
  FInVT52Mode := False;
End;

Destructor TIpVT100Parser.Destroy;
Begin
  FSequence := ReAllocSeq(FSequence, 0);
  FSavedSeq := ReAllocSeq(FSavedSeq, 0);
  If (FArgs <> Nil) Then
  Begin
    FreeMem(FArgs, sizeof(Integer) * FArgCountMax);
    FArgs := Nil;
    FArgCountMax := 0;
  End;
  Inherited Destroy;
End;

Procedure TIpVT100Parser.Clear;
Begin
  ClearSeq(FSequence);
  FCommand := eNone;
  If (FArgCount <> 0) Then
  Begin
    FillChar(FArgs^, sizeof(Integer) * FArgCount, 0);
    FArgCount := 0;
  End;
End;

Function TIpVT100Parser.ProcessChar(aCh: AnsiChar): TIpParserCmdType;
Begin
  {if the current state is psGotCommand, the previous character
   managed to complete a command. Before comtinuing we should clear
   all traces of the previous command and sequence}
  If (FState = psGotCommand) Then
  Begin
    FArgCount := 0;
    ClearSeq(FSequence);
    FCommand := eNone;
    FState := psIdle;
  End;

  {if the current state is psGotInterCommand, the previous character
   was non-displayable and a command; restore the previously saved
   state}
  If (FState = psGotInterCommand) Then
  Begin
    FArgCount := 0;
    FCommand := eNone;
    CopySeq(FSavedSeq, PSeq(FSequence));
    FState := FSavedState;
  End;

  {assume that the result is going to be that we are building up a
   command escape sequence}
  Result := pctPending;

  {add the character to the sequence string we're building up,
   although we may delete it later}
  AddCharToSeq(PSeq(FSequence), aCh);

  {if the character is non-displayable, process it immediately, even
   if we're in the middle of parsing some other command}
  If (aCh < ' ') Then
  Begin
    FSavedState := FState;
    DelCharFromSeq(FSequence);
    CopySeq(FSequence, PSeq(FSavedSeq));
    FState := psGotInterCommand;
    Result := pctComplete;
    Case aCh Of
      cENQ:
        Begin {enquiry request}
          AssignSeqToChar(PSeq(FSequence), cENQ);
          FCommand := eENQ;
        End;
      cBel:
        Begin {sound bell}
          AssignSeqToChar(PSeq(FSequence), cBel);
          FCommand := eBel;
        End;
      cBS:
        Begin {backspace}
          AssignSeqToChar(PSeq(FSequence), cBS);
          FCommand := eBS;
        End;
      cTab:
        Begin {horizontal tab}
          AssignSeqToChar(PSeq(FSequence), cTab);
          FCommand := eCHT;
          FArgCount := 1;
          FArgs^[0] := 1; {ie a single tab}
        End;
      cLF:
        Begin
          AssignSeqToChar(PSeq(FSequence), cLF);
          FCommand := eLF;
        End;
      cVT:
        Begin
          AssignSeqToChar(PSeq(FSequence), cVT);
          FCommand := eCVT;
          FArgCount := 1;
          FArgs^[0] := 1; {ie a single tab}
        End;
      cFF:
        Begin {formfeed, equals clear screen}
          AssignSeqToChar(PSeq(FSequence), cFF);
          FCommand := eED;
          FArgCount := 1;
          FArgs^[0] := 2; {ie <esc>[2J}
        End;
      cCR:
        Begin {carriage return}
          AssignSeqToChar(PSeq(FSequence), cCR);
          FCommand := eCR;
        End;
      cSO:
        Begin {shift out character set, ie use G0}
          AssignSeqToChar(PSeq(FSequence), cSO);
          FCommand := eSO;
        End;
      cSI:
        Begin {shift in character set, ie use G1}
          AssignSeqToChar(PSeq(FSequence), cSI);
          FCommand := eSI;
        End;
      cCan,
        cSub:
        Begin {abandon current escape sequence}
          Result := pctNone;
        End;
      cEsc:
        Begin {start a new escape sequence}
          {abandon whatever escape sequence we're in}
          AssignSeqToChar(PSeq(FSequence), cEsc);
          FArgCount := 0;
          FState := psGotEscape;
          Result := pctPending;
        End;
    Else
        {otherwise ignore the non-displayable char}
      DelCharFromSeq(FSequence);
      Result := pctNone;
    End; {case}
  End
    {otherwise parse the character}
  Else
  Begin
    Case FState Of
      psIdle:
        Begin
          If (aCh < #127) Then
          Begin
            FState := psGotCommand;
            FCommand := eChar;
            Result := pctChar;
          End
          Else {full 8-bit char}
          Begin
            FState := psGotCommand;
            FCommand := eChar;
            Result := pct8bitChar;
          End;
        End;
      psGotEscape:
        If InVT52Mode Then
        Begin
          Result := vtpProcessVT52(aCh);
        End
        Else {in VT100 mode}
        Begin
          Case aCh Of
            '[': FState := psParsingANSI;
            '(': FState := psParsingLeftParen;
            ')': FState := psParsingRightParen;
            '#': FState := psParsingHash;
            '*', '+', '-', '.', '/': FState := psParsingCharSet;
          Else {it's a two character esc. seq.}
            FState := psGotCommand;
            Result := pctComplete;
            Case aCh Of
              '1':
                Begin {set graphics processor option on}
                    { NOT SUPPORTED }
                  FCommand := eNone;
                End;
              '2':
                Begin {set graphics processor option off}
                    { NOT SUPPORTED }
                  FCommand := eNone;
                End;
              '7':
                Begin {save cursor pos}
                  FCommand := eSaveCursorPos;
                End;
              '8':
                Begin {restore cursor pos}
                  FCommand := eRestoreCursorPos;
                End;
              '<':
                Begin {switch to ANSI--ie, do nothing}
                  FCommand := eNone;
                  Result := pctNone;
                End;
              '=':
                Begin {set application keypad mode}
                  FCommand := eSM;
                  FArgCount := 2;
                  FArgs^[0] := -2;
                  FArgs^[1] := 999; {special iPRO code!}
                End;
              '>':
                Begin {set numeric keypad mode}
                  FCommand := eRM;
                  FArgCount := 2;
                  FArgs^[0] := -2;
                  FArgs^[1] := 999; {special iPRO code!}
                End;
              'D':
                Begin {index = cursor down + scroll}
                  FCommand := eIND2;
                End;
              'E':
                Begin {next line}
                  FCommand := eNEL;
                End;
              'H':
                Begin {set horx tab stop}
                  FCommand := eHTS;
                End;
              'M':
                Begin {reverse index = cursor up + scroll}
                  FCommand := eRI;
                End;

              'Z':
                Begin {device attributes}
                  FCommand := eDA;
                  FArgCount := 1;
                  FArgs^[0] := 0; {stands for VT100}
                End;
              'c':
                Begin
                  FCommand := eRIS;
                End;
            Else
                  {ignore the char & seq.--it's not one we know}
              Result := pctNone;
            End; {case}
          End; {case}
        End;
      psParsingANSI:
        Begin
          If (#$40 <= aCh) And (aCh < #$7F) Then
          Begin
            {the command is now complete-see if we know about it}
            FState := psGotCommand;
            Result := vtpParseANSISeq(aCh);
          End;
          {otherwise, the next character has already been added to
           the sequence string, so there's nothing extra to do}
        End;
      psParsingLeftParen:
        Begin
          If ('0' <= aCh) And (aCh <= '~') Then
          Begin
            {the command is complete}
            If (GetSeqLength(FSequence) = 3) Then
            Begin
              FState := psGotCommand;
              Result := pctComplete;
              FCommand := eDECSCS;
              FArgCount := 2;
              FArgs^[0] := 0; {0 = set G0 charset}
              Case aCh Of
                'A': FArgs^[1] := ord('A');
                'B': FArgs^[1] := ord('B');
                '0': FArgs^[1] := 0;
                '1': FArgs^[1] := 1;
                '2': FArgs^[1] := 2;
              Else
                  {ignore the char & seq.--it's not one we know}
                FState := psGotCommand;
                Result := pctNone;
                FCommand := eNone;
                FArgCount := 0;
              End; {case}
            End
            Else {sequence is too long}
            Begin
              FState := psGotCommand;
              Result := pctNone;
              FCommand := eNone;
              FArgCount := 0;
            End;
          End;
        End;
      psParsingRightParen:
        Begin
          If ('0' <= aCh) And (aCh <= '~') Then
          Begin
            {the command is complete}
            If (GetSeqLength(FSequence) = 3) Then
            Begin
              FState := psGotCommand;
              Result := pctComplete;
              FCommand := eDECSCS;
              FArgCount := 2;
              FArgs^[0] := 1; {0 = set G1 charset}
              Case aCh Of
                'A': FArgs^[1] := ord('A');
                'B': FArgs^[1] := ord('B');
                '0': FArgs^[1] := 0;
                '1': FArgs^[1] := 1;
                '2': FArgs^[1] := 2;
              Else
                  {ignore the char & seq.--it's not one we know}
                FState := psGotCommand;
                Result := pctNone;
                FCommand := eNone;
                FArgCount := 0;
              End; {case}
            End
            Else {sequence is too long}
            Begin
              FState := psGotCommand;
              Result := pctNone;
              FCommand := eNone;
              FArgCount := 0;
            End;
          End;
        End;
      psParsingCharSet:
        Begin
          {these are the VT200+ "switch charset" sequences: we ignore
           them after finding the first char in range $30..$7E}
          If ('0' <= aCh) And (aCh <= '~') Then
          Begin
            FState := psGotCommand;
            Result := pctNone;
            FCommand := eNone;
            FArgCount := 0;
          End;
        End;
      psParsingHash:
        Begin
          FState := psGotCommand;
          Result := pctComplete;
          Case aCh Of
            '3':
              Begin
                FCommand := eDECDHL;
                FArgCount := 1;
                FArgs^[0] := 0; {0 = top half}
              End;
            '4':
              Begin
                FCommand := eDECDHL;
                FArgCount := 1;
                FArgs^[0] := 1; {1 = bottom half}
              End;
            '5':
              Begin
                FCommand := eDECSWL;
              End;
            '6':
              Begin
                FCommand := eDECDWL;
              End;
            '8':
              Begin
                FCommand := eDECALN;
              End;
          Else
              {ignore the char & seq.--it's not one we know}
            FState := psGotCommand;
            Result := pctNone;
          End; {case}
        End;
      psParsingCUP52:
        Begin
          If (FArgCount = 0) Then
          Begin
            FArgs^[0] := ord(aCh) - $1F;
            inc(FArgCount);
          End
          Else
          Begin
            FState := psGotCommand;
            FCommand := eCUP;
            FArgs^[1] := ord(aCh) - $1F;
            inc(FArgCount);
            Result := pctComplete;
          End;
        End;
    Else
        {invalid state?}
    End; {case}
  End;
End;

Function TIpVT100Parser.ProcessWideChar(aCh: WideChar): TIpParserCmdType;
Begin
  Result := pctNone;
End;

Function TIpVT100Parser.tpGetArgument(aInx: Integer): Integer;
Begin
  If (aInx < 0) Or (aInx >= FArgCount) Then
    Result := 0
  Else
    Result := FArgs^[aInx];
End;

Function TIpVT100Parser.tpGetSequence: String;
Begin
  If (FCommand <> eNone) Then
    Result := GetStringFromSeq(FSequence)
  Else
    Result := '';
End;

Function TIpVT100Parser.vtpGetArguments: Boolean;
Var
  ChInx: Integer;
  StartInx: Integer;
  Ch: char;
  ec: Integer;
  TempStr: String[255];
Begin
  {for this parser, we assume
     1. arguments consist of numeric digits only
     2. arguments are separated by ';'
     3. the first argument can be ? (DEC VT100 special)
     4. argument parsing stops at the first character #$20 - #$2F, or
        #$40 - #$7E}

  {assume the sequence is badly formed}
  Result := False;

  {first check for the third character being ?}
  If (PSeq(FSequence)^.sText[3] = '?') Then
  Begin
    FArgCount := 1;
    FArgs^[0] := -2;
    StartInx := 4;
  End
  Else
    StartInx := 3;

  {scan the rest of the characters until we reach a char in the range
   $20-$2F, or $40-$7E; look out for numeric digits and semi-colons}
  TempStr := '';
  For ChInx := StartInx To PSeq(FSequence)^.sLen Do
  Begin
    Ch := PSeq(FSequence)^.sText[ChInx];
    If ((#$20 <= Ch) And (Ch <= #$2F)) Or
      ((#$40 <= Ch) And (Ch <= #$7E)) Then
      Break;
    If (Ch = ';') Then
    Begin
      If (FArgCountMax = FArgCount) Then
        vtpGrowArgs;
      If (TempStr = '') Then
      Begin
        FArgs^[FArgCount] := -1;
        inc(FArgCount);
      End
      Else
      Begin
        Val(TempStr, FArgs^[FArgCount], ec);
        If (ec <> 0) Then
          Exit;
        inc(FArgCount);
        TempStr := '';
      End;
    End
    Else
      If ('0' <= Ch) And (Ch <= '9') Then
      Begin
        TempStr := TempStr + Ch;
      End
      Else {bad character}
        Exit;
  End;

  {convert the final argument}
  If (FArgCountMax = FArgCount) Then
    vtpGrowArgs;
  If (TempStr = '') Then
  Begin
    FArgs^[FArgCount] := -1;
    inc(FArgCount);
  End
  Else
  Begin
    Val(TempStr, FArgs^[FArgCount], ec);
    If (ec <> 0) Then
      Exit;
    inc(FArgCount);
  End;

  {if we got here, everything was all right}
  Result := True;
End;

Procedure TIpVT100Parser.vtpGrowArgs;
Var
  NewMax: Integer;
  NewArray: PAdIntegerArray;
Begin
  {use a simple increase-by-half algorithm}
  If (FArgCountMax = 0) Then
    NewMax := 16
  Else
    NewMax := (FArgCountMax * 3) Div 2;
  {alloc the new array, zeroed}
  NewArray := AllocMem(sizeof(Integer) * NewMax);
  {if there's any data in the old array copy it over, delete it}
  If (FArgs <> Nil) Then
  Begin
    Move(FArgs^, NewArray^, sizeof(Integer) * FArgCount);
    FreeMem(FArgs, sizeof(Integer) * FArgCountMax);
  End;
  {remember the new details}
  FArgs := NewArray;
  FArgCountMax := NewMax;
End;

Function TIpVT100Parser.vtpParseANSISeq(aCh: char): TIpParserCmdType;
Begin
  {when this method is called FSequence has the full escape sequence,
   and FArgCount, FArgs, FCommand have to be set; for convenience aCh
   is the final character in FSequence--the command identifier--and
   FSequence must have at least three characters in it}

  {assume the sequence is invalid}
  Result := pctNone;

  {special case: DECSCL}
  If (GetStringFromSeq(FSequence) = DECSCLseq) Then
  Begin
    FCommand := eRIS;
    Result := pctComplete;
  End;

  {split out the arguments in the sequence, build up the FArgs array;
   note that an arg of -1 means 'default', and -2 means ? (a special
   DEConly parameter)}
  If Not vtpGetArguments Then
    Exit;

  {identify the command character}
  Case aCh Of
    '@':
      Begin {insert character--VT102}
        FCommand := eICH;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'A':
      Begin {Cursor up}
        FCommand := eCUU;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'B':
      Begin {Cursor down}
        FCommand := eCUD;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'C':
      Begin {Cursor right}
        FCommand := eCUF;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'D':
      Begin {cursor left}
        FCommand := eCUB;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'H':
      Begin {cursor position}
        FCommand := eCUP;
        {should have two parameters, both default of 1}
        If Not vtpValidateArgsPrim(2, 2, 1) Then
          Exit;
      End;
    'J':
      Begin {Erase in display}
        FCommand := eED;
        {should only have one parameter, default of 0}
        If Not vtpValidateArgsPrim(1, 1, 0) Then
          Exit;
      End;
    'K':
      Begin {Erase in line}
        FCommand := eEL;
        {should only have one parameter, default of 0}
        If Not vtpValidateArgsPrim(1, 1, 0) Then
          Exit;
      End;
    'L':
      Begin {Insert line--VT102}
        FCommand := eIL;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'M':
      Begin {Delete line--VT102}
        FCommand := eDL;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'P':
      Begin {delete character--VT102}
        FCommand := eDCH;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'X':
      Begin {erase character--VT102}
        FCommand := eECH;
        {should only have one parameter, default of 1}
        If Not vtpValidateArgsPrim(1, 1, 1) Then
          Exit;
      End;
    'c':
      Begin {Device attributes}
        FCommand := eDA;
        {should only have one parameter, default of 0}
        If Not vtpValidateArgsPrim(1, 1, 0) Then
          Exit;
      End;
    'f':
      Begin {cursor position}
        FCommand := eCUP;
        {should have two parameters, both default of 1}
        If Not vtpValidateArgsPrim(2, 2, 1) Then
          Exit;
      End;
    'g':
      Begin {clear horizontal tabs}
        FCommand := eTBC;
        {should only have one parameter, default of 0}
        If Not vtpValidateArgsPrim(1, 1, 0) Then
          Exit;
      End;
    'h':
      Begin {set mode}
        FCommand := eSM;
        {should have one parameter, or 2 if the first is ?, no
         defaults}
      End;
    'l':
      Begin {reset mode}
        FCommand := eRM;
        {should have one parameter, or 2 if the first is ?, no
         defaults}
        {we have to try and spot one command in particular: the
         switch to VT52 mode}
        If (FArgCount = 2) And
          (FArgs^[0] = -2) And (FArgs^[1] = 2) Then
          FInVT52Mode := True;
      End;
    'm':
      Begin
        FCommand := eSGR;
        {should have at least one parameter, default of 0 for all
         parameters}
        If Not vtpValidateArgsPrim(1, 30000, 0) Then
          Exit;
      End;
    'n':
      Begin {Device status report}
        FCommand := eDSR;
        {should only have one parameter, no default}
        If Not vtpValidateArgsPrim(1, 1, -1) Then
          Exit;
      End;
    'q':
      Begin {DEC PRIVATE-set/clear LEDs}
        FCommand := eDECLL;
        {should have at least one parameter, default of 0 for all
         parameters}
        If Not vtpValidateArgsPrim(1, 30000, 0) Then
          Exit;
      End;
    'r':
      Begin {DEC PRIVATE-set top/bottom margins}
        FCommand := eDECSTBM;
        {should have two parameters, first default of 1, second
         default unknowable by this class}
      End;
    's':
      Begin {save cursor pos - ANSI.SYS escape sequence}
        FCommand := eSaveCursorPos;
      End;
    'u':
      Begin {restore cursor pos - ANSI.SYS escape sequence}
        FCommand := eRestoreCursorPos;
      End;
    'x':
      Begin {DEC PRIVATE-request terminal parameters}
        FCommand := eDECREQTPARM;
        {should only have one parameter, no default}
        If Not vtpValidateArgsPrim(1, 1, -1) Then
          Exit;
      End;
    'y':
      Begin {DEC PRIVATE-invoke confidence test}
        FCommand := eDECTST;
        {should have two parameters, no default for first, second
         default to 0}
      End;
  Else {the command letter is unknown}
    Exit;
  End; {case}

  {if we get here the sequence is valid and we've patched up the
   arguments list and count}
  Result := pctComplete;
End;

Function TIpVT100Parser.vtpProcessVT52(aCh: char): TIpParserCmdType;
Begin
  FState := psGotCommand;
  Result := pctComplete;
  Case aCh Of
    '<':
      Begin {switch to ANSI mode}
        FCommand := eSM;
        FArgCount := 2; {pretend it's Esc[?2h}
        FArgs^[0] := -2;
        FArgs^[1] := 2;
        FInVT52Mode := False;
      End;
    '=':
      Begin {enter alternate keypad mode}
        FCommand := eSM;
        FArgCount := 2;
        FArgs^[0] := -2;
        FArgs^[1] := 999; {special iPRO code!}
      End;
    '>':
      Begin {leave alternate keypad mode}
        FCommand := eRM;
        FArgCount := 2;
        FArgs^[0] := -2;
        FArgs^[1] := 999; {special iPRO code!}
      End;
    'A':
      Begin {cursor up}
        FCommand := eCUU;
        FArgCount := 1;
        FArgs^[0] := 1;
      End;
    'B':
      Begin {cursor down}
        FCommand := eCUD;
        FArgCount := 1;
        FArgs^[0] := 1;
      End;
    'C':
      Begin {cursor right}
        FCommand := eCUF;
        FArgCount := 1;
        FArgs^[0] := 1;
      End;
    'D':
      Begin {cursor left}
        FCommand := eCUB;
        FArgCount := 1;
        FArgs^[0] := 1;
      End;
    'F':
      Begin {switch to graphics characters}
        FCommand := eSO;
      End;
    'G':
      Begin {switch to ASCII characters}
        FCommand := eSI;
      End;
    'H':
      Begin {move cursor home}
        FCommand := eCUP;
        FArgCount := 2;
        FArgs^[0] := 1;
        FArgs^[1] := 1;
      End;
    'I':
      Begin {reverse index = cursor up + scroll}
        FCommand := eRI;
      End;
    'J':
      Begin {erase to end of screen}
        FCommand := eED;
        FArgCount := 1;
        FArgs^[0] := 0; {ie <esc>[0J}
      End;
    'K':
      Begin {erase to end of line}
        FCommand := eEL;
        FArgCount := 1;
        FArgs^[0] := 0; {ie <esc>[0K}
      End;
    'Y':
      Begin {position cursor}
        FState := psParsingCUP52;
        FCommand := eCUP;
        Result := pctPending;
      End;
    'Z':
      Begin {device attributes, identify}
        FCommand := eDA;
        FArgCount := 1;
        FArgs^[0] := 52; {ie VT52 emulation}
      End;
  Else
    Result := pctNone;
  End; {case}
End;

Function TIpVT100Parser.vtpValidateArgsPrim(aMinArgs: Integer;
  aMaxArgs: Integer;
  aDefault: Integer): Boolean;
Var
  i: Integer;
Begin
  Result := False;
  {if we have too many arguments, something's obviously wrong}
  If (FArgCount > aMaxArgs) Then
    Exit;
  {if we have too few, make the missing ones the default}
  While (FArgCount < aMinArgs) Do
  Begin
    If (FArgCountMax = FArgCount) Then
      vtpGrowArgs;
    FArgs^[FArgCount] := aDefault;
    inc(FArgCount);
  End;
  {convert any -1 arguments to the default}
  For i := 0 To pred(FArgCount) Do
    If (FArgs^[i] = -1) Then
      FArgs^[i] := aDefault;
  {and we're done}
  Result := True;
End;

{===TIpTerminal======================================================}

Constructor TIpCustomTerminal.Create(aOwner: TComponent);
Begin
  //ipc_TermBufForeColor:= clsilver;
  //ipc_TermBufBackColor:= clblack;

  { create the default emulator }
  FDefEmulator := TIpTTYEmulator.Create(Nil);
  FDefEmulator.FIsDefault := True;
  FDefEmulator.Terminal := Self;
  If (FEmulator = Nil) Then
    FEmulator := FDefEmulator;

  Inherited Create(aOwner);

  { create our canvas }
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;

  { set up heartbeat timer }
  FHeartbeat := TTimer.Create(Self);
  FHeartbeat.Interval := BeatInterval;
  FHeartbeat.OnTimer := tmBeat;
  FHeartbeat.Enabled := False;

  { create a Byte queue to receive data }
  FByteQueue := TaaByteQueue.Create;

  { make sure the origin is at (1,1) }
  FOriginCol := 1;
  FOriginRow := 1;

  { set up the default values of the terminal's properties }
  Active := ipc_TermActive; { the set access method must be called }
  FBlinkTime := ipc_TermBlinkTime;
  FBorderStyle := ipc_TermBorderStyle;
  FCapture := ipc_TermCapture;
  FCaptureFile := ipc_TermCaptureFile;
  FCursorType := ipc_TermCursorType;
  FHalfDuplex := ipc_TermHalfDuplex;
  FLazyByteDelay := ipc_TermLazyByteDelay;
  FLazyTimeDelay := ipc_TermLazyTimeDelay;
  FTermWidth := ipc_TermWidth;
  FTermHeight := ipc_TermHeight;
  FUseLazyDisplay := ipc_TermUseLazyDisplay;
  FWantAllKeys := ipc_TermWantAllKeys;
  FFreezeScrollBack := ipc_FreezeScrollBack; {!!.03}

  { Initialize socket }
  FSocket := Invalid_Socket;

  Width := ipc_TermWidth;
  Height := ipc_TermHeight;
  Font.Name := ipc_TermFontName;
  Font.Size := 9;
  Font.Color := ipc_TermForeColor;
  Color := ipc_TermBackColor;

  { miscellaneous }
  With FScrollHorzInfo Do
    cbSize := sizeof(FScrollHorzInfo);
  With FScrollVertInfo Do
    cbSize := sizeof(FScrollVertInfo);

End;

Destructor TIpCustomTerminal.Destroy;
Begin
  Emulator := Nil;
  FEmulator := Nil;
  FDefEmulator.Free;
  FCanvas.Free;
  FHeartbeat.Free;
  RemoveTermEmuLink(Self, False);
  TaaByteQueue(FByteQueue).Free;
  Inherited Destroy;
End;

Procedure TIpCustomTerminal.CMIpSocketStatus(Var Msg: TCMIpSocketStatus);
Begin
  If Not Assigned(SockControl) Then
    Exit;

  Case Msg.StatusEvent Of
    IpssConnect:
      Begin
        { Handle single-socket "attach to anything" situation }
        If (FSocket = Invalid_Socket) And (Msg.SingleSocket = 1) Then
          { Go through SetSocket, so we attach to socket }
          Socket := Msg.Socket;
      End;

    IpssDisconnect:
      Begin
        { Disconnect and set Socket to Invalid_Socket }
        If Msg.Socket = FSocket Then
          Socket := Invalid_Socket;
      End;
  End;
End;

Procedure TIpCustomTerminal.CMIpTermForceSize(Var Msg: TMessage);
Begin
  RecreateWnd;
End;

Procedure TIpCustomTerminal.CMIpTermStuff(Var Msg: TMessage);
Var
  DataLen: Integer;
  DataPtr: Pointer;
  PaintRgn: HRGN;
Begin
  { This message is sent by CMIpTermData, and the WriteChar/WriteString }
  { methods; note that if there are two or more CM_IPTERMSTUFF messages }
  { in the message queue, the first one to be accepted will clear the   }
  { byte queue, so the others need to take account of the fact that     }
  { there may be no data             }

  DataLen := TaaByteQueue(FByteQueue).Count;
  If (DataLen > 0) Then
  Begin
    DataPtr := TaaByteQueue(FByteQueue).Peek(DataLen);

    { if we are capturing, save the data to file }
    If (Capture = cmOn) Then
    Begin
      If (FCaptureStream <> Nil) Then
        FCaptureStream.Write(DataPtr^, DataLen);
    End;

    { otherwise get the emulator to process the block }
    FEmulator.ProcessBlock(DataPtr, DataLen);
    If (Not UseLazyDisplay) And FEmulator.NeedsUpdate Then
    Begin
      { hide the caret }
      tmHideCaret;
      Try
        PaintRgn := CreateRectRgn(0, 0,
          ClientCols * CharWidth,
          ClientRows * CharHeight);
        SelectClipRgn(Canvas.Handle, PaintRgn);
        DeleteObject(PaintRgn);
        FEmulator.LazyPaint;
      Finally
        tmShowCaret;
      End;
    End;

    { increment the byte count for the lazy painting }
    inc(FLazyByteCount, DataLen);

    { now we've processed all the data, clear the byte queue }
    If (DataLen <> TaaByteQueue(FByteQueue).Count) Then {!!.01}
      TaaByteQueue(FByteQueue).Remove(DataLen) {!!.01}
    Else {!!.01}
      TaaByteQueue(FByteQueue).Clear;
  End;
End;

Procedure TIpCustomTerminal.Clear;
Begin
  FEmulator.teClear;
End;

Procedure TIpCustomTerminal.ClearAll;
Begin
  FEmulator.teClearAll;
End;

Procedure TIpCustomTerminal.CMCtl3DChanged(Var Msg: TMessage);
Begin
  If (csLoading In ComponentState) Or Not HandleAllocated Then
    Exit;

  If NewStyleControls And (FBorderStyle = bsSingle) Then
    RecreateWnd;

  Inherited;
End;

Procedure TIpCustomTerminal.CMFontChanged(Var Msg: TMessage);
Begin
  { make sure our ancestors do their stuff }
  Inherited;
  { if we're loading, don't bother doing anything }
  If (csLoading In ComponentState) Then
    Exit;
  { if we have no handle, there's nothing we can do }
  If Not HandleAllocated Then
    Exit;
  { otherwise, work out the character size }
  tmGetFontInfo;
End;

Procedure TIpCustomTerminal.CMIpTermData(Var Msg: TCMIpTermData);
Var
  QBuffer: Pointer;
Begin
  Try
    { get all the data available and buffer it elsewhere }
    QBuffer := TaaByteQueue(FByteQueue).PutPeek(Msg.BufSize);
    Move(Msg.Buffer^, QBuffer^, Msg.BufSize);
    TaaByteQueue(FByteQueue).AdvanceAfterPut(Msg.BufSize);
    { tell ourselves that we have more data }
    PostMessage(Handle, CM_IPTERMSTUFF, 0, 0);
  Finally
    FreeMem(Msg.Buffer);
  End;
End;

Procedure TIpCustomTerminal.CNKeyDown(Var Msg: TWMKeyDown);
Var
  Shift: TShiftState;
  EnhKey: Boolean;
  Key: Word;
  NumLockState: TKeyboardState;
Begin
  { if the left button is down (we're selecting) and the key is Escape, }
  { cancel the mode }
  If FLButtonDown And (Msg.CharCode = VK_ESCAPE) Then
  Begin
    Perform(WM_CANCELMODE, 0, 0);
    Msg.Result := 1;
    Exit;
  End;
  { if we have a selection and the key is Ctrl+C or Ctrl+Insert, copy }
  { the selection to the clipboard }
  If tmProcessClipboardCopy(Msg) Then
  Begin
    Msg.Result := 1;
    Exit;
  End;
  { process the keystroke }
  With Msg Do
  Begin
    { ignore all shift and supershift keys }
    If (CharCode = VK_SHIFT) Or
      (CharCode = VK_CONTROL) Or
      (CharCode = VK_MENU) Then
      Exit;
    {calculate the shift state}
    Shift := KeyDataToShiftState(KeyData);
    {calculate whether this is an enhanced key}
    EnhKey := (KeyData And $1000000) <> 0;
    {get the key}
    Key := CharCode;
    {convert some keys}
    If EnhKey Then
    Begin
      {convert VK_RETURN to VK_EXECUTE for the Keypad}
      If (Key = VK_RETURN) Then
        Key := VK_EXECUTE;
    End
    Else {not an enhanced keyboard}
    Begin
      {convert unNumLocked keys on Keypad to their Keypad values}
      GetKeyboardState(NumLockState);
      If ((NumLockState[VK_NUMLOCK] And $01) = 0) Then
        Case Key Of
          VK_CLEAR: Key := VK_NUMPAD5;
          VK_PRIOR: Key := VK_NUMPAD9;
          VK_NEXT: Key := VK_NUMPAD3;
          VK_END: Key := VK_NUMPAD1;
          VK_HOME: Key := VK_NUMPAD7;
          VK_LEFT: Key := VK_NUMPAD4;
          VK_UP: Key := VK_NUMPAD8;
          VK_RIGHT: Key := VK_NUMPAD6;
          VK_DOWN: Key := VK_NUMPAD2;
          VK_INSERT: Key := VK_NUMPAD0;
          VK_DELETE: Key := VK_DECIMAL;
        End; {case}
    End;
    { calculate whether this is an autorepeat }
    If ((KeyData And $40000000) <> 0) Then
      Key := Key Or $8000;
    { pass the key, etc, onto the emulator }
    FEmulator.KeyDown(Key, Shift);
    {if the emulator successfully processed the key, let the caller
     know that it's been processed}
    If (Key = 0) Then
      Result := 1;
  End;
End;

Procedure TIpCustomTerminal.CNSysKeyDown(Var Msg: TWMKeyDown);
Begin
  CNKeyDown(Msg);
End;

Procedure TIpCustomTerminal.CopyToClipboard;
Var
  i: Integer;
  CharCount: Integer;
  StartCol: Integer;
  ColCount: Integer;
  CurInx: Integer;
  RowText: PAnsiChar;
  TextPtr: PAnsiChar;
Begin
  { calculate the amount of text in the selection; this equals the }
  { number of characters for each line, plus CR/LF per line }
  ColCount := FEmulator.Buffer.ColCount;
  CharCount := 0;
  StartCol := FLButtonRect.Left;
  For i := FLButtonRect.Top To pred(FLButtonRect.Bottom) Do
  Begin
    CharCount := CharCount + (ColCount - StartCol + 1) + 2;
    StartCol := 1;
  End;
  CharCount := CharCount + FLButtonRect.Right + 2;
  { allocate enough memory to store this set of characters, plus the }
  { terminating null }
  GetMem(TextPtr, CharCount + 1);
  { copy all the selected characters }
  CurInx := 0;
  StartCol := FLButtonRect.Left;
  For i := FLButtonRect.Top To pred(FLButtonRect.Bottom) Do
  Begin
    RowText := FEmulator.Buffer.GetLineCharPtr(i);
    Move(RowText[StartCol - 1], TextPtr[CurInx],
      ColCount - StartCol + 1);
    inc(CurInx, ColCount - StartCol + 1);
    TextPtr[CurInx] := ^M;
    TextPtr[CurInx + 1] := ^J;
    inc(CurInx, 2);
    StartCol := 1;
  End;
  RowText := FEmulator.Buffer.GetLineCharPtr(FLButtonRect.Bottom);
  Move(RowText[StartCol - 1], TextPtr[CurInx],
    FLButtonRect.Right - StartCol + 1);
  inc(CurInx, FLButtonRect.Right - StartCol + 1);
  TextPtr[CurInx] := ^M;
  TextPtr[CurInx + 1] := ^J;
  inc(CurInx, 2);
  TextPtr[CurInx] := #0;

  Clipboard.Open;
  Clipboard.SetTextBuf(TextPtr);
  Clipboard.Close;
End;

Function TIpCustomTerminal.GetSelectedText: String;
Var
  i: Integer;
  CharCount: Integer;
  StartCol: Integer;
  ColCount: Integer;
  CurInx: Integer;
  RowText: PAnsiChar;
  TextPtr: PAnsiChar;
Begin
  { calculate the amount of text in the selection; this equals the }
  { number of characters for each line, plus CR/LF per line }
  ColCount := FEmulator.Buffer.ColCount;
  CharCount := 0;
  StartCol := FLButtonRect.Left;
  For i := FLButtonRect.Top To pred(FLButtonRect.Bottom) Do
  Begin
    CharCount := CharCount + (ColCount - StartCol + 1) + 2;
    StartCol := 1;
  End;
  CharCount := CharCount + FLButtonRect.Right + 2;
  { allocate enough memory to store this set of characters, plus the }
  { terminating null }
  GetMem(TextPtr, CharCount + 1);
  { copy all the selected characters }
  CurInx := 0;
  StartCol := FLButtonRect.Left;
  For i := FLButtonRect.Top To pred(FLButtonRect.Bottom) Do
  Begin
    RowText := FEmulator.Buffer.GetLineCharPtr(i);
    Move(RowText[StartCol - 1], TextPtr[CurInx],
      ColCount - StartCol + 1);
    inc(CurInx, ColCount - StartCol + 1);
    TextPtr[CurInx] := ^M;
    TextPtr[CurInx + 1] := ^J;
    inc(CurInx, 2);
    StartCol := 1;
  End;
  RowText := FEmulator.Buffer.GetLineCharPtr(FLButtonRect.Bottom);
  Move(RowText[StartCol - 1], TextPtr[CurInx],
    FLButtonRect.Right - StartCol + 1);
  inc(CurInx, FLButtonRect.Right - StartCol + 1);
  TextPtr[CurInx] := ^M;
  TextPtr[CurInx + 1] := ^J;
  inc(CurInx, 2);
  TextPtr[CurInx] := #0;

  If String(TextPtr) <> #13 Then
    Result := String(TextPtr);
End;

Procedure TIpCustomTerminal.CreateParams(Var Params: TCreateParams);
Const
  BorderStyles: Array[TBorderStyle] Of Longint =
  (0, WS_BORDER);
Begin
  Inherited CreateParams(Params);

  With Params Do
  Begin
    Style := Longint(Style) Or BorderStyles[FBorderStyle];
    If FUseHScrollBar Then
      Style := Longint(Style) Or WS_HSCROLL;
    If FUseVScrollBar Or (ScrollbackRows > Rows) Then
      Style := Longint(Style) Or WS_VSCROLL;
  End;

  If NewStyleControls And Ctl3D And (FBorderStyle = bsSingle) Then
  Begin
    Params.Style := Params.Style And Not WS_BORDER;
    Params.ExStyle := Params.ExStyle Or WS_EX_CLIENTEDGE;
  End;
End;

Procedure TIpCustomTerminal.CreateWnd;

{ Search for an existing TIpSockControl }

Function FindSockControl(aForm: TWinControl): TIpSockControl;
  Var
    I: Integer;
  Begin
    For i := 0 To pred(aForm.ComponentCount) Do
      If aForm.Components[i] Is TIpSockControl Then
      Begin
        Result := TIpSockControl(aForm.Components[i]);
        Exit;
      End;
    Result := Nil;
  End;

Var
  ParentForm: TWinControl;
Begin
  { It is possible that the last message to be sent to the old window }
  { handle was a WM_SETFOCUS: this would have created the caret, so   }
  { free it: it's attached to a window handle that no longer exists   }
  tmFreeCaret;
  { call our ancestor to create the window }
  Inherited CreateWnd;
  { Hopefully, a handle was created... }
  If HandleAllocated Then
  Begin
    { if we don't have a SockControl, go find one }
    If (SockControl = Nil) Then
    Begin
      ParentForm := GetParentForm(Self);
      If (ParentForm <> Nil) Then
        FSockControl := FindSockControl(ParentForm);
    End;
    { check to see if we're attached to a SockControl, if so, register }
    If (SockControl <> Nil) And Not (csDesigning In ComponentState) Then
    Begin
      SockControl.RegisterTerminal(Handle);
      If (Socket <> Invalid_Socket) Then
        tmAttachToSocket;
    End;
    { calculate the character sizes }
    tmGetFontInfo;
    { start the heartbeat }
    FHeartbeat.Enabled := True;
    If (csDesigning In ComponentState) Then
      tmDrawDefaultText;
  End;
End;

Procedure TIpCustomTerminal.DestroyWnd;
Begin
  If HandleAllocated Then
  Begin
    { check to see if we're attached to a socket, if so, deregister }
    If (SockControl <> Nil) And Not (csDesigning In ComponentState) Then
    Begin
      If (Socket <> Invalid_Socket) Then
        tmDetachFromSocket;
      SockControl.DeregisterTerminal(Handle);
    End;
    {stop the heartbeat}
    FHeartbeat.Enabled := False;
    {destroy the caret}
    tmFreeCaret;
  End;
  Inherited DestroyWnd;
End;

Procedure TIpCustomTerminal.HideSelection;
Var
  i: Integer;
  ColCount: Integer;
Begin
  {clear the current selection}
  ColCount := FEmulator.Buffer.ColCount;
  tmMarkDeselected(FLButtonRect.Top, FLButtonRect.Left, ColCount);
  For i := succ(FLButtonRect.Top) To pred(FLButtonRect.Bottom) Do
    tmMarkDeselected(i, 1, ColCount);
  tmMarkDeselected(FLButtonRect.Bottom, 1, FLButtonRect.Right);
  {initialize the selection variables}
  SetRectEmpty(FLButtonRect);
  FLButtonAnchor.X := 0;
  FLButtonAnchor.Y := 0;
  FHaveSelection := False;
End;

Procedure TIpCustomTerminal.KeyDown(Var Key: Word; Shift: TShiftState);
Begin
  Inherited KeyDown(Key, Shift);
  If (Not WantAllKeys) And (Key <> 0) Then
    FEmulator.KeyDown(Key, Shift);
End;

Procedure TIpCustomTerminal.KeyPress(Var Key: Char);
Begin
  Inherited KeyPress(Key);
  If (Key <> #0) Then
    FEmulator.KeyPress(Key);
End;

Procedure TIpCustomTerminal.MouseDown(Button: TMouseButton;
  Shift: TShiftState;
  X, Y: Integer);
Begin
  Inherited MouseDown(Button, Shift, X, Y);
  If (Button = mbLeft) Then
  Begin
    If Not Focused And CanFocus Then
      SetFocus;
    FLButtonDown := True;
    {force mouse position into bounds}
    X := BoundI(0, X, pred(ClientCols * CharWidth));
    Y := BoundI(0, Y, pred(ClientRows * CharHeight));
    {start the selection}
    tmStartSelect(X, Y);
  End;
End;

Procedure TIpCustomTerminal.MouseMove(Shift: TShiftState; X, Y: Integer);
Begin
  Inherited MouseMove(Shift, X, Y);
  If FLButtonDown Then
  Begin
    {force mouse position into bounds}
    X := BoundI(0, X, pred(ClientCols * CharWidth));
    Y := BoundI(0, Y, pred(ClientRows * CharHeight));
    {update the selection}
    tmGrowSelect(X, Y);
  End;
End;

Procedure TIpCustomTerminal.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
Begin
  Inherited MouseUp(Button, Shift, X, Y);
  If FLButtonDown And (Button = mbLeft) Then
  Begin
    FLButtonDown := False;
  End;
End;

Procedure TIpCustomTerminal.Notification(AComponent: TComponent;
  Operation: TOperation);
Begin
  If (Operation = opRemove) Then
  Begin
    If (AComponent = Emulator) Then
      Emulator := Nil
    Else
      If (AComponent = SockControl) Then
      Begin
        Active := False;
        SockControl := Nil;
        Socket := Invalid_Socket;
      End;
  End
  Else {Operation = opInsert}
  Begin
    If (AComponent Is TIpTerminalEmulator) Then
    Begin
      If (Emulator = Nil) Then
        Emulator := TIpTerminalEmulator(AComponent);
    End
    Else
      If (AComponent Is TIpSockControl) Then
      Begin
        If (SockControl = Nil) Then
          SockControl := TIpSockControl(AComponent);
      End;
  End;
  Inherited Notification(AComponent, Operation);
End;

Function TermIntersectRect(Var lprcDst: TRect;
  Const lprcSrc1, lprcSrc2: TRect): Boolean;
Begin
  Result := IntersectRect(lprcDst, lprcSrc1, lprcSrc2);
End;

Procedure TIpCustomTerminal.Paint;
Var
  DirtyRect: TRect;
  PaintRect: TRect;
  PaintRgn: HRGN;
Begin
  {hide the caret}
  tmHideCaret;
  Try
    {get the current clip rect--ie, the rect that needs painting}
    DirtyRect := Canvas.ClipRect;
    {paint any unused bits of that rect}
    If TermIntersectRect(PaintRect, DirtyRect, FUnusedRightRect) Then
    Begin
      Canvas.Brush.Color := Color;
      Canvas.FillRect(PaintRect);
    End;
    If TermIntersectRect(PaintRect, DirtyRect, FUnusedBottomRect) Then
    Begin
      Canvas.Brush.Color := Color;
      Canvas.FillRect(PaintRect);
    End;
    {find out if there is anything else to paint}
    If TermIntersectRect(PaintRect, DirtyRect, FUsedRect) Then
    Begin
      {create a clipping region and select it into the canvas, so that
       the emulator only gets to know about bits it can paint}
      With PaintRect Do
        PaintRgn := CreateRectRgn(Left, Top, Right, Bottom);
      SelectClipRgn(Canvas.Handle, PaintRgn);
      DeleteObject(PaintRgn);
      {tell the emulator to paint what it has}
      FEmulator.Paint
    End;
  Finally
    {show the caret again}
    tmShowCaret;
  End;
  TIpCustomTerminalScrollBarPresent := False;
  If FUseHScrollbar Then
    TIpCustomTerminalScrollBarPresent := True;
  TerminalScreenWidth := 80 * CharWidth;
  TerminalScreenHeight := 24 * CharHeight;

  TermWidth := PaintRect.Right - PaintRect.Left;
  TermHeight := PaintRect.Bottom - PaintRect.Top;
End;

Procedure TIpCustomTerminal.PaintWindow(DC: HDC);
Begin
  Canvas.Handle := DC;
  Try
    Paint;
  Finally
    Canvas.Handle := 0;
  End;
End;

Procedure TIpCustomTerminal.tmAttachToSocket;
Begin
  { Assumptions: not designtime; SockControl <> nil; Active = True; }
  If TIpSockControlFriend(SockControl).scAttachTerminal(Socket, Handle) Then
  Begin

    { Set MasterTerminal if not currently set }
    If (SockControl.MasterTerminal[Socket] = Nil) Then
      SockControl.MasterTerminal[Socket] := Self;

  End
  Else
  Begin

    { Socket apparently doesn't exist }
    Socket := Invalid_Socket;
  End;
End;

Procedure TIpCustomTerminal.tmBeat(Sender: TObject);
Var
  PaintRgn: HRGN;
  MousePt: TPoint;
  Msg: TWMScroll;
Begin
  {if the left mouse button is down we may need to scroll to help the
   user select text}
  If FLButtonDown Then
  Begin
    GetCursorPos(MousePt);
    MousePt := ScreenToClient(MousePt);
    If FUseHScrollBar Then
      If (MousePt.X < 0) Then
      Begin
        Msg.ScrollCode := SB_LINELEFT;
        WMHScroll(Msg);
      End
      Else
        If (MousePt.X >= ClientWidth) Then
        Begin
          Msg.ScrollCode := SB_LINERIGHT;
          WMHScroll(Msg);
        End;
    If (MousePt.Y < 0) Then
    Begin
      Msg.ScrollCode := SB_LINEUP;
      WMVScroll(Msg);
    End
    Else
      If (MousePt.Y >= ClientHeight) Then
      Begin
        Msg.ScrollCode := SB_LINEDOWN;
        WMVScroll(Msg);
      End;
    MouseMove([ssLeft], MousePt.X, MousePt.Y);
  End;
  {only blink if the blink time is not zero}
  If (BlinkTime > 0) Then
  Begin
    inc(FBlinkTimeCount, BeatInterval);
    If (FBlinkTimeCount > BlinkTime) Then
    Begin
      FBlinkTextVisible := Not FBlinkTextVisible;
      If FEmulator.HasBlinkingText Then
      Begin
        tmHideCaret;
        Try
          PaintRgn := CreateRectRgn(0, 0,
            ClientCols * CharWidth,
            ClientRows * CharHeight);
          SelectClipRgn(Canvas.Handle, PaintRgn);
          DeleteObject(PaintRgn);
          FEmulator.BlinkPaint(FBlinkTextVisible);
        Finally
          tmShowCaret;
        End;
      End;
      FBlinkTimeCount := 0;
    End;
  End;

  {only use the lazy display option if requested to}
  If UseLazyDisplay Then
  Begin
    {check for a lazy display time or byte count}
    inc(FLazyTimeCount, BeatInterval);
    If FEmulator.NeedsUpdate And
      ((FLazyTimeCount > FLazyTimeDelay) Or
      (FLazyByteCount > FLazyByteDelay)) Then
    Begin
      tmHideCaret;
      Try
        PaintRgn := CreateRectRgn(0, 0,
          ClientCols * CharWidth,
          ClientRows * CharHeight);
        SelectClipRgn(Canvas.Handle, PaintRgn);
        DeleteObject(PaintRgn);
        FEmulator.LazyPaint;
      Finally
        tmShowCaret;
      End;
      FLazyTimeCount := 0;
      FLazyByteCount := 0;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmCalcExtent;
Var
  WorkColCount: Integer;
  WorkRowCount: Integer;
Begin
  { ASSUMPTION: there is a handle allocated, but an emulator may not be }
  {             attached. Since there is a handle, we've already        }
  {             calculated the character width and height.              }

  { first calculate the max number of columns and rows in the client area }
  If (FEmulator = Nil) Then
  Begin
    FClientCols := 0;
    FClientRows := 0;
    Exit;
  End;

  WorkColCount := Columns;
  WorkRowCount := Rows;

  FClientCols := (ClientWidth + pred(FCharWidth)) Div FCharWidth;
  FClientCols := IpMinInt(FClientCols, WorkColCount - ClientOriginCol + 1);
  FClientFullCols := ClientWidth Div FCharWidth;
  FClientFullCols := IpMinInt(FClientFullCols, WorkColCount - ClientOriginCol + 1);

  FClientRows := (ClientHeight + pred(FCharHeight)) Div FCharHeight;
  FClientRows := IpMinInt(FClientRows, WorkRowCount - ClientOriginRow + 1);
  FClientFullRows := ClientHeight Div FCharHeight;
  FClientFullRows := IpMinInt(FClientFullRows, WorkRowCount - ClientOriginRow + 1);

  {if there is a handle for the terminal, we have to worry about
   whether the scrollbars are visible or not}
  If HandleAllocated Then
  Begin
    {if the width is too small, force a horizontal scroll bar; if too
     large, force the horizontal scroll bar off}
    If (FClientFullCols < WorkColCount) Then
    Begin
      If Not FUseHScrollBar Then
      Begin
        FUseHScrollBar := True;
        PostMessage(Handle, CM_IPTERMFORCESIZE, 0, 0);
        Exit;
      End
    End
    Else
    Begin
      If FUseHScrollBar Then
      Begin
        FUseHScrollBar := False;
        PostMessage(Handle, CM_IPTERMFORCESIZE, 0, 0);
        Exit;
      End
    End;
    { if the height is too small, force a vertical scroll bar; if too }
    { large, force the vertical scroll bar off }
    If (FClientFullRows < WorkRowCount) Then
    Begin
      If Not FUseVScrollBar Then
      Begin
        FUseVScrollBar := True;
        {note: if we have scrollback rows we already have a vertical
               scrollbar}
        If (ScrollbackRows <= Rows) Then
        Begin
          PostMessage(Handle, CM_IPTERMFORCESIZE, 0, 0);
          Exit;
        End
      End
    End
    Else
    Begin
      If FUseVScrollBar Then
      Begin
        FUseVScrollBar := False;
        PostMessage(Handle, CM_IPTERMFORCESIZE, 0, 0);
        Exit;
      End
    End;
  End;

  { now calculate the used and unused area rects }
  FUsedRect := Rect(0, 0, ClientWidth, ClientHeight);
  FUnusedRightRect := FUsedRect;
  FUnusedBottomRect := FUsedRect;

  FUsedRect.Right := IpMinInt(ClientCols * CharWidth, ClientWidth);
  FUsedRect.Bottom := IpMinInt(ClientRows * CharHeight, ClientHeight);

  FUnusedRightRect.Left := FUsedRect.Right;
  FUnusedRightRect.Bottom := FUsedRect.Bottom;
  FUnusedBottomRect.Top := FUsedRect.Bottom;

  {if we are using a horizontal scroll bar, set the range}
  If FUseHScrollBar Then
  Begin
    tmInitHScrollBar;
  End;

  {if we are using a vertical scroll bar, set the range}
  If FUseVScrollBar Or (ScrollbackRows > Rows) Then
  Begin
    tmInitVScrollBar;
  End;
End;

Procedure TIpCustomTerminal.tmDetachFromSocket;
Begin
  If Assigned(SockControl) Then
  Begin
    If IsMasterTerminal Then
      SockControl.MasterTerminal[Socket] := Nil;

    TIpSockControlFriend(SockControl).scDetachTerminal(Socket, Handle);
  End;
End;

Procedure TIpCustomTerminal.tmDrawDefaultText;
Const
  FormatMost = 'Design mode; line %.2d'^M^J;
  FormatLast = 'Design mode; line %.2d'^M;
  VT100Most = #27'[0;%dmDesign mode; line %.2d'^M^J;
  VT100Last = #27'[0;%dmDesign mode; line %.2d'^M;
Var
  i: Integer;
  S: String;
  VTColor: Integer;
Begin
  {ASSUMPTION: Handle is allocated}

  If (FByteQueue = Nil) Or
    (FEmulator.Buffer = Nil) Then
    Exit;

  For i := 1 To ScrollbackRows Do
    WriteString(^M^J);

      If (FEmulator Is TIpVT100Emulator) Then
      Begin
        VTColor := 31;
        For i := 1 To pred(Rows) Do
        Begin
          S := Format(VT100Most, [VTColor, i]);
          WriteString(S);
          inc(VTColor);
          If (VTColor = 38) Then
            VTColor := 31;
        End;
        S := Format(VT100Last, [VTColor, Rows]);
        WriteString(S);
      End
      Else
      Begin
        For i := 1 To pred(Rows) Do
        Begin
          S := Format(FormatMost, [i]);
          WriteString(S);
        End;
        S := Format(FormatLast, [Rows]);
        WriteString(S);
      End;
      Invalidate;
End;

Procedure TIpCustomTerminal.tmFreeCaret;
Begin
  If FCreatedCaret Then
  Begin
    DestroyCaret;
    FCreatedCaret := False;
  End;
End;

Function TIpCustomTerminal.tmGetAttributes(aRow, aCol: Integer): TIpTerminalCharAttrs;
Var
  AttrArray: PByteArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    AttrArray := FEmulator.Buffer.GetLineAttrPtr(aRow);
    Result := TIpTerminalCharAttrs(AttrArray^[pred(aCol)]);
  End
  Else
    Result := [];
End;

Function TIpCustomTerminal.tmGetBackColor(aRow, aCol: Integer): TColor;
Var
  ColorArray: PIptLongArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    ColorArray := FEmulator.Buffer.GetLineBackColorPtr(aRow);
    Result := TColor(ColorArray^[pred(aCol)]);
  End
  Else
    Result := ipc_TermBackColor;
End;

Function TIpCustomTerminal.tmGetCharSet(aRow, aCol: Integer): Byte;
Var
  CharSetArray: PByteArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    CharSetArray := FEmulator.Buffer.GetLineCharSetPtr(aRow);
    Result := CharSetArray^[pred(aCol)];
  End
  Else
    Result := 0;
End;

Function TIpCustomTerminal.tmGetColumns: Integer;
Begin
  If (FEmulator.Buffer <> Nil) Then
    Result := FEmulator.Buffer.ColCount
  Else
    Result := 0;
End;

Function TIpCustomTerminal.tmGetEmulator: TIpTerminalEmulator;
Begin
  If (FEmulator = FDefEmulator) Then
    Result := Nil
  Else
    Result := FEmulator;
End;

Procedure TIpCustomTerminal.tmGetFontInfo;
Var
  DC: HDC;
  i: Integer;
  SavedFontHandle: THandle;
  Metrics: TTextMetric;
  SizeChanged: Boolean;
  NewValue: Integer;
  TempFont: TFont;
  FontList: TStringList;
  {------}

  Procedure UpdateCharSizes;
  Begin
    With Metrics Do
    Begin
      {calculate the height}
      NewValue := tmHeight + tmExternalLeading;
      If (NewValue > FCharHeight) Then
      Begin
        SizeChanged := True;
        FCharHeight := NewValue;
      End;
      {if it's a fixed pitch font, use the average char width}
      If ((tmPitchAndFamily And TMPF_FIXED_PITCH) <> 0) Then
        NewValue := tmAveCharWidth
          {otherwise use the average char width, weighted towards the max
         char width}
      Else
        NewValue := tmAveCharWidth +
          (tmMaxCharWidth - tmAveCharWidth) Div 2;
      If (NewValue > FCharWidth) Then
      Begin
        SizeChanged := True;
        FCharWidth := NewValue;
      End;
    End;
  End;
  {------}
Begin
  If (FEmulator.CharSetMapping <> Nil) Then
  Begin
    FontList := TStringList.Create;
    Try
      FEmulator.CharSetMapping.GetFontNames(FontList);
    Except
      FontList.Free;
      Raise;
    End;
  End
  Else
    FontList := Nil;

  {get a DC, in order that we can get the font metrics for the current
   font; release the DC afterwards}
  DC := GetDC(0);
  SavedFontHandle := SelectObject(DC, Font.Handle);
  TempFont := Nil;
  FCharWidth := 0;
  FCharHeight := 0;
  Try
    {obtain the character cell height and width from the metrics}
    SizeChanged := False;
    If (FontList = Nil) Then
    Begin
      GetTextMetrics(DC, Metrics);
      UpdateCharSizes;
    End
    Else
    Begin
      TempFont := TFont.Create;
      TempFont.Assign(Font);
{$IFDEF Version3}
      TempFont.CharSet := DEFAULT_CHARSET;
{$ENDIF}
      For i := 0 To pred(FontList.Count) Do
      Begin
        If (FontList[i] = DefaultFontName) Then
          TempFont.Name := Font.Name
        Else
          TempFont.Name := FontList[i];
        SelectObject(DC, Font.Handle);
        GetTextMetrics(DC, Metrics);
        UpdateCharSizes;
      End;
    End;
    {if either the width or height changed, invalidate the display}
    If HandleAllocated Then
    Begin
      If SizeChanged Then
      Begin
        tmCalcExtent;
        Invalidate;
      End;
    End;
  Finally
    SelectObject(DC, SavedFontHandle);
    ReleaseDC(0, DC);
    FontList.Free;
    TempFont.Free;
  End;
End;

Function TIpCustomTerminal.tmGetForeColor(aRow, aCol: Integer): TColor;
Var
  ColorArray: PIptLongArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    ColorArray := FEmulator.Buffer.GetLineForeColorPtr(aRow);
    Result := TColor(ColorArray^[pred(aCol)]);
  End
  Else
    Result := clWhite;
End;

Function TIpCustomTerminal.tmGetIsMasterTerminal: Boolean;
Begin
  If Assigned(SockControl) And (Socket <> Invalid_Socket) Then
    Result := (SockControl.MasterTerminal[Socket] = Self)
  Else
    Result := False;
End;

Function TIpCustomTerminal.tmGetLine(aRow: Integer): String;
Var
  CharArray: PAnsiChar;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    CharArray := FEmulator.Buffer.GetLineCharPtr(aRow);
    SetLength(Result, FEmulator.Buffer.ColCount);
    Move(CharArray^, Result[1], FEmulator.Buffer.ColCount);
  End
  Else
    Result := '';
End;

Function TIpCustomTerminal.tmGetRows: Integer;
Begin
  If (FEmulator.Buffer <> Nil) Then
    Result := FEmulator.Buffer.RowCount
  Else
    Result := 0;
End;

Function TIpCustomTerminal.tmGetScrollbackRows: Integer;
Begin
  If (FEmulator.Buffer <> Nil) Then
    Result := FEmulator.Buffer.SVRowCount
  Else
    Result := 0;
End;

Function TIpCustomTerminal.tmGetTelnetTermSize: String;
Begin
  Result := Result + AnsiChar(Hi(Columns));
  Result := Result + AnsiChar(Lo(Columns));
  Result := Result + AnsiChar(Hi(Rows));
  Result := Result + AnsiChar(Lo(Rows));
End;

Procedure TIpCustomTerminal.tmGrowSelect(X, Y: Integer);
Var
  NewRow: Integer;
  NewCol: Integer;
  StartCol: Integer;
  ColCount: Integer;
  NewRgn: TRect;
  i: Integer;
Begin
  {if this gets called, we have a selection}
  FHaveSelection := True;

  {get the column count as a local variable: we'll be using it a lot}
  ColCount := FEmulator.Buffer.ColCount;

  {X and Y are mouse coordinates on the terminal display; convert to a
   row/col position}
  NewCol := (X Div CharWidth) + ClientOriginCol;
  NewCol := IpMinInt(IpMaxInt(NewCol, 1), ColCount);
  NewRow := (Y Div CharHeight) + ClientOriginRow;
  With FEmulator.Buffer Do
    NewRow := IpMinInt(IpMaxInt(NewRow, RowCount - SVRowCount), RowCount);

  {generate the new selected region}
  If (NewRow < FLButtonAnchor.Y) Then
  Begin
    NewRgn.Top := NewRow;
    NewRgn.Left := NewCol;
    NewRgn.Bottom := FLButtonAnchor.Y;
    NewRgn.Right := FLButtonAnchor.X;
  End
  Else
    If (NewRow > FLButtonAnchor.Y) Then
    Begin
      NewRgn.Top := FLButtonAnchor.Y;
      NewRgn.Left := FLButtonAnchor.X;
      NewRgn.Bottom := NewRow;
      NewRgn.Right := NewCol;
    End
    Else {NewRow and the anchor row are the same}
    Begin
      NewRgn.Top := NewRow;
      NewRgn.Bottom := NewRow;
      If (NewCol <= FLButtonAnchor.X) Then
      Begin
        NewRgn.Left := NewCol;
        NewRgn.Right := FLButtonAnchor.X;
      End
      Else
      Begin
        NewRgn.Left := FLButtonAnchor.X;
        NewRgn.Right := NewCol;
      End;
    End;

  {check to see how the current selection grew (we need to mark the
   new bits as selected) or shrank (the new bits need to be
   deselected)}
  If (NewRgn.Top = NewRgn.Bottom) Then
  Begin
    StartCol := FLButtonRect.Left;
    For i := FLButtonRect.Top To pred(FLButtonRect.Bottom) Do
    Begin
      tmMarkDeselected(i, StartCol, ColCount);
      StartCol := 1;
    End;
    tmMarkDeselected(FLButtonRect.Bottom, 1, FLButtonRect.Right);
    tmMarkSelected(NewRgn.Top, NewRgn.Left, NewRgn.Right);
  End
  Else
  Begin
    If (NewRgn.Top < FLButtonRect.Top) Then
    Begin
      {the selection grew upwards}
      StartCol := NewRgn.Left;
      For i := NewRgn.Top To pred(FLButtonRect.Top) Do
      Begin
        tmMarkSelected(i, StartCol, ColCount);
        StartCol := 1;
      End;
      tmMarkSelected(FLButtonRect.Top, 1, FLButtonRect.Left);
    End
    Else
      If (NewRgn.Top = FLButtonRect.Top) Then
      Begin
        {the selection might have changed on the top row}
        If (NewRgn.Left < FLButtonRect.Left) Then
          tmMarkSelected(NewRgn.Top, NewRgn.Left, FLButtonRect.Left)
        Else
          If (NewRgn.Left > FLButtonRect.Left) Then
            tmMarkDeselected(NewRgn.Top, FLButtonRect.Left, NewRgn.Left);
      End
      Else {new top > old top}
      Begin
        StartCol := FLButtonRect.Left;
        For i := FLButtonRect.Top To pred(NewRgn.Top) Do
        Begin
          tmMarkDeselected(i, StartCol, ColCount);
          StartCol := 1;
        End;
        tmMarkDeselected(NewRgn.Top, 1, NewRgn.Left);
      End;
    If (NewRgn.Bottom > FLButtonRect.Bottom) Then
    Begin
      {the selection grew downwards}
      StartCol := FLButtonRect.Right;
      tmMarkSelected(FLButtonRect.Bottom, FLButtonRect.Right, ColCount);
      For i := FLButtonRect.Bottom To pred(NewRgn.Bottom) Do
      Begin
        tmMarkSelected(i, StartCol, ColCount);
        StartCol := 1;
      End;
      tmMarkSelected(NewRgn.Bottom, 1, NewRgn.Right);
    End
    Else
      If (NewRgn.Bottom = FLButtonRect.Bottom) Then
      Begin
        {the selection might have changed on the bottom row}
        If (NewRgn.Right > FLButtonRect.Right) Then
          tmMarkSelected(NewRgn.Bottom, FLButtonRect.Right, NewRgn.Right)
        Else
          If (NewRgn.Right < FLButtonRect.Right) Then
            tmMarkDeselected(NewRgn.Bottom, NewRgn.Right, FLButtonRect.Right);
      End
      Else {new bottom < old bottom}
      Begin
        StartCol := NewRgn.Right;
        For i := NewRgn.Bottom To pred(FLButtonRect.Bottom) Do
        Begin
          tmMarkDeselected(i, StartCol, ColCount);
          StartCol := 1;
        End;
        tmMarkDeselected(FLButtonRect.Bottom, 1, FLButtonRect.Right);
      End;
  End;
  FLButtonRect := NewRgn;
End;

Procedure TIpCustomTerminal.tmHideCaret;
Begin
  If FShowingCaret Then
  Begin
    HideCaret(Handle);
    FShowingCaret := False;
  End;
End;

Procedure TIpCustomTerminal.tmInitHScrollBar;
Begin
  {Assumptions: FUseHScrollBar = True
                FClientFullCols < Columns}
  With FScrollHorzInfo Do
  Begin
    fMask := SIF_ALL;
    nMin := 0;
    nMax := Columns - 1;
    nPage := FClientFullCols;
    nPos := ClientOriginCol - 1;
  End;
  SetScrollInfo(Handle, SB_HORZ, FScrollHorzInfo, True);
End;

Procedure TIpCustomTerminal.tmInitVScrollBar;
Begin
  {Assumptions: FUseVScrollBar = True AND FClientFullRows < Rows
                OR ScrollbackRows > Rows}
  If FUseVScrollBar And (Not Scrollback) Then
  Begin
    With FScrollVertInfo Do
    Begin
      fMask := SIF_ALL;
      nMin := 0;
      nMax := Rows - 1;
      nPage := FClientFullRows;
      nPos := ClientOriginRow - 1;
    End;
  End
  Else
  Begin
    With FScrollVertInfo Do
    Begin
      fMask := SIF_ALL;
      nMin := Rows - ScrollbackRows;
      nMax := Rows - 1;
      nPage := FClientFullRows;
      nPos := ClientOriginRow - 1;
    End;
  End;
  //here............................................
  SetScrollInfo(Handle, SB_VERT, FScrollVertInfo, True);
End;

Procedure TIpCustomTerminal.tmMakeCaret;
Begin
  FCreatedCaret := True;
  If (CursorType = ctUnderline) Then
    CreateCaret(Handle, 0, CharWidth, 2)
  Else
    If (CursorType = ctBlock) Then
      CreateCaret(Handle, 0, CharWidth, CharHeight)
    Else
      FCreatedCaret := False;
End;

Procedure TIpCustomTerminal.tmInvalidateRow(aRow: Integer);
Var
  InvRect: TRect;
Begin
  If HandleAllocated And (ClientOriginRow <= aRow) And
    (aRow < ClientOriginRow + ClientRows) Then
  Begin
    InvRect.Left := 0;
    InvRect.Top := (aRow - ClientOriginRow) * CharHeight;
    InvRect.Right := ClientWidth;
    InvRect.Bottom := InvRect.Top + CharHeight;
    InvalidateRect(Handle, @InvRect, False);
  End;
End;

Procedure TIpCustomTerminal.tmMarkDeselected(aRow: Integer;
  aFromCol, aToCol: Integer);
Var
  AttrArray: PByteArray;
  i: Integer;
  CharAttrs: TIpTerminalCharAttrs;
Begin
  {get the attributes for this line}
  AttrArray := FEmulator.Buffer.GetLineAttrPtr(aRow);
  For i := pred(aFromCol) To pred(aToCol) Do
  Begin
    CharAttrs := TIpTerminalCharAttrs(AttrArray^[i]);
    Exclude(CharAttrs, tcaSelected);
    AttrArray^[i] := Byte(CharAttrs);
  End;
  tmInvalidateRow(aRow);
End;

Procedure TIpCustomTerminal.tmMarkSelected(aRow: Integer;
  aFromCol, aToCol: Integer);
Var
  AttrArray: PByteArray;
  i: Integer;
  CharAttrs: TIpTerminalCharAttrs;
Begin
  {get the attributes for this line}
  AttrArray := FEmulator.Buffer.GetLineAttrPtr(aRow);
  For i := pred(aFromCol) To pred(aToCol) Do
  Begin
    CharAttrs := TIpTerminalCharAttrs(AttrArray^[i]);
    Include(CharAttrs, tcaSelected);
    AttrArray^[i] := Byte(CharAttrs);
  End;
  tmInvalidateRow(aRow);
End;

Procedure TIpCustomTerminal.tmPositionCaret;
Var
  X, Y: Integer;
Begin
  With FEmulator.Buffer Do
  Begin
    If UseAbsAddress Then
    Begin
      X := (Col - FOriginCol) * CharWidth;
      Y := (Row - FOriginRow) * CharHeight;
    End
    Else
    Begin
      X := (Col + OriginCol - 1 - FOriginCol) * CharWidth;
      Y := (Row + OriginRow - 1 - FOriginRow) * CharHeight;
    End;
  End;
  If (CursorType = ctUnderline) Then
    inc(Y, CharHeight - 2);
  If (0 <= X) And (X < ClientWidth) And
    (0 <= Y) And (Y < ClientHeight) Then
  Begin
    ShowCaret(Handle);
    SetCaretPos(X, Y);
    FShowingCaret := True;
  End;
End;

Function TIpCustomTerminal.tmProcessClipboardCopy(
  Var Msg: TWMKeyDown): Boolean;
Const
  VK_C = $43;
Var
  Shift: TShiftState;
Begin
  {this method is called from the main keydown methods to check for
   Ctrl+C or Ctrl+Insert, so that the selected text can get copied to
   the clipboard}
  Result := False;
  {check whether we have a selection}
  If Not FHaveSelection Then
    Exit;
  {check whether the key is a C or an Insert}
  If (Msg.CharCode <> VK_C) And (Msg.CharCode <> VK_INSERT) Then
    Exit;
  {see if the Ctrl key is down and the shift and Alt keys are not}
  Shift := KeyDataToShiftState(Msg.KeyData);
  If (Not (ssCtrl In Shift)) Or
    (ssShift In Shift) Or (ssAlt In Shift) Then
    Exit;
  {we have the required keystroke and a selection: copy to clipboard}
  Result := True;
  CopyToClipboard;
End;

Procedure TIpCustomTerminal.tmScrollHorz(aDist: Integer);
Begin
  {aDist is +ve if we are scrolling to the right (ie the current
   window contents are moved to a position aDist pixels to the right),
   and is -ve for a leftward scroll}
  ScrollWindow(Handle, aDist, 0, Nil, Nil);
End;

Procedure TIpCustomTerminal.tmScrollVert(aDist: Integer);
Begin
  { aDist is +ve if we are scrolling downwards (ie the current window   }
  { contents are moved to a position aDist pixels down), and is -ve for }
  { a upward scroll }
  ScrollWindow(Handle, 0, aDist, Nil, Nil);
End;

Procedure TIpCustomTerminal.tmSetActive(aValue: Boolean);
Begin
  If (aValue <> Active) Then
  Begin
    FActive := aValue;
    { if we have a socket then either attach to or detach from it }
    If (SockControl <> Nil) And (Socket <> Invalid_Socket) Then
    Begin
      If Not (csDesigning In ComponentState) Then
        If Active Then
          tmAttachToSocket
        Else
          tmDetachFromSocket;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetAttributes(aRow, aCol: Integer;
  Const aAttr: TIpTerminalCharAttrs);
Var
  AttrArray: PByteArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    AttrArray := FEmulator.Buffer.GetLineAttrPtr(aRow);
    TIpTerminalCharAttrs(AttrArray^[pred(aCol)]) := aAttr;
  End;
End;

Procedure TIpCustomTerminal.tmSetBackColor(aRow, aCol: Integer;
  aValue: TColor);
Var
  ColorArray: PIptLongArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    ColorArray := FEmulator.Buffer.GetLineBackColorPtr(aRow);
    TColor(ColorArray^[pred(aCol)]) := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetBlinkTime(aValue: Integer);
Begin
  If (aValue <> BlinkTime) Then
  Begin
    If (aValue <= 0) Then
      aValue := 0
    Else
      If (aValue <= 250) Then
        aValue := 250;
    FBlinkTime := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetBorderStyle(aBS: TBorderStyle);
Begin
  If (aBS <> BorderStyle) Then
  Begin
    FBorderStyle := aBS;
    RecreateWnd;
  End;
End;

Procedure TIpCustomTerminal.tmSetCapture(aValue: TIpCaptureMode);
Begin
  If (aValue <> Capture) Then
  Begin
    If (aValue = cmAppend) And (Capture = cmOn) Then
      Exit;
    {if we're capturing now, close the stream}
    If (Capture = cmOn) Then
    Begin
      FCaptureStream.Free;
    End;
    {save the new value}
    FCapture := aValue;
    {if we should now be capturing, open the stream}
    If (Capture <> cmOff) Then
    Begin
      If (CaptureFile = '') Then
        FCaptureFile := ipc_TermCaptureFile;
      If Not (csDesigning In ComponentState) Then
      Begin
        If (Capture = cmOn) Then
          FCaptureStream := TFileStream.Create(CaptureFile, fmCreate)
        Else
          If (Capture = cmAppend) Then
          Begin
            FCaptureStream := TFileStream.Create(CaptureFile, fmOpenReadWrite);
            FCaptureStream.Seek(0, soFromEnd);
            FCapture := cmOn;
          End;
      End;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetCaptureFile(Const aValue: String);
Var
  WasCapturing: Boolean;
Begin
  If (aValue <> CaptureFile) Then
  Begin
    {first turn off capturing if needed}
    If (Capture = cmOff) Then
      WasCapturing := False
    Else
    Begin
      WasCapturing := True;
      Capture := cmOff;
    End;
    {save the new filename}
    FCaptureFile := aValue;
    {now turn capturing back on}
    If WasCapturing Then
      Capture := cmOn;
  End;
End;

Procedure TIpCustomTerminal.tmSetCharSet(aRow, aCol: Integer; aValue: Byte);
Var
  CharSetArray: PByteArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    CharSetArray := FEmulator.Buffer.GetLineCharSetPtr(aRow);
    CharSetArray^[pred(aCol)] := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetColumns(aValue: Integer);
Var
  OldValue: Integer;
Begin
  If (FEmulator.Buffer <> Nil) And
    (aValue > 0) Then
  Begin
    OldValue := FEmulator.Buffer.ColCount;
    If (OldValue <> aValue) Then
    Begin
      FEmulator.Buffer.ColCount := aValue;
      FOriginCol := 1;
      FOriginRow := 1;
      tmCalcExtent;
      tmTermResize;
      Invalidate;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetCursorType(aValue: TIpCursorType);
Begin
  If (aValue <> CursorType) Then
  Begin
    tmHideCaret;
    tmFreeCaret;
    FCursorType := aValue;
    tmMakeCaret;
    tmShowCaret;
  End;
End;

Procedure TIpCustomTerminal.tmSetEmulator(aValue: TIpTerminalEmulator);
Var
  OldRowCount: Integer;
  OldColCount: Integer;
  OldSVRowCount: Integer;
Begin
  If (aValue <> Emulator) Then
  Begin
    If HandleAllocated Then
      FHeartbeat.Enabled := False;
    {if we were attached to an emulator, remove its link}
    OldSVRowCount := FEmulator.Buffer.SVRowCount;
    OldRowCount := FEmulator.Buffer.RowCount;
    OldColCount := FEmulator.Buffer.ColCount;
    If (Emulator <> Nil) Then
    Begin
      FEmulator := Nil; {to stop recursion with the next call}
      RemoveTermEmuLink(Self, True);
    End;
    {attach ourselves to the new emulator}
    If (aValue = Nil) Then
      FEmulator := FDefEmulator
    Else
    Begin
      FEmulator := aValue;
      AddTermEmuLink(Self, Emulator);
    End;
    {set the new emulator up}
    If (OldSVRowCount > 0) Then
    Begin
      FEmulator.Buffer.SVRowCount := OldSVRowCount;
      FEmulator.Buffer.RowCount := OldRowCount;
      FEmulator.Buffer.ColCount := OldColCount;
    End;
    If HandleAllocated Then
    Begin
      FHeartbeat.Enabled := True;
      If (csDesigning In ComponentState) Then
        tmDrawDefaultText;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetForeColor(aRow, aCol: Integer;
  aValue: TColor);
Var
  ColorArray: PIptLongArray;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    ColorArray := FEmulator.Buffer.GetLineForeColorPtr(aRow);
    TColor(ColorArray^[pred(aCol)]) := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetLazyByteDelay(aValue: Integer);
Begin
  If (0 < aValue) And (aValue <= 1024) And
    (aValue <> LazyByteDelay) Then
  Begin
    FLazyByteDelay := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetLazyTimeDelay(aValue: Integer);
Begin
  If (0 < aValue) And (aValue <= 1000) And
    (aValue <> LazyTimeDelay) Then
  Begin
    FLazyTimeDelay := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetTermWidth(aValue: Integer);
Begin
  FTermWidth := aValue;
End;

Procedure TIpCustomTerminal.tmSetTermHeight(aValue: Integer);
Begin
  FTermHeight := aValue;
End;

Procedure TIpCustomTerminal.tmSetLine(aRow: Integer; Const aValue: String);
Var
  CharArray: PAnsiChar;
  StLen: Integer;
  i: Integer;
Begin
  If (FEmulator.Buffer <> Nil) Then
  Begin
    CharArray := FEmulator.Buffer.GetLineCharPtr(aRow);
    StLen := length(aValue);
    If (StLen > FEmulator.Buffer.ColCount) Then
      Move(aValue[1], CharArray[0], FEmulator.Buffer.ColCount)
    Else
    Begin
      Move(aValue[1], CharArray[0], StLen);
      For i := StLen To pred(FEmulator.Buffer.ColCount) Do
        CharArray[i] := ' ';
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetOriginCol(aValue: Integer);
Var
  MaxOriginCol: Integer;
  OldOrigin: Integer;
Begin
  If (aValue <> ClientOriginCol) Then
  Begin
    {work out the maximum value}
    MaxOriginCol := Columns - FClientFullCols + 1;
    If (MaxOriginCol < 1) Then
      MaxOriginCol := 1;
    {save the old value}
    OldOrigin := FOriginCol;
    {set the new value}
    If (aValue < 1) Then
      FOriginCol := 1
    Else
      If (aValue > MaxOriginCol) Then
        FOriginCol := MaxOriginCol
      Else
        FOriginCol := aValue;
    {scroll the window, set the scrollbar position}
    If (OldOrigin <> ClientOriginCol) Then
    Begin
      tmScrollHorz((OldOrigin - ClientOriginCol) * CharWidth);
      tmCalcExtent;

      With FScrollHorzInfo Do
      Begin
        fMask := SIF_POS;
        nPos := ClientOriginCol - 1;
      End;
      SetScrollInfo(Handle, SB_HORZ, FScrollHorzInfo, True);
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetOriginRow(aValue: Integer);
Var
  MinOriginRow: Integer;
  MaxOriginRow: Integer;
  OldOrigin: Integer;
Begin
  If (aValue <> ClientOriginRow) Then
  Begin
    {work out the minimum value}
    If FUseVScrollBar And (Not Scrollback) Then
      MinOriginRow := 1
    Else
      MinOriginRow := Rows - ScrollbackRows + 1;
    {work out the maximum value}
    MaxOriginRow := Rows - FClientFullRows + 1;
    If (MaxOriginRow < 1) Then
      MaxOriginRow := 1;
    {save the old value}
    OldOrigin := FOriginRow;
    {set the new value}
    If (aValue < MinOriginRow) Then
      FOriginRow := MinOriginRow
    Else
      If (aValue > MaxOriginRow) Then
        FOriginRow := MaxOriginRow
      Else
        FOriginRow := aValue;
    {scroll the window, set the scrollbar position}
    If (OldOrigin <> ClientOriginRow) Then
    Begin
      tmScrollVert((OldOrigin - ClientOriginRow) * CharHeight);
      tmCalcExtent;

      With FScrollVertInfo Do
      Begin
        fMask := SIF_POS;
        nPos := ClientOriginRow - 1;
      End;

      // This happens when the vertical scrollbar is moved manually
      SetScrollInfo(Handle, SB_VERT, FScrollVertInfo, True);
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetRows(aValue: Integer);
Var
  OldValue: Integer;
Begin
  If (FEmulator.Buffer <> Nil) And
    (aValue > 0) Then
  Begin
    OldValue := FEmulator.Buffer.RowCount;
    If (OldValue <> aValue) Then
    Begin
      FEmulator.Buffer.RowCount := aValue;
      tmCalcExtent;
      tmTermResize;
      Invalidate;
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetScrollback(aValue: Boolean);
Begin
  If (aValue <> Scrollback) Then
  Begin
    FScrollback := aValue;
    tmInitVScrollBar;
  End;
End;

Procedure TIpCustomTerminal.tmSetScrollbackRows(aValue: Integer);
Begin
  If (FEmulator.Buffer <> Nil) And (aValue > 0) Then
    FEmulator.Buffer.SVRowCount := aValue;
End;

Procedure TIpCustomTerminal.tmSetSockControl(aValue: TIpSockControl);
Begin
  If (aValue <> SockControl) Then
  Begin
    { if we don't yet have a handle, just save the new value }
    If Not HandleAllocated Then
    Begin
      { Make sure Socket is reset }
      tmDetachFromSocket;
      { Store new value }
      FSockControl := aValue;
    End
    Else
    Begin
      { otherwise we need to do some registration }
        { detach/deregister from the old socket & sockcontrol }
      If (SockControl <> Nil) And Not (csDesigning In ComponentState) Then
      Begin
        tmDetachFromSocket;
        SockControl.DeregisterTerminal(Handle);
      End;
      { save the new value }
      FSockControl := aValue;
      { register with the new SockControl }
      If (SockControl <> Nil) And Not (csDesigning In ComponentState) Then
        SockControl.RegisterTerminal(Handle);
    End;
  End;
End;

Procedure TIpCustomTerminal.tmSetSocket(aValue: TSocket);
Begin
  If (aValue <> Socket) Then
  Begin
    { Detach from old socket }
    tmDetachFromSocket;
    { Store value }
    FSocket := aValue;
    { Attach to new socket if necessary }
    If Active And Not (csDesigning In ComponentState) Then
      tmAttachToSocket;
  End;
End;

Procedure TIpCustomTerminal.tmSetUseLazyDisplay(aValue: Boolean);
Begin
  FUseLazyDisplay := aValue;
End;

Procedure TIpCustomTerminal.tmSetWantAllKeys(aValue: Boolean);
Begin
  If (aValue <> WantAllKeys) Then
  Begin
    FWantAllKeys := aValue;
  End;
End;

Procedure TIpCustomTerminal.tmSetFreezeScrollback(v: boolean); {!!.03}
Begin {!!.03}
  If (v <> FFreezeScrollback) Then {!!.03}
    FFreezeScrollback := v; {!!.03}
End; {!!.03}

Procedure TIpCustomTerminal.tmShowCaret;
Begin
  If (Not FShowingCaret) And FCreatedCaret And
    (Not (csDesigning In ComponentState)) And
    Focused Then
    tmPositionCaret;
End;

Procedure TIpCustomTerminal.tmStartSelect(X, Y: Integer);
Begin
  { clear the current selection }
  If (FLButtonRect.Left > 0) And (FLButtonRect.Top > 0) And {!!.01}
    (FLButtonRect.Bottom > 0) And (FLButtonRect.Right > 0) Then {!!.01}
    HideSelection;

  { X and Y are mouse coordinates on the terminal display; convert to a }
  { row/col position }
  FLButtonAnchor.X := (X Div CharWidth) + ClientOriginCol;
  FLButtonAnchor.Y := (Y Div CharHeight) + ClientOriginRow;

  { reset the current selection }
  FLButtonRect.Left := FLButtonAnchor.X;
  FLButtonRect.Top := FLButtonAnchor.Y;
  FLButtonRect.Right := FLButtonAnchor.X;
  FLButtonRect.Bottom := FLButtonAnchor.Y;
End;

Procedure TIpCustomTerminal.tmTermResize;
Begin
  If Assigned(SockControl) And (Socket <> Invalid_Socket) Then
    PostMessage(SockControl.Handle, CM_IPTERMRESIZE, Socket, 0);
End;

Procedure TIpCustomTerminal.WMCancelMode(Var Msg: TMessage);
Begin
  HideSelection;
  FLButtonDown := False;
End;

Procedure TIpCustomTerminal.WMCopy(Var Msg: TMessage);
Begin
  CopyToClipboard;
  Msg.Result := 1;
End;

Procedure TIpCustomTerminal.WMEraseBkgnd(Var Msg: TMessage);
Begin
  Msg.Result := 1
End;

Procedure TIpCustomTerminal.WMGetDlgCode(Var Msg: TMessage);
Begin
  {we want as many keys as we can get for the emulatation}
  If WantAllKeys Then
    Msg.Result := DLGC_WANTALLKEYS +
      DLGC_WANTARROWS +
      DLGC_WANTCHARS +
      DLGC_WANTMESSAGE +
      DLGC_WANTTAB
  Else
    Inherited;
End;

Procedure TIpCustomTerminal.WMHScroll(Var Msg: TWMScroll);
Var
  PageSize: Integer;
  NewPos: Integer;
  MaxOriginCol: Integer;
Begin
  PageSize := FClientFullCols;
  MaxOriginCol := Columns - PageSize + 1;
  Case Msg.ScrollCode Of
    SB_LINELEFT: NewPos := ClientOriginCol - 1;
    SB_LINERIGHT: NewPos := ClientOriginCol + 1;
    SB_PAGELEFT: NewPos := ClientOriginCol - PageSize;
    SB_PAGERIGHT: NewPos := ClientOriginCol + PageSize;
    SB_THUMBPOSITION: NewPos := Msg.Pos + 1;
    SB_THUMBTRACK: NewPos := Msg.Pos + 1;
  Else
    Exit; {ignore it}
  End;
  If (NewPos < 1) Then
    NewPos := 1
  Else
    If (NewPos > MaxOriginCol) Then
      NewPos := MaxOriginCol;
  If (NewPos <> ClientOriginCol) Then
    ClientOriginCol := NewPos;
End;

Procedure TIpCustomTerminal.WMKillFocus(Var Msg: TWMSetFocus);
Begin
  tmHideCaret;
  tmFreeCaret;
  Inherited;
End;

Procedure TIpCustomTerminal.WMPaint(Var Msg: TWMPaint);
Begin
  PaintHandler(Msg);
End;

Procedure TIpCustomTerminal.WMSetFocus(Var Msg: TWMSetFocus);
Begin
  Inherited;
  If Not FCreatedCaret Then
    tmHideCaret;
  tmMakeCaret;
  If Not FShowingCaret Then
    tmShowCaret;
End;

Procedure TIpCustomTerminal.WMSize(Var Msg: TWMSize);
Begin
  If (CharWidth > 0) Then
  Begin
    { calculate the new values for the extent of the window }
    tmCalcExtent;
    { if the current origin column is not 1 and we can show more of the }
    { terminal if it were less, then do so }
    If (ClientOriginCol > 1) Then
    Begin
      If ((ClientCols * CharWidth) < ClientWidth) Then
        ClientOriginCol := Columns - (ClientWidth Div CharWidth) + 1;
    End;
    { if the current origin row is not 1 and we can show more of the }
    { terminal if it were less, then do so }
    If (ClientOriginRow > 1) Then
    Begin
      If ((ClientRows * CharHeight) < ClientHeight) Then
        ClientOriginRow := Rows - (ClientHeight Div CharHeight) + 1;
    End;
  End;
End;

Procedure TIpCustomTerminal.WMVScroll(Var Msg: TWMScroll);
Var
  PageSize: Integer;
  NewPos: Integer;
  MinOriginRow: Integer;
  MaxOriginRow: Integer;
Begin
  {if we've a vertical scrollbar because the window is smaller than
   the terminal screen...}
  If FUseVScrollBar Then
  Begin
    {if we're not currently going through the scrollback buffer and
     the current origin row is 1 and the user clicks the up arrow on
     the scrollbar, auto switch into scrollback mode}
    If (Not Scrollback) Then
    Begin
      If (Msg.ScrollCode = SB_LINEUP) And (ClientOriginRow = 1) Then
        Scrollback := True;
    End
      {if we're currently going through the scrollback buffer and the
       current origin row is greater than 1, auto switch into
       non-scrollback mode}
    Else {Scrollback is True}
    Begin
      If (ClientOriginRow > 1) Then
        Scrollback := False;
    End;
  End;

  PageSize := FClientFullRows;
  If FUseVScrollBar And (Not Scrollback) Then
    MinOriginRow := 1
  Else
    MinOriginRow := Rows - ScrollbackRows - 1;
  MaxOriginRow := Rows - PageSize + 1;
  If (MaxOriginRow < 1) Then
    MaxOriginRow := 1;
  Case Msg.ScrollCode Of
    SB_LINEUP: NewPos := ClientOriginRow - 1;
    SB_LINEDOWN: NewPos := ClientOriginRow + 1;
    SB_PAGEUP: NewPos := ClientOriginRow - PageSize;
    SB_PAGEDOWN: NewPos := ClientOriginRow + PageSize;
    SB_THUMBPOSITION: NewPos := Msg.Pos + 1;
    SB_THUMBTRACK: NewPos := Msg.Pos + 1;
  Else
    Exit; {if anything else, ignore it}
  End;
  If (NewPos < MinOriginRow) Then
    NewPos := MinOriginRow
  Else
    If (NewPos > MaxOriginRow) Then
      NewPos := MaxOriginRow;
  If (NewPos <> ClientOriginRow) Then
    ClientOriginRow := NewPos;
End;

Procedure TIpCustomTerminal.WriteChar(aCh: AnsiChar);
Begin
  {stuff the data into the byte queue}
  TaaByteQueue(FByteQueue).Put(aCh, sizeof(AnsiChar));
  {tell ourselves that we have more data}
  PostMessage(Handle, CM_IPTERMSTUFF, 0, 0);
End;

Procedure TIpCustomTerminal.WriteString(Const aSt: String);
Begin
  {stuff the data into the byte queue}
  TaaByteQueue(FByteQueue).Put(aSt[1], length(aSt));
  {tell ourselves that we have more data}
  PostMessage(Handle, CM_IPTERMSTUFF, 0, 0);
End;
{====================================================================}

{===TIpTerminalEmulator==============================================}

Constructor TIpTerminalEmulator.Create(aOwner: TComponent);
Begin
  Inherited Create(aOwner);
End;

Destructor TIpTerminalEmulator.Destroy;
Begin
  If Not FIsDefault Then
    Terminal := Nil;
  Inherited Destroy;
End;

Procedure TIpTerminalEmulator.BlinkPaint(aVisible: Boolean);
Begin
  {do nothing}
End;

Function TIpTerminalEmulator.HasBlinkingText: Boolean;
Begin
  Result := False;
End;

Procedure TIpTerminalEmulator.KeyDown(Var Key: Word; Shift: TShiftState);
Begin
  {do nothing}
End;

Procedure TIpTerminalEmulator.KeyPress(Var Key: AnsiChar);
Begin
  {do nothing}
End;

Procedure TIpTerminalEmulator.LazyPaint;
Begin
  {do nothing}
End;

Procedure TIpTerminalEmulator.Notification(AComponent: TComponent;
  Operation: TOperation);
Begin
  If (Operation = opRemove) And (AComponent = Terminal) Then
  Begin
    FTerminal := Nil;
  End;
  If (Operation = opInsert) And
    (AComponent Is TIpTerminal) And
    (Terminal = Nil) Then
  Begin
    Terminal := TIpTerminal(AComponent);
  End;
  Inherited Notification(AComponent, Operation);
End;

Procedure TIpTerminalEmulator.Paint;
Begin
  {do nothing}
End;

Procedure TIpTerminalEmulator.ProcessBlock(aData: Pointer; aDataLen: Longint);
Begin
  {do nothing}
End;

Procedure TIpTerminalEmulator.teClear;
Begin
  {do nothing};
End;

Procedure TIpTerminalEmulator.teClearAll;
Begin
  {do nothing};
End;

Function TIpTerminalEmulator.teGetNeedsUpdate: Boolean;
Begin
  Result := Buffer.HasDisplayChanged Or Buffer.HasCursorMoved; {!!.01}
End;

Procedure TIpTerminalEmulator.teHandleCursorMovement(Sender: TObject; {!!.01}
  Row: Integer; {!!.01}
  Col: Integer); {!!.01}
{ HandleCursorMovement

  Method is called by the Terminal Buffer's OnCursorMoved event}
Begin {!!.01}
  If assigned(FTerminal) Then
  Begin {!!.01}
    If assigned(FTerminal.OnCursorMoved) Then {!!.01}
      FTerminal.OnCursorMoved(Sender, Row, Col); {!!.01}
  End; {!!.01}
End; {!!.01}

Procedure TIpTerminalEmulator.teSendChar(aCh: char; aCanEcho: Boolean);
Begin
  { Assumptions: aCh is a printable character or is one of the standard }
  { non-printable characters }
  If (Terminal <> Nil) Then
  Begin
    If aCanEcho And Terminal.HalfDuplex Then
      ProcessBlock(@aCh, 1);
    If (Terminal.SockControl <> Nil) And (Terminal.Socket <> Invalid_Socket) Then
      TIpSockControlFriend(Terminal.SockControl).scPutBlock(Terminal.Socket, aCh, SizeOf(aCh), False);
  End;
End;

Procedure TIpTerminalEmulator.teSetTerminal(aValue: TIpCustomTerminal);
Var
  OldTerminal: TIpCustomTerminal;
Begin
  If (aValue <> Terminal) Then
  Begin
    {if we were attached to a terminal, remove its link}
    If (Terminal <> Nil) And (Not FIsDefault) Then
    Begin
      OldTerminal := Terminal;
      FTerminal := Nil; {to stop recursion in the following call}
      RemoveTermEmuLink(OldTerminal, True);
    End;
    FTerminal := aValue;
    {attach ourselves to the new terminal}
    If (Terminal <> Nil) And (Not FIsDefault) Then
    Begin
      If (Terminal.Emulator <> Nil) And (Terminal.Emulator <> Self) Then
        RemoveTermEmuLink(Terminal, True);
      AddTermEmuLink(Terminal, Self);
    End;
  End;
End;

Constructor TIpTTYEmulator.Create(aOwner: TComponent);
Begin
  {Note: the buffer *must* be created before the ancestor can perform
         initialization. The reason is that at design time dropping an
         emulator on the form will cause a series of Notification
         calls to take place. This in turn could cause a terminal's
         tmSetEmulator method to be called, which would then set up
         some default text in the emulator's buffer.}

  {create the buffer}
  FTerminalBuffer := TIpTerminalBuffer.Create(False);
  FTelnetTermType := 'dumb';

  {now let the ancestor do his stuff}
  Inherited Create(aOwner);

  {set up the terminal buffer and ourselves}
  FDisplayStrSize := 256; {enough for standard terminals}
  GetMem(FDisplayStr, FDisplayStrSize);
  GetMem(FCellWidths, 255 * sizeof(Integer));
  FillChar(FCellWidths^, 255 * sizeof(Integer), 0);
  FTerminalBuffer.OnCursorMoved := teHandleCursorMovement; {!!.01}
End;

Destructor TIpTTYEmulator.Destroy;
Begin
  {free the paint node free list}
  ttyFreeAllPaintNodes;
  {free the cell widths array}
  If (FCellWidths <> Nil) Then
    FreeMem(FCellWidths, 255 * SizeOf(Integer));
  {free the display string}
  If (FDisplayStr <> Nil) Then
    FreeMem(FDisplayStr, FDisplayStrSize);
  {free the internal objects}
  FTerminalBuffer.Free;
  Inherited Destroy;
End;

Procedure TIpTTYEmulator.KeyPress(Var Key: AnsiChar);
Begin
  {send it to the host}
  If (Key = ^H) Then
    Key := #127;
  teSendChar(Key, True);
End;

Procedure TIpTTYEmulator.LazyPaint;
Var
  DirtyRect: TRect;
  Row: Integer;
Begin
  { the LazyPaint method is called by the terminal, if and only if      }
  { either of the lazy timers expired; in other words, either a certain }
  { amount of data has been received by the terminal, or a certain time }
  { has elapsed since the last update                                   }

  If (Terminal.Color <> Buffer.BackColor) Or
    (Terminal.Font.Color <> Buffer.ForeColor) Then
  Begin
    Buffer.DefBackColor := Terminal.Color;
    Buffer.DefForeColor := Terminal.Font.Color;
    Buffer.BackColor := Terminal.Color;
    Buffer.ForeColor := Terminal.Font.Color;
    FRefresh := True;
  End;

  { if we have to refresh the display, do it, and throw away all the }
  { 'dirty' data }
  If FRefresh Then
  Begin
    FRefresh := False;
    For Row := 1 To Buffer.RowCount Do
      ttyDrawChars(Row, 1, Buffer.ColCount, True);
    While Buffer.GetInvalidRect(DirtyRect) Do {nothing}
  End
  Else
  Begin
    { using the 'dirty' data in the buffer, draw the required characters }
    While Buffer.GetInvalidRect(DirtyRect) Do
    Begin
      If Not Terminal.FreezeScrollBack Then
      Begin {!!.03}
        Inc(DirtyRect.Top, Terminal.FScrollVertInfo.nPos); {!!.03}
        Inc(DirtyRect.Bottom, Terminal.FScrollVertInfo.nPos + 1); {!!.03}
      End; {!!.03}
      For Row := DirtyRect.Top To DirtyRect.Bottom Do
        ttyDrawChars(Row, DirtyRect.Left, DirtyRect.Right, True);
    End;
  End;
End;

Procedure TIpTTYEmulator.Paint;
Var
  DirtyRect: TRect;
  Row: Integer;
Begin
  { the Paint method is called by the terminal, if and only if the }
  { terminal component received a WM_PAINT message }

  If (Terminal.Color <> Buffer.BackColor) Or
    (Terminal.Font.Color <> Buffer.ForeColor) Then
  Begin
    Buffer.DefBackColor := Terminal.Color;
    Buffer.DefForeColor := Terminal.Font.Color;
    Buffer.BackColor := Terminal.Color;
    Buffer.ForeColor := Terminal.Font.Color;
    FRefresh := true;
  End;

  {repaint the clip region}
  DirtyRect := Terminal.Canvas.ClipRect;
  With Terminal Do
  Begin
    DirtyRect.Left :=
      (DirtyRect.Left Div CharWidth) + ClientOriginCol;
    DirtyRect.Right :=
      (pred(DirtyRect.Right) Div CharWidth) + ClientOriginCol;
    DirtyRect.Top :=
      (DirtyRect.Top Div CharHeight) + ClientOriginRow;
    DirtyRect.Bottom :=
      (pred(DirtyRect.Bottom) Div CharHeight) + ClientOriginRow;
  End;
  If Not Terminal.FreezeScrollBack Then
  Begin {!!.03}
    Inc(DirtyRect.Top, Terminal.FScrollVertInfo.nPos); {!!.03}
    Inc(DirtyRect.Bottom, Terminal.FScrollVertInfo.nPos + 1); {!!.03}
  End; {!!.03}
  For Row := DirtyRect.Top To DirtyRect.Bottom Do
    ttyDrawChars(Row, DirtyRect.Left, DirtyRect.Right, True);
End;

Procedure TIpTTYEmulator.ProcessBlock(aData: Pointer; aDataLen: Longint);
Var
  DataAsChar: PAnsiChar Absolute aData;
  i: Integer;
  Ch: AnsiChar;
Begin
  For i := 0 To pred(aDataLen) Do
  Begin
    Ch := DataAsChar[i];
    If (Ch < ' ') Then
      ttyProcessCommand(Ch)
    Else
      Buffer.WriteChar(DataAsChar[i]);
  End;
End;

Procedure TIpTTYEmulator.teClear;
Begin
  Buffer.EraseScreen;
End;

Procedure TIpTTYEmulator.teClearAll;
Begin
  Buffer.EraseAll;
End;

Function TIpTTYEmulator.teGetNeedsUpdate: Boolean;
Begin
  Result := Buffer.HasDisplayChanged Or FRefresh Or {!!.01}
    Buffer.HasCursorMoved; {!!.01}
End;

Procedure TIpTTYEmulator.teSetTerminal(aValue: TIpCustomTerminal);
Begin
  Inherited teSetTerminal(aValue);
End;

Procedure TIpTTYEmulator.ttyDrawChars(aRow, aStartCol, aEndCol: Integer;
  aVisible: Boolean);
Var
  ColNum: Integer;
  StartColNum: Integer;
  BackColor: TColor;
  ForeColor: TColor;
  Attr: TIpTerminalCharAttrs;
  ForeColors: PIptLongArray;
  BackColors: PIptLongArray;
  Attrs: PByteArray;
  Script: PPaintNode;
  PaintNode: PPaintNode;
Begin
  {ASSUMPTION: aStartCol <= aEndCol}

  {avoid any drawing if the row simply is not visible}
  If (aRow < Terminal.ClientOriginRow) Or
    (aRow >= Terminal.ClientOriginRow + Terminal.ClientRows) Then
    Exit;
  {same if the range of columns is not visible}
  If (aEndCol < Terminal.ClientOriginCol) Or
    (aStartCol >= Terminal.ClientOriginCol + Terminal.ClientCols) Then
    Exit;

  {if this point is reached, we have to paint *something*}

  {force the parameter values in range}
  If (aStartCol < Terminal.ClientOriginCol) Then
    aStartCol := Terminal.ClientOriginCol;
  If (aEndCol > Terminal.ClientOriginCol + Terminal.ClientCols) Then
    aEndCol := Terminal.ClientOriginCol + Terminal.ClientCols;

  {this is the main processing: in general we'll be displaying text in
   this section; what will happen here, is that we first generate a
   script of drawing commands (which background color, which text
   color, which attributes, which text), and then we execute the
   script}

  {get the pointers to the colors, the attributes}
  BackColors := Buffer.GetLineBackColorPtr(aRow);
  ForeColors := Buffer.GetLineForeColorPtr(aRow);
  Attrs := Buffer.GetLineAttrPtr(aRow);

  {get the initial values for the display variables}
  BackColor := BackColors^[aStartCol - 1];
  ForeColor := ForeColors^[aStartCol - 1];
  Attr := TIpTerminalCharAttrs(Attrs^[aStartCol - 1]);

  {make a note of the start column}
  StartColNum := aStartCol;
  ColNum := aStartCol;

  {build the script as a stack of paint commands}
  Script := Nil;
  While (ColNum < aEndCol) Do
  Begin
    {look at the next column}
    inc(ColNum);
    {if any info has changed...}
    If (ForeColor <> ForeColors^[ColNum - 1]) Or
      (BackColor <> BackColors^[ColNum - 1]) Or
      (Attr <> TIpTerminalCharAttrs(Attrs^[ColNum - 1])) Then
    Begin
      {get a new node, initialize it}
      PaintNode := ttyNewPaintNode;
      PaintNode^.pnStart := StartColNum;
      PaintNode^.pnEnd := pred(ColNum);
      PaintNode^.pnFore := ForeColor;
      PaintNode^.pnBack := BackColor;
      PaintNode^.pnAttr := Attr;
      PaintNode^.pnCSet := 0;
      {add it to the script}
      PaintNode^.pnNext := Script;
      Script := PaintNode;
      {save the new values of the variables}
      ForeColor := ForeColors^[ColNum - 1];
      BackColor := BackColors^[ColNum - 1];
      Attr := TIpTerminalCharAttrs(Attrs^[ColNum - 1]);
      StartColNum := ColNum;
    End;
  End;
  {create the final paint command}
  PaintNode := ttyNewPaintNode;
  PaintNode^.pnStart := StartColNum;
  PaintNode^.pnEnd := aEndCol;
  PaintNode^.pnFore := ForeColor;
  PaintNode^.pnBack := BackColor;
  PaintNode^.pnAttr := Attr;
  PaintNode^.pnCSet := 0;
  {add it to the script}
  PaintNode^.pnNext := Script;
  Script := PaintNode;

  {now execute the paint script}
  ttyExecutePaintScript(aRow, Script);
End;

Procedure TIpTTYEmulator.ttyExecutePaintScript(aRow: Integer;
  aScript: Pointer);
Var
  Canvas: TCanvas;
  Font: TFont;
  Walker, Temp: PPaintNode;
  ForeColor: TColor;
  BackColor: TColor;
  CharWidth: Integer;
  CharHeight: Integer;
  OriginCol: Integer;
  TextStrLen: Integer;
  TextChars: PAnsiChar;
  WorkRect: TRect;
  Y: Integer;
  R, G, B: Integer;
  Reversed: Boolean;
Begin
  {get some values as local variables}
  Canvas := Terminal.Canvas;
  Font := Terminal.Font;
  TextChars := Buffer.GetLineCharPtr(aRow);
  CharHeight := Terminal.CharHeight;
  WorkRect.Top := (aRow - Terminal.ClientOriginRow) * CharHeight;
  WorkRect.Bottom := WorkRect.Top + CharHeight;
  CharWidth := Terminal.CharWidth;
  OriginCol := Terminal.ClientOriginCol;
  {set the cell widths}
  For Y := 0 To pred(Buffer.ColCount) Do
    FCellWidths^[Y] := CharWidth;
  {process the script}
  Walker := PPaintNode(aScript);
  While (Walker <> Nil) Do
  Begin
    {check for reverse}
    Reversed := (tcaReverse In Walker^.pnAttr) Xor
      (tcaSelected In Walker^.pnAttr);
    If Reversed Then
    Begin
      ForeColor := Walker^.pnBack;
      BackColor := Walker^.pnFore;
    End
    Else
    Begin
      ForeColor := Walker^.pnFore;
      BackColor := Walker^.pnBack;
    End;
    {check for invisible}
    If (tcaInvisible In Walker^.pnAttr) Then
      ForeColor := BackColor
        {check for bold}
    Else
      If (tcaBold In Walker^.pnAttr) Then
      Begin
        If Reversed Then
        Begin
          R := IpMinInt(Integer(GetRValue(BackColor)) + $80, $FF);
          G := IpMinInt(Integer(GetGValue(BackColor)) + $80, $FF);
          B := IpMinInt(Integer(GetBValue(BackColor)) + $80, $FF);
          BackColor := RGB(R, G, B);
        End
        Else
        Begin
          R := IpMinInt(Integer(GetRValue(ForeColor)) + $80, $FF);
          G := IpMinInt(Integer(GetGValue(ForeColor)) + $80, $FF);
          B := IpMinInt(Integer(GetBValue(ForeColor)) + $80, $FF);
          ForeColor := RGB(R, G, B);
        End;
      End;

    {get the length of the text to display}
    TextStrLen := succ(Walker^.pnEnd - Walker^.pnStart);

    {move the required text to our display string}
    Move(TextChars[Walker^.pnStart - 1], FDisplayStr^, TextStrLen);
    FDisplayStr[TextStrLen] := #0;

    {set the correct background}
    Canvas.Brush.Color := BackColor;

    {calculate the left and right values for the rect}
    WorkRect.Left := (Walker^.pnStart - OriginCol) * CharWidth;
    WorkRect.Right := WorkRect.Left + (TextStrLen * CharWidth);

    {display the bit o'text}
    Canvas.Font := Font;
    Canvas.Font.Color := ForeColor;

    ExtTextOut(Canvas.Handle,
      WorkRect.Left,
      WorkRect.Top,
      ETO_OPAQUE,
      @WorkRect,
      FDisplayStr,
      TextStrLen,
      @FCellWidths^);

    {finally, draw the underline and/or strike through}
    Canvas.Pen.Color := ForeColor;
    If (tcaUnderline In Walker^.pnAttr) Then
    Begin
      Y := WorkRect.Bottom - 2;
      Canvas.MoveTo(WorkRect.Left, Y);
      Canvas.LineTo(WorkRect.Right, Y);
    End;
    If (tcaStrikeThrough In Walker^.pnAttr) Then
    Begin
      Y := WorkRect.Top + (WorkRect.Bottom - WorkRect.Top) Div 2;
      Canvas.MoveTo(WorkRect.Left, Y);
      Canvas.LineTo(WorkRect.Right, Y);
    End;

    {walk to the next paint node, free this one}
    Temp := Walker;
    Walker := Walker^.pnNext;
    ttyFreePaintNode(Temp);
  End;
End;

Procedure TIpTTYEmulator.ttyFreeAllPaintNodes;
Var
  Walker, Temp: PPaintNode;
Begin
  Walker := FPaintFreeList;
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.pnNext;
    Dispose(Temp);
  End;
  FPaintFreeList := Nil;
End;

Procedure TIpTTYEmulator.ttyFreePaintNode(aNode: Pointer);
Begin
  PPaintNode(aNode)^.pnNext := FPaintFreeList;
  FPaintFreeList := aNode;
End;

Function TIpTTYEmulator.ttyNewPaintNode: Pointer;
Begin
  If (FPaintFreeList = Nil) Then
    New(PPaintNode(Result))
  Else
  Begin
    Result := FPaintFreeList;
    FPaintFreeList := PPaintNode(Result)^.pnNext;
  End;
End;

Procedure TIpTTYEmulator.ttyProcessCommand(aCh: AnsiChar);
Begin
  {Assumption: aCh is less than space, i.e., is one of the unprintable
               characters}
  Case aCh Of
    ^G: {bell}
      Begin
        MessageBeep(MB_ICONASTERISK);
      End;
    ^H: {backspace}
      Begin
        Buffer.DoBackspace;
      End;
    ^I: {tab}
      Begin
        Buffer.DoHorzTab;
      End;
    ^J: {linefeed}
      Begin
        Buffer.DoLineFeed;
      End;
    ^M: {carriage return}
      Begin
        Buffer.DoCarriageReturn;
      End;
  End; {case}
End;

Constructor TIpVT100Emulator.Create(aOwner: TComponent);
Begin
  {Note: the buffer *must* be created before the ancestor can perform
         initialization. The reason is that at design time dropping an
         emulator on the form will cause a series of Notification
         calls to take place. This in turn could cause a terminal's
         tmSetEmulator method to be called, which would then set up
         some default text in the emulator's buffer.}

  {create the buffer and parser and keyboard mapper}
  FTerminalBuffer := TIpTerminalBuffer.Create(False);
  FParser := TIpVT100Parser.Create(False);
  FKeyboardMapping := TIpKeyboardMapping.Create;
  FCharSetMapping := TIpCharSetMapping.Create;

  {now let the ancestor do his stuff}
  Inherited Create(aOwner);

  {set up our default values}
  ANSIMode := ipc_VT100ANSIMode;
  AppKeyMode := ipc_VT100AppKeyMode;
  AppKeypadMode := ipc_VT100AppKeypadMode;
  AutoRepeat := ipc_VT100AutoRepeat;
  FCol132Mode := ipc_VT100Col132Mode;
  GPOMode := ipc_VT100GPOMode;
  InsertMode := ipc_VT100InsertMode;
  Interlace := ipc_VT100Interlace;
  NewLineMode := ipc_VT100NewLineMode;
  FRelOriginMode := ipc_VT100RelOriginMode;
  FRevScreenMode := ipc_VT100RevScreenMode;
  SmoothScrollMode := ipc_VT100SmoothScrollMode;
  WrapAround := ipc_VT100WrapAround;

  FAnswerback := ipc_VT100Answerback;
  FG0CharSet := ipc_VT100G0CharSet;
  FG1CharSet := ipc_VT100G1CharSet;
  FTelnetTermType := 'vt100';

  FDisplayStrSize := 133; {enough for both 80- and 132-column mode}
  GetMem(FDisplayStr, FDisplayStrSize);

  {make sure we're told about scrolling rows so we can track double
   height and double width lines}
  FLineAttrArray := TIpLineAttrArray.Create(Self);
  FTerminalBuffer.OnScrollRows := vttScrollRowsHandler;

  {make sure the mappers are initialized}
  FKeyboardMapping.LoadFromRes(hInstance, 'IPVT100KeyMap'); {!!.10}
  FCharSetMapping.LoadFromRes(hInstance, 'IPVT100CharSet'); {!!.10}

  {initialize the secondary font--a cache to help charset switches}
  FSecondaryFont := TFont.Create;
  If (Terminal <> Nil) Then
    FSecondaryFont.Assign(Terminal.Font);
  GetMem(FCellWidths, 132 * sizeof(Integer));
  FillChar(FCellWidths^, 132 * sizeof(Integer), 0);
  FTerminalBuffer.OnCursorMoved := teHandleCursorMovement; {!!.01}
End;

Destructor TIpVT100Emulator.Destroy;
Begin
  {clear the current blink script, free the blink node free list}
  vttClearBlinkScript;
  vttFreeAllBlinkNodes;
  {free the paint node free list}
  vttFreeAllPaintNodes;
  FreeMem(FCellWidths, 132 * sizeof(Integer));
  {free the display string}
  If (FDisplayStr <> Nil) Then
    FreeMem(FDisplayStr, FDisplayStrSize);
  {free the internal objects}
  FSecondaryFont.Free;
  FLineAttrArray.Free;
  FKeyboardMapping.Free;
  FCharSetMapping.Free;
  FParser.Free;
  FTerminalBuffer.Free;
  Inherited Destroy;
End;

Procedure TIpVT100Emulator.BlinkPaint(aVisible: Boolean);
Var
  Walker: PBlinkNode;
Begin
  {the BlinkPaint method is called by the terminal, if and only if
   the blink timer expired; in other words, the blinking text must be
   displayed in the opposite sense, on or off}

  {read all the nodes in the blink linked list, redraw them}
  Walker := PBlinkNode(FBlinkers);
  While (Walker <> Nil) Do
    With Walker^ Do
    Begin
      vttDrawChars(bnRow, bnStartCh, bnEndCh, aVisible, True);
      Walker := bnNext;
    End;
End;

Function TIpVT100Emulator.HasBlinkingText: Boolean;
Begin
  Result := FBlinkers <> Nil;
End;

Procedure TIpVT100Emulator.KeyDown(Var Key: Word; Shift: TShiftState);

Function CvtHexChar(Const S: String; aInx: Integer): AnsiChar;
  Var
    Hex: String[3];
    Value: Integer;
    ec: Integer;
  Begin
    Hex := '$  ';
    Hex[2] := S[aInx];
    Hex[3] := S[aInx + 1];
    Val(Hex, Value, ec);
    If (ec = 0) Then
      Result := chr(Value)
    Else
      Result := '?';
  End;
Var
  VKKey: String;
  VTKey: String;
  VTKeyString: String;
  i: Integer;
  EscChar: Boolean;
  HexChar1: Boolean;
  HexChar2: Boolean;
  IsEscSeq: Boolean;
  IsRepeat: Boolean;
  IsNumLock: Boolean;
  Ch: AnsiChar;
Begin
  {check for a repeated key and AutoRepeat is off}
  IsRepeat := (Key And $8000) <> 0;
  If IsRepeat Then
  Begin
    If Not AutoRepeat Then
    Begin
      Key := 0;
      Exit;
    End;
    Key := Key And $7FFF;
  End;
  {make a note in case we hit the numlock key}
  IsNumLock := Key = VK_NUMLOCK;
  {we need to convert this keystroke into a VT100 string to pass back
   to the server; this is a three step process...}
  {first, convert the keystroke to its name}
  VKKey := Format('\x%.2x', [Key]);
  VKKey := KeyboardMapping.Get(VKKey);
  {if we can continue, add the shift state in the order shift, ctrl,
   then alt}
  If (VKKey = '') Then
    Exit;
  If ssAlt In Shift Then
    VKKey := 'alt+' + VKKey;
  If ssCtrl In Shift Then
    VKKey := 'ctrl+' + VKKey;
  If ssShift In Shift Then
    VKKey := 'shift+' + VKKey;
  {now convert this into a DEC VT100 keyname}
  VTKey := KeyboardMapping.Get(VKKey);
  {if we managed to convert it to a VT100 key, modify the name to
   include the application/cursor key mode}
  If (VTKey = '') Then
    Exit;
  {if this is a cursor key the name ends in 'c', replace it with 0, 1,
   or 2, depending on whether we're in VT52 mode, ANSI mode with
   cursor key mode reset, or ANSI mode with cursor key mode set}
  If (VTKey[length(VTKey)] = 'c') Then
  Begin
    If Not ANSIMode Then
      VTKey[length(VTKey)] := '0'
    Else
      If Not AppKeyMode Then
        VTKey[length(VTKey)] := '1'
      Else
        VTKey[length(VTKey)] := '2';
  End
    {if this is a Keypad key the name ends in 'k', replace it with 3, 4,
     5, or 6, depending on whether we're in VT52 numeric mode, VT52
     application mode, ANSI numeric mode, or ANSI application mode}
  Else
    If (VTKey[length(VTKey)] = 'k') Then
    Begin
      If Not ANSIMode Then
        If Not AppKeypadMode Then
          VTKey[length(VTKey)] := '3'
        Else
          VTKey[length(VTKey)] := '4'
      Else
        If Not AppKeypadMode Then
          VTKey[length(VTKey)] := '5'
        Else
          VTKey[length(VTKey)] := '6';
    End;
  {now get the string we need to send to the host for this key}
  VTKeyString := KeyboardMapping.Get(VTKey);
  If (VTKeyString <> '') Then
  Begin
    IsEscSeq := False;
    EscChar := False;
    HexChar1 := False;
    HexChar2 := False;
    {interpret the key string}
    For i := 1 To length(VTKeyString) Do
    Begin
      {get the next character}
      Ch := VTKeyString[i];
      {if the previous character was a '\' then we're reading an
       escaped character, either of the form \e or \xhh}
      If EscChar Then
      Begin
        If (Ch = 'e') Then
        Begin
          teSendChar(#27, False);
          IsEscSeq := True;
        End
        Else
          If (Ch = 'x') Then
          Begin
            HexChar1 := True;
            Ch := CvtHexChar(VTKeyString, i + 1);
            If (Ch = #13) Then
            Begin
              teSendChar(Ch, True);
              If NewLineMode Then
                teSendChar(#10, True);
            End
            Else
              If (Ch = #27) Then
              Begin
                teSendChar(#27, False);
                IsEscSeq := True;
              End
              Else
                teSendChar(Ch, Not IsEscSeq);
          End
          Else {it's not \e or \xhh}
          Begin
            teSendChar('\', True);
            teSendChar(Ch, True);
          End;
        EscChar := False;
      End
        {if the previous character was the x in \xhh, ignore this one:
         we've already interpreted it with the \x}
      Else
        If HexChar1 Then
        Begin
          HexChar1 := False;
          HexChar2 := True;
        End
          {if the previous character was the first h in \xhh, ignore this
           one: we've already interpreted it with the \x}
        Else
          If HexChar2 Then
          Begin
            HexChar2 := False;
          End
            {if this character is a '\' we're starting an escaped character}
          Else
            If (Ch = '\') Then
              EscChar := True
                {otherwise there's nothing special about this character}
            Else
              teSendChar(Ch, Not IsEscSeq);
    End;
    {we've now fully processed the key, so make sure it doesn't come
     back to bite us}
    Key := 0;
    {if this was the NumLock key, we'd better set it on again}
    If IsNumLock Then
    Begin
      vttToggleNumLock;
    End;
  End;
End;

Procedure TIpVT100Emulator.KeyPress(Var Key: AnsiChar);
Begin
  {on a key press, send the key to the host; for a CR character we
   either send CR (NewLineMode is OFF) or a CR/LF (NewLineMode is on}
  If (Key = #13) Then
  Begin
    teSendChar(Key, True);
    If NewLineMode Then
      teSendChar(#10, True);
  End
  Else
    teSendChar(Key, True);
End;

Procedure TIpVT100Emulator.LazyPaint;
Var
  DirtyRect: TRect;
  Row: Integer;
Begin
  {the LazyPaint method is called by the terminal, if and only if
   either of the lazy timers expired; in other words, either a certain
   amount of data has been received by the terminal, or a certain time
   has elapsed since the last update}

  {if we have to refresh the display, do it, and throw away all the
   'dirty' data}
  If FRefresh Then
  Begin
    FRefresh := False;
    For Row := 1 To Buffer.RowCount Do
      vttDrawChars(Row, 1, Buffer.ColCount, True, False);
    While Buffer.GetInvalidRect(DirtyRect) Do {nothing}
  End
  Else
  Begin
    {using the 'dirty' data in the buffer, draw the required
     characters}
    While Buffer.GetInvalidRect(DirtyRect) Do
    Begin
      If Not Terminal.FreezeScrollBack Then
      Begin {!!.03}
        Inc(DirtyRect.Top, Terminal.FScrollVertInfo.nPos); {!!.03}
        Inc(DirtyRect.Bottom, Terminal.FScrollVertInfo.nPos + 1); {!!.03}
      End; {!!.03}
      For Row := DirtyRect.Top To DirtyRect.Bottom Do
        vttDrawChars(Row, DirtyRect.Left, DirtyRect.Right, True, False);
    End;
  End;
End;

Procedure TIpVT100Emulator.Paint;
Var
  DirtyRect: TRect;
  Row: Integer;
Begin
  {the Paint method is called by the terminal, if and only if the
   terminal component received a WM_PAINT message}

  {repaint the clip region}
  DirtyRect := Terminal.Canvas.ClipRect;
  With Terminal Do
  Begin
    DirtyRect.Left :=
      (DirtyRect.Left Div CharWidth) + ClientOriginCol;
    DirtyRect.Right :=
      (pred(DirtyRect.Right) Div CharWidth) + ClientOriginCol;
    DirtyRect.Top :=
      (DirtyRect.Top Div CharHeight) + ClientOriginRow;
    DirtyRect.Bottom :=
      (pred(DirtyRect.Bottom) Div CharHeight) + ClientOriginRow;
  End;
  If Not Terminal.FreezeScrollBack Then
  Begin {!!.03}
    Inc(DirtyRect.Top, Terminal.FScrollVertInfo.nPos); {!!.03}
    Inc(DirtyRect.Bottom, Terminal.FScrollVertInfo.nPos + 1); {!!.03}
  End; {!!.03}
  For Row := DirtyRect.Top To DirtyRect.Bottom Do
    vttDrawChars(Row, DirtyRect.Left, DirtyRect.Right, True, False);
End;

Procedure TIpVT100Emulator.ProcessBlock(aData: Pointer; aDataLen: Longint);
Var
  DataAsChar: PChar Absolute aData;
  i: Integer;
Begin
  For i := 0 To pred(aDataLen) Do
  Begin
    Case Parser.ProcessChar(DataAsChar[i]) Of
      pctNone:
        {ignore it};
      pctChar:
        Begin
          {Terminal.ComPort.AddStringToLog(DataAsChar[i]);}
          Buffer.WriteChar(DataAsChar[i]);
        End;
      pct8BitChar:
        Begin
          If DisplayUpperASCII Then
            Buffer.WriteChar(DataAsChar[i])
          Else
            vttProcess8bitChar(DataAsChar[i]);
        End;
      pctPending:
        {nothing to do yet};
      pctComplete:
        vttProcessCommand;
    End;
  End;
  vttCalcBlinkScript;
End;

Procedure TIpVT100Emulator.teClear;
Begin
  Buffer.EraseScreen;
End;

Procedure TIpVT100Emulator.teClearAll;
Begin
  Buffer.EraseAll;
End;

Function TIpVT100Emulator.teGetNeedsUpdate: Boolean;
Begin
  Result := Buffer.HasDisplayChanged Or FRefresh Or {!!.01}
    Buffer.HasCursorMoved; {!!.01}
End;

Procedure TIpVT100Emulator.teSetTerminal(aValue: TIpCustomTerminal);
Begin
  Inherited teSetTerminal(aValue);
  If (aValue <> Nil) And (aValue.Font <> Nil) And
    (FSecondaryFont <> Nil) Then
    FSecondaryFont.Assign(aValue.Font);
End;

Procedure TIpVT100Emulator.vttCalcBlinkScript;
Var
  RowInx: Integer;
  ColInx: Integer;
  StartColInx: Integer;
  Temp: PBlinkNode;
  Attrs: TIpTerminalCharAttrs;
  AttrArray: PByteArray;
  InBlinkRegion: Boolean;
Begin
  If Buffer.HasDisplayChanged Then
  Begin
    {clear the current script of blink regions}
    vttClearBlinkScript;
    {search for the current blink regions in the buffer}
    InBlinkRegion := False;
    StartColInx := 0;
    With Buffer Do
      For RowInx := Terminal.ClientOriginRow To
        pred(Terminal.ClientOriginRow + RowCount) Do
      Begin
        AttrArray := GetLineAttrPtr(RowInx);
        For ColInx := 0 To pred(ColCount) Do
        Begin
          Attrs := TIpTerminalCharAttrs(AttrArray^[ColInx]);
          If InBlinkRegion Then
          Begin
            If Not (tcaBlink In Attrs) Then
            Begin
              InBlinkRegion := False;
              Temp := vttNewBlinkNode;
              Temp^.bnRow := RowInx;
              Temp^.bnStartCh := succ(StartColInx);
              Temp^.bnEndCh := ColInx;
              Temp^.bnNext := PBlinkNode(FBlinkers);
              FBlinkers := Temp;
            End;
          End
          Else {not tracking blink region}
          Begin
            If (tcaBlink In Attrs) Then
            Begin
              InBlinkRegion := True;
              StartColInx := ColInx;
            End;
          End;
        End;
      End;

    {close off the final blink region if there is one}
    If InBlinkRegion Then
    Begin
      Temp := vttNewBlinkNode;
      Temp^.bnRow := Buffer.RowCount;
      Temp^.bnStartCh := succ(StartColInx);
      Temp^.bnEndCh := Buffer.ColCount;
      Temp^.bnNext := PBlinkNode(FBlinkers);
      FBlinkers := Temp;
    End;
  End;
End;

Procedure TIpVT100Emulator.vttClearBlinkScript;
Var
  Walker, Temp: PBlinkNode;
Begin
  Walker := PBlinkNode(FBlinkers);
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.bnNext;
    vttFreeBlinkNode(Temp);
  End;
  FBlinkers := Nil;
End;

Procedure TIpVT100Emulator.vttDrawBlinkOffCycle(
  aRow, aStartCh, aEndCh: Integer);
Var
  ChNum: Integer;
  BackColor: TColor;
  BackColors: PIptLongArray;
  WorkRect: TRect;
  CharWidth: Integer;
  OriginCh: Integer;
Begin
  {get the character width}
  If (TIpLineAttrArray(FLineAttrArray).Attr[aRow] = ltNormal) Then
  Begin
    CharWidth := Terminal.CharWidth;
    OriginCh := Terminal.ClientOriginCol;
  End
  Else
  Begin
    CharWidth := Terminal.CharWidth * 2;
    OriginCh := ((Terminal.ClientOriginCol - 1) Div 2) + 1;
  End;

  {get the pointers to the fore/background colors}
  If RevScreenMode Then
    BackColors := Buffer.GetLineForeColorPtr(aRow)
  Else
    BackColors := Buffer.GetLineBackColorPtr(aRow);

  {just paint the background}
  With Terminal.Canvas Do
  Begin
    {we have to display the background color in separate steps since
     different characters across the display may have different
     background colors; set to the loop variables}
    BackColor := BackColors^[aStartCh - 1];
    ChNum := aStartCh;
    WorkRect :=
      Rect((aStartCh - OriginCh) * CharWidth,
      (aRow - Terminal.ClientOriginRow) * Terminal.CharHeight,
      0, {to be set in the loop}
      (aRow - Terminal.ClientOriginRow + 1) * Terminal.CharHeight);
    While (ChNum < aEndCh) Do
    Begin
      inc(ChNum);
      {if the background color changes, draw the rect in the old
       color, and then save the new color and start point}
      If (BackColor <> BackColors^[ChNum - 1]) Then
      Begin
        WorkRect.Right := (ChNum - OriginCh) * CharWidth;
        Brush.Color := BackColor;
        FillRect(WorkRect);
        BackColor := BackColors^[ChNum - 1];
        WorkRect.Left := WorkRect.Right;
      End;
    End;
    {finish off the last rect in the final color}
    WorkRect.Right := (aEndCh - OriginCh + 1) * CharWidth;
    Brush.Color := BackColor;
    FillRect(WorkRect);
  End;
End;

Procedure TIpVT100Emulator.vttDrawChars(aRow, aStartVal, aEndVal: Integer;
  aVisible: Boolean;
  aCharValues: Boolean);
Var
  ChNum: Integer;
  StartChNum: Integer;
  StartChar: Integer;
  EndChar: Integer;
  BackColor: TColor;
  ForeColor: TColor;
  Attr: TIpTerminalCharAttrs;
  CharSet: Byte;
  ForeColors: PIptLongArray;
  BackColors: PIptLongArray;
  Attrs: PByteArray;
  CharSets: PByteArray;
  Script: PPaintNode;
  PaintNode: PPaintNode;
  LineAttr: TIpVT100LineAttr;
Begin
  {ASSUMPTION: aStartVal <= aEndVal}

  {Notes: The terminal display can be viewed in two ways: either a
          certain number of characters across, or a certain number of
          columns across. When each character is exactly one column
          (or cell) in width it doesn't matter whether we talk about
          characters or columns. If the characters are double width,
          it *does* matter, since 1 character unit is then equivalent
          to two column units. aCharValues True means that aStartVal
          and aEndVal are character values; False means that they are
          column values. In the latter case, if the row consists of
          double-width characters then we'll have to convert the
          column values to character values.}

  {avoid any drawing if the row simply is not visible}
  If (aRow < Terminal.ClientOriginRow) Or
    (aRow >= Terminal.ClientOriginRow + Terminal.ClientRows) Then
    Exit;

  {get the line attribute}
  Try
    LineAttr := TIpLineAttrArray(FlineAttrArray).Attr[aRow];
  Except
    //zzz rlm
    LineAttr := ltNormal;
  End;

  {if the start/end parameters are column values, convert to character
   values}
  If Not aCharValues Then
  Begin
    If (LineAttr <> ltNormal) Then
    Begin
      aStartVal := ((aStartVal - 1) Div 2) + 1;
      aEndVal := ((aEndVal - 1) Div 2) + 1;
    End;
  End;

  {check to see whether the range of characters is visible, force
   values into visible range}
  If (LineAttr = ltNormal) Then
  Begin
    StartChar := Terminal.ClientOriginCol;
    EndChar := Terminal.ClientOriginCol + Terminal.ClientCols;
  End
  Else {double-width}
  Begin
    StartChar := ((Terminal.ClientOriginCol - 1) Div 2) + 1;
    EndChar := ((Terminal.ClientOriginCol + Terminal.ClientCols - 1)
      Div 2) + 1;
  End;
  If (aEndVal < StartChar) Or (aStartVal >= EndChar) Then
    Exit;
  aStartVal := IpMaxInt(aStartVal, StartChar);
  aEndVal := IpMinInt(aEndVal, EndChar);

  {if this point is reached, we have to paint *something*}

  {there will be calls to this method that are just drawing blinking
   characters for the 'off' cycle (aVisible = False), so make this a
   special case}
  If Not aVisible Then
  Begin
    vttDrawBlinkOffCycle(aRow, aStartVal, aEndVal);
    Exit;
  End;

  {if this row is the bottom half of a double-height row we don't have
   to display any text since it is displayed with the top half}
  If (LineAttr = ltDblHeightBottom) Then
    dec(aRow);

  {this is the main processing: in general we'll be displaying text in
   this section; what will happen here, is that we first generate a
   script of drawing commands (which background color, which text
   color, which attributes, which character set, which text), and then
   we execute the script}

  {get the pointers to the colors, the attributes, the charsets}
  BackColors := Buffer.GetLineBackColorPtr(aRow);
  ForeColors := Buffer.GetLineForeColorPtr(aRow);
  Attrs := Buffer.GetLineAttrPtr(aRow);
  CharSets := Buffer.GetLineCharSetPtr(aRow);

  {get the initial values for the display variables}
  BackColor := BackColors^[aStartVal - 1];
  ForeColor := ForeColors^[aStartVal - 1];
  Attr := TIpTerminalCharAttrs(Attrs^[aStartVal - 1]);
  CharSet := CharSets^[aStartVal - 1];

  {make a note of the start char number}
  StartChNum := aStartVal;
  ChNum := aStartVal;

  {build the script as a stack of paint commands}
  Script := Nil;
  While (ChNum < aEndVal) Do
  Begin
    {look at the next char number}
    inc(ChNum);
    {if any info has changed...}
    If (ForeColor <> ForeColors^[ChNum - 1]) Or
      (BackColor <> BackColors^[ChNum - 1]) Or
      (Attr <> TIpTerminalCharAttrs(Attrs^[ChNum - 1])) Or
      (CharSet <> CharSets^[ChNum - 1]) Then
    Begin
      {get a new node, initialize it}
      PaintNode := vttNewPaintNode;
      PaintNode^.pnStart := StartChNum;
      PaintNode^.pnEnd := pred(ChNum);
      PaintNode^.pnFore := ForeColor;
      PaintNode^.pnBack := BackColor;
      PaintNode^.pnAttr := Attr;
      PaintNode^.pnCSet := CharSet;
      {add it to the script}
      PaintNode^.pnNext := Script;
      Script := PaintNode;
      {save the new values of the variables}
      ForeColor := ForeColors^[ChNum - 1];
      BackColor := BackColors^[ChNum - 1];
      Attr := TIpTerminalCharAttrs(Attrs^[ChNum - 1]);
      CharSet := CharSets^[ChNum - 1];
      StartChNum := ChNum;
    End;
  End;
  {create the final paint command}
  PaintNode := vttNewPaintNode;
  PaintNode^.pnStart := StartChNum;
  PaintNode^.pnEnd := aEndVal;
  PaintNode^.pnFore := ForeColor;
  PaintNode^.pnBack := BackColor;
  PaintNode^.pnAttr := Attr;
  PaintNode^.pnCSet := CharSet;
  {add it to the script}
  PaintNode^.pnNext := Script;
  Script := PaintNode;

  {now execute the paint script}
  vttExecutePaintScript(aRow, Script);
End;

Procedure TIpVT100Emulator.vttExecutePaintScript(aRow: Integer;
  aScript: Pointer);
Var
  Canvas: TCanvas;
  Font: TFont;
  Walker, Temp: PPaintNode;
  ForeColor: TColor;
  BackColor: TColor;
  CharWidth: Integer;
  CharHeight: Integer;
  OriginCol: Integer;
  OffsetCol: Integer;
  TextStrLen: Integer;
  TextChars: PAnsiChar;
  PartTextLen: Integer;
  TextCol: Integer;
  FontName: TIpKeyString;
  WorkRect: TRect;
  Y: Integer;
  R, G, B: Integer;
  Reversed: Boolean;
  DblHeight: Boolean;
Begin
  Case TIpLineAttrArray(FLineAttrArray).Attr[aRow] Of
    ltNormal: DblHeight := False;
    ltDblHeightTop: DblHeight := True;
    ltDblHeightBottom: DblHeight := True;
    ltDblWidth: DblHeight := False;
  Else
    DblHeight := False;
  End;
  {get some values as local variables}
  Canvas := Terminal.Canvas;
  Font := Terminal.Font;
  TextChars := Buffer.GetLineCharPtr(aRow);
  CharHeight := Terminal.CharHeight;
  WorkRect.Top := (aRow - Terminal.ClientOriginRow) * CharHeight;
  If DblHeight Then
    WorkRect.Bottom := WorkRect.Top + (CharHeight * 2)
  Else
    WorkRect.Bottom := WorkRect.Top + CharHeight;
  If (TIpLineAttrArray(FLineAttrArray).Attr[aRow] = ltNormal) Then
  Begin
    CharWidth := Terminal.CharWidth;
    OriginCol := Terminal.ClientOriginCol;
    OffsetCol := 0;
  End
  Else
  Begin
    CharWidth := Terminal.CharWidth * 2;
    OriginCol := ((Terminal.ClientOriginCol - 1) Div 2) + 1;
    If Odd(Terminal.ClientOriginCol) Then
      OffsetCol := 0
    Else
      OffsetCol := -Terminal.CharWidth;
  End;
  {set the cell widths}
  For Y := 0 To pred(Buffer.ColCount) Do
    FCellWidths^[Y] := CharWidth;
  {process the script}
  Walker := PPaintNode(aScript);
  While (Walker <> Nil) Do
  Begin
    {check for reverse}
    Reversed := RevScreenMode Xor
      (tcaReverse In Walker^.pnAttr) Xor
      (tcaSelected In Walker^.pnAttr);
    If Reversed Then
    Begin
      ForeColor := Walker^.pnBack;
      BackColor := Walker^.pnFore;
    End
    Else
    Begin
      ForeColor := Walker^.pnFore;
      BackColor := Walker^.pnBack;
    End;
    {check for invisible}
    If (tcaInvisible In Walker^.pnAttr) Then
      ForeColor := BackColor
        {check for bold}
    Else
      If (tcaBold In Walker^.pnAttr) Then
      Begin
        If Reversed Then
        Begin
          R := IpMinInt(Integer(GetRValue(BackColor)) + $80, $FF);
          G := IpMinInt(Integer(GetGValue(BackColor)) + $80, $FF);
          B := IpMinInt(Integer(GetBValue(BackColor)) + $80, $FF);
          BackColor := RGB(R, G, B);
        End
        Else
        Begin
          R := IpMinInt(Integer(GetRValue(ForeColor)) + $80, $FF);
          G := IpMinInt(Integer(GetGValue(ForeColor)) + $80, $FF);
          B := IpMinInt(Integer(GetBValue(ForeColor)) + $80, $FF);
          ForeColor := RGB(R, G, B);
        End;
      End;

    {get the length of the text to display}
    TextStrLen := succ(Walker^.pnEnd - Walker^.pnStart);

    {move the required text to our display string}
    Move(TextChars[Walker^.pnStart - 1], FDisplayStr^, TextStrLen);
    FDisplayStr[TextStrLen] := #0;

    {set the correct background}
    Canvas.Brush.Color := BackColor;

    {ask the charset mapping to create a draw script for this text}
    Try
      FCharSetMapping.GenerateDrawScript(VT100CharSetNames[Walker^.pnCSet], FDisplayStr);
    Except
      //ShowMessage(
      //  'Error:'+#13+#10+
      //  'Unit=IpTerm.pas;'+#13+#10+
      //  'Proc=TIpVT100Emulator.vttExecutePaintScript;'+#13+#10+
      //  'line=FCharSetMapping.GenerateDrawScript(VT100..');
    End;
    {while the charset mapping passes us back draw commands, draw}
    TextCol := Walker^.pnStart - OriginCol;
    While FCharSetMapping.GetNextDrawCommand(FontName, FDisplayStr) Do
    Begin
      {calculate the length of this bit o' text}
      PartTextLen := StrLen(FDisplayStr);

      { calculate the left and right values for the rect }
      WorkRect.Left := (TextCol * CharWidth) + OffsetCol;
      WorkRect.Right := WorkRect.Left + (PartTextLen * CharWidth);

      { display the bit o'text }
      If (FontName = DefaultFontName) Then
        Canvas.Font := Font
      Else
      Begin
        FSecondaryFont.Name := FontName;
        Canvas.Font := FSecondaryFont;
      End;
      Canvas.Font.Color := ForeColor;
{$IFDEF VERSION3}
      Canvas.Font.CharSet := DEFAULT_CHARSET;
{$ENDIF}
      If DblHeight Then
        Canvas.Font.Size := Canvas.Font.Size * 2;

      ExtTextOut(Canvas.Handle,
        WorkRect.Left,
        WorkRect.Top,
        ETO_OPAQUE,
        @WorkRect,
        FDisplayStr,
        PartTextLen,
        @FCellWidths^);

      { finally, draw the underline and/or strike through }
      Canvas.Pen.Color := ForeColor;
      If (tcaUnderline In Walker^.pnAttr) Then
      Begin
        Y := WorkRect.Bottom - 2;
        Canvas.MoveTo(WorkRect.Left, Y);
        Canvas.LineTo(WorkRect.Right, Y);
      End;
      If (tcaStrikeThrough In Walker^.pnAttr) Then
      Begin
        Y := WorkRect.Top + (WorkRect.Bottom - WorkRect.Top) Div 2;
        Canvas.MoveTo(WorkRect.Left, Y);
        Canvas.LineTo(WorkRect.Right, Y);
      End;

      {advance past this bit o'text}
      inc(TextCol, PartTextLen);
    End;

    {walk to the next paint node, free this one}
    Temp := Walker;
    Walker := Walker^.pnNext;
    vttFreePaintNode(Temp);
  End;
End;

Procedure TIpVT100Emulator.vttFreeAllBlinkNodes;
Var
  Walker, Temp: PBlinkNode;
Begin
  Walker := FBlinkFreeList;
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.bnNext;
    Dispose(Temp);
  End;
  FBlinkFreeList := Nil;
End;

Procedure TIpVT100Emulator.vttFreeAllPaintNodes;
Var
  Walker, Temp: PPaintNode;
Begin
  Walker := FPaintFreeList;
  While (Walker <> Nil) Do
  Begin
    Temp := Walker;
    Walker := Walker^.pnNext;
    Dispose(Temp);
  End;
  FPaintFreeList := Nil;
End;

Procedure TIpVT100Emulator.vttFreeBlinkNode(aNode: Pointer);
Begin
  PBlinkNode(aNode)^.bnNext := FBlinkFreeList;
  FBlinkFreeList := aNode;
End;

Procedure TIpVT100Emulator.vttFreePaintNode(aNode: Pointer);
Begin
  PPaintNode(aNode)^.pnNext := FPaintFreeList;
  FPaintFreeList := aNode;
End;

Function TIpVT100Emulator.vttGenerateDECREPTPARM(aArg: Integer): String;
Const
  par = 1;
  nbits = 1;
  xspd = 120;
  rspd = 120;
Begin
  { Hard-coded values, since these really don't apply very well to WinSock }
  Result := Format(VT100ReportParm, [aArg + 2, par, nbits, xspd, rspd, 1, 0]);
End;

Procedure TIpVT100Emulator.vttInvalidateRow(aRow: Integer);
Var
  InvRect: TRect;
Begin
  If Terminal.HandleAllocated And
    (Terminal.ClientOriginRow <= aRow) And
    (aRow < Terminal.ClientOriginRow + Terminal.ClientRows) Then
  Begin
    InvRect.Left := 0;
    InvRect.Top := (aRow - Terminal.ClientOriginRow) * Terminal.CharHeight;
    InvRect.Right := Terminal.ClientWidth;
    InvRect.Bottom := InvRect.Top + Terminal.CharHeight;
    InvalidateRect(Terminal.Handle, @InvRect, False);
  End;
End;

Function TIpVT100Emulator.vttNewBlinkNode: Pointer;
Begin
  If (FBlinkFreeList = Nil) Then
    New(PBlinkNode(Result))
  Else
  Begin
    Result := FBlinkFreeList;
    FBlinkFreeList := PBlinkNode(Result)^.bnNext;
  End;
End;

Function TIpVT100Emulator.vttNewPaintNode: Pointer;
Begin
  If (FPaintFreeList = Nil) Then
    New(PPaintNode(Result))
  Else
  Begin
    Result := FPaintFreeList;
    FPaintFreeList := PPaintNode(Result)^.pnNext;
  End;
End;

Procedure TIpVT100Emulator.vttProcess8bitChar(aCh: AnsiChar);
Const
  FirstChar = #176;
  LastChar = #218;
  CvtChars: Array[FirstChar..LastChar] Of AnsiChar =
  'aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjl';
Var
  OldCharSet: Byte;
Begin
  { convert the line draw characters from the OEM character set to the }
  { VT100 linedraw charset }
  If (FirstChar <= aCh) And (aCh <= LastChar) Then
  Begin
    OldCharSet := Buffer.CharSet;
    Buffer.CharSet := 2;
    Buffer.WriteChar(CvtChars[aCh]);
    Buffer.CharSet := OldCharSet;
  End;
End;

Procedure TIpVT100Emulator.vttProcessCommand;
Var
  i: Integer;
  Arg: Integer;
  Arg2: Integer;
  OldDefChar: AnsiChar;
  Attrs: TIpTerminalCharAttrs;
Begin
  { assumption: this method is called immediately Parser.ProcessChar }
  {             returns pctComplete. Hence Parser.ArgumentCount,     }
  {             Parser.Argument, Parser.Command, Parser.Sequence     }
  {             are all set properly                                 }

{$IFDEF TermDebug}
  If (Terminal.SockControl <> Nil) And (Terminal.Socket <> Invalid_Socket) Then
    Terminal.SockControl.DebugLog.WriteDebugString(Parser.Sequence);
{$ENDIF}

  {WARNING: this case statement is in numeric order!}
  Case Parser.Command Of
    eGotoXY {or eCUP, eHVP}:
      Begin
        If (Parser.ArgumentCount = 2) Then
          Buffer.SetCursorPosition(Parser.Argument[0], Parser.Argument[1]);
      End;
    eUp {or eCUU}:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Arg := Parser.Argument[0];
          If (Arg = 0) Then
            Arg := 1;
          For i := 1 To Arg Do
            Buffer.MoveCursorUp(False);
        End;
      End;
    eDown {or eCUD, eIND, eVPR}:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Arg := Parser.Argument[0];
          If (Arg = 0) Then
            Arg := 1;
          For i := 1 To Arg Do
            Buffer.MoveCursorDown(False);
        End;
      End;
    eRight {or eCUF, eHPR}:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Arg := Parser.Argument[0];
          If (Arg = 0) Then
            Arg := 1;
          For i := 1 To Arg Do
            Buffer.MoveCursorRight(False, False);
        End;
      End;
    eLeft {or eCUB}:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Arg := Parser.Argument[0];
          If (Arg = 0) Then
            Arg := 1;
          For i := 1 To Arg Do
            Buffer.MoveCursorLeft(False, False);
        End;
      End;
    eSetMode {or eSM}:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Case Parser.Argument[0] Of
            4:
              Begin
                InsertMode := True;
                Buffer.UseInsertMode := True;
              End;
            20:
              Begin
                NewLineMode := True;
                Buffer.UseNewLineMode := True;
              End;
          End; {case}
        End
        Else
          If (Parser.ArgumentCount = 2) And
            (Parser.Argument[0] = -2) Then
          Begin
            Case Parser.Argument[1] Of
              1: AppKeyMode := True;
              2: ANSIMode := True;
              3:
                Begin
                  Col132Mode := True;
                End;
              4: SmoothScrollMode := True;
              5:
                Begin
                  RevScreenMode := True;
                End;
              6:
                Begin
                  RelOriginMode := True;
                End;
              7:
                Begin
                  WrapAround := True;
                  Buffer.UseAutoWrap := True;
                End;
              8: AutoRepeat := True;
              9: Interlace := True;
              999 {special IPRO value}: AppKeypadMode := True;
            End; {case}
          End;
      End;
    eSetAttribute {or eSGR}:
      Begin
        For i := 0 To pred(Parser.ArgumentCount) Do
        Begin
          Case Parser.Argument[i] Of
            0:
              Begin
                Buffer.SetCharAttrs([]);
                Buffer.ForeColor := Buffer.DefForeColor;
                Buffer.BackColor := Buffer.DefBackColor;
              End;
            1:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Include(Attrs, tcaBold);
                Buffer.SetCharAttrs(Attrs);
              End;
            4:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Include(Attrs, tcaUnderline);
                Buffer.SetCharAttrs(Attrs);
              End;
            5:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Include(Attrs, tcaBlink);
                Buffer.SetCharAttrs(Attrs);
              End;
            7:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Include(Attrs, tcaReverse);
                Buffer.SetCharAttrs(Attrs);
              End;
            8:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Include(Attrs, tcaInvisible);
                Buffer.SetCharAttrs(Attrs);
              End;
            22:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Exclude(Attrs, tcaBold);
                Buffer.SetCharAttrs(Attrs);
              End;
            24:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Exclude(Attrs, tcaUnderline);
                Buffer.SetCharAttrs(Attrs);
              End;
            25:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Exclude(Attrs, tcaBlink);
                Buffer.SetCharAttrs(Attrs);
              End;
            27:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Exclude(Attrs, tcaReverse);
                Buffer.SetCharAttrs(Attrs);
              End;
            28:
              Begin
                Buffer.GetCharAttrs(Attrs);
                Exclude(Attrs, tcaInvisible);
                Buffer.SetCharAttrs(Attrs);
              End;
            30: Buffer.ForeColor := clBlack;
            31: Buffer.ForeColor := clMaroon;
            32: Buffer.ForeColor := clGreen;
            33: Buffer.ForeColor := clOlive;
            34: Buffer.ForeColor := clNavy;
            35: Buffer.ForeColor := clPurple;
            36: Buffer.ForeColor := clTeal;
            37: Buffer.ForeColor := clSilver;
            40: Buffer.BackColor := clBlack;
            41: Buffer.BackColor := clMaroon;
            42: Buffer.BackColor := clGreen;
            43: Buffer.BackColor := clOlive;
            44: Buffer.BackColor := clNavy;
            45: Buffer.BackColor := clPurple;
            46: Buffer.BackColor := clTeal;
            47: Buffer.BackColor := clSilver;
          End; {case}
        End;
      End;
    eSaveCursorPos:
      Begin
        Buffer.GetCharAttrs(FSavedAttrs);
        FSavedBackColor := Buffer.BackColor;
        FSavedCharSet := Buffer.CharSet;
        FSavedCol := Buffer.Col;
        FSavedForeColor := Buffer.ForeColor;
        FSavedRow := Buffer.Row;
      End;
    eRestoreCursorPos:
      Begin
        Buffer.SetCharAttrs(FSavedAttrs);
        Buffer.BackColor := FSavedBackColor;
        Buffer.CharSet := FSavedCharSet;
        Buffer.Col := FSavedCol;
        Buffer.ForeColor := FSavedForeColor;
        Buffer.Row := FSavedRow;
      End;
    eDeviceStatusReport {or eDSR}:
      Begin
        { we *only* issue a reply if we are the master terminal,   }
        { otherwise if there were several terminals on a form, all }
        { attached to the same comport, the host would get several }
        { responses }
        If Terminal.IsMasterTerminal Then
        Begin
          If (Parser.ArgumentCount = 1) Then
            If (Parser.Argument[0] = 5) Then
              TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
                VT100StatusRpt, False)
            Else
              If (Parser.Argument[0] = 6) Then
                TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
                  Format(VT100CursorPos, [Buffer.Row, Buffer.Col]), False);
        End;
      End;
    eCHT:
      Begin
        If (Parser.ArgumentCount = 1) Then
          For i := 1 To Parser.Argument[0] Do
            Buffer.DoHorzTab;
      End;
    eCVT:
      Begin
        If (Parser.ArgumentCount = 1) Then
          For i := 1 To Parser.Argument[0] Do
            Buffer.DoVertTab;
      End;
    eDA:
      Begin
        { we *only* issue a reply if we are the master terminal,   }
        { otherwise if there were several terminals on a form, all }
        { attached to the same comport, the host would get several }
        { responses }
        If Terminal.IsMasterTerminal Then
        Begin
          If (Parser.ArgumentCount = 1) Then
            If (Parser.Argument[0] = 0) Then
              TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
                VT100DeviceAttrs, False)
            Else
              If (Parser.Argument[0] = 52) Then
                TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
                  VT52DeviceAttrs, False);
        End;
      End;
    eDCH:
      Begin
        If (Parser.ArgumentCount > 0) Then
          Buffer.DeleteChars(Parser.Argument[0]);
      End;
    eDL:
      Begin
        If (Parser.ArgumentCount > 0) Then
          Buffer.DeleteLines(Parser.Argument[0]);
      End;
    eECH:
      Begin
        If (Parser.ArgumentCount > 0) Then
          Buffer.EraseChars(Parser.Argument[0]);
      End;
    eED:
      Begin
        If (Parser.ArgumentCount = 1) Then
          Case Parser.Argument[0] Of
            0: Buffer.EraseToEOW;
            1: Buffer.EraseFromBOW;
            2: Buffer.EraseScreen;
          End;
      End;
    eEL:
      Begin
        If (Parser.ArgumentCount = 1) Then
          Case Parser.Argument[0] Of
            0: Buffer.EraseToEOL;
            1: Buffer.EraseFromBOL;
            2: Buffer.EraseLine;
          End;
      End;
    eHTS:
      Begin
        Buffer.SetHorzTabStop;
      End;
    eICH:
      Begin
        If (Parser.ArgumentCount > 0) Then
          Buffer.InsertChars(Parser.Argument[0]);
      End;
    eIL:
      Begin
        If (Parser.ArgumentCount > 0) Then
          Buffer.InsertLines(Parser.Argument[0]);
      End;
    eNEL:
      Begin
        Buffer.DoLineFeed;
        If Not NewLineMode Then
          Buffer.DoCarriageReturn;
      End;
    eRI:
      Begin
        Buffer.MoveCursorUp(True);
      End;
    eRIS:
      Begin
        Buffer.Reset; {also clears the screen}

        ANSIMode := ipc_VT100ANSIMode;
        AppKeyMode := ipc_VT100AppKeyMode;
        AppKeypadMode := ipc_VT100AppKeypadMode;
        AutoRepeat := ipc_VT100AutoRepeat;
        Col132Mode := ipc_VT100Col132Mode;
        GPOMode := ipc_VT100GPOMode;
        InsertMode := ipc_VT100InsertMode;
        Interlace := ipc_VT100Interlace;
        NewLineMode := ipc_VT100NewLineMode;
        RelOriginMode := ipc_VT100RelOriginMode;
        RevScreenMode := ipc_VT100RevScreenMode;
        SmoothScrollMode := ipc_VT100SmoothScrollMode;
        WrapAround := ipc_VT100WrapAround;

        FUsingG1 := False;
        FG0CharSet := ipc_VT100G0CharSet;
        FG1CharSet := ipc_VT100G1CharSet;
      End;
    eRM:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          Case Parser.Argument[0] Of
            4:
              Begin
                InsertMode := False;
                Buffer.UseInsertMode := False;
              End;
            20:
              Begin
                NewLineMode := False;
                Buffer.UseNewLineMode := False;
              End;
          End; {case}
        End
        Else
          If (Parser.ArgumentCount = 2) And
            (Parser.Argument[0] = -2) Then
          Begin
            Case Parser.Argument[1] Of
              1: AppKeyMode := False;
              2: ANSIMode := False;
              3:
                Begin
                  Col132Mode := False;
                End;
              4: SmoothScrollMode := False;
              5:
                Begin
                  RevScreenMode := False;
                End;
              6:
                Begin
                  RelOriginMode := False;
                End;
              7:
                Begin
                  WrapAround := False;
                  Buffer.UseAutoWrap := False;
                End;
              8: AutoRepeat := False;
              9: Interlace := False;
              999 {special IPRO value}: AppKeypadMode := False;
            End; {case}
          End;
      End;
    eTBC:
      Begin
        If (Parser.Argument[0] = 0) Then
          Buffer.ClearHorzTabStop
        Else
          If (Parser.Argument[0] = 3) Then
            Buffer.ClearAllHorzTabStops;
      End;
    eDECSTBM:
      Begin
        Arg := Parser.Argument[0];
        If (Arg = -1) Or (Arg = 0) Then
          Arg := 1;
        Arg2 := Parser.Argument[1];
        If (Arg2 = -1) Or (Arg2 = 0) Then
          Arg2 := Buffer.RowCount;
        Buffer.SetScrollRegion(Arg, Arg2);
      End;
    eENQ:
      Begin
        { we *only* issue a reply if we are the master terminal,   }
        { otherwise if there were several terminals on a form, all }
        { attached to the same comport, the host would get several }
        { responses }
        If Terminal.IsMasterTerminal Then
        Begin
          TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
            Answerback, False);
        End;
      End;
    eBel:
      Begin
        MessageBeep(MB_ICONASTERISK);
      End;
    eBS:
      Begin
        Buffer.DoBackspace;
      End;
    eLF:
      Begin
        Buffer.DoLineFeed;
      End;
    eCR:
      Begin
        Buffer.DoCarriageReturn;
      End;
    eSO:
      Begin
        If Not ANSIMode Then
          Buffer.CharSet := 2
        Else
          Buffer.Charset := FG1CharSet;
        FUsingG1 := True;
      End;
    eSI:
      Begin
        If Not ANSIMode Then
          Buffer.CharSet := 0
        Else
          Buffer.Charset := FG0CharSet;
        FUsingG1 := False;
      End;
    eIND2:
      Begin
        Buffer.MoveCursorDown(True);
      End;
    eDECALN:
      Begin
        {scroll buffer up, fill screen with character E's}
        OldDefChar := Buffer.DefAnsiChar;
        Buffer.DefAnsiChar := 'E';
        Buffer.EraseScreen;
        Buffer.DefAnsiChar := OldDefChar;
      End;
    eDECDHL:
      Begin
        If (Parser.ArgumentCount = 1) Then
        Begin
          With TIpLineAttrArray(FLineAttrArray) Do
            If (Parser.Argument[0] = 0) Then
              Attr[Buffer.Row] := ltDblHeightTop
            Else
              Attr[Buffer.Row] := ltDblHeightBottom;
          vttInvalidateRow(Buffer.Row);
        End;
      End;
    eDECDWL:
      Begin
        With TIpLineAttrArray(FLineAttrArray) Do
          Attr[Buffer.Row] := ltDblWidth;
        vttInvalidateRow(Buffer.Row);
      End;
    eDECLL:
      Begin
        For i := 0 To pred(Parser.ArgumentCount) Do
          Case Parser.Argument[i] Of
            0: LEDs := 0;
            1: LEDs := LEDs Or $1;
            2: LEDs := LEDs Or $2;
            3: LEDs := LEDs Or $4;
            4: LEDs := LEDs Or $8;
          End; {case}
      End;
    eDECREQTPARM:
      Begin
        { we *only* issue a reply if we are the master terminal,   }
        { otherwise if there were several terminals on a form, all }
        { attached to the same comport, the host would get several }
        { responses }
        If Terminal.IsMasterTerminal Then
        Begin
          Arg := 1;
          If (Parser.ArgumentCount = 1) Then
          Begin
            Arg := Parser.Argument[0];
            If (Arg < 0) Then
              Arg := 0
            Else
              If (Arg > 1) Then
                Arg := 1;
          End;
          TIpSockControlFriend(Terminal.SockControl).scPutString(Terminal.Socket,
            vttGenerateDECREPTPARM(Arg), False);
        End;
      End;
    eDECSWL:
      Begin
        With TIpLineAttrArray(FLineAttrArray) Do
          Attr[Buffer.Row] := ltNormal;
        vttInvalidateRow(Buffer.Row);
      End;
    eDECTST:
      Begin
        {}
      End;
    eDECSCS:
      Begin
        If Parser.ArgumentCount = 2 Then
          Case Parser.Argument[0] Of
            0:
              Begin
                Case Parser.Argument[1] Of
                  0: FG0CharSet := 2;
                  1: FG0CharSet := 3;
                  2: FG0CharSet := 4;
                  ord('A'): FG0CharSet := 1;
                  ord('B'): FG0CharSet := 0;
                End;
                If Not FUsingG1 Then
                  Buffer.Charset := FG0CharSet;
              End;
            1:
              Begin
                Case Parser.Argument[1] Of
                  0: FG1CharSet := 2;
                  1: FG1CharSet := 3;
                  2: FG1CharSet := 4;
                  ord('A'): FG1CharSet := 1;
                  ord('B'): FG1CharSet := 0;
                End;
                If FUsingG1 Then
                  Buffer.Charset := FG1CharSet;
              End;
          End {case}
      End;
  End; {case}
End;

Procedure TIpVT100Emulator.vttScrollRowsHandler(aSender: TObject;
  aCount, aTop, aBottom: Integer);
Begin
  TIpLineAttrArray(FLIneAttrArray).Scroll(aCount, aTop, aBottom);
End;

Procedure TIpVT100Emulator.vttSetCol132Mode(aValue: Boolean);
Begin
  If (aValue <> Col132Mode) Then
  Begin
    FCol132Mode := aValue;
    If Col132Mode Then
      Terminal.Columns := 132
    Else
      Terminal.Columns := 80;
  End;
End;

Procedure TIpVT100Emulator.vttSetRelOriginMode(aValue: Boolean);
Begin
  If (aValue <> RelOriginMode) Then
  Begin
    FRelOriginMode := aValue;
    Buffer.UseAbsAddress := Not aValue;
  End;
End;

Procedure TIpVT100Emulator.vttSetRevScreenMode(aValue: Boolean);
Begin
  If (aValue <> RevScreenMode) Then
  Begin
    FRevScreenMode := aValue;
    FRefresh := True;
  End;
End;

Procedure TIpVT100Emulator.vttToggleNumLock;
Var
  NumLockState: TKeyboardState;
Begin
  GetKeyboardState(NumLockState);
  If ((NumLockState[VK_NUMLOCK] And $01) = 0) Then
  Begin
    NumLockState[VK_NUMLOCK] := NumLockState[VK_NUMLOCK] Or $01;
  End
  Else
  Begin
    NumLockState[VK_NUMLOCK] := NumLockState[VK_NUMLOCK] And $FE;
  End;
  SetKeyboardState(NumLockState);
End;
{====================================================================}

{===Initialization/finalization======================================}

Procedure IPTrmEmuDone; Far;
Var
  Node, Temp: PTermEmuLink;
Begin
  {dispose of active links}
  Node := TermEmuLink;
  While (Node <> Nil) Do
  Begin
    Temp := Node;
    Node := Node^.telNext;
    Dispose(Temp);
  End;
  {dispose of inactive links}
  Node := TermEmuLinkFreeList;
  While (Node <> Nil) Do
  Begin
    Temp := Node;
    Node := Node^.telNext;
    Dispose(Temp);
  End;
End;

Initialization
  TermEmuLink := Nil;
  TermEmuLinkFreeList := Nil;
  InvRectFreeList := Nil;
  InvRectPageList := Nil;

Finalization
  IPTrmEmuDone;
  IPTFreeInvRectPages;
End.
